! //   ********************************************************************
! //                              CoPSE3d_LST Code
! //   ********************************************************************
! //
! //  For more information, please contact:
! //  Author         : M. S. Mughal 
! //  Contacts       : M. S. Mughal (s.mughal@imperial.ac.uk) 
! //                   Department of Mathematics, 
! //                   Imperial College, London
! //                   London SW7 2AZ, UK.
! //
! //
! //  Copyright (c) 2021 Imperial College London (UK),  
! //
! //  Permission is hereby granted, free of charge, to any person obtaining a
! //  copy of this software and associated documentation files (the "Software"),
! //  to deal in the Software without restriction, including without limitation
! //  the rights to use, copy, modify, merge, publish, distribute, sublicense,
! //  and/or sell copies of the Software, and to permit persons to whom the
! //  Software is furnished to do so, subject to the following conditions:
! //
! //  The above copyright notice and this permission notice shall be included
! //  in all copies or substantial portions of the Software.
! //
! //  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
! //  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
! //  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
! //  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
! //  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
! //  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
! //  DEALINGS IN THE SOFTWARE.
! //  -----------------------*- CoPSE3d : Background -*------------------------
! // 
! //   Project        : Advanced Boundary-layer Transition modeling using LST, PSE
! //                    Based Approaches.
! // 
! //   CoPSE3d        : Identical to CoPSE_d + True 3d analysis capability +
! //                  : Receptivity analysis using residues.
! // 
! //        Program Solving 3d Compressible Non-Parallel, Linear and
! //        Non-Linear Parabolised Stability Equations.
! //        Fourth-order Accurate Finite-Difference Discretisation.
! // 
! //        Solves for Infinite-Swept 3d Wing Flows, 2d Aerofoil or
! //        Flat-Plate Type Flows. True 3d-Linear analysis capability
! //        in generalised non-orthogonal coordinates.
! // 
! //  -------------------------*- Fortran-90 -*--------------------------------
! // 	version='3.b.0.00' :: Nektar++ integration, Only the LST solvers have 
! //                              been retained. (10/March/2021
! //                              The more complete CoPSE3d solver version= '3.a.0.09' 
! //                              includes : LST, LPSE, NL-PSE and PPM-PSE3D
! // 
! //  -------------------------*- Fortran-90 Compilation -*-----------------------
! // 
! //  Compilation: (some warning signs but non-important)
! //     gfortran: gfortran -O2 -ffree-line-length-0 -fdefault-double-8 -fdefault-integer-8 -fdefault-real-8
! //        intel: ifort -O3 -r8 -i8 -prec-div -fp-model precise
! //  For bugs etc. :
! //  Contact       : M. S. Mughal (s.mughal@imperial.ac.uk) 
! //  
! //   ********************************************************************  
! //   ********************************************************************
! //  April/2021::   
! //  NOTE:  Complex polynomial Laguerre method code by 
! //  Jan Skowron & Andrew Gould    
! //  refer to: http://www.apache.org/licenses/LICENSE-2.0
! // 
! accuracy module
        module header_info
	CHARACTER*8, parameter :: version='3.b.0.00'
        integer, parameter :: ndigit=12, npower=35
        integer, parameter :: long = selected_real_kind(ndigit,npower)
     end module header_info
!    
! basic array setup module     
        Module param_info
!   *******************************************************************
! 
!       Important Control Variables Set via the PARAMETER Statement :
! 
!       MP_LOC  : Maximum number of grid points allowable in the wall
!                 normal direction for space marching using the IPSE,
!                 IOSE & INONLIN options
!       MP_GLOB : Maximum number of grid points allowable in the wall
!                 normal direction for the Global Method
!       NQ      : Maximum number of data points allowable in defining
!                 the steady boundary-layer data, which is read from
!                 file PSEMEAN.PRO or PSECoDS.PRO
!       NPOST   : Maximum number of grid points allowable in the
!                 streamwise, space marching direction
!       MODE    : Maximum number of modes allowable in the streamwise
!                 direction in the nonlinear PSE.
!                 Also controls the maximum number of frequencies or
!                 waves which can be tracked in the linear PSE - with
!                 this option, the maximum available is 2*MODE+1.
!       KMODE   : Maximum number of modes allowable in the crossflow
!                 direction in the nonlinear PSE.
!                 Also controls the maximum number of multi-frequency
!                 waves which can be tracked in the linear PSE, over a
!                 range of crossflow wave numbers  - with
!                 this option, the maximum available is 2*KMODE+1.
!                 ie. Both MODE & KMODE serve a dual role
!                 KMODE in addition serves as the array to analyse
!                 linear fully 3d analysis. i.e. no. of stations in z
!                 spanwise direction.
!
	integer, parameter :: MP_LOC = 402, MP_GLOB = 162
	integer, parameter :: MODE = 17, KMODE = 17
	integer, parameter :: NPOST = 2002
	integer, parameter :: MP = MP_LOC, MPG = MP_GLOB
     end module param_info 
     
     
        Module param_blprof
	integer, parameter :: NQ = 650
     end module param_blprof
     
     	Module param_NUMBERCRUNCH
	use param_info, only : MP, MODE, KMODE, NPOST
	PARAMETER (ILQQ=9,KEO=(MP-1)*ILQQ)
	PARAMETER (KFRO=2*MODE+1,KBETAO=2*KMODE+1)
	COMPLEX*16, dimension(KFRO,KBETAO,KEO) :: ORRRHS, ORRDCRHS
     end module param_NUMBERCRUNCH           
     
     	Module param_FILEADD
        CHARACTER*40 LABEL
	integer :: ICHART
     end module param_FILEADD 
              
     	Module GPL_STUFF
	parameter (NQYR = 1249, NQMN=6, NQDy=1)
	INTEGER :: idateLC(3)
	integer :: ILCxx
	REAL*8 :: pLC
	end module GPL_STUFF
	
	! Input parameters
	module cmdInput
	    implicit none
		integer :: option
	    integer :: numSteps1,   numSteps2
		real*8  :: finalValue1, finalValue2	
	end module cmdInput
	
	
! Start of main program  : More general calls to LPSE, NL-PSE and PPM-PSE3D commented out.
!                          
!     Version capability : Spatial analysis for LST, Neutral curves analysis,
!                          LST N-factors analysis, LST envelopes analysis,
!                          global call to QZ routine, Marching in x - Global QZx call.
!                          Temporal local analysis : global call to QZ temporal analysis.
!                          Spatial and Temporal Eigenvalues and spectrum outputs.
!                          
!                          (10/March/2021
!
! 
! Main driver routine:
	! Parameter passing test
	!subroutine pass_int2(a)
	!	integer :: a
    !	write(*,*)"  In copse3d, a = ", a
	!end subroutine pass_int2
	!
	!subroutine pass_double2(b)
	!	real*8 :: b
	!	write(*,*)"  b = ", b
	!end subroutine pass_double2
	subroutine helloWorld
	    implicit none
	    write(*,*) "Hello Fortran!"
	end subroutine helloWorld
	
	
	subroutine CoPSE3d(opt, val1, num1, val2, num2)
	    use cmdInput
		integer :: opt
	    integer :: num1, num2
	    real*8  :: val1, val2
	
		! Save the inpute parameters
		option      = opt
		numSteps1   = num1
		finalValue1 = val1
		numSteps2   = num2
		finalValue2 = val2

	
	
        CALL FRONT(IERR)
        IF(IERR == 1)GOTO 9099
        CALL ZDEFLT(I3D,ISTRIP,IPHINS,IOP,INON,I3DNEQ,IGLBX,IJOK,IDOZ, &
     & IRECEPT,IPOSTENV,ISETUP3D,ISEP3D,IPROPT, IERR)
     
     	IF(IPROPT == 1)THEN
	CALL INITLIZ_OPTIM
	STOP
	END IF
     
        IF(IERR == 1)GOTO 9099
! directs program to post-processing routines....
     	IF(IPOSTENV == 1 .and. ISETUP3D == 0) GOTO 9199
! SETUP_PSE3D
	IF(ISETUP3D == 1 .and. I3D == 1)GOTO 7766
         
        CALL ZEROAL(I3DNEQ)


	IF(I3D == 1)THEN
	
! SETUP_PSE3D

! many many options possible here..especially regarding
! treatemnt of beta.
		IF(ISTRIP == 1  .AND.  IOP == 1)THEN
!		CALL STRIP3D(I3DNEQ,IDOZ,IERR) <== this works too?...
! in fact above was written earlier for cartesian coordintes. since now
! work in h1, h2's   redundant....works but fails when getting to end3d
! calc..as this is now only for horton-formatted data.
! this option is that Beta is kept fixed (infinite swept wing assumption)
!		CALL STRIP3Dx(I3DNEQ,IDOZ,IERR)
		ELSE IF(IOP == 0)THEN
	PRINT*,' Sorry this option (OSE3d) not set up yet'
		IF(ISTRIP == 1)THEN
!		CALL OSE3D(I3DNEQ,IDOZ,IERR)
!  in actutal fact is set up..but needs to be finally checked out.
! in that all parameters are scaled correctly.
    PRINT*,' In fact is, OSE3d_strip requires further fine tuning/checks'
		ELSE
    PRINT*,' Future work :: OSE3d '
		END IF
	WRITE(*,*) CHAR(7)
		
		ELSE 
!		PRINT*,' input type 1(new) or 0 (old)'
!		READ*,ISEP3D
!		IF(ISEP3D == 0)THEN
!		CALL PSE3D(1,I3DNEQ,IDOZ,IERR)
!		ELSE
!		CALL PSE3D_SEP(1,I3DNEQ,IDOZ,IERR)
!		END IF
!		CALL PSE3D2D(1,I3DNEQ,IDOZ,IERR)
		end if
        ELSE  IF(IPHINS == 1)THEN
        CALL PHABINS(I3DNEQ,IERR)
        ELSE IF(IOP == 1  .AND.  INON == 0)THEN
!        CALL PSELIN(I3DNEQ,IERR)
        ELSE IF(IOP == 1  .AND.  INON == 1)THEN
		IF(IRECEPT == 1)THEN
!       		CALL RECNON(I3DNEQ,IERR)
		else
!       		CALL PSENON(I3DNEQ,IERR)
		end if
        ELSE
		IF(IGLBX == 1)THEN
                CALL OSEGLBX(I3DNEQ,IERR,1)
                ELSE
                CALL OSE(I3DNEQ,IERR)
                END IF
        END IF
        IF(IERR == 1)GOTO 9099

	IF(IJOK == 0)THEN
	WRITE(*,*)' Finished !!!'
	WRITE(*,*) CHAR(7)
	PRINT*,CHAR(7)
	END IF

	STOP
9099    CONTINUE
	IF(IERR == 1)WRITE(*,*) CHAR(7)
	stop
9199    CONTINUE
! directs program to post-processing routines....
	IOK=0
	IF(I3D  == 1)THEN
!	CALL POST_COPSE3D(IOK)
	ELSE
!     	CALL POSTENV(IOK)
	END IF

	IF(IOK == 1)THEN
	WRITE(*,*)' Finished succesfully!!!'
	WRITE(*,*) CHAR(7)
	END IF	
	
	IF(IOK == -1)THEN
	WRITE(*,*)' Error in envelope files!!!'
	WRITE(*,*) CHAR(7)
	END IF	
	stop
	
7766	CONTINUE
	IOK=0
!	CALL SETUP_PSE3D(ISTRIP,IOK)
        END subroutine copse3d



	SUBROUTINE ZDEFLT(I3D,JSTRIP,IPHINS,IOP,INON,I3DNEQ,     & 
     & IGLOBX,IJOK,IDOZ,IREXEPT,IPOSTENV,ISETUP3D,ISEP3D, IPROPT,IERR)
        use param_info, only : MP, MODE, KMODE, MPG
	use param_FILEADD
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	IMPLICIT INTEGER (I-N)
        PARAMETER (IPROB=9, IL=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*72 USERENV
	CHARACTER*80 DESTIN,SUBDIR
	DOUBLE PRECISION AMP(0:MODE,-KMODE:KMODE)
	DOUBLE PRECISION DIMSCL(4)
        DOUBLE PRECISION, DIMENSION(IPROB) :: AI, BC
	INTEGER JOUTSC(2)
	COMPLEX*16 SING,ALFA,DADX,AUN
	COMPLEX*16 QZPRED(0:MODE,-KMODE:KMODE)
	COMPLEX*16 QZLIN(KSTR)
	REAL*8 FRENCY(KSTR),WAVBETA(KCRF),WAVALFA(KSTR)
	REAL*8 WAVALIM(KSTR),CRFBETA(KCRF),OMRANGE(KSTR)
        REAL*8 STRALFA(KSTR),STRALIM(KSTR)
	CHARACTER*11 frm
	COMPLEX*16 WAVLOCL(KSTR)
        INTEGER LTMP(0:MODE,-KMODE:KMODE,2)
	INTEGER ITRANS(0:MODE,-KMODE:KMODE)
        INTEGER IPLATE(-MODE:MODE,-KMODE:KMODE)
	INTEGER IPLOCK(0:MODE,-KMODE:KMODE)
	INTEGER IFLOQ(-MODE:MODE,-KMODE:KMODE)
	INTEGER NFFLAG(KSTR,KCRF)
	LOGICAL YUP
	EXTERNAL TENPOW
	NAMELIST /PSEUSER/NSTART,NFINAL,NINTR,IUmax,ICRSFLO,      & 
     & KEU,KEUV,KEUVW,IUHerb,MASFLX,KEMAX,IGlb,IOSE,IPSE,INONLIN, & 
     & YLAMBDA,WOMG,FREQLIN,BETALIN,NFSWICH,IATTACH,IVARIP, &
     & I2DCOMP,I3DCOMP,ITSE,WAVELIN,WAVEIM,IGLBX,NFACTOR, &
     & ITMAX,ICURVI,ICURVV,IC2UBAR,IWMAX,Lookup,IMULTF2,IDO3D,NZDIR, &
     & FDXDZ,IPSEZ,IELLIPT,IUPWIND,ICHARBX,IUPSMRT,IZERO,ISTRIP, &
     & Ineutrl,Isim,Ibuoy,IUPIMP,IUPEDGE,IRECEPT,NZSTART, &
     & IPOSTENV,xtransition, XDLIMIT, IOVERIDE, ISETUP3D, &
     & ITRANSONIC, IREVFLOW, INDENT, IGLB3D, IEXTRAP3D, ISEP3D
	NAMELIST /PSENONL/IAlfA,IBeta,AMP,IPLOCK,IOB,MAGNIFY,NSTAT, &
     & Freq,Beta,IFLOQ,BC,IADIAB,ITHRESH,ISYMM,IQE,IPRCENT, &
     & IEADSIW, XFORMIN, XFORMAX,RIBLET_URMS,IRIB_URMS,ILoF 
        NAMELIST /PSECNTL/YGlobal,YLocal,NGlobal,NLocal,ITolRay,Grid, &
     & NitIray,ITolSpc,ITolNon,IRAyCnt,NitMode,NGlbMd,DIFFER, &
     & IASYMP,xstrech,ythick,INAG, IQZHI8, IROBUST, ixx33, &
     & IBACOMPAT, IPROPT
        NAMELIST /PSEOUT/IeigL,IeigS,IntvAl,RCptur,IWVLNTH, &
     & ItSpres,NRMLOCL,NRMSTRT,NRMDIM,LABEL,ISCREEN,IWVDUMP,IQZFULL, &
     & IOUTXC,IOUTSC,RESCALE,NRMCORD,IBAE,NATURAL,IEURO,ICoDS, &
     & I2NDARY, USERENV
	COMMON /UMAXDIR/ILoF
	COMMON /SUBSONIC/ITRANSONIC
	COMMON /EADS_ROB/XFORMIN, XFORMAX,RIBLET_URMS
	COMMON /EADS_IFLAG/IEADSIW,IRIB_URMS
	COMMON /xtpos/xtransition, XDLIMIT
	COMMON /ENVIT/USERENV
	COMMON /NAGBIT/INAG,IWVLNTH
	COMMON /K3D/IDO3D, IGLB3D, IEXTRAP3D, IGLB3D_EXIST
	COMMON /SXXXXX/ISIM,Ibuoy
	COMMON /flexib/xstrech,udel
        COMMON /NCURVES/INEUTRL     
	COMMON /FSU/FoSQ
        COMMON /OBLIQUE/IOB
        COMMON /ITESTIT/IZERO
        COMMON /IHOW3D/IUPWIND,ICHARBX,IUPSMRT,IUPIMP,IUPEDGE
	COMMON /DIRTYPE/frm
	COMMON /DISCR3D/xellips,ox
     	COMMON /CRANCKN/e,em1
	COMMON /MAD3DP/FDXDZ
	COMMON /WING3DP/NZDIR,NZSTART
        COMMON /MALCROS/KEMAX,IQE,I100
	COMMON /HLPLOC/DESTIN,SUBDIR
	COMMON /AWESOME/IOSE
	COMMON /HLPINT/IDEST,IDEST2
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /DONALD/IKEJUMP
	COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR
	COMMON /MFDADIB/IADIAB
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /PHCUTTY/WAVLOCL	
	COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /VENUS/IATTACH
        COMMON /FAIRY/TOLCUT
	COMMON /QZAUTO/NGLOBAL
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
        COMMON /QZBETA/WAVBETA
        COMMON /QZXXXX/NCROSS
	COMMON /PATERN/LTMP,ITRANS,IPLATE
        COMMON /FOURIR/AMP
        COMMON /FOURINT/IALFA,IBETA,IFL
	COMMON /QCNTRL/ND
	COMMON /EQTYPE/AI,BC
        COMMON /DECIDE/YEDGE,OUTER,RESHMA,BIBI
	COMMON /ZCLEAR/SING
	COMMON /DISATO/AUN,frAct
	COMMON /GLOBAL/ALFA,TolRAy,ToLocal,TolSpc,TolNon
        COMMON /NLOGAL/NitMode,NGlbMd
	COMMON /WHATP/IEIGL,NITIRAY,IRAYCNT
        COMMON /WHREL/PXFACT
	COMMON /SPFLG/IEIGS,INTVAL,IHOW,IPAR
        COMMON /QZMETH/INGUESS,IGLB
        COMMON /ZEBRAQ/QZPRED
	COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB, & 
     & IWSQ,IWMAX
	COMMON /TWOMODE/ITMAX,IUNOMAL,IFTX,ISELECT
	COMMON /AMPLIT/NFACTOR,NFFLAG
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /NICEINF/RESIDU
        COMMON /NICINTG/ICNORM
	COMMON /TRAPST/RCPTUR,IWETH
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /PHASIT/ITWO,KCARRY
        COMMON /PAFLQO/IFLOQ
	COMMON /PHASEC/IPLOCK
        COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /STRETCH/GRID
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
	COMMON /POLISH/IOMSW
        COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TO,T1,STOKES
        COMMON /PISAVE/PI
	COMMON /REVERSE/ISYMM
	COMMON /HOHOHO/IMULTF2, IROBUST
	COMMON /GUI/IGUI
	COMMON /KEEEEP/IKIK
	COMMON /TRYIT/MAGNIFY,NSTAT,IDONEIT,IMANYS
	COMMON /FARFELD/IASYMP
	COMMON /HUSSOK/IONE2
	COMMON /ALEENAQZ/IQZHI8
	COMMON /IFLARE/IREVFLOW,IBACOMPAT
	common /argbargy/ixx33
	COMMON /EEC/INDENT
	IKIK=0
        PI=4.D0*DATAN(1.D0)
	IOMSW=0
	Lookup=0
	ixx33=9999999999
	
! REVERSE FLOW FLAG set convective terms to zero ..flare type approx.
	IREVFLOW=0
! EMMA COOKE indentation analysis flag:
	INDENT=0
! EC_To output more balanced .PRO file
	IPROPT=0

! EADS forced problem...
	IEADSIW=0
	XFORMIN=1.E11
	XFORMAX=1.E22
	RIBLET_URMS=1.d0
	IRIB_URMS=0
	
! PSE3D flag, which if 1, then 	
	ITRANSONIC=0
	ILoF=0   ! wing-chord umax in nonlinear....	

	MAGNIFY=1
	NSTAT=99999
	IDONEIT=0
	
! this set to 1 calls nag routine to evaluate roots exactly
! in eigenvalue search: otherwise set to zero. uses incompressible values!
! so will have problems in compressible limit....
! replaced with alternative by Skowron & Gould 2012 :: 04/2021 smughal	
	INAG=1
! to output wavelenths as opposed to wavenumbers... set this to 1
! but only works for RESLIN and not elsehwere....and 1/0 problems
! so code will crash if this occurs.  abit troublesome to sort out error
! handling..then! so beware..only use if know what your doing.
	IWVLNTH=0
	
        BC(1:IL)=0.D0

!	flag for 3d wing analysis....
	IDO3D=0
! just to see hwhether setting all other palces to zero works!
	IZERO=0
!	IPSEZ=1 (PSEin Z) or OSE calc to initiate!
	IPSEZ=0

! ISTRIP
!	this controls whether truley 3d or striop based nalayiss
	ISTRIP=0
	ISETUP3D=0
! PPM-PSE3D initialisation options...Nov. 2018
	IGLB3D= 0
	IEXTRAP3D=-99
!		
! above for seting up lots of runs....
!	no. of Z stations
	NZSTART=1
	NZDIR=NZSTART
	FDXDZ=1.D0
	
! moved over to read in from CoBLc3D or QIC3D (24 jan 2005)
! this makes life much easier for black box use.
!       data in form of Horton non-orthogonal data set if NONORTH=1
!	NONORTH=0
!	data in PSEDIRECT.PRO is direct access formatted type.
!	IFORMAT=1
	
!    neutral curves flag, and bl similarity flag
        INEUTRL=0
	ISIM=0
! bouyoancy term switched on......in this version FoSQ will not work..its 
! been effectively swotched off for good..simple to chnage..PRFCoPS routine
! check and compare with FSU version of code 
	Ibuoy=0		
	FoSQ=1.D66

! flag for receptivity analysis
	IRECEPT=0

!	flag for elliptic analysis
!	i.e ==1 does elliptic bit.
! ok have in fact siwtched the true ellipticity off in equations.
! so in fact only does centered differencing either explicit or
! implicit or semi-implicit, with differ controlling this.
! ie. only marching parabolic system of equiations....
	IELLIPT=0
	ISEP3D=0

! only use outer inviscid flow vector to detrmine differencing.
	IUPEDGE=0

	IUPWIND=1
	IUPSMRT=1
! fully upwinded implicit diffing then sett this to 1 too.
	IUPIMP=0
! setting iupwind to zero forces central differnced scheme to be operative.
!	if you want acharcteristic based step size .. experimenatl..
!     in fact much of the 3d stuff still is!
	ICHARBX=0

!	flag to use puvwt variables (primitive)
	IVARIP=0
!	IVARIP=1 (ro,uvwt varibales density)

	SING=DCMPLX(0.D0,1.D0)
	SILLYB=-999999.0D10
	SILLYZ=-999999.0D9
	ISILLY=-999999
	Freq=SILLYB
	FREQLIN=SILLYB
	BETALIN=SILLYB
	WAVELIN=SILLYB
	WAVEIM=0.D0
	RGORTL=SILLYB

	DO I=0,MODE
	DO J=-KMODE,KMODE
	IPLOCK(I,J)=0
	AMP(I,J)=SILLYB
	IFLOQ(-I,J)=1
	IFLOQ(I,J)=1
	END DO
	END DO

!       DefAult vAlues for dimensionalising scale

	NRMLOCL=1
	NRMCORD=0
	NRMSTRT=0
	NRMDIM=0
	DIMSCL(1)=0.D0
	DIMSCL(2)=0.D0
	DIMSCL(3)=0.D0
	DIMSCL(4)=0.D0

!	cranck nicolson switch in pse3d
	DIFFER=0.D0

!       final output of results scaling.. altering rescale forces
!       wavenumbers and local reynolds number to be rescaled
!       with *RESCALE
!       ioutxc & ioutsc selection of whether output with respect
!       to x/c or s/ci - default will be with s/c.
        RESCALE=1.D0
        IOUTXC=0
        IOUTSC=0
	
! Backward compatbility in reading .PRO data files.. tdisp variable,
	IBACOMPAT=0

!       DefAult vAlues
! DESTIN etc..was the help file facility..which was an online lookup of
! each namelist command and short description of it...works automatically
! if the appropriate help files are in the DESTIN directory.
        LABEL='copse'
        DESTIN='/usr/local/CoPSEBL/'
        SUBDIR='hlp/'
        ISCREEN=1
!  this flag routes to post-processing bits...
	IPOSTENV=0
	USERENV='PSE.ENV.cases'
	xtransition= -99999.
	XDLIMIT=0.000000002d0

!       Curvature flag
        ICURVI=0
!       for later use, which switches on the viscouse nonlinear contrib.
        ICURVV=0
!
	IC2UBAR=1
! above is not used..I suspect, neither is ICURVV yet

! 	2d compressible or 3d compressible....not done yet wholey
!	3d will be default
	I3DCOMP=0
	I2DCOMP=0

!	flag if global method required at all x
	IGLBX=0
	IMULTF2=1

! 	flag which uses asymptotic bc's in far-field for IOSE option.
	IASYMP=1

	ITSE=0
	IPHINS=0
        IOSE=1
	IPSE=0
	IOP=1
	INONLIN=0

!	flag to select whether cross-flow or TS instability to
!	be examined : 0 (TS) , 1 (Cross-Flow)...rubbish..not used!
! used this very early on..in InPSE days...
	I3DSTAB=0
!	full eigenvalue spectrum output
	IQZFULL=1
! call's 8th-order accurate QZ solver....
	IQZHI8=0

!	attachment line analysis  If IATTACH=1 correct only for
!	attachment point..... use with care!!!
	IATTACH=0

!  cut off thresh-hold
        ITHRESH=15

!	output amplitudes as percentages
	IPRCENT=0

! 	normalise nonlinear amplitudes with Qe 
	IQE=0

!	Active control bit
	YLAMBDA=0.D0
	WOMG=0.D0

! GLBX range of analysis for OSE robustness.
	IROBUST=-1
	NSTART=2
	NFINAL=NSTART
	NINTR=0

!	non-parallel iteration history... redundant...in In2D it does work.
!       In this code. DADX is zero. FRACT not used.
	IPAR=0
	DADx=(0.0D0,0.0D0)
	FrAct=1.D-4 

	IAlfA=ISILLY
	IBeta=ISILLY
	BETA=0.D0
!	does a symettrical disturbanceif ISYMM=1
	ISYMM=0

!	cross flow flAg if zero Assumes only ts wAve Analysis required
!	overidden by specifying betalin in namelist
	ICRSFLO=0

!	nfActor switches... first Allows nfActors to be cALCulated 
!	NFSWICH Allows nfActors to be oprAtionAl even if nf curve shows
!	return to stAble regime. 
	NFACTOR=1
	NFSWICH=0

	IeigL=0
	PxFAct=100.D0
	IeigS=0
	IntvAl=1
	RCptur=0.D0
	ItSpres=0

	IGlb=0
	ISPEC=0
	IUMAX=0
	KEU=0
	KEUV=0
        KEUVW=0
	MASFLX=0
	ITMAX=0
	IWMAX=0
	IUHerb=1
        KEMAX=0

!	evaluation of closure based on Eurotrans approach and
!	subsequent output of results in eurotrans format
	IEURO=0
!	flag to output nonlinear results for secondary-instability analysis
	I2NDARY=0
!	input data in CoDS /Eurotrans format
	ICoDS=0
!	for output of results according to BAe (Airbus) requirements.
	IBAE=0
!	Natural output of n factors, preffered by me !!!
	NATURAL=1

!	Adiabatic wall flag : IADIAB =1 ON
	IADIAB=0

!	perhAps mAke this A multiple of first profile looked At
!	15*Yedge sAy ? or look at AsymptotIC bc's <= old versions of CoPSE
! 	ok above is now done...use SATURN to dtermnie bl thickness
! coupled with namelist parameter ythick. This is much better
! and makes code more general. (Feb/9/2005)
	YGLOBAL=45.D0
	YLOCAL=100.D0

!	Grid stretching pArAmeters
	xstrech=1.d0
	Grid=0.835D0
	ythick=0.9998
!
	NGlobal=39
	NLocal=121
	NitIray=25
	NGlbMd=65
	NGlbMd2=65
	NitMode=ISILLY
	IRAyCnt=0
	ToLocal=1.D-2

	ITolSpc=ISILLY
	ITolRAy=ISILLY
	ITolNon=ISILLY

        IWVDUMP=ISILLY

! scenario to do nonlinear pure oblique wave quickly.
	IOB=0
	IOVERIDE=0
	
        READ(7,PSEUSER,END=10,IOSTAT=JFK,ERR=1099)
10      CONTINUE

	IF(Lookup == 1)THEN
	CALL HELP(0)
	IERR=1
	END IF
	
	IF(IPOSTENV == 1 .and. ISETUP3D == 1)THEN
	IERR=1
	PRINT*,' *** Error : Both IPOSTENV and ISETUP3D flagged up!'
	END IF
	IF(IERR == 1)RETURN
	IF(IPOSTENV == 1)RETURN
	I3D=IDO3D
	JSTRIP=ISTRIP
	IF(ISETUP3D == 1 .and. IDO3D == 1)RETURN
	
        IF(INONLIN /= 1)GOTO 20
        READ(7,PSENONL,END=20,IOSTAT=JFK,ERR=2099)
20      READ(7,PSECNTL,END=30,IOSTAT=JFK,ERR=3099)
30      READ(7,PSEOUT,END=40,IOSTAT=JFK,ERR=4099)
40	CONTINUE

	IJOK=ISCREEN
	
	IF(INEUTRL == 1)THEN
	ITSPRES=0
	END IF	
	
	IF(NFINAL == 2)THEN
	IROBUST=-1
	NFINAL=NSTART
	END IF
	
	
	IF(ITSE == 1)THEN
	IOSE=0
	END IF
	
	I3D=IDO3D
	IDOZ=IPSEZ
	
	IREXEPT=IRECEPT
	
	IF(IDO3D == 1)THEN
	
	IERR=0
	IF(IGLB3D == 1 .and. IEXTRAP3D == -99)THEN
	IEXTRAP3D=0
	ELSE IF(IGLB3D == 1 .and. IEXTRAP3D /= -99)THEN
	ELSE
	IEXTRAP3D=1
	END IF
	
	IF(IEXTRAP3D /= 0 .AND. IEXTRAP3D /= 1)THEN
	PRINT*,' *** Error :: IEXTRAP3D must be 0 or 1 '
	IERR=1
	RETURN
	END IF

! to prevent silly anlysis..
	IF(NZDIR == 1)ISTRIP=1

	IF(IELLIPT == 0)THEN
	xellips=0.D0
	END IF
	IF(IELLIPT == 1)THEN
! this is now set according to direction of We
	DIFFER=1.D0
	xellips=1.D0
	PRINT*,' Beware the Pz terms may be still zero! See SET3CRx'
! this has been corrected now automatic... setting of Pz terms ox bit
! though to be tested to amke sure it does what its supposed to do!
	END IF

! jsut so as not to cause any problemssss though not used differ.then

	IF(IUPWIND == 1)THEN
	DIFFER=0.D0
	ELSE
	IUPSMRT=0
	END IF

	e=DIFFER
	em1=1.D0-DIFFER	
	


! so as to stop calcs and exit nicely when not converging in 3d calcs.

	IF(IELLIPT == 1  .AND.  DIFFER < 0.9D0)THEN
	PRINT*,'Sorry your attempting an elliptic analysis but...#@!'
	IERR=1
	WRITE(2,'(A)')'PSE3d analysis selected :'
	WRITE(2,'(A)')'Attempting an elliptic analysis but not set yet!'
	END IF

	IF(NZDIR > KCRF)THEN
	WRITE(*,11021)KCRF
11021	FORMAT(1x,'Error !  Nzdir must be <= ',I5)
	IERR=1
	END IF
	
	IF(IERR == 0)THEN
	WRITE(2,'(A)')'PSE3d analysis selected :'
	IF(NZDIR == 1 .OR. ISTRIP == 1)THEN
	WRITE(2,'(A)')' Performing a strip-based pseudo-3d-analysis'
        WRITE(2,*)
	else IF(IUPWIND == 0)THEN
		WRITE(2,'(a,I0)')'  IUPWIND = ',IUPWIND
		WRITE(2,'(a,I0)')'  IUPSMRT = ',IUPSMRT
		WRITE(2,'(a,I0)')'   IUPIMP = ',IUPIMP
		IF(DIFFER < 0.49)THEN
		WRITE(2,'(A, F5.3)')     & 
     & ' Using explicit central-differencing, Differ = ',DIFFER
        	WRITE(2,*)
		else IF(DIFFER > 0.51)THEN
		WRITE(2,'(A, F5.3)')     & 
     & ' Using implicit central-differencing, Differ = ',DIFFER
        	WRITE(2,*)
		else
		WRITE(2,'(A, F5.3)')     & 
     & ' Using semi-implicit central-differencing, Differ = ',DIFFER
        	WRITE(2,*)
		end if
     	ELSE IF(IUPWIND == 1)THEN
		WRITE(2,'(a,I0)')'  IUPWIND = ',IUPWIND
		WRITE(2,'(a,I0)')'  IUPSMRT = ',IUPSMRT
		WRITE(2,'(a,I0)')'   IUPIMP = ',IUPIMP
		IF(IUPSMRT == 0)THEN
		WRITE(2,'(A)')     & 
     & ' Using uni-direction, one-sided, un-smart implicit differencing'
                WRITE(2,*)
     		else if(IUPIMP == 0  .AND.  IUPSMRT == 1)THEN
		WRITE(2,'(A)')     & 
     & ' Using (+)ve locally-upwinded, smart explicit differencing'
                WRITE(2,*)
     		else if(IUPIMP == 1  .AND.  IUPSMRT == 1)THEN 
		WRITE(2,'(A)')     & 
     & ' Using (+)ve locally-upwinded, smart implicit differencing'
                WRITE(2,*)
     		end if
	end if 
	end if

	IF(NGlbMd == NGlbMd2)THEN
	NGlbMd=25
	end if
	end if
	
	JSTRIP=ISTRIP
! ok done the 3d default settings.

!	just to ensure that this is set (bl thickness parameter)
! 	in automatic determination of bl thickness.
	udel=ythick
	
        IF(IERR == 1)RETURN

        IGLOBX=IGLBX
	
        IF(IGLBX == 1) IGlb=1

	IF(INONLIN == 0)THEN
		IF(ITolSpc < -1)THEN
			TolSpc=TENPOW(2+IDO3D)
			ELSE
			TolSpc=TENPOW(ITolSpc)
		END IF 
		IF(ITolRAy < -1)THEN
		        TolRAy=TENPOW(2+IDO3D)
			ELSE
			TolRAy=TENPOW(ITolRAy)
		END IF
	IF(NitMode < -1) NitMode=25
	IF(IWVDUMP < -1) IWVDUMP=1
	END IF

	IDEST=INDEX(DESTIN,' ')-1
	IDEST2=INDEX(SUBDIR,' ')-1

	AUN=DADX

	CALL GASPROP(IERR)
        IF(IERR == 1)RETURN

	IF(IQE /= 0)THEN
	IQE=1
	END IF

	IF(IPRCENT == 0)THEN
	I100=1
	ELSE
	I100=100
	END IF

	IF(IEURO > 0)THEN
	IWVDUMP=0
	NATURAL=0
        IUMAX=0
        KEU=0
        KEUV=0
        KEUVW=0
        MASFLX=0
        ITMAX=0
        IWMAX=0
        IUHerb=1
        KEMAX=0
! this makes things really inflexible.....
! as gives end-user no choice!
!
! perhaps should also have this for non linear!
	IQE=1
	END IF

	YQZ=YGLOBAL

	IF(ISCREEN == 1)THEN
!	ITV=5
	ITV=6
	ELSE
	ITV=3
	END IF
	
	IF(IPSE == 1)THEN
        IERR=1
	PRINT*,' '
        PRINT*,'Error : IPSE option not supported in this version'
        PRINT*,'        Contact author ...'
	RETURN
	END IF
	
	IF(INONLIN == 1)THEN
        IERR=1
	PRINT*,' '
        PRINT*,'Error : NL-PSE option not supported in this version'
        PRINT*,'        Contact author ...'
	RETURN
	END IF	
	
	IF(IDO3D == 1)THEN
        IERR=1
	PRINT*,' '
        PRINT*,'Error : PPM-PSE3D option not supported in this version'
        PRINT*,'        Contact author ...'
	RETURN
	END IF


	KTKL=IOSE+IPSE+INONLIN+ITSE
	IMPBIT=IOSE+ITSE
        INONBIT=IPSE+INONLIN
	IF(KTKL == 0)THEN
        IOP=1
        INON=0
        ICHART=INDEX(LABEL,' ')-1
	ELSE IF (INONBIT == 2  .AND.  IMPBIT == 0)THEN
        INON=1
        ICHART=INDEX(LABEL,' ')-1
	ELSE IF (INONLIN == 1  .AND.  IMPBIT == 0)THEN
        INON=1
        ICHART=INDEX(LABEL,' ')-1
	ELSE IF (IPSE == 1  .AND.  IMPBIT == 0)THEN
        IOP=1
        INON=0
        ICHART=INDEX(LABEL,' ')-1
	ELSE IF (IMPBIT > 0  .AND.  INONBIT > 0)THEN
        IERR=1
        PRINT*,'Error : Confusion over type of analysis requested'
        PRINT*,'Select 1 from : IPSE=1,IOSE=1,INONLIN=1 or ITSE=1'
	ELSE IF(IOSE == 1  .AND.  ITSE == 0)THEN
        IOP=0
	IPHINS=0
        INON=0
	
! overide switch for routine N-factors. Forces ICURVI=0, for IOSE=1 option!
		IF(IOVERIDE == 0)THEN
		ICURVI=0
        PRINT*,'  !!! Overide switch : Curvature model set to OFF'
		end if
        ICHART=INDEX(LABEL,' ')-1
		IF(ICURVI == 1)THEN
        	LABEL=LABEL(1:ICHART)//'ose_Crv'
		else
        	LABEL=LABEL(1:ICHART)//'ose'
		end if
        ICHART=INDEX(LABEL,' ')-1
	
	ELSE IF(IOSE == 0  .AND.  ITSE == 1)THEN
        IPHINS=1
        IOP=0
        INON=0
        ICHART=INDEX(LABEL,' ')-1
        LABEL=LABEL(1:ICHART)//'tse'
        ICHART=INDEX(LABEL,' ')-1
	ELSE
        IERR=1
        PRINT*,'Error : Confusion over type of analysis requested'
        PRINT*,'Select 1 from : IPSE=1,IOSE=1,INONLIN=1 or ITSE=1'
	END IF	
	
	
	IF(IDO3D == 1)THEN
	IF(ISTRIP == 1)THEN
	ICHART=INDEX(LABEL,' ')-1
        LABEL=LABEL(1:ICHART)//'_s'
        ICHART=INDEX(LABEL,' ')-1
	END IF
	END IF
	
	IF(ICURVI == 1 .and. IOSE == 0)THEN
	ICHART=INDEX(LABEL,' ')-1
	IF(IDO3D == 1 .and. ISTRIP == 1)THEN
        LABEL=LABEL(1:ICHART)//'Crv'
	ELSE
        LABEL=LABEL(1:ICHART)//'_Crv'
	END IF
        ICHART=INDEX(LABEL,' ')-1
	end if
	

	IF(IERR == 1)RETURN

66054	CONTINUE

	INGUESS=INON
	ICNORM=IOP
	ITGUESS=IPHINS

	IF(ITGUESS == 1)THEN
	I3DNEQ=5
	ELSE
	I3DNEQ=9
	END IF

	KXCOUT=IOUTXC+IOUTSC
	IF(KXCOUT >= 2)THEN
	WRITE(*,*)' Error : Confusion over type of output requested'
	WRITE(*,*)' Flag either IOUTSC or IOUTXC , NOT both ! '
	IERR=1
	ELSE IF(KXCOUT == 0)THEN
	JOUTSC(1)=0
	JOUTSC(2)=1
	ELSE IF(KXCOUT == 1)THEN
	JOUTSC(1)=IOUTSC
	JOUTSC(2)=IOUTXC
	ELSE
	END IF
	IF(IERR == 1)RETURN



	IF(NSTART > 2  .AND.  NFINAL == 2)THEN
	NFINAL=NSTART
	ELSE
	END IF

!	TrAcking silly errors by users ... incomplete !!!
	IF(NSTART <= NFINAL)GOTO 8241
	IERR=1
	PRINT*,' Error : You Have Set NSTART > NFINAL '
	PRINT*,'      Check Namelist : USER'
	RETURN
8241	CONTINUE

        IF(NINTR >= 0)GOTO 8421
        IERR=1
        PRINT*,' Error : You Have Set a (-)ve NINTR Value'
	PRINT*,' Upstream Marching Not Possible !!'
        PRINT*,'      Check Namelist : USER'
        RETURN
8421    CONTINUE

        IF(NSTART >= 1)GOTO 8124
        IERR=1
        PRINT*,' Error : You Have Set a Silly NSTART Value'
        PRINT*,' You Must Have NSTART >= 1  !!'
        PRINT*,'      Check Namelist : USER'
        RETURN
8124    CONTINUE

        IF(NINTR <= (NFINAL-NSTART))GOTO 8142
        WRITE(2,*)' Caution : You Have Set NINTR > NFINAL '
        WRITE(2,*)' Only 1 Station Will be Computed'
	WRITE(2,*)
8142    CONTINUE


	IODDY=MOD(NLOCAL,2)
	IF(IODDY /= 1)THEN
	NLOCAL=NLOCAL-1
	WRITE(2,*)'** Even Number of Points Across Boundary Layer **'
	WRITE(2,*)'   Odd Number Required -- Reducing By 1 '
	ELSE
	END IF


	IF(IGLBX == 1)THEN
	IGLB=1
	ELSE
	END IF

	IF(IGLB == 0)THEN
        YEDGE=YLOCAL
        ND=NLOCAL
        MPMAX=MP
	ELSE
	IGLBX=1 ; IGLOBX=IGLBX
        YEDGE=YQZ
        ND=NGLOBAL
        MPMAX=MPG
	END IF
	
	IF(ND <= MPMAX)GOTO 1111
        PRINT*,'             '
	WRITE(*,22)
	WRITE(*,33)MP
	WRITE(*,44)MPG
	IERR=1
22	FORMAT(' Have exceeded no. of allowable grid points in wall')
33	FORMAT(' normal direction - Maximum = NLOCAL(',I0,')')
44	FORMAT('                  - Maximum = NGLOBAL(',I0,')')
	RETURN

1111	CONTINUE

	IRECEPT=IRECEPT*INONLIN
	IONE2=IRECEPT

	IF(INONLIN == 0)GOTO 8801
	WRITE(2,*)
	WRITE(2,441)
	WRITE(2,*)
        PRINT*,'             '
        PRINT*,' **************************************************'
	WRITE(*,441)
        PRINT*,' **************************************************'
441	FORMAT(2X,'***',6X,'NON-LINEAR PSE ANALYSIS SELECTED',6X, & 
     & '***')
!	nonlinear scenArio being set up...
! force output into IEURO form.
	IEURO=1

	IF(IWVDUMP < -1) IWVDUMP=0

        IF(ITolSpc < -1)THEN
        TolSpc=TENPOW(3)
        ELSE
        TolSpc=TENPOW(ITolSpc)
        END IF
        IF(ITolRAy < -1)THEN
        TolRAy=TENPOW(9)
        ELSE
        TolRAy=TENPOW(ITolRAy)
        END IF
	IF(ITolNon < -1)THEN
	TolNon=1.2D0*TolSpc
	ELSE
	TolNon=TENPOW(ITolNon)
	END IF

	IF(NitMode < -1) NitMode=200

	ITS=0
	IVX=0
	INTOB=0
	DO 70001 IJK=1,MODE
	IF(AMP(IJK,0) > -1.) ITS=1
70001   CONTINUE
	DO 70002 IJK=1,KMODE
	IF(AMP(0,IJK) > -1) IVX=1
70002	CONTINUE
	DO 70003 IJK=1,KMODE
!	DO 70003 IJK=-KMODE,-1
	DO 70003 IKJ=1,MODE
	IF(AMP(IKJ,IJK) > -1.) INTOB=1
70003	CONTINUE

	IERR=0
	IF(INTOB == 0.) THEN
		IF(IVX == 0  .AND.  ITS == 1)THEN
		IF(IALFA < -1) IALFA=4
		IF(IBETA < -1) IBETA=0
		ELSE IF(IVX == 1  .AND.  ITS == 0)THEN
		IF(IBETA < -1) IBETA=4
		IF(IALFA < -1) IALFA=0
		ELSE IF(ITS == 1  .AND.  IVX == 1)THEN
		IF(IALFA < -1) IALFA=4
                IF(IBETA < -1) IBETA=0
		ELSE IF(ITS+IVX == 0)THEN
		IERR=1
	WRITE(*,*)' *** ERROR - All modes have been set to have *** '
	WRITE(*,*)'             zero amplitudes ! '
		ELSE
		END IF
	ELSE 
	IF(IALFA < -1) IALFA=4
	IF(IBETA < -1) IBETA=0
	END IF

	IF(IERR == 1)RETURN

        YUHU=ITHRESH
        TEN=9.98D0
        TOLCUT=TEN**(YUHU)
        TOLCUT=1.D0/TOLCUT

	IF(IBETA == 0)GOTO 1212
	IF(DABS(BETA) > 1.D-10)GOTO 1212
	IERR=1
	PRINT*,'You have selected a crossflow type interaction'
        PRINT*,' but Beta remains undefined !'
	RETURN
1212	CONTINUE

	IF(IOB == 0)THEN
	IF(IBETA == 0  .AND.  DABS(BETA) > 1.D-10)THEN
	IERR=1
	PRINT*,'You have defined a Non-Zero Beta value and yet '
	PRINT*,' IBETA = 0 ! : Have you set up the required scenario'
	PRINT*,' I think you need to have a Non-Zero IBETA value'
	RETURN
	ELSE
	END IF
	end if
	IF(IERR == 1)RETURN
	
	IF(IALFA <= MODE)GOTO 2222
	IERR=1
        PRINT*,'             '
	PRINT*,' Have exceeded no. of allowable streamwise Fourier modes'
	WRITE(*,55)MODE
55	FORMAT('                    Maximum = IALFA(',I0,')')
	RETURN

2222	IF(IBETA <= KMODE)GOTO 3333
	IERR=1
        PRINT*,'             '
	PRINT*,' Have exceeded no. of allowable crossflow Fourier modes'
	WRITE(*,65)KMODE
65	FORMAT('                    Maximum = IBETA(',I0,')')
	RETURN

3333	CONTINUE

!	mAke template for fourier modes
	MTOTAL=(IALFA+1)*(IBETA+1)
!	CALL TMPLAT(IBOUND,IFULL,ICHEK,INCORR,IVORTX,ISIGX,IERRN)


	IF(ISIGX == 0)THEN
	IDIV=1
	ELSE
	IDIV=ISIGX
	END IF

	IF(IERRN /= 1)GOTO 14786
	WRITE(*,*)'    *** Error in nonlinear scenario setup'
	IERR=1
	RETURN
14786	CONTINUE

        IF(INCORR == 0)GOTO 8888
        PRINT*,'       '
	IERR=1
        IF(INCORR == 1)THEN
        WRITE(*,*)'    *** Error - variable AMP is an Array ! ***  '
        RETURN
	ELSE IF(INCORR == 2)THEN
        WRITE(*,*)' Note - though IBETA > 0, All modes (IALFA,k) (k>0)' 
	WRITE(*,*)' are set to have a Zero amplitude : You must set'    
	WRITE(*,*)' either Amp(1,k),Amp(2,k) or Amp(0,k) to be Non-Zero'
        RETURN
	ELSE IF(INCORR == 3)THEN
        WRITE(*,*)' Note - Though IALFA > 0, All modes (n,IBETA) (n>0)'
	WRITE(*,*)' are set to have a Zero amplitude : You must set'    
	WRITE(*,*)' either Amp(n,1),Amp(n,2) or Amp(n,0) to be Non-Zero'
        RETURN
	ELSE IF(INCORR == 4  .AND.  IBOUND == 0)THEN
	WRITE(*,*)' *** ERROR - All modes have been set to have *** '
	WRITE(*,*)'             zero amplitudes ! '
        RETURN
	ELSE
	END IF
8888    CONTINUE

	IF(IBOUND == 0 .OR. IOB == 1)GOTO 7777
        IERR=1
        PRINT*,'       '
        WRITE(*,*)'Some of your Non-Zero Fourier modes input as initial'
	WRITE(*,*)'amplitudes lie beyond the range (0:IALFA,0:IBETA)'
	WRITE(*,66)IALFA,IBETA
        RETURN
66	FORMAT(' Range at present - IALFA = ',I0,' ; IBETA = ',I0) 
7777    CONTINUE

	IF(IFULL >= 1  .AND.  IFULL <= MTOTAL)GOTO 4444
	IERR=1
	PRINT*,'       '
	WRITE(*,*)'You have not specified any amplitudes for the'
	WRITE(*,*)'initial disturbances -'
	WRITE(*,*)'You MUST specify at least one Non-Zero'
	WRITE(*,*)'amplitude...  eg. : Amp(2,0)=0.25 '
	RETURN
4444	CONTINUE

!	Check if user defined frequency is set
	IF(FREQ > SILLYB)GOTO 4774
	IERR=1
	PRINT*,'       '
	WRITE(*,76)
76	FORMAT(' Run aborted - Frequency variable FREQ is Undefined')
	RETURN
	
4774	CONTINUE
!     CALL SETFTC(IALFA,IBETA,IERR)
        IF(IERR == 1)RETURN

	IF(IGLB == 1)GOTO 7766

        INQUIRE(FILE='PSEIGEN.DAT',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 9776
	OPEN(13,FILE='PSEIGEN.DAT',STATUS='OLD')
        READ(13,*,END=9796,ERR=9226)IDUM,IDUM,POLK,POLK
	REWIND(13)

        DO 101 IZDIR=-IBETA,IBETA
        DO 101 IXWVE=0,IALFA
        IA=LTMP(IXWVE,IZDIR,1)
        IB=LTMP(IXWVE,IZDIR,2)
        IF(ITRANS(IA,IB) == 0)GOTO 101
        READ(13,*,END=9116,ERR=9226)IMA,IKB,ALFR,ALFI
	IF(ITRANS(IMA,IKB) == 0)GOTO 9336
        QZPRED(IMA,IKB)=DCMPLX(ALFI,ALFR)
101     CONTINUE
        GOTO 7766

9796    CONTINUE
	CLOSE(13)
	GOTO 9776

9116    IERR=1
        WRITE(*,*)'End of file attained in PSEIGEN.DAT - Check whether'
        WRITE(*,*)'Wave & Grwth for each mode input as an initial'
        WRITE(*,*)'condition is specified'
        RETURN
9226    IERR=1
        WRITE(*,*)'Check PSEIGEN.DAT - Error in file format '
	RETURN
9336	IERR=1
	WRITE(*,*)'Check PSEIGEN.DAT - Mis-match in modal wavenumbers'
	WRITE(*,*)'and requested initial conditions - Use the Global'
	WRITE(*,*)'method to generate a new PSEIGEN.DAT file : '
	WRITE(*,*)'Either set IGLB=1 in Namelist USER or remove the '
	WRITE(*,*)'existing PSEIGEN.DAT file'
	RETURN
9776	CONTINUE
        PRINT*,'       '
        PRINT*,'Though you have selected IGLB=0, the file which '
        PRINT*,'contains the initial guesses for the eigenvalues does'
        PRINT*,'not exist - Am automatically selecting the Global '
        PRINT*,'Method to generate these estimates : '
        PRINT*,'Results stored in PSEIGEN.DAT'
        IGLB=1
	IGLBX=1 ; IGLOBX=IGLBX
	IGLOB=IGLB
        YEDGE=YQZ
        ND=NGLOBAL
        MPMAX=MPG

7766    CONTINUE

	IF(RGORTL > SILLYB)THEN
	IERR=1
	PRINT*,'       '
	PRINT*,'You Have Selected A Vortex Type Interaction'
	PRINT*,'As An Initial Condition - Not Implemented Yet !!!'
	RETURN
	ELSE
	RGORTL=0.D0
	END IF

!	nonlineAr scenArio finished...
	IF(INONLIN == 1)GOTO 8803

8801	CONTINUE

	ITWO=1
!	floquet Analysis set here to zero for lineAr pse
	DO I=-MODE,MODE
	IFLOQ(I,-KMODE:KMODE)=0
	END DO

	IGLB3D_EXIST=1
	IF(IGLB == 1)GOTO 6666

        INQUIRE(FILE='PSEMULTF.DAT',EXIST=YUP)
        IF(YUP.EQV..FALSE.)GOTO 6746
	OPEN(12,FILE='PSEMULTF.DAT',STATUS='OLD')

        READ(12,*,END=6746,ERR=6246)NOOMEG
        READ(12,*,END=6776,ERR=6246)RALF,RIALF,XCDUMY
	IGLB3D_EXIST=0
	REWIND(12)
	READ(12,*)LUCKY
	IF(NOOMEG > KSTR)GOTO 6706
	IF(NOOMEG < 0)GOTO 6246
	IFRICA=NOOMEG
	IF(ITGUESS == 1)THEN
	NWAVE=IFRICA
	ELSE
	NFREQ=IFRICA
	END IF

	DO 6006 I=1,NOOMEG
	IF(ITGUESS == 1)THEN
        READ(12,*,END=6146,ERR=6246)RALF,RIALF,XCDUMY,XCI
	WAVALFA(I)=XCDUMY
	WAVALIM(I)=XCI
        QZLIN(I)=DCMPLX(RALF,RIALF)
	ELSE
        READ(12,*,END=6146,ERR=6246)RALF,RIALF,XCDUMY
	FRENCY(I)=XCDUMY
        QZLIN(I)=DCMPLX(RIALF,RALF)
	END IF
6006	CONTINUE
       	GOTO 1001 

6776	CONTINUE
	CLOSE(12)
	GOTO 6746

6146    IERR=1
        WRITE(*,*)'End of file attained in PSEMULTF.DAT - Check whether'
	IF(ITGUESS == 1)THEN
        WRITE(*,*)'the No. of Wavenumbers Specified Equals NWave'
        WRITE(*,*)'Note : Wavenumber is now complex! '
	WRITE(*,*)'PSEMULTF.DAT file format now ...'
        WRITE(*,*)'Nwave     No. of complex valued wave nos.'
	WRITE(*,*)'omega_R  omega_I  Alfa_R Alfa_I'
	WRITE(*,*)'0.1228   0.0012    20.0  1.0 (example)'
	ELSE
        WRITE(*,*)'the no. of frequencies specified equals NFreq'
	END IF
        RETURN
6246    IERR=1
        WRITE(*,*)'Check PSEMULTF.DAT - Error in file format'
	RETURN
6706	IERR=1
	IF(ITGUESS == 1)THEN
	WRITE(*,*)'Have exceeded no. of allowable wavenumbers'
	WRITE(*,61017)KSTR,NOOMEG
	ELSE
	WRITE(*,*)'Have exceeded no. of allowable frequencies'
	WRITE(*,6101)KSTR,NOOMEG
	END IF
6101	FORMAT(' Nfreq_Max = ',I0,' : You have Nfreq = ',I0)
61017	FORMAT(' NWave_Max = ',I0,' : You have NWave = ',I0)
	RETURN
6746	CONTINUE
        PRINT*,'       '
	PRINT*,' IGLB=0, But PSEMULTF.DAT does not exist - This file'
        PRINT*,' must contain the eigenvalues for each frequency or'
	PRINT*,' streamwise wavenumber (temporal option). '
        PRINT*,' Am Automatically selecting the QZ-Routine to generate'
        PRINT*,' these estimates'
	PRINT*,'    '
        IGLB=1
        IGLBx=1 ; IGLOBX=IGLBX
	IGLOB=IGLB
        YEDGE=YQZ
        ND=NGLOBAL
        MPMAX=MPG

	IF(ND <= MPMAX)GOTO 6666
        PRINT*,'             '
	WRITE(*,22)
	WRITE(*,33)MP
	WRITE(*,44)MPG
	IERR=1
	RETURN

6666    CONTINUE


	IF(ITGUESS /= 1)GOTO 10001
        IF(WAVELIN <= SILLYZ)GOTO 10002
        NWAVE=1
        WAVALFA(1)=WAVELIN
	WAVALIM(1)=WAVEIM
        GOTO 10003

10002   INQUIRE(FILE='PSEWAVE.DAT',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 60676
        OPEN(9,FILE='PSEWAVE.DAT',STATUS='OLD')

        READ(9,*,END=60516,ERR=60526)NOOMEG
        IF(NOOMEG > KSTR)GOTO 60706
        NWAVE=NOOMEG
        DO 50661 I=1,NWAVE
        READ(9,*,END=60516,ERR=60526)WAVALFA(I),WAVALIM(I)
50661    CONTINUE
	CLOSE(9)
10003   OPEN(12,FILE='PSEMULTF.DAT',STATUS='UNKNOWN')
        WRITE(12,20031)NWave
20031     FORMAT(i6)
        IGLB=1
        IGLBx=1 ; IGLOBX=IGLBX
        IGLOB=IGLB
        YEDGE=YQZ
        ND=NGLOBAL
        MPMAX=MPG
        GOTO 1001

60516   IERR=1
        WRITE(*,*)'End of File Attained in PSEWAVE.DAT - Check Whether'
        WRITE(*,*)'the No. of alphas Specified Equals NWave '
	WRITE(*,*)'Note : require Alfa_R Alfa_I - as Alpha is Complex '
        RETURN
60526    IERR=1
        WRITE(*,*)' Check PSEWAVE.DAT - Error in File Format '
        RETURN
60706   IERR=1
        WRITE(*,*)
        WRITE(*,*)'Have exceeded no. of allowable Alfas'
        WRITE(*,10101)KSTR,NOOMEG
10101    FORMAT(' NWave_Max = ',I0,' : You have NWave = ',I0)
        RETURN
60676   CONTINUE
        IERR=1
        WRITE(*,*)' File PSEWAVE.DAT Does Not Exist !!! '
        WRITE(*,*)' Create this file ... This file contains the range'
        WRITE(*,*)' of wavenumbers for which eigenvalues are computed'
        WRITE(*,*)' Alternatively specify WAVELIN in namelist   '
        WRITE(*,*)'    '
        WRITE(*,*)' Basic File Format - PSEWAVE.DAT'
        WRITE(*,*)' Line 1      : Nwave     (No. of Streamwise Waves)'
        WRITE(*,*)' Nwave Lines : Alfa_R Alfa_I (Refered to Chord)   '
        RETURN

!     *******************************************************

10001	CONTINUE
	IF(FREQLIN <= SILLYZ)GOTO 11112
	NFREQ=1
	FRENCY(1)=FREQLIN
	GOTO 11111

11112   INQUIRE(FILE='PSEFREQ.DAT',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 6676
	OPEN(16,FILE='PSEFREQ.DAT',STATUS='OLD')

        READ(16,*,END=6516,ERR=6526)NOOMEG
	IF(NOOMEG > KSTR)GOTO 6706
	NFREQ=NOOMEG
	DO 5661 I=1,NFREQ
        READ(16,*,END=6516,ERR=6526)FRENCY(I)
5661	CONTINUE
11111	OPEN(12,FILE='PSEMULTF.DAT',STATUS='UNKNOWN')
	WRITE(12,231)NFreq
231	FORMAT(i6)
        IGLB=1
        IGLBx=1 ;  IGLOBX=IGLBX
	IGLOB=IGLB
        YEDGE=YQZ
        ND=NGLOBAL
        MPMAX=MPG
	GOTO 1001
	
6516    IERR=1
        WRITE(*,*)'End of file attained in PSEFREQ.DAT - Check whether'
        WRITE(*,*)'the no. of frequencies specified equals NFreq '
        RETURN
6526    IERR=1
        WRITE(*,*)' Check PSEFREQ.DAT - Error in file format '
	RETURN
6676	CONTINUE
	IERR=1
	WRITE(*,*)' File PSEFREQ.DAT does not exist !!! '
	WRITE(*,*)' Create this file ... This file contains the range of'
	WRITE(*,*)' frequencies for which N-factors are desired.'
        WRITE(*,*)' Alternatively specify FREQLIN in namelist   '
        WRITE(*,*)'    '
	WRITE(*,*)' Basic file format - PSEFREQ.DAT'
	WRITE(*,*)' Line 1      : Nfreq        (No. of frequencies)'
	WRITE(*,*)' Nfreq lines : Frequency    (Hz) ' 
	RETURN

!     *******************************************************
1001	CONTINUE

	IF(BETALIN <= SILLYZ)GOTO 22223
	NCROSS=1
	WAVBETA(1)=BETALIN
	GOTO 1081

22223	IF(ICRSFLO /= 0)GOTO 77777
	NCROSS=1
	WAVBETA(1)=0.D0
	GOTO 8802

77777	CONTINUE
        INQUIRE(FILE='PSEBETA.DAT',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 1676
	OPEN(19,FILE='PSEBETA.DAT',STATUS='OLD')

        READ(19,*,END=1516,ERR=1526)NOOMEG
	NCROSS=NOOMEG
	IF(NCROSS > KCRF)GOTO 1706
	DO 1661 I=1,NCROSS
        READ(19,*,END=1516,ERR=1526)WAVBETA(I)
1661	CONTINUE

	GOTO 1081
	
1516    IERR=1
	WRITE(*,*)
        WRITE(*,*)'End of file attained in PSEBETA.DAT - Check whether'
        WRITE(*,*)'the no. of crossflow wave nos specified equals NBeta'
        RETURN
1526    IERR=1
	WRITE(*,*)
        WRITE(*,*)' Check PSEBETA.DAT - Error in file format '
	RETURN
1706	IERR=1
	WRITE(*,*)
	WRITE(*,*)'Have exceeded no. of allowable Betas'
	WRITE(*,1101)KCRF,NOOMEG
1101	FORMAT(' NBeta_max = ',I0,' : You have NBeta = ',I0)
	RETURN
1676	CONTINUE
	IERR=1
	WRITE(*,*)
	WRITE(*,*)' File PSEBETA.DAT does not exist !!! '
	WRITE(*,*)' Create this file. This file contains the '
	WRITE(*,*)' Crossflow Wavenumbers... or alternatively to perform'
	WRITE(*,*)' a single wave analysis Only Reset ICRSFLO = 0'
	WRITE(*,*)' in Namelist USER'
        WRITE(*,*)'    '
	WRITE(*,*)' Basic file format - PSEBETA.DAT'
	WRITE(*,*)' Line 1       : NCross (no. of crossflow waves)'
	WRITE(*,*)' NCross lines : Beta   (wave no.- refered to Chord)' 
	RETURN

1081	CONTINUE

8802	CONTINUE

	IF(ITGUESS == 0)THEN

!	deciding between type of output
	IF(NCROSS == 1  .AND.  NFREQ == 1)THEN
	NOUTPUT=1
	WRITE(2,*)'A single oblique wave being tracked'
	ELSE IF(NCROSS > 1 .AND. NFREQ == 1)THEN
	NOUTPUT=2
	WRITE(2,*)'Oblique waves with A fixed frequency being tracked'
	ELSE IF(NCROSS == 1 .AND. NFREQ > 1)THEN
	NOUTPUT=3
	WRITE(2,*)'Multifrequencied oblique waves with a fixed'
	WRITE(2,*)'crossflow wavenumber being tracked'
	ELSE
	NOUTPUT=4
	WRITE(2,*)'Multivalued frequency & crossflow wavenumber tracking'
	WRITE(2,*)'option selected - this may take some time!'
	END IF


	ELSE

!       deciding between type of output
        IF(NCROSS == 1 .AND. NWAVE == 1)THEN
        NOUTPUT=1
        WRITE(2,*)'A single oblique wave being tracked'
        ELSE IF(NCROSS > 1 .AND. NWAVE == 1)THEN
        NOUTPUT=2
        WRITE(2,*)'Oblique waves at a fixed Alfa being tracked'
        ELSE IF(NCROSS == 1 .AND. NWAVE > 1)THEN
        NOUTPUT=3
        WRITE(2,*)'Multivalued Alfa waves with a fixed'
        WRITE(2,*)'crossflow wavenumber being tracked'
        ELSE
        NOUTPUT=4
        WRITE(2,*)'Multivalued Alfa & crossflow wavenumber tracking'
        WRITE(2,*)'option selected - this may take some time!'
        END IF

	END IF

!     *******************************************************
8803	CONTINUE


	IF(RCPTUR > 1.D-8)THEN
	IWETH=1
	ELSE
	IWETH=0
	END IF

	NONDIM=0
	NSTRAT=NRMLOCL+NRMCORD+NRMSTRT+NRMDIM
	IF(NSTRAT > 2)GOTO 7124
	IF(NSTRAT == 2)THEN
	NRMLOCL=0
	ELSE
	NONDIM=2
	DIMSCL(2)=1.D0
	WRITE(2,*)
	WRITE(2,*)'Results Non-Dimensionalised with LOCAL Length Scale'
	WRITE(2,*)'Based on Boundary Layer Thickness - Delta' 
	WRITE(2,*)
	END IF

	IF(NONDIM == 2)GOTO 4217

!  PRINT*,'       '
        WRITE(2,*)
        IF(NRMCORD /= 0)THEN
        NONDIM=1
        DIMSCL(1)=1.D0
        WRITE(2,*)'Results Non-Dimensionalised with Global Length '
        WRITE(2,*)'Scale Based on Chord'
        ELSE IF(NRMSTRT /= 0)THEN
        NONDIM=3
        DIMSCL(3)=1.D0
        WRITE(2,*)'Results Non-Dimensionalised with Length Scale'
        WRITE(2,*)'Based on Boundary Layer Thickness at Start of'
        WRITE(2,*)'Analysis : Delta(Nstart)'
        ELSE
        NONDIM=4
        DIMSCL(4)=1.D0
        WRITE(2,*)'Results in Dimensional form'
        END IF
        WRITE(2,*)
!  PRINT*,'       '

        GOTO 4217

7124    PRINT*,'             '
        IERR=1
        PRINT*,'  Confusion Over Scalings to be Used Whilst '
        PRINT*,'  Out-Putting Results ...'
        PRINT*,'  Either 2 or More Scalings Have Been Selected -'
        PRINT*,'  Flag Only 1 From : NRMCORD, NRMSTRT, NRMDIM'
        RETURN

4217	CONTINUE

	IF(IOP == 0 .AND. ITGUESS == 0)THEN
        PRINT*,' **************************************************'
	WRITE(*,443)
	WRITE(*,447)
	WRITE(2,443)
	WRITE(2,447)
        PRINT*,' **************************************************'
	ELSE IF(IOP == 1 .AND. INONLIN == 0)THEN
        PRINT*,' **************************************************'
	WRITE(*,442)
	WRITE(2,442)
        PRINT*,' **************************************************'
	ELSE IF(ITGUESS == 1 .AND. INONLIN == 0)THEN
        PRINT*,' **************************************************'
	WRITE(*,443)
	WRITE(2,443)
	WRITE(*,445)
	WRITE(2,445)
        PRINT*,' **************************************************'
	ELSE
	END IF
442	FORMAT(2X,'*',1X,'PARABOLISED LINEAR STABILITY ANALYSIS', & 
     & 1X,'SELECTED',1X,'*')
443     FORMAT(2X,'***',5X,'LINEAR STABILITY ANALYSIS SELECTED', &
     & 5X,'***')
445     FORMAT(2X,'***',5X,'             (TEMPORAL)           ', &
     & 5X,'***')
447     FORMAT(2X,'***',5X,'             (SPATIAL)            ', &
     & 5X,'***')

        ISTR=IUmax+KEU+KEUV+IUHerb+KEUVW+ITMAX+MASFLX+IWMAX
        IF(ISTR == 2)THEN
        IUHERB=0
        END IF

	IF(IUMAX == 1)THEN
	IUNOMAL=2
	ELSE IF(IWMAX == 1)THEN
	IUNOMAL=4
	ELSE IF(ITMAX == 1)THEN
	IUNOMAL=5
	ELSE
	IUNOMAL=2
	END IF

	IF(IOSE == 1)THEN
	ISELECT=2
	IUNOMAL=2
	END IF

        IFTX=2*IUNOMAL-2

	IF(IOP == 0 .OR. ITGUESS == 1)RETURN
        IKEJUMP=0


	ISELECT=0
        ISTR=IUmax+KEU+KEUV+IUHerb+KEUVW+ITMAX+MASFLX+IWMAX
	If(ISTR == 1)GOTO 7634
	IERR=1
	IF(ISTR /= 0)GOTO 4367
        PRINT*,'             '
	PRINT*,'  Space Marching Strategy Incorrectly Set !!!  '
	PRINT*,'  You Have Not Specified the Norm that Needs to '
	PRINT*,'  be Satisfied Whilst Space Marching - Select From:'
	PRINT*,'  KEU,KEUV,KEUVW,IUHerb,ITmax,MASFLX,IWmax'
	RETURN

4367	CONTINUE
        PRINT*,'             '
	PRINT*,'  Space Marching Strategy Incorrectly Set !!!  '
	PRINT*,'  Confusion Over Which Norm You Wish to Satisfy...'
	PRINT*,'  Either 2 or More Norms Have Been Selected - '
	PRINT*,'  KEU,KEUV,KEUVW,IUHerb,ITmax,MASFLX,IWmax'
	RETURN
7634	CONTINUE
 
	IUSQ=0
	PRINT*,'    '
	IF(IUMAX == 1)THEN
	PRINT*,'         Growth Rates Based on :   U M A X '
	WRITE(2,*)'Growth Rates Based on :   U M A X '
	ISELECT=2
	ELSE IF(IWMAX == 1)THEN
        PRINT*,' Growth Rates Based on Crossflow Disturbance,IWmax '
        WRITE(2,*)'Growth Rates Based on :   W M A X '
	ISELECT=4
	ELSE IF(MASFLX == 1)THEN
	PRINT*,'     Growth Rates Based on :   M A S S   F L U X '
	WRITE(2,*)'Growth Rates Based on :   M A S S   F L U X '
!	IVARIP=1
!	WRITE(*,*)'Warning.. Am automatically setting IVARIP=1 and'
!        WRITE(*,*)'solving equations in density form as pressure'
!        WRITE(*,*)'form not set up yet for this option'
	ISELECT=0
	ELSE IF(IUHERB == 1)THEN
	PRINT*,'      Growth Rates Based on : Velocity Integral '
	WRITE(2,*)'  Growth Rates Based on : Velocity Integral '
        ISELECT=2
        IKEJUMP=1
!	ELSE IF(MHERB == 1)THEN
!	PRINT*,'      Growth Rates Based on : Mass Flux Integral '
!	WRITE(2,*)'Growth Rates Based on : Mass Flux Integral '
	ELSE IF(KEUV == 1)THEN
	IENRGY=1
	IUSQ=1
        IWSQ=0
	PRINT*,'       Growth Rates Based on :   K E  = U*U+V*V   '
	WRITE(2,*)'Growth Rates Based on :   K E  = U*U+V*V   '
	ISELECT=2
        IKEJUMP=1
	ELSE IF(KEU == 1)THEN
	IENRGY=1
	IUSQ=0
        IWSQ=0
	PRINT*,'         Growth Rates Based on :   K E  = U*U   '
	WRITE(2,*)'Growth Rates Based on :   K E  = U*U   '
	ISELECT=2
        IKEJUMP=1
	ELSE IF(KEUVW == 1)THEN
        IENRGY=1
        IUSQ=1
        IWSQ=1
        PRINT*,'     Growth Rates Based on :   K E  = U*U+V*V+W*W '
        WRITE(2,*)'Growth Rates Based on :   K E  = U*U+V*V+W*W   '
	ISELECT=2
        IKEJUMP=1
        ELSE IF(ITMAX == 1)THEN
        PRINT*,'   Growth Rates Based on Temperature Maximum, ITmax'
        WRITE(2,*)'Growth Rates Based on :   ITmax '
	ISELECT=5
	ELSE
	END IF
	PRINT*,'       '
	WRITE(*,440)
440     FORMAT(2X,50('*'))
	PRINT*,'       '

        IF(ISCREEN == 0)THEN
        WRITE(*,*)' Executing ...'
        END IF


        IF(IEURO == 1)THEN                              
        IUSQ=1                                                       
        IWSQ=1                                                       
        ELSE                                                        
        END IF    

	RETURN

1099	PRINT*,' Error in Namelist : USER - Unexpected Variable'
	IERR=1
	RETURN
2099	IF(JFK == 121 .OR. JFK == 115)THEN
        PRINT*,' Error in Namelist : NONL - '
	PRINT*,' Check Dimensions of Arrays : Amp, Wave & Grwth'
	WRITE(*,9902)MODE,KMODE
9902	FORMAT(2x,'Maximum Dimensions Allowable (0:',I0,';0:',I0,')')
	ELSE
        PRINT*,' Error in Namelist : NONL - Unexpected Variable' 
	END IF
	IERR=1
	RETURN
3099    PRINT*,' Error in Namelist : CNTL - Unexpected Variable'
	IERR=1
	RETURN
4099	PRINT*,' Error in Namelist : OUT - Unexpected Variable'
	IERR=1
	RETURN
	END


        SUBROUTINE GASPROP(IERR)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        NAMELIST /PSEGAS/RGAS,GAMMA,PRANDTL,STOKES,ISUTHER,ICHAPMN
        COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
        COMMON /TRATUR/TOMEG,TMIN1,TMIN2
        COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TO,T1,STOKES
        COMMON /VISMOD/ISUTHER,ICHAPMN
        RGAS=287.4D0
        GAMMA=1.4D0
        PRANDTL=0.72D0
!        UMUO=1.7890E-05
        UMUO=1.79320E-05
        TO=288.2D0
        T1=110.4D0

!       *****************************************************
!       STOKES :  YLMU=-2/3        RM=4/3
!       MALIK  :      = 2/15         =32/15    DENSE(?) GASES ?
        STOKES=-2.D0/3.D0

!
!       FUNCTIONAL VISCOSITY DEPENDENCE ON TEMP : MU=T**TOMEG
        TOMEG=0.76D0

        READ(7,PSEGAS,END=40,IOSTAT=JFK,ERR=4099)
40      CONTINUE

        YLMU=STOKES
        RS=1.D0+YLMU
        RM=2.D0+YLMU
        REM=RM+1.D0

        RMINV=1.D0/RM

        TMIN1=TOMEG-1.D0
        TMIN2=TOMEG-2.D0
        IERR=0
        RETURN
4099    PRINT*,' Error in namelist : GAS - Unexpected variable'
        IERR=1
        RETURN
        END SUBROUTINE GASPROP

	SUBROUTINE SETGLB(NDIV,IL)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NEQ=9,KE=(MP-1)*NEQ,KPG=NEQ*5)
	COMPLEX*16, DIMENSION(KE,KPG) :: A, C
	COMPLEX*16, DIMENSION(NEQ,NEQ,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION, DIMENSION(NEQ) :: AI, BC
	COMMON /EQTYPE/AI,BC
	COMMON /DICTAT/Y,YA,Z,ZA
        COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /TESTEP/HAFH,H2,HSQ12,H8
	COMMON /GLPROP/A,C
	COMMON /FARFELD/IASYMP

	MEQ=2*IL
	JEND=NDIV-2

	NP=1
	IC=1

        A(IC,IC)=H2*Z(1,1,NP)-3.D0*HAFH*Y(1,1,NP)
        C(IC,IC)=H2*ZA(1,1,NP)-3.D0*HAFH*YA(1,1,NP)
        A(IC,IC+IL)=4.D0*HAFH*Y(1,1,NP)
        C(IC,IC+IL)=4.D0*HAFH*YA(1,1,NP)
        A(IC,IC+MEQ)=-HAFH*Y(1,1,NP)
        C(IC,IC+MEQ)=-HAFH*YA(1,1,NP)

	DO J=2,IL
        A(IC,J)=H2*Z(1,J,NP)-3.D0*HAFH*Y(1,J,NP)
        C(IC,J)=H2*ZA(1,J,NP)-3.D0*HAFH*YA(1,J,NP)
	A(IC,J+IL)=4.D0*HAFH*Y(1,J,NP)
	C(IC,J+IL)=4.D0*HAFH*YA(1,J,NP)
	A(IC,J+MEQ)=-HAFH*Y(1,J,NP)
	C(IC,J+MEQ)=-HAFH*YA(1,J,NP)
	END DO


!	for temporal bit IL is 5 or 4(2d-form) therfor need if
	
	IF(IL == 9)THEN

	DO IJ=2,3
        A(IJ,IJ)=1.D0
	END DO

	IJ=4
        A(4,1)=H2*Z(4,1,1)
        A(4,2)=-3.D0*HAFH*Y(4,2,1)
        A(4,2+IL)=4.D0*HAFH*Y(4,2,1)
        A(4,2+MEQ)=-HAFH*Y(4,2,1)
        A(4,4)=H2*Z(4,4,1)
        A(4,8)=H2*Z(4,8,1)

	A(5,5)=1.D0
	C(5,4)=-1.D0

	DO IJ=6,IL
        A(IJ,IJ)=1.D0
	END DO

	ELSE

        IJ=2
        A(IJ,IJ)=1.D0

        IJ=3
        A(IJ,1)=H2*Z(IJ,1,1)
        A(IJ,2)=-3.D0*HAFH*Y(IJ,2,1)
        A(IJ,2+IL)=4.D0*HAFH*Y(IJ,2,1)
        A(IJ,2+MEQ)=-HAFH*Y(IJ,2,1)
        A(IJ,3)=H2*Z(IJ,3,1)

        DO IJ=4,IL
        A(IJ,IJ)=1.D0
	END DO

	END IF

!  ***********   first block incorporates boundary conditions at wall *****
	NB=2
	NP=NB
	IC=(NB-1)*IL+1

        A(IC,IC)=-2.D0*AI(1)+H2*Z(1,1,NP)
        C(IC,IC)=H2*ZA(1,1,NP)
        A(IC,IC+IL)=AI(1)+HAFH*Y(1,1,NP)
        C(IC,IC+IL)=HAFH*YA(1,1,NP)
        A(IC,IC-IL)=AI(1)-HAFH*Y(1,1,NP)
        C(IC,IC-IL)=-HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=IC+J-1
	A(IC,JT)=H2*Z(1,J,NP)
	C(IC,JT)=H2*ZA(1,J,NP)
	A(IC,JT+IL)=HAFH*Y(1,J,NP)
	C(IC,JT+IL)=HAFH*YA(1,J,NP)
	A(IC,JT-IL)=-HAFH*Y(1,J,NP)
	C(IC,JT-IL)=-HAFH*YA(1,J,NP)
	END DO

	DO 30 IJ=2,IL
	NIJ=IC+IJ-1
	DO I=1,IJ-1
	JT=I+IC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
	C(NIJ,JT)=H2*ZA(IJ,I,NP)
	A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
	C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
	A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
	C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO
        A(NIJ,NIJ)=-2.D0*AI(IJ)+H2*Z(IJ,IJ,NP)
	C(NIJ,NIJ)=H2*ZA(IJ,IJ,NP)
	A(NIJ,NIJ+IL)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
	C(NIJ,NIJ+IL)=HAFH*YA(IJ,IJ,NP)
	A(NIJ,NIJ-IL)=AI(IJ)-HAFH*Y(IJ,IJ,NP)
	C(NIJ,NIJ-IL)=-HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+IC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
	C(NIJ,JT)=H2*ZA(IJ,I,NP)
	A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
	C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
	A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
	C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO
30	CONTINUE


!  ***************************************************
        LC=2*IL+1
        DO 601 NB=3,JEND-1
	NP=NB
	IC=(NB-1)*IL+1
	
  	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC+IL)=16.D0*AI(1)+H8*Y(1,1,NP)
	C(IC,LC+IL)=H8*YA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC+MEQ)=-AI(1)-DY*Y(1,1,NP)
	C(IC,LC+MEQ)=-DY*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)
	
	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT+IL)=H8*Y(1,J,NP)
        C(IC,JT+IL)=H8*YA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT+MEQ)=-DY*Y(1,J,NP)
        C(IC,JT+MEQ)=-DY*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT+MEQ)=-DY*Y(IJ,I,NP)
        C(NIJ,JT+MEQ)=-DY*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=16.D0*AI(IJ)+H8*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP+MEQ)=-AI(IJ)-DY*Y(IJ,IJ,NP)
        C(NIJ,NIP+MEQ)=-DY*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)

	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT+MEQ)=-DY*Y(IJ,I,NP)
        C(NIJ,JT+MEQ)=-DY*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO
601	CONTINUE
 
!  *****************************************************************
	NB=JEND
	NP=NB
	IC=(NB-1)*IL+1

	IF(IASYMP == 1)THEN

  	A(IC,LC)=-2.D0*AI(1)+H2*Z(1,1,NP)
	C(IC,LC)=H2*ZA(1,1,NP)
	A(IC,LC+IL)=AI(1)+HAFH*Y(1,1,NP)
	C(IC,LC+IL)=HAFH*YA(1,1,NP)
	A(IC,LC-IL)=AI(1)-HAFH*Y(1,1,NP)
	C(IC,LC-IL)=-HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=H2*Z(1,J,NP)
        C(IC,JT)=H2*ZA(1,J,NP)
        A(IC,JT+IL)=HAFH*Y(1,J,NP)
        C(IC,JT+IL)=HAFH*YA(1,J,NP)
        A(IC,JT-IL)=-HAFH*Y(1,J,NP)
        C(IC,JT-IL)=-HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-2.D0*AI(IJ)+H2*Z(IJ,IJ,NP)
        C(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=AI(IJ)-HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE

  	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC+IL)=16.D0*AI(1)+H8*Y(1,1,NP)
	C(IC,LC+IL)=H8*YA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT+IL)=H8*Y(1,J,NP)
        C(IC,JT+IL)=H8*YA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=16.D0*AI(IJ)+H8*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO
	
	END IF

 
!  *****************************************************************
	NB=JEND+1
	NP=NB
	IC=(NB-1)*IL+1
	
	IF(IASYMP == 1)THEN

  	A(IC,LC)=AI(1)+H2*Z(1,1,NP)+3.D0*HAFH*Y(1,1,NP)
	C(IC,LC)=H2*ZA(1,1,NP)+3.D0*HAFH*YA(1,1,NP)
	A(IC,LC-IL)=-2.D0*AI(1)-4.d0*HAFH*Y(1,1,NP)
	C(IC,LC-IL)=-4.d0*HAFH*YA(1,1,NP)
	A(IC,LC-MEQ)=AI(1)+HAFH*Y(1,1,NP)
	C(IC,LC-MEQ)=HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=H2*Z(1,J,NP)+3.D0*HAFH*Y(1,J,NP)
        C(IC,JT)=H2*ZA(1,J,NP)+3.D0*HAFH*YA(1,J,NP)
        A(IC,JT-IL)=-4.d0*HAFH*Y(1,J,NP)
        C(IC,JT-IL)=-4.d0*HAFH*YA(1,J,NP)
        A(IC,JT-MEQ)=HAFH*Y(1,J,NP)
        C(IC,JT-MEQ)=HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
        A(NIJ,NIP)=AI(IJ)+H2*Z(IJ,IJ,NP)+3.D0*HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP)=H2*ZA(IJ,IJ,NP)+3.D0*HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=-2.D0*AI(IJ)-4.d0*HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-4.d0*HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE
	  	
	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO

	END IF


	RETURN
	END



	SUBROUTINE SETGLB_QZ4(NDIV,IL)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NEQ=9,KE=(MP-1)*NEQ,KPG=NEQ*5)
	COMPLEX*16, DIMENSION(KE,KPG) :: A, C
	COMPLEX*16, DIMENSION(NEQ,NEQ,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION, DIMENSION(NEQ) :: AI, BC
	COMMON /EQTYPE/AI,BC
	COMMON /DICTAT/Y,YA,Z,ZA
        COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /TESTEP/HAFH,H2,HSQ12,H8
	COMMON /GLPROP/A,C
	COMMON /FARFELD/IASYMP
	PRINT*,' calling 4-4th exact..' 
!	HAFH=0.5D0*DY
	H26=H2*6.d0
	
	MEQ=2*IL
	ME3Q=3*IL
	ME4Q=4*IL
	JEND=NDIV-2

	NP=1
	IC=1

        A(IC,IC)=H26*Z(1,1,NP)-12.5D0*HAFH*Y(1,1,NP)
        C(IC,IC)=H26*ZA(1,1,NP)-12.5D0*HAFH*YA(1,1,NP)
        A(IC,IC+IL)=24.D0*HAFH*Y(1,1,NP)
        C(IC,IC+IL)=24.D0*HAFH*YA(1,1,NP)
        A(IC,IC+MEQ)=-18.d0*HAFH*Y(1,1,NP)
        C(IC,IC+MEQ)=-18.d0*HAFH*YA(1,1,NP)
        A(IC,IC+ME3Q)=8.d0*HAFH*Y(1,1,NP)
        C(IC,IC+ME3Q)=8.d0*HAFH*YA(1,1,NP)
        A(IC,IC+ME4Q)=-1.5d0*HAFH*Y(1,1,NP)
        C(IC,IC+ME4Q)=-1.5d0*HAFH*YA(1,1,NP)

	DO J=2,IL
        A(IC,J)=H26*Z(1,J,NP)-12.5D0*HAFH*Y(1,J,NP)
        C(IC,J)=H26*ZA(1,J,NP)-12.5D0*HAFH*YA(1,J,NP)
	A(IC,J+IL)=24.D0*HAFH*Y(1,J,NP)
	C(IC,J+IL)=24.D0*HAFH*YA(1,J,NP)
	A(IC,J+MEQ)=-18.d0*HAFH*Y(1,J,NP)
	C(IC,J+MEQ)=-18.d0*HAFH*YA(1,J,NP)
	A(IC,J+ME3Q)=8.d0*HAFH*Y(1,J,NP)
	C(IC,J+ME3Q)=8.d0*HAFH*YA(1,J,NP)
	A(IC,J+ME4Q)=-1.5d0*HAFH*Y(1,J,NP)
	C(IC,J+ME4Q)=-1.5d0*HAFH*YA(1,J,NP)
	END DO


!	for temporal bit IL is 5 or 4(2d-form) therfor need if
	
	IF(IL == 9)THEN

	DO IJ=2,3
        A(IJ,IJ)=1.D0
	END DO

	IJ=4
        A(4,1)=H26*Z(4,1,1)
        A(4,2)=-12.5D0*HAFH*Y(4,2,1)
        A(4,2+IL)=24.D0*HAFH*Y(4,2,1)
        A(4,2+MEQ)=-18.d0*HAFH*Y(4,2,1)
        A(4,2+ME3Q)=8.d0*HAFH*Y(4,2,1)
        A(4,2+ME4Q)=-1.5d0*HAFH*Y(4,2,1)
        A(4,4)=H26*Z(4,4,1)
        A(4,8)=H26*Z(4,8,1)

	A(5,5)=1.D0
	C(5,4)=-1.D0

	DO IJ=6,IL
        A(IJ,IJ)=1.D0
	END DO

	ELSE

        IJ=2
        A(IJ,IJ)=1.D0

        IJ=3
        A(IJ,1)=H26*Z(IJ,1,1)
        A(IJ,2)=-12.5D0*HAFH*Y(IJ,2,1)
        A(IJ,2+IL)=24.D0*HAFH*Y(IJ,2,1)
        A(IJ,2+MEQ)=-18.d0*HAFH*Y(IJ,2,1)
        A(IJ,2+ME3Q)=8.d0*HAFH*Y(IJ,2,1)
        A(IJ,2+ME4Q)=-1.5d0*HAFH*Y(IJ,2,1)
        A(IJ,3)=H26*Z(IJ,3,1)

        DO IJ=4,IL
        A(IJ,IJ)=1.D0
	END DO

	END IF

!  ***********   first block incorporates boundary conditions at wall *****
	NB=2
	NP=NB
	IC=(NB-1)*IL+1

        A(IC,IC)=-10.D0*AI(1)+H26*Z(1,1,NP)-5.d0*HAFH*Y(1,1,NP)
        C(IC,IC)=H26*ZA(1,1,NP)-5.d0*HAFH*YA(1,1,NP)
        A(IC,IC+IL)=3.d0*AI(1)+9.d0*HAFH*Y(1,1,NP)
        A(IC,IC+MEQ)=2.d0*AI(1)-3.d0*HAFH*Y(1,1,NP)
        A(IC,IC+ME3Q)=-0.5d0*AI(1)+0.5d0*HAFH*Y(1,1,NP)
        C(IC,IC+IL)=9.d0*HAFH*YA(1,1,NP)
        C(IC,IC+MEQ)=-3.d0*HAFH*YA(1,1,NP)
        C(IC,IC+ME3Q)=0.5d0*HAFH*YA(1,1,NP)
        A(IC,IC-IL)=5.5d0*AI(1)-1.5d0*HAFH*Y(1,1,NP)
        C(IC,IC-IL)=-1.5d0*HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=IC+J-1
	A(IC,JT)=H26*Z(1,J,NP)-5.d0*HAFH*Y(1,J,NP)
	C(IC,JT)=H26*ZA(1,J,NP)-5.d0*HAFH*YA(1,J,NP)
	A(IC,JT+IL)=9.d0*HAFH*Y(1,J,NP)
	A(IC,JT+MEQ)=-3.d0*HAFH*Y(1,J,NP)
	A(IC,JT+ME3Q)=0.5d0*HAFH*Y(1,J,NP)
	C(IC,JT+IL)=9.d0*HAFH*YA(1,J,NP)
	C(IC,JT+MEQ)=-3.d0*HAFH*YA(1,J,NP)
	C(IC,JT+ME3Q)=0.5d0*HAFH*YA(1,J,NP)
	A(IC,JT-IL)=-1.5d0*HAFH*Y(1,J,NP)
	C(IC,JT-IL)=-1.5d0*HAFH*YA(1,J,NP)
	END DO

	DO 30 IJ=2,IL
	NIJ=IC+IJ-1
	DO I=1,IJ-1
	JT=I+IC-1
        A(NIJ,JT)=H26*Z(IJ,I,NP)-5.d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT)=H26*ZA(IJ,I,NP)-5.d0*HAFH*YA(IJ,I,NP)
	A(NIJ,JT+IL)=9.d0*HAFH*Y(IJ,I,NP)
	A(NIJ,JT+MEQ)=-3.d0*HAFH*Y(IJ,I,NP)
	A(NIJ,JT+ME3Q)=0.5d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT+IL)=9.d0*HAFH*YA(IJ,I,NP)
	C(NIJ,JT+MEQ)=-3.d0*HAFH*YA(IJ,I,NP)
	C(NIJ,JT+ME3Q)=0.5d0*HAFH*YA(IJ,I,NP)
	A(NIJ,JT-IL)=-1.5d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT-IL)=-1.5d0*HAFH*YA(IJ,I,NP)
	END DO
        A(NIJ,NIJ)=-10.D0*AI(IJ)+H26*Z(IJ,IJ,NP)-5.d0*HAFH*Y(IJ,IJ,NP)
	C(NIJ,NIJ)=H26*ZA(IJ,IJ,NP)-5.d0*HAFH*YA(IJ,IJ,NP)
	A(NIJ,NIJ+IL)=3.d0*AI(IJ)+9.d0*HAFH*Y(IJ,IJ,NP)
	A(NIJ,NIJ+MEQ)=2.d0*AI(IJ)-3.d0*HAFH*Y(IJ,IJ,NP)
	A(NIJ,NIJ+ME3Q)=-0.5d0*AI(IJ)+0.5d0*HAFH*Y(IJ,IJ,NP)
	C(NIJ,NIJ+IL)=HAFH*YA(IJ,IJ,NP)
	C(NIJ,NIJ+MEQ)=-3.d0*HAFH*YA(IJ,IJ,NP)
	C(NIJ,NIJ+ME3Q)=0.5d0*HAFH*YA(IJ,IJ,NP)
	A(NIJ,NIJ-IL)=5.5d0*AI(IJ)-1.5d0*HAFH*Y(IJ,IJ,NP)
	C(NIJ,NIJ-IL)=-1.5d0*HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+IC-1
        A(NIJ,JT)=H26*Z(IJ,I,NP)-5.d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT)=H26*ZA(IJ,I,NP)-5.d0*HAFH*YA(IJ,I,NP)
	A(NIJ,JT+IL)=9.d0*HAFH*Y(IJ,I,NP)
	A(NIJ,JT+MEQ)=-3.d0*HAFH*Y(IJ,I,NP)
	A(NIJ,JT+ME3Q)=0.5d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT+IL)=9.d0*HAFH*YA(IJ,I,NP)
	C(NIJ,JT+MEQ)=-3.d0*HAFH*YA(IJ,I,NP)
	C(NIJ,JT+ME3Q)=0.5d0*HAFH*YA(IJ,I,NP)
	A(NIJ,JT-IL)=-1.5d0*HAFH*Y(IJ,I,NP)
	C(NIJ,JT-IL)=-1.5d0*HAFH*YA(IJ,I,NP)
	END DO
30	CONTINUE

!  ***************************************************
        LC=2*IL+1
        DO 601 NB=3,JEND-1
	NP=NB
	IC=(NB-1)*IL+1
	
  	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC+IL)=16.D0*AI(1)+H8*Y(1,1,NP)
	C(IC,LC+IL)=H8*YA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC+MEQ)=-AI(1)-DY*Y(1,1,NP)
	C(IC,LC+MEQ)=-DY*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)
	
	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT+IL)=H8*Y(1,J,NP)
        C(IC,JT+IL)=H8*YA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT+MEQ)=-DY*Y(1,J,NP)
        C(IC,JT+MEQ)=-DY*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT+MEQ)=-DY*Y(IJ,I,NP)
        C(NIJ,JT+MEQ)=-DY*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=16.D0*AI(IJ)+H8*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP+MEQ)=-AI(IJ)-DY*Y(IJ,IJ,NP)
        C(NIJ,NIP+MEQ)=-DY*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)

	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT+MEQ)=-DY*Y(IJ,I,NP)
        C(NIJ,JT+MEQ)=-DY*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO
601	CONTINUE
 
!  *****************************************************************
	NB=JEND
	NP=NB
	IC=(NB-1)*IL+1

	IF(IASYMP == 1)THEN

  	A(IC,LC)=-2.D0*AI(1)+H2*Z(1,1,NP)
	C(IC,LC)=H2*ZA(1,1,NP)
	A(IC,LC+IL)=AI(1)+HAFH*Y(1,1,NP)
	C(IC,LC+IL)=HAFH*YA(1,1,NP)
	A(IC,LC-IL)=AI(1)-HAFH*Y(1,1,NP)
	C(IC,LC-IL)=-HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=H2*Z(1,J,NP)
        C(IC,JT)=H2*ZA(1,J,NP)
        A(IC,JT+IL)=HAFH*Y(1,J,NP)
        C(IC,JT+IL)=HAFH*YA(1,J,NP)
        A(IC,JT-IL)=-HAFH*Y(1,J,NP)
        C(IC,JT-IL)=-HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-2.D0*AI(IJ)+H2*Z(IJ,IJ,NP)
        C(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=AI(IJ)-HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE

  	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC+IL)=16.D0*AI(1)+H8*Y(1,1,NP)
	C(IC,LC+IL)=H8*YA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT+IL)=H8*Y(1,J,NP)
        C(IC,JT+IL)=H8*YA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO

        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP+IL)=16.D0*AI(IJ)+H8*Y(IJ,IJ,NP)
        C(NIJ,NIP+IL)=H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        C(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO
	
	END IF

 
!  *****************************************************************
	NB=JEND+1
	NP=NB
	IC=(NB-1)*IL+1
	
	IF(IASYMP == 1)THEN

  	A(IC,LC)=AI(1)+H2*Z(1,1,NP)+3.D0*HAFH*Y(1,1,NP)
	C(IC,LC)=H2*ZA(1,1,NP)+3.D0*HAFH*YA(1,1,NP)
	A(IC,LC-IL)=-2.D0*AI(1)-4.d0*HAFH*Y(1,1,NP)
	C(IC,LC-IL)=-4.d0*HAFH*YA(1,1,NP)
	A(IC,LC-MEQ)=AI(1)+HAFH*Y(1,1,NP)
	C(IC,LC-MEQ)=HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=H2*Z(1,J,NP)+3.D0*HAFH*Y(1,J,NP)
        C(IC,JT)=H2*ZA(1,J,NP)+3.D0*HAFH*YA(1,J,NP)
        A(IC,JT-IL)=-4.d0*HAFH*Y(1,J,NP)
        C(IC,JT-IL)=-4.d0*HAFH*YA(1,J,NP)
        A(IC,JT-MEQ)=HAFH*Y(1,J,NP)
        C(IC,JT-MEQ)=HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
        A(NIJ,NIP)=AI(IJ)+H2*Z(IJ,IJ,NP)+3.D0*HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP)=H2*ZA(IJ,IJ,NP)+3.D0*HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=-2.D0*AI(IJ)-4.d0*HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-4.d0*HAFH*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE
	  	
	A(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	C(IC,LC)=HSQ12*ZA(1,1,NP)
	A(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	C(IC,LC-IL)=-H8*YA(1,1,NP)
	A(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	C(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        A(IC,JT)=HSQ12*Z(1,J,NP)
        C(IC,JT)=HSQ12*ZA(1,J,NP)
        A(IC,JT-IL)=-H8*Y(1,J,NP)
        C(IC,JT-IL)=-H8*YA(1,J,NP)
        A(IC,JT-MEQ)=DY*Y(1,J,NP)
        C(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=LC+IJ-1
	DO I=1,IJ-1
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
        A(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        C(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        A(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        C(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        A(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        C(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=I+LC-1
        A(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        C(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        A(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        C(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        A(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        C(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO

	END IF


	RETURN
	END SUBROUTINE SETGLB_QZ4



	SUBROUTINE SETGLB_QZ(NDIV,IL)
        use param_info, only : MP, MPG
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NEQ=9,KE=(MP-1)*NEQ,KPG=NEQ*5)
	PARAMETER (KEG=(MPG-1)*NEQ+2)
	COMPLEX*16 AQZ(KEG, KEG),BQZ(KEG, KEG)
	COMPLEX*16, DIMENSION(NEQ,NEQ,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION, DIMENSION(NEQ) :: AI, BC
	DOUBLE PRECISION, dimension(5) ::  fd1,fd2,gd1,gd2,f41,f42
	DOUBLE PRECISION, dimension(7) ::  f61, f62
	DOUBLE PRECISION, dimension(9) ::  f81, f82,f811,f812 , &
     & f821,f822, f831,f832 ,f841,f842
	DOUBLE PRECISION, dimension(4,9) ::  g81, g82
! 4th order
        data fd1/-25.d0,48.d0,-36.d0,16.d0,-3.d0/
 	data fd2/35.d0,-104.d0,114.d0,-56.d0,11.d0/	       
        data gd1/-3.d0,-10.d0,18.d0, -6.d0, 1.d0/
	data gd2/11.d0,-20.d0,6.d0,4.d0,-1.d0/
     	data f61/-0.3d0,2.7d0,-13.5d0,0.0d0,13.5d0,-2.7d0,0.3d0/
        data f62/0.2d0,-2.7d0,27.d0,-49.d0,27.d0,-2.7d0,0.2d0/
	data f41/1.0d0,-8.d0,0.0d0,8.d0,-1.d0/
        data f42/-1.0d0,16.d0,-30.d0,16.d0,-1.d0/
	data f81/0.281250d0,-3.0d0,15.750d0,-63.0d0,0.0d0, 63.0d0,  &
     & -15.750d0,3.0d0,-0.281250d0/
        data f82/-0.140625d0,2.d0,-15.75d0,126.d0,-224.21875d0, &
     & 126.d0,-15.75d0,2.d0, -0.140625d0/	
        data f811/-57.075d0,168.d0,-294.d0,392.d0,-367.5d0,235.2d0, &
     & -98.d0,24.d0,-2.625d0/
  	data f812/369.1375d0,-1731.6d0,3912.3d0, -5608.4d0,5441.625d0,&
     & -3553.2d0,1500.1d0,-370.8d0,40.8375d0/
	data f821/-1.575d0,-20.07d0,44.1d0,-44.1d0,36.75d0,-22.05d0, &
     & 8.82d0,-2.1d0,0.225d0/
 	data f822/8.1675d0,0.32d0, -52.29d0,96.39d0,-92.575d0, &
     & 59.22d0,-24.57d0,5.99d0,-0.6525d0/
        data f831/1.125d0,-18.d0,-59.85d0,126.d0,-78.75d0,42.d0, &
     & -15.75d0,3.6d0,-0.375d0/
        data f832/-3.2625d0,70.2d0,-115.85d0,12.6d0,70.875d0, &
     & -51.8d0,22.05d0,-5.4d0,0.5875d0/	   
     	data f841/-0.375d0,4.5d0,-31.5d0,-28.35d0,78.75d0,-31.5d0, &
     & 10.5d0,-2.25d0,0.225d0/
	data f842/0.5875d0,-8.55d0,91.35d0,-165.2d0,86.625d0, &
     & -3.15d0,-2.45d0,0.9d0,-0.1125d0/	   	          
	COMMON /EQTYPE/AI,BC
	COMMON /DICTAT/Y,YA,Z,ZA
        COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /TESTEP/HAFH,H2,HSQ12,H8
	COMMON /GLPQZ/AQZ,BQZ
	COMMON /FARFELD/IASYMP
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	IQZ=1

	AQZ=0.d0 ; BQZ=0.d0


	MEQ=2*IL
	JEND=NDIV-2

	NP=1
	IC=1
	twelvth=1.d0/12.d0
	DO KKi=1,5
	gd2(kki)=gd2(kki)*twelvth
	gd1(kki)=gd1(kki)*DY*twelvth
	f41(kki)=f41(kki)*DY*twelvth
	f42(kki)=f42(kki)*twelvth
	end do
	
	eitennth=1.d0/18.d0
	DO KKi=1,7
	f61(kki)=f61(kki)*DY*eitennth
	f62(kki)=f62(kki)*eitennth
	end do	
	
	s63rd=1.d0/63.D0
	DO KKi=1,9
	f81(kki)=f81(kki)*DY/78.75D0
	f82(kki)=f82(kki)/78.75D0
	g81(1,kki)=f811(kki)*DY*3.d0*s63rd
	g82(1,kki)=f812(kki)*s63rd
	g81(2,kki)=f821(kki)*DY/12.60D0
	g82(2,kki)=f822(kki)/12.60D0
	g81(3,kki)=f831(kki)*DY*s63rd
	g82(3,kki)=f832(kki)*s63rd
	g81(4,kki)=f841(kki)*DY*s63rd
	g82(4,kki)=f842(kki)*s63rd
	end do
	
	HAFH12=HAFH/6.d0

	
	KORD=8
	Mi=KORD/2

        AQZ(IC,IC)=H2*Z(1,1,NP)+g81(1,1)*Y(1,1,NP)
        BQZ(IC,IC)=H2*ZA(1,1,NP)+g81(1,1)*YA(1,1,NP)
	DO kki=1,KORD
        AQZ(IC,IC+IL*kki)=g81(1,kki+1)*Y(1,1,NP)
        BQZ(IC,IC+IL*kki)=g81(1,kki+1)*YA(1,1,NP)
	end do

	DO J=2,IL
        AQZ(IC,J)=H2*Z(1,J,NP)+g81(1,1)*Y(1,J,NP)
        BQZ(IC,J)=H2*ZA(1,J,NP)+g81(1,1)*YA(1,J,NP)
	DO kki=1,KORD
	AQZ(IC,J+IL*kki)=g81(1,kki+1)*Y(1,J,NP)
	BQZ(IC,J+IL*kki)=g81(1,kki+1)*YA(1,J,NP)
	end do
	end do


!	for temporal bit IL is 5 or 4(2d-form) therfor need if
	
	IF(IL == 9)THEN

	DO IJ=2,3
        AQZ(IJ,IJ)=1.D0
	end do

	IJ=4
        AQZ(4,1)=H2*Z(4,1,1)
	DO kki=0,KORD
        AQZ(4,2+IL*kki)=g81(1,kki+1)*Y(4,2,1)
	end do
        AQZ(4,4)=H2*Z(4,4,1)
        AQZ(4,8)=H2*Z(4,8,1)

	AQZ(5,5)=1.D0
	BQZ(5,4)=-1.D0

	DO  IJ=6,IL
        AQZ(IJ,IJ)=1.D0
	end do

	ELSE

        IJ=2
        AQZ(IJ,IJ)=1.D0

        IJ=3
        AQZ(IJ,1)=H2*Z(IJ,1,1)
	DO kki=0,KORD
        AQZ(IJ,2+IL*kki)=g81(1,kki+1)*Y(IJ,2,1)
	end do
        AQZ(IJ,3)=H2*Z(IJ,3,1)

        DO IJ=4,IL
        AQZ(IJ,IJ)=1.D0
	end do

	END IF
	
!  ***********   first block incorporates boundary conditions at wall *****
	KORD4=5
	NB=2
	NP=NB
	IC=(NB-1)*IL+1

        AQZ(IC,IC)=gd2(NB)*AI(1)+H2*Z(1,1,NP)+gd1(NB)*Y(1,1,NP)
        BQZ(IC,IC)=H2*ZA(1,1,NP)+gd1(NB)*YA(1,1,NP)
	DO KKi=1,KORD4-NB
        AQZ(IC,IC+KKi*IL)=gd2(NB+KKi)*AI(1)+gd1(NB+KKi)*Y(1,1,NP)
        BQZ(IC,IC+KKi*IL)=gd1(NB+KKi)*YA(1,1,NP)
	end do
	DO KKi=1,NB-1
        AQZ(IC,IC-IL*KKi)=gd2(KKi)*AI(1)+gd1(KKi)*Y(1,1,NP)
        BQZ(IC,IC-IL*KKi)=gd1(KKi)*YA(1,1,NP)
	end do

	DO  J=2,IL
	JT=IC+J-1
	AQZ(IC,JT)=H2*Z(1,J,NP)+gd1(NB)*Y(1,J,NP)
	BQZ(IC,JT)=H2*ZA(1,J,NP)+gd1(NB)*YA(1,J,NP)
	DO KKi=1,KORD4-NB
	AQZ(IC,JT+KKi*IL)=gd1(NB+KKi)*Y(1,J,NP)
	BQZ(IC,JT+KKi*IL)=gd1(NB+KKi)*YA(1,J,NP)
	end do
	DO KKi=1,NB-1
	AQZ(IC,JT-IL*KKi)=gd1(KKi)*Y(1,J,NP)
	BQZ(IC,JT-IL*KKi)=gd1(KKi)*YA(1,J,NP)
	end do
	end do

	DO IJ=2,IL
	NIJ=IC+IJ-1
	DO I=1,IJ-1
	JT=I+IC-1
	
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+gd1(NB)*Y(IJ,I,NP)
	BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+gd1(NB)*YA(IJ,I,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,JT+KKi*IL)=gd1(NB+KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT+KKi*IL)=gd1(NB+KKi)*YA(IJ,I,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,JT-IL*KKi)=gd1(KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT-IL*KKi)=gd1(KKi)*YA(IJ,I,NP)
	end do
	end do
	
        AQZ(NIJ,NIJ)=gd2(NB)*AI(IJ)+H2*Z(IJ,IJ,NP)+gd1(NB)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ)=H2*ZA(IJ,IJ,NP)+gd1(NB)*YA(IJ,IJ,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,NIJ+KKi*IL)=gd2(NB+KKi)*AI(IJ)+ &
     & gd1(NB+KKi)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ+KKi*IL)=gd1(NB+KKi)*YA(IJ,IJ,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,NIJ-IL*KKi)=gd2(KKi)*AI(IJ)+gd1(KKi)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ-IL*KKi)=gd1(KKi)*YA(IJ,IJ,NP)
	end do
	
	DO I=IJ+1,IL
	JT=I+IC-1
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+gd1(NB)*Y(IJ,I,NP)
	BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+gd1(NB)*YA(IJ,I,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,JT+KKi*IL)=gd1(NB+KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT+KKi*IL)=gd1(NB+KKi)*YA(IJ,I,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,JT-IL*KKi)=gd1(KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT-IL*KKi)=gd1(KKi)*YA(IJ,I,NP)
	end do
	end do
	end do
	
!  *****************************************************************
	KORD4=9
        DO  NB=3,4
	NP=NB
	IC=(NB-1)*IL+1
	LC=IC
  	AQZ(IC,LC)=g82(NB,NB)*AI(1)+H2*Z(1,1,NP)+g81(NB,NB)*Y(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)+g81(NB,NB)*YA(1,1,NP)
	DO KKi=1,KORD4-NB
        AQZ(IC,LC+KKi*IL)=g82(NB,NB+KKi)*AI(1)+g81(NB,NB+KKi)*Y(1,1,NP)
        BQZ(IC,LC+KKi*IL)=g81(NB,NB+KKi)*YA(1,1,NP)
	end do
	DO KKi=1,NB-1
        AQZ(IC,LC-IL*KKi)=g82(NB,NB-KKi)*AI(1)+g81(NB,NB-KKi)*Y(1,1,NP)
        BQZ(IC,LC-IL*KKi)=g81(NB,NB-KKi)*YA(1,1,NP)
	end do
	
	DO  J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)+g81(NB,NB)*Y(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)+g81(NB,NB)*YA(1,J,NP)
	DO KKi=1,KORD4-NB
	AQZ(IC,JT+KKi*IL)=g81(NB,NB+KKi)*Y(1,J,NP)
	BQZ(IC,JT+KKi*IL)=g81(NB,NB+KKi)*YA(1,J,NP)
	end do
	DO KKi=1,NB-1
	AQZ(IC,JT-IL*KKi)=g81(NB,NB-KKi)*Y(1,J,NP)
	BQZ(IC,JT-IL*KKi)=g81(NB,NB-KKi)*YA(1,J,NP)
	end do
	end do

	DO  IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIP
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+g81(NB,NB)*Y(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+g81(NB,NB)*YA(IJ,I,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,JT+KKi*IL)=g81(NB,NB+KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT+KKi*IL)=g81(NB,NB+KKi)*YA(IJ,I,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,JT-IL*KKi)=g81(NB,NB-KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT-IL*KKi)=g81(NB,NB-KKi)*YA(IJ,I,NP)
	end do
	end do

        AQZ(NIJ,NIJ)=g82(NB,NB)*AI(IJ)+H2*Z(IJ,IJ,NP)+ &
     & g81(NB,NB)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ)=H2*ZA(IJ,IJ,NP)+g81(NB,NB)*YA(IJ,IJ,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,NIJ+KKi*IL)=g82(NB,NB+KKi)*AI(IJ)+ &
     & g81(NB,NB+KKi)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ+KKi*IL)=g81(NB,NB+KKi)*YA(IJ,IJ,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,NIJ-IL*KKi)=g82(NB,NB-KKi)*AI(IJ)+ &
     & g81(NB,NB-KKi)*Y(IJ,IJ,NP)
	BQZ(NIJ,NIJ-IL*KKi)=g81(NB,NB-KKi)*YA(IJ,IJ,NP)
	end do

	DO  I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+g81(NB,NB)*Y(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+g81(NB,NB)*YA(IJ,I,NP)
	DO KKi=1,KORD4-NB
	AQZ(NIJ,JT+KKi*IL)=g81(NB,NB+KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT+KKi*IL)=g81(NB,NB+KKi)*YA(IJ,I,NP)
	end do
	DO KKi=1,NB-1
	AQZ(NIJ,JT-IL*KKi)=g81(NB,NB-KKi)*Y(IJ,I,NP)
	BQZ(NIJ,JT-IL*KKi)=g81(NB,NB-KKi)*YA(IJ,I,NP)
	end do
	end do
	
	end do
	end do
 	
!  ***************************************************
	Mi=5
        DO  NB=Mi,JEND-(Mi-2)
	NP=NB

	IC=(NB-1)*IL+1
	LC=IC
  	AQZ(IC,LC)=f82(Mi)*AI(1)+H2*Z(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)
	do kki=1,Mi-1
	AQZ(IC,LC+IL*kki)=f82(Mi+kki)*AI(1)+f81(Mi+kki)*Y(1,1,NP)
	BQZ(IC,LC+IL*kki)=f81(Mi+kki)*YA(1,1,NP)
	AQZ(IC,LC-IL*kki)=f82(Mi-kki)*AI(1)+f81(Mi-kki)*Y(1,1,NP)
	BQZ(IC,LC-IL*kki)=f81(Mi-kki)*YA(1,1,NP)
	end do
	
	DO  J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)
	do kki=1,Mi-1
        AQZ(IC,JT+IL*kki)=f81(Mi+kki)*Y(1,J,NP)
        BQZ(IC,JT+IL*kki)=f81(Mi+kki)*YA(1,J,NP)
        AQZ(IC,JT-IL*kki)=f81(Mi-kki)*Y(1,J,NP)
        BQZ(IC,JT-IL*kki)=f81(Mi-kki)*YA(1,J,NP)
	end do
	end do

	DO  IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIP
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,Mi-1
        AQZ(NIJ,JT+IL*kki)=f81(Mi+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f81(Mi+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f81(Mi-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f81(Mi-kki)*YA(IJ,I,NP)
	end do
	end do

        AQZ(NIJ,NIP)=f82(Mi)*AI(IJ)+H2*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
	do kki=1,Mi-1
        AQZ(NIJ,NIP+IL*kki)=f82(Mi+kki)*AI(IJ)+f81(Mi+kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP+IL*kki)=f81(Mi+kki)*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL*kki)=f82(Mi-kki)*AI(IJ)+f81(Mi-kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL*kki)=f81(Mi-kki)*YA(IJ,IJ,NP)
	end do

	DO  I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,Mi-1
        AQZ(NIJ,JT+IL*kki)=f81(Mi+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f81(Mi+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f81(Mi-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f81(Mi-kki)*YA(IJ,I,NP)
	end do
	end do
	
	end do	
	
	
	end do
 
!  *****************************************************************
 
 	Mi=4
        DO  NB=JEND-(Mi-2),JEND-(Mi-2)
	NP=NB
	IC=(NB-1)*IL+1
	LC=IC
  	AQZ(IC,LC)=f62(Mi)*AI(1)+H2*Z(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)
	do kki=1,Mi-1
	AQZ(IC,LC+IL*kki)=f62(Mi+kki)*AI(1)+f61(Mi+kki)*Y(1,1,NP)
	BQZ(IC,LC+IL*kki)=f61(Mi+kki)*YA(1,1,NP)
	AQZ(IC,LC-IL*kki)=f62(Mi-kki)*AI(1)+f61(Mi-kki)*Y(1,1,NP)
	BQZ(IC,LC-IL*kki)=f61(Mi-kki)*YA(1,1,NP)
	end do
	
	DO  J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)
	do kki=1,Mi-1
        AQZ(IC,JT+IL*kki)=f61(Mi+kki)*Y(1,J,NP)
        BQZ(IC,JT+IL*kki)=f61(Mi+kki)*YA(1,J,NP)
        AQZ(IC,JT-IL*kki)=f61(Mi-kki)*Y(1,J,NP)
        BQZ(IC,JT-IL*kki)=f61(Mi-kki)*YA(1,J,NP)
	end do
	end do

	DO  IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIP
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,Mi-1
        AQZ(NIJ,JT+IL*kki)=f61(Mi+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f61(Mi+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f61(Mi-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f61(Mi-kki)*YA(IJ,I,NP)
	end do
	end do

        AQZ(NIJ,NIP)=f62(Mi)*AI(IJ)+H2*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
	do kki=1,Mi-1
        AQZ(NIJ,NIP+IL*kki)=f62(Mi+kki)*AI(IJ)+f61(Mi+kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP+IL*kki)=f61(Mi+kki)*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL*kki)=f62(Mi-kki)*AI(IJ)+f61(Mi-kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL*kki)=f61(Mi-kki)*YA(IJ,IJ,NP)
	end do

	DO  I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,Mi-1
        AQZ(NIJ,JT+IL*kki)=f61(Mi+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f61(Mi+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f61(Mi-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f61(Mi-kki)*YA(IJ,I,NP)
	end do
	end do
	
	end do
	end do
 
 
!  *****************************************************************

        DO  NB=JEND-1,JEND-1
	NP=NB
	IC=(NB-1)*IL+1
	LC=IC
  	AQZ(IC,LC)=f42(3)*AI(1)+H2*Z(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)
	do kki=1,2
	AQZ(IC,LC+IL*kki)=f42(3+kki)*AI(1)+f41(3+kki)*Y(1,1,NP)
	BQZ(IC,LC+IL*kki)=f41(3+kki)*YA(1,1,NP)
	AQZ(IC,LC-IL*kki)=f42(3-kki)*AI(1)+f41(3-kki)*Y(1,1,NP)
	BQZ(IC,LC-IL*kki)=f41(3-kki)*YA(1,1,NP)
	end do
	
	DO  J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)
	do kki=1,2
        AQZ(IC,JT+IL*kki)=f41(3+kki)*Y(1,J,NP)
        BQZ(IC,JT+IL*kki)=f41(3+kki)*YA(1,J,NP)
        AQZ(IC,JT-IL*kki)=f41(3-kki)*Y(1,J,NP)
        BQZ(IC,JT-IL*kki)=f41(3-kki)*YA(1,J,NP)
	end do
	end do

	DO  IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIP
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,2
        AQZ(NIJ,JT+IL*kki)=f41(3+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f41(3+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f41(3-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f41(3-kki)*YA(IJ,I,NP)
	end do
	end do

        AQZ(NIJ,NIP)=f42(3)*AI(IJ)+H2*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
	do kki=1,2
        AQZ(NIJ,NIP+IL*kki)=f42(3+kki)*AI(IJ)+f41(3+kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP+IL*kki)=f41(3+kki)*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL*kki)=f42(3-kki)*AI(IJ)+f41(3-kki)*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL*kki)=f41(3-kki)*YA(IJ,IJ,NP)
	end do

	DO  I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
	do kki=1,2
        AQZ(NIJ,JT+IL*kki)=f41(3+kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL*kki)=f41(3+kki)*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL*kki)=f41(3-kki)*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL*kki)=f41(3-kki)*YA(IJ,I,NP)
	end do
	end do
	
	end do
	end do
 
	
!  *****************************************************************
	NB=JEND
	NP=NB
	IC=(NB-1)*IL+1

	IF(IASYMP == 1)THEN

	LC=IC
  	AQZ(IC,LC)=-2.D0*AI(1)+H2*Z(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)
	AQZ(IC,LC+IL)=AI(1)+HAFH*Y(1,1,NP)
	BQZ(IC,LC+IL)=HAFH*YA(1,1,NP)
	AQZ(IC,LC-IL)=AI(1)-HAFH*Y(1,1,NP)
	BQZ(IC,LC-IL)=-HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=IC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)
        AQZ(IC,JT+IL)=HAFH*Y(1,J,NP)
        BQZ(IC,JT+IL)=HAFH*YA(1,J,NP)
        AQZ(IC,JT-IL)=-HAFH*Y(1,J,NP)
        BQZ(IC,JT-IL)=-HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIJ
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
        AQZ(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO

        AQZ(NIJ,NIP)=-2.D0*AI(IJ)+H2*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=H2*ZA(IJ,IJ,NP)
        AQZ(NIJ,NIP+IL)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP+IL)=HAFH*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL)=AI(IJ)-HAFH*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL)=-HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)
        AQZ(NIJ,JT+IL)=HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL)=HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE
	LC=IC
  	AQZ(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	BQZ(IC,LC)=HSQ12*ZA(1,1,NP)
	AQZ(IC,LC+IL)=16.D0*AI(1)+H8*Y(1,1,NP)
	BQZ(IC,LC+IL)=H8*YA(1,1,NP)
	AQZ(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	BQZ(IC,LC-IL)=-H8*YA(1,1,NP)
	AQZ(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	BQZ(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=HSQ12*Z(1,J,NP)
        BQZ(IC,JT)=HSQ12*ZA(1,J,NP)
        AQZ(IC,JT+IL)=H8*Y(1,J,NP)
        BQZ(IC,JT+IL)=H8*YA(1,J,NP)
        AQZ(IC,JT-IL)=-H8*Y(1,J,NP)
        BQZ(IC,JT-IL)=-H8*YA(1,J,NP)
        AQZ(IC,JT-MEQ)=DY*Y(1,J,NP)
        BQZ(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIJ
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        BQZ(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        AQZ(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO

        AQZ(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        AQZ(NIJ,NIP+IL)=16.D0*AI(IJ)+H8*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP+IL)=H8*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        BQZ(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        AQZ(NIJ,JT+IL)=H8*Y(IJ,I,NP)
        BQZ(NIJ,JT+IL)=H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO
	
	END IF

 
!  *****************************************************************
	NB=JEND+1
	NP=NB
	IC=(NB-1)*IL+1
	LC=IC
	IF(IASYMP == 1)THEN

  	AQZ(IC,LC)=AI(1)+H2*Z(1,1,NP)+3.D0*HAFH*Y(1,1,NP)
	BQZ(IC,LC)=H2*ZA(1,1,NP)+3.D0*HAFH*YA(1,1,NP)
	AQZ(IC,LC-IL)=-2.D0*AI(1)-4.d0*HAFH*Y(1,1,NP)
	BQZ(IC,LC-IL)=-4.d0*HAFH*YA(1,1,NP)
	AQZ(IC,LC-MEQ)=AI(1)+HAFH*Y(1,1,NP)
	BQZ(IC,LC-MEQ)=HAFH*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=H2*Z(1,J,NP)+3.D0*HAFH*Y(1,J,NP)
        BQZ(IC,JT)=H2*ZA(1,J,NP)+3.D0*HAFH*YA(1,J,NP)
        AQZ(IC,JT-IL)=-4.d0*HAFH*Y(1,J,NP)
        BQZ(IC,JT-IL)=-4.d0*HAFH*YA(1,J,NP)
        AQZ(IC,JT-MEQ)=HAFH*Y(1,J,NP)
        BQZ(IC,JT-MEQ)=HAFH*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIJ
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
        AQZ(NIJ,NIP)=AI(IJ)+H2*Z(IJ,IJ,NP)+3.D0*HAFH*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP)=H2*ZA(IJ,IJ,NP)+3.D0*HAFH*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL)=-2.D0*AI(IJ)-4.d0*HAFH*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL)=-4.d0*HAFH*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-MEQ)=AI(IJ)+HAFH*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-MEQ)=HAFH*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=H2*Z(IJ,I,NP)+3.D0*HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT)=H2*ZA(IJ,I,NP)+3.D0*HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-4.d0*HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-4.d0*HAFH*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=HAFH*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=HAFH*YA(IJ,I,NP)
	END DO
	END DO
	
	ELSE
	  	
	AQZ(IC,LC)=-30.D0*AI(1)+HSQ12*Z(1,1,NP)
	BQZ(IC,LC)=HSQ12*ZA(1,1,NP)
	AQZ(IC,LC-IL)=16.D0*AI(1)-H8*Y(1,1,NP)
	BQZ(IC,LC-IL)=-H8*YA(1,1,NP)
	AQZ(IC,LC-MEQ)=-AI(1)+DY*Y(1,1,NP)
	BQZ(IC,LC-MEQ)=DY*YA(1,1,NP)

	DO J=2,IL
	JT=LC+J-1
        AQZ(IC,JT)=HSQ12*Z(1,J,NP)
        BQZ(IC,JT)=HSQ12*ZA(1,J,NP)
        AQZ(IC,JT-IL)=-H8*Y(1,J,NP)
        BQZ(IC,JT-IL)=-H8*YA(1,J,NP)
        AQZ(IC,JT-MEQ)=DY*Y(1,J,NP)
        BQZ(IC,JT-MEQ)=DY*YA(1,J,NP)
	END DO

	DO IJ=2,IL
	NIJ=IC+IJ-1
	NIP=NIJ
	LC=NIJ
	DO I=1,IJ-1
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        BQZ(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
        AQZ(NIJ,NIP)=-30.D0*AI(IJ)+HSQ12*Z(IJ,IJ,NP)
        BQZ(NIJ,NIP)=HSQ12*ZA(IJ,IJ,NP)
        AQZ(NIJ,NIP-IL)=16.D0*AI(IJ)-H8*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-IL)=-H8*YA(IJ,IJ,NP)
        AQZ(NIJ,NIP-MEQ)=-AI(IJ)+DY*Y(IJ,IJ,NP)
        BQZ(NIJ,NIP-MEQ)=DY*YA(IJ,IJ,NP)
	DO I=IJ+1,IL
	JT=NIJ-IJ+I
        AQZ(NIJ,JT)=HSQ12*Z(IJ,I,NP)
        BQZ(NIJ,JT)=HSQ12*ZA(IJ,I,NP)
        AQZ(NIJ,JT-IL)=-H8*Y(IJ,I,NP)
        BQZ(NIJ,JT-IL)=-H8*YA(IJ,I,NP)
        AQZ(NIJ,JT-MEQ)=DY*Y(IJ,I,NP)
        BQZ(NIJ,JT-MEQ)=DY*YA(IJ,I,NP)
	END DO
	END DO

	END IF


	RETURN
	END




      	SUBROUTINE QZLIB(N,NM,AR,AI,BR,BI,EPS1,ALFR,ALFI, & 
     &                 BETA,MATZ,ZR,ZI,ITER,IFAIL)

      	REAL*8 EPS1
      	LOGICAL MATZ

      	REAL*8 AI(NM,N), ALFI(N), ALFR(N), AR(NM,N), BETA(N), & 
     &       BI(NM,N), BR(NM,N), ZI(NM,N), ZR(NM,N)
      	INTEGER ITER(N)

      	EPS1=1.1102230246251D-16
      	CALL  CQZHES(NM,N,AR,AI,BR,BI,MATZ,ZR,ZI)
      	CALL  CQZVAL(NM,N,AR,AI,BR,BI,EPS1,ALFR,ALFI,BETA, & 
     &             MATZ,ZR,ZI,IERROR)

      	RETURN
      	END


      	SUBROUTINE CQZHES(NM,N,AR,AI,BR,BI,MATZ,ZR,ZI)                    
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
!	
      INTEGER I,J,K,L,N,K1,LB,L1,NM,NK1,NM1
      REAL*8 AR(NM,N),AI(NM,N),BR(NM,N),BI(NM,N),ZR(NM,N),ZI(NM,N)
	LOGICAL MATZ
!
!     THIS SUBROUTINE IS A COMPLEX ANALOGUE OF THE FIRST STEP OF THE
!     QZ ALGORITHM FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
!     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART.
!
!     THIS SUBROUTINE ACCEPTS A PAIR OF COMPLEX GENERAL MATRICES AND
!     REDUCES ONE OF THEM TO UPPER HESSENBERG FORM WITH REAL (AND NON-
!     NEGATIVE) SUBDIAGONAL ELEMENTS AND THE OTHER TO UPPER TRIANGULAR
!     FORM USING UNITARY TRANSFORMATIONS.  IT IS USUALLY FOLLOWED BY
!     CQZVAL  AND POSSIBLY  CQZVEC.
!
!     ON INPUT-
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!          DIMENSION STATEMENT,
!
!        N IS THE ORDER OF THE MATRICES,
!
!        A=(AR,AI) CONTAINS A COMPLEX GENERAL MATRIX,
!
!        B=(BR,BI) CONTAINS A COMPLEX GENERAL MATRIX,
!
!           MATZ SHOULD BE SET TO .TRUE. IF THE RIGHT HAND TRANSFORMATIONS
!             ARE TO BE ACCUMULATED FOR LATER USE IN COMPUTING
!             EIGENVECTORS, AND TO .FALSE. OTHERWISE.
!   
!        ON OUTPUT-
!   
!           A HAS BEEN REDUCED TO UPPER HESSENBERG FORM.  THE ELEMENTS
!             BELOW THE FIRST SUBDIAGONAL HAVE BEEN SET TO ZERO, AND THE
!             SUBDIAGONAL ELEMENTS HAVE BEEN MADE REAL (AND NON-NEGATIVE),
!   
!           B HAS BEEN REDUCED TO UPPER TRIANGULAR FORM.  THE ELEMENTS
!             BELOW THE MAIN DIAGONAL HAVE BEEN SET TO ZERO,
!   
!           Z=(ZR,ZI) CONTAINS THE PRODUCT OF THE RIGHT HAND
!             TRANSFORMATIONS IF MATZ HAS BEEN SET TO .TRUE.
!             OTHERWISE, Z IS NOT REFERENCED.
!   
!        QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
!        APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
!   
!        ------------------------------------------------------------------
!   
!        ********** INITIALIZE Z **********
	DATA ZERO/0.D0/, ONE/1.D0/
      IF (.NOT. MATZ) GO TO 10
!   
      DO 3 I = 1, N
!   
         DO 2 J = 1, N
            ZR(I,J) = ZERO
            ZI(I,J) = ZERO
    2    CONTINUE
!   
         ZR(I,I) = ONE
    3 CONTINUE
!        ********** REDUCE B TO UPPER TRIANGULAR FORM WITH
!                   TEMPORARILY REAL DIAGONAL ELEMENTS **********
   10 IF (N  <=  1) GO TO 170
      NM1 = N - 1
!   
      DO 100 L = 1, NM1
         L1 = L + 1
         S = ZERO
!   
         DO 20 I = L, N
            S = S + DABS(BR(I,L)) + DABS(BI(I,L))
   20    CONTINUE
!   
         IF (S  ==  ZERO) GO TO 100
         RHO = ZERO
!   
         DO 25 I = L, N
            BR(I,L) = BR(I,L) / S
            BI(I,L) = BI(I,L) / S
            RHO = RHO + BR(I,L)**2 + BI(I,L)**2
   25    CONTINUE
!   
         R = DSQRT(RHO)
         XR = CDABS(DCMPLX(BR(L,L),BI(L,L)))
         IF (XR  ==  ZERO) GO TO 27
         RHO = RHO + XR * R
         U1 = -BR(L,L) / XR
         U1I = -BI(L,L) / XR
         YR = R / XR + ONE
         BR(L,L) = YR * BR(L,L)
         BI(L,L) = YR * BI(L,L)
         GO TO 28
!   
   27    BR(L,L) = R
         U1 = -ONE
         U1I = ZERO
!   
   28    DO 50 J = L1, N
            T = ZERO
            TI = ZERO
!   
            DO 30 I = L, N
               T = T + BR(I,L) * BR(I,J) + BI(I,L) * BI(I,J)
               TI = TI + BR(I,L) * BI(I,J) - BI(I,L) * BR(I,J)
   30       CONTINUE
!   
            T = T / RHO
            TI = TI / RHO
!   
            DO 40 I = L, N
               BR(I,J) = BR(I,J) - T * BR(I,L) + TI * BI(I,L)
               BI(I,J) = BI(I,J) - T * BI(I,L) - TI * BR(I,L)
   40       CONTINUE
!   
            XI = U1 * BI(L,J) - U1I * BR(L,J)
            BR(L,J) = U1 * BR(L,J) + U1I * BI(L,J)
            BI(L,J) = XI
   50    CONTINUE
!   
         DO 80 J = 1, N
            T = ZERO
            TI = ZERO
!   
            DO 60 I = L, N
               T = T + BR(I,L) * AR(I,J) + BI(I,L) * AI(I,J)
               TI = TI + BR(I,L) * AI(I,J) - BI(I,L) * AR(I,J)
   60       CONTINUE
!   
            T = T / RHO
            TI = TI / RHO
!   
            DO 70 I = L, N
               AR(I,J) = AR(I,J) - T * BR(I,L) + TI * BI(I,L)
               AI(I,J) = AI(I,J) - T * BI(I,L) - TI * BR(I,L)
   70       CONTINUE
!   
            XI = U1 * AI(L,J) - U1I * AR(L,J)
            AR(L,J) = U1 * AR(L,J) + U1I * AI(L,J)
            AI(L,J) = XI
   80    CONTINUE
!   
         BR(L,L) = R * S
         BI(L,L) = ZERO
!   
         DO 90 I = L1, N
            BR(I,L) = ZERO
            BI(I,L) = ZERO
   90    CONTINUE
!   
  100 CONTINUE
!        ********** REDUCE A TO UPPER HESSENBERG FORM WITH REAL SUBDIAGONAL
!                   ELEMENTS, WHILE KEEPING B TRIANGULAR **********
      DO 160 K = 1, NM1
         K1 = K + 1
!        ********** SET BOTTOM ELEMENT IN K-TH COLUMN OF A REAL **********
         IF (AI(N,K)  ==  ZERO) GO TO 105
         R = CDABS(DCMPLX(AR(N,K),AI(N,K)))
         U1 = AR(N,K) / R
         U1I = AI(N,K) / R
         AR(N,K) = R
         AI(N,K) = ZERO
!   
         DO 103 J = K1, N
            XI = U1 * AI(N,J) - U1I * AR(N,J)
            AR(N,J) = U1 * AR(N,J) + U1I * AI(N,J)
            AI(N,J) = XI
  103    CONTINUE
!   
         XI = U1 * BI(N,N) - U1I * BR(N,N)
         BR(N,N) = U1 * BR(N,N) + U1I * BI(N,N)
         BI(N,N) = XI
  105    IF (K  ==  NM1) GO TO 170
         NK1 = NM1 - K
!        ********** FOR L=N-1 STEP -1 UNTIL K+1 DO -- **********
         DO 150 LB = 1, NK1
            L = N - LB
            L1 = L + 1
!        ********** ZERO A(L+1,K) **********
            S = DABS(AR(L,K)) + DABS(AI(L,K)) + AR(L1,K)
            IF (S  ==  ZERO) GO TO 150
            U1 = AR(L,K) / S
            U1I = AI(L,K) / S
            U2 = AR(L1,K) / S
            R = DSQRT(U1*U1+U1I*U1I+U2*U2)
            U1 = U1 / R
            U1I = U1I / R
            U2 = U2 / R
            AR(L,K) = R * S
            AI(L,K) = ZERO
            AR(L1,K) = ZERO
!   
            DO 110 J = K1, N
               XR = AR(L,J)
               XI = AI(L,J)
               YR = AR(L1,J)
               YI = AI(L1,J)
               AR(L,J) = U1 * XR + U1I * XI + U2 * YR
               AI(L,J) = U1 * XI - U1I * XR + U2 * YI
               AR(L1,J) = U1 * YR - U1I * YI - U2 * XR
               AI(L1,J) = U1 * YI + U1I * YR - U2 * XI
  110       CONTINUE
!   
            XR = BR(L,L)
            BR(L,L) = U1 * XR
            BI(L,L) = -U1I * XR
            BR(L1,L) = -U2 * XR
!   
            DO 120 J = L1, N
               XR = BR(L,J)
               XI = BI(L,J)
               YR = BR(L1,J)
               YI = BI(L1,J)
               BR(L,J) = U1 * XR + U1I * XI + U2 * YR
               BI(L,J) = U1 * XI - U1I * XR + U2 * YI
               BR(L1,J) = U1 * YR - U1I * YI - U2 * XR
               BI(L1,J) = U1 * YI + U1I * YR - U2 * XI
  120       CONTINUE
!        ********** ZERO B(L+1,L) **********
            S = DABS(BR(L1,L1)) + DABS(BI(L1,L1)) + DABS(BR(L1,L))
            IF (S  ==  ZERO) GO TO 150
            U1 = BR(L1,L1) / S
            U1I = BI(L1,L1) / S
            U2 = BR(L1,L) / S
            R = DSQRT(U1*U1+U1I*U1I+U2*U2)
            U1 = U1 / R
            U1I = U1I / R
            U2 = U2 / R
            BR(L1,L1) = R * S
            BI(L1,L1) = ZERO
            BR(L1,L) = ZERO
!   
            DO 130 I = 1, L
               XR = BR(I,L1)
               XI = BI(I,L1)
               YR = BR(I,L)
               YI = BI(I,L)
               BR(I,L1) = U1 * XR + U1I * XI + U2 * YR
               BI(I,L1) = U1 * XI - U1I * XR + U2 * YI
               BR(I,L) = U1 * YR - U1I * YI - U2 * XR
               BI(I,L) = U1 * YI + U1I * YR - U2 * XI
  130       CONTINUE
!   
            DO 140 I = 1, N
               XR = AR(I,L1)
               XI = AI(I,L1)
               YR = AR(I,L)
               YI = AI(I,L)
               AR(I,L1) = U1 * XR + U1I * XI + U2 * YR
               AI(I,L1) = U1 * XI - U1I * XR + U2 * YI
               AR(I,L) = U1 * YR - U1I * YI - U2 * XR
               AI(I,L) = U1 * YI + U1I * YR - U2 * XI
  140       CONTINUE
!   
            IF (.NOT. MATZ) GO TO 150
!   
            DO 145 I = 1, N
               XR = ZR(I,L1)
               XI = ZI(I,L1)
               YR = ZR(I,L)
               YI = ZI(I,L)
               ZR(I,L1) = U1 * XR + U1I * XI + U2 * YR
               ZI(I,L1) = U1 * XI - U1I * XR + U2 * YI
               ZR(I,L) = U1 * YR - U1I * YI - U2 * XR
               ZI(I,L) = U1 * YI + U1I * YR - U2 * XI
  145       CONTINUE
!   
  150    CONTINUE
!   
  160 CONTINUE
!   
  170 RETURN
!        ********** LAST CARD OF CQZHES **********
      END
      SUBROUTINE CQZVAL(NM,N,AR,AI,BR,BI,EPS1,ALFR,ALFI,BETA, &         
     &                                       MATZ,ZR,ZI,IERR)
     	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
!   	
      INTEGER EN,ENM2,ENORN
      REAL*8 AR(NM,N),AI(NM,N),BR(NM,N),BI(NM,N),ALFR(N),ALFI(N),&
     &       BETA(N),ZR(NM,N),ZI(NM,N)

      INTEGER MAX0
      LOGICAL MATZ
      COMPLEX*16 Z3
!
!
!
!
!
!     THIS SUBROUTINE IS A COMPLEX ANALOGUE OF STEPS 2 AND 3 OF THE
!     QZ ALGORITHM FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
!     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART,
!     AS MODIFIED IN TECHNICAL NOTE NASA TN E-7305(1973) BY WARD.
!
!     THIS SUBROUTINE ACCEPTS A PAIR OF COMPLEX MATRICES, ONE OF THEM
!     IN UPPER HESSENBERG FORM AND THE OTHER IN UPPER TRIANGULAR FORM,
!     THE HESSENBERG MATRIX MUST FURTHER HAVE REAL SUBDIAGONAL ELEMENTS.
!     IT REDUCES THE HESSENBERG MATRIX TO TRIANGULAR FORM USING
!     UNITARY TRANSFORMATIONS WHILE MAINTAINING THE TRIANGULAR FORM
!     OF THE OTHER MATRIX AND FURTHER MAKING ITS DIAGONAL ELEMENTS
!     REAL AND NON-NEGATIVE.  IT THEN RETURNS QUANTITIES WHOSE RATIOS
!     GIVE THE GENERALIZED EIGENVALUES.  IT IS USUALLY PRECEDED BY
!     CQZHES  AND POSSIBLY FOLLOWED BY  CQZVEC.
!
!     ON INPUT-
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!          DIMENSION STATEMENT,
!
!        N IS THE ORDER OF THE MATRICES,
!
!        A=(AR,AI) CONTAINS A COMPLEX UPPER HESSENBERG MATRIX
!          WITH REAL SUBDIAGONAL ELEMENTS,
!
!        B=(BR,BI) CONTAINS A COMPLEX UPPER TRIANGULAR MATRIX,
!
!        EPS1 IS A TOLERANCE USED TO DETERMINE NEGLIGIBLE ELEMENTS.
!          EPS1 = 0.0 (OR NEGATIVE) MAY BE INPUT, IN WHICH CASE AN
!          ELEMENT WILL BE NEGLECTED ONLY IF IT IS LESS THAN ROUNDOFF
!          ERROR TIMES THE NORM OF ITS MATRIX.  IF THE INPUT EPS1 IS
!          POSITIVE, THEN AN ELEMENT WILL BE CONSIDERED NEGLIGIBLE
!          IF IT IS LESS THAN EPS1 TIMES THE NORM OF ITS MATRIX.  A
!          POSITIVE VALUE OF EPS1 MAY RESULT IN FASTER EXECUTION,
!          BUT LESS ACCURATE RESULTS,
!
!        MATZ SHOULD BE SET TO .TRUE. IF THE RIGHT HAND TRANSFORMATIONS
!          ARE TO BE ACCUMULATED FOR LATER USE IN COMPUTING
!          EIGENVECTORS, AND TO .FALSE. OTHERWISE,
!
!        Z=(ZR,ZI) CONTAINS, IF MATZ HAS BEEN SET TO .TRUE., THE
!          TRANSFORMATION MATRIX PRODUCED IN THE REDUCTION
!          BY  CQZHES, IF PERFORMED, OR ELSE THE IDENTITY MATRIX.
!          IF MATZ HAS BEEN SET TO .FALSE., Z IS NOT REFERENCED.
!
!     ON OUTPUT-
!
!        A HAS BEEN REDUCED TO UPPER TRIANGULAR FORM.  THE ELEMENTS
!          BELOW THE MAIN DIAGONAL HAVE BEEN SET TO ZERO,
!
!        B IS STILL IN UPPER TRIANGULAR FORM, ALTHOUGH ITS ELEMENTS
!          HAVE BEEN ALTERED.  IN PARTICULAR, ITS DIAGONAL HAS BEEN SET
!          REAL AND NON-NEGATIVE.  THE LOCATION BR(N,1) IS USED TO
!          STORE EPS1 TIMES THE NORM OF B FOR LATER USE BY  CQZVEC,
!
!        ALFR AND ALFI CONTAIN THE REAL AND IMAGINARY PARTS OF THE
!          DIAGONAL ELEMENTS OF THE TRIANGULARIZED A MATRIX,
!
!        BETA CONTAINS THE REAL NON-NEGATIVE DIAGONAL ELEMENTS OF THE
!          CORRESPONDING B.  THE GENERALIZED EIGENVALUES ARE THEN
!          THE RATIOS ((ALFR+I*ALFI)/BETA),
!
!        Z CONTAINS THE PRODUCT OF THE RIGHT HAND TRANSFORMATIONS
!          (FOR BOTH STEPS) IF MATZ HAS BEEN SET TO .TRUE.,
!
!        IERR IS SET TO
!          ZERO       FOR NORMAL RETURN,
!          J          IF AR(J,J-1) HAS NOT BECOME
!                     ZERO AFTER 50 ITERATIONS.
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
!     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
!
!     ------------------------------------------------------------------
!
	DATA ZERO/0.D0/, ONE/1.D0/
      IERR = 0
!     ********** COMPUTE EPSA,EPSB **********
      ANORM = ZERO
      BNORM = ZERO
! C
      DO 30 I = 1, N
         ANI = ZERO
         IF (I  /=  1) ANI = DABS(AR(I,I-1))
         BNI = ZERO
! C
         DO 20 J = I, N
            ANI = ANI + DABS(AR(I,J)) + DABS(AI(I,J))
            BNI = BNI + DABS(BR(I,J)) + DABS(BI(I,J))
   20    CONTINUE
! C
         IF (ANI  >  ANORM) ANORM = ANI
         IF (BNI  >  BNORM) BNORM = BNI
   30 CONTINUE
! C
      IF (ANORM  ==  ZERO) ANORM = ONE
      IF (BNORM  ==  ZERO) BNORM = ONE
      EP = EPS1
      IF (EP  >  ZERO) GO TO 50
! C     ********** COMPUTE ROUNDOFF LEVEL IF EPS1 IS ZERO **********
      EP = ONE
   40 EP = 0.5D0*EP 
      IF (ONE + EP  >  ONE) GO TO 40
   50 EPSA = EP * ANORM
      EPSB = EP * BNORM
! C     ********** REDUCE A TO TRIANGULAR FORM, WHILE
! C                KEEPING B TRIANGULAR **********
      LOR1 = 1
      ENORN = N
      EN = N
! C     ********** BEGIN QZ STEP **********
   60 IF (EN  ==  0) GO TO 1001
      IF (.NOT. MATZ) ENORN = EN
      ITS = 0
      NA = EN - 1
      ENM2 = NA - 1
! C     ********** CHECK FOR CONVERGENCE OR REDUCIBILITY.
! C                FOR L=EN STEP -1 UNTIL 1 DO -- **********
   70 DO 80 LL = 1, EN
         LM1 = EN - LL
         L = LM1 + 1
         IF (L  ==  1) GO TO 95
         IF (DABS(AR(L,LM1))  <=  EPSA) GO TO 90
   80 CONTINUE
! C
   90 AR(L,LM1) = ZERO
! C     ********** SET DIAGONAL ELEMENT AT TOP OF B REAL **********
   95 B11 = CDABS(DCMPLX(BR(L,L),BI(L,L)))
      IF (B11      ==  ZERO) GO TO 98
      U1 = BR(L,L) / B11
      U1I = BI(L,L) / B11
! C
      DO 97 J = L, ENORN
         XI = U1 * AI(L,J) - U1I * AR(L,J)
         AR(L,J) = U1 * AR(L,J) + U1I * AI(L,J)
         AI(L,J) = XI
         XI = U1 * BI(L,J) - U1I * BR(L,J)
         BR(L,J) = U1 * BR(L,J) + U1I * BI(L,J)
         BI(L,J) = XI
   97 CONTINUE
! C
      BI(L,L) = ZERO
   98 IF (L  /=  EN) GO TO 100
! C     ********** 1-BY-1 BLOCK ISOLATED **********
      ALFR(EN) = AR(EN,EN)
      ALFI(EN) = AI(EN,EN)
      BETA(EN) = B11
      EN = NA
      GO TO 60
! C     ********** CHECK FOR SMALL TOP OF B **********
  100 L1 = L + 1
      IF (B11  >  EPSB) GO TO 120
      BR(L,L) = ZERO
      S = DABS(AR(L,L)) + DABS(AI(L,L)) + DABS(AR(L1,L))
      U1 = AR(L,L) / S
      U1I = AI(L,L) / S
      U2 = AR(L1,L) / S
      R = DSQRT(U1*U1+U1I*U1I+U2*U2)
      U1 = U1 / R
      U1I = U1I / R
      U2 = U2 / R
      AR(L,L) = R * S
      AI(L,L) = ZERO
! C
      DO 110 J = L1, ENORN
         XR = AR(L,J)
         XI = AI(L,J)
         YR = AR(L1,J)
         YI = AI(L1,J)
         AR(L,J) = U1 * XR + U1I * XI + U2 * YR
         AI(L,J) = U1 * XI - U1I * XR + U2 * YI
         AR(L1,J) = U1 * YR - U1I * YI - U2 * XR
         AI(L1,J) = U1 * YI + U1I * YR - U2 * XI
         XR = BR(L,J)
         XI = BI(L,J)
         YR = BR(L1,J)
         YI = BI(L1,J)
         BR(L1,J) = U1 * YR - U1I * YI - U2 * XR
         BR(L,J) = U1 * XR + U1I * XI + U2 * YR
         BI(L,J) = U1 * XI - U1I * XR + U2 * YI
         BI(L1,J) = U1 * YI + U1I * YR - U2 * XI
  110 CONTINUE
! C
      LM1 = L
      L = L1
      GO TO 90
! C     ********** ITERATION STRATEGY **********
  120 IF (ITS  ==  50) GO TO 1000
      IF (ITS  ==  10) GO TO 135
! C     ********** DETERMINE SHIFT **********
      B33 = BR(NA,NA)
      B33I = BI(NA,NA)
      IF (CDABS(DCMPLX(B33,B33I))  >=  EPSB) GO TO 122
      B33 = EPSB
      B33I = ZERO
  122 B44 = BR(EN,EN)
      B44I = BI(EN,EN)
      IF (CDABS(DCMPLX(B44,B44I)) >= EPSB) GO TO 124
      B44 = EPSB
      B44I = ZERO
  124 B3344 = B33 * B44 - B33I * B44I
      B3344I = B33 * B44I + B33I * B44
      A33 = AR(NA,NA) * B44 - AI(NA,NA) * B44I
      A33I = AR(NA,NA) * B44I + AI(NA,NA) * B44
      A34 = AR(NA,EN) * B33 - AI(NA,EN) * B33I &  
     &      - AR(NA,NA) * BR(NA,EN) + AI(NA,NA) * BI(NA,EN)
      A34I = AR(NA,EN) * B33I + AI(NA,EN) * B33 &  
     &       - AR(NA,NA) * BI(NA,EN) - AI(NA,NA) * BR(NA,EN)
      A43 = AR(EN,NA) * B44
      A43I = AR(EN,NA) * B44I
      A44 = AR(EN,EN) * B33 - AI(EN,EN) * B33I - AR(EN,NA) * BR(NA,EN)
      A44I = AR(EN,EN) * B33I + AI(EN,EN) * B33 - AR(EN,NA) * BI(NA,EN)
      SH = A44
      SHI = A44I
      XR = A34 * A43 - A34I * A43I
      XI = A34 * A43I + A34I * A43
      IF (XR  ==  ZERO  .AND.  XI  ==  ZERO) GO TO 140
      YR = 0.5D0*(A33 - SH) 
      YI = 0.5D0*(A33I - SHI) 
      Z3 = CDSQRT(DCMPLX(YR**2-YI**2+XR,2.D0*YR*YI+XI))
      U1 = DBLE(Z3)
      U1I = DIMAG(Z3)
      IF (YR * U1 + YI * U1I  >=  ZERO) GO TO 125
      U1 = -U1
      U1I = -U1I
  125 Z3 = (DCMPLX(SH,SHI) - DCMPLX(XR,XI) / DCMPLX(YR+U1,YI+U1I)) &  
     &   / DCMPLX(B3344,B3344I)
      SH = DBLE(Z3)
      SHI = DIMAG(Z3)
      GO TO 140
! C     ********** AD HOC SHIFT **********
  135 SH = AR(EN,NA) + AR(NA,ENM2)
      SHI = ZERO
! C     ********** DETERMINE ZEROTH COLUMN OF A **********
  140 A1 = AR(L,L) / B11 - SH
      A1I = AI(L,L) / B11 - SHI
      A2 = AR(L1,L) / B11
      ITS = ITS + 1
      IF (.NOT. MATZ) LOR1 = L
! C     ********** MAIN LOOP **********
      DO 260 K = L, NA
         K1 = K + 1
         K2 = K + 2
         KM1 = MAX0(K-1,L)
! C     ********** ZERO A(K+1,K-1) **********
         IF (K  ==  L) GO TO 170
         A1 = AR(K,KM1)
         A1I = AI(K,KM1)
         A2 = AR(K1,KM1)
  170    S = DABS(A1) + DABS(A1I) + DABS(A2)
         U1 = A1 / S
         U1I = A1I / S
         U2 = A2 / S
         R = DSQRT(U1*U1+U1I*U1I+U2*U2)
         U1 = U1 / R
         U1I = U1I / R
         U2 = U2 / R
! C
         DO 180 J = KM1, ENORN
            XR = AR(K,J)
            XI = AI(K,J)
            YR = AR(K1,J)
            YI = AI(K1,J)
            AR(K,J) = U1 * XR + U1I * XI + U2 * YR
            AI(K,J) = U1 * XI - U1I * XR + U2 * YI
            AR(K1,J) = U1 * YR - U1I * YI - U2 * XR
            AI(K1,J) = U1 * YI + U1I * YR - U2 * XI
            XR = BR(K,J)
            XI = BI(K,J)
            YR = BR(K1,J)
            YI = BI(K1,J)
            BR(K,J) = U1 * XR + U1I * XI + U2 * YR
            BI(K,J) = U1 * XI - U1I * XR + U2 * YI
            BR(K1,J) = U1 * YR - U1I * YI - U2 * XR
            BI(K1,J) = U1 * YI + U1I * YR - U2 * XI
  180    CONTINUE
! C
         IF (K  ==  L) GO TO 240
         AI(K,KM1) = ZERO
         AR(K1,KM1) = ZERO
         AI(K1,KM1) = ZERO
! C     ********** ZERO B(K+1,K) **********
  240    S = DABS(BR(K1,K1)) + DABS(BI(K1,K1)) + DABS(BR(K1,K))
         U1 = BR(K1,K1) / S
         U1I = BI(K1,K1) / S
         U2 = BR(K1,K) / S
         R = DSQRT(U1*U1+U1I*U1I+U2*U2)
         U1 = U1 / R
         U1I = U1I / R
         U2 = U2 / R
         IF (K  ==  NA) GO TO 245
         XR = AR(K2,K1)
         AR(K2,K1) = U1 * XR
         AI(K2,K1) = -U1I * XR
         AR(K2,K) = -U2 * XR
! C
  245    DO 250 I = LOR1, K1
            XR = AR(I,K1)
            XI = AI(I,K1)
            YR = AR(I,K)
            YI = AI(I,K)
            AR(I,K1) = U1 * XR + U1I * XI + U2 * YR
            AI(I,K1) = U1 * XI - U1I * XR + U2 * YI
            AR(I,K) = U1 * YR - U1I * YI - U2 * XR
            AI(I,K) = U1 * YI + U1I * YR - U2 * XI
            XR = BR(I,K1)
            XI = BI(I,K1)
            YR = BR(I,K)
            YI = BI(I,K)
            BR(I,K1) = U1 * XR + U1I * XI + U2 * YR
            BI(I,K1) = U1 * XI - U1I * XR + U2 * YI
            BR(I,K) = U1 * YR - U1I * YI - U2 * XR
            BI(I,K) = U1 * YI + U1I * YR - U2 * XI
  250    CONTINUE
! C
         BI(K1,K1) = ZERO
         BR(K1,K) = ZERO
         BI(K1,K) = ZERO
         IF (.NOT. MATZ) GO TO 260
! C
         DO 255 I = 1, N
            XR = ZR(I,K1)
            XI = ZI(I,K1)
            YR = ZR(I,K)
            YI = ZI(I,K)
            ZR(I,K1) = U1 * XR + U1I * XI + U2 * YR
            ZI(I,K1) = U1 * XI - U1I * XR + U2 * YI
            ZR(I,K) = U1 * YR - U1I * YI - U2 * XR
            ZI(I,K) = U1 * YI + U1I * YR - U2 * XI
  255    CONTINUE
! C
  260 CONTINUE
! C     ********** SET LAST A SUBDIAGONAL REAL AND END QZ STEP **********
      IF (AI(EN,NA)  ==  ZERO) GO TO 70
      R = CDABS(DCMPLX(AR(EN,NA),AI(EN,NA)))
      U1 = AR(EN,NA) / R
      U1I = AI(EN,NA) / R
      AR(EN,NA) = R
      AI(EN,NA) = ZERO
! C
      DO 270 J = EN, ENORN
         XI = U1 * AI(EN,J) - U1I * AR(EN,J)
         AR(EN,J) = U1 * AR(EN,J) + U1I * AI(EN,J)
         AI(EN,J) = XI
         XI = U1 * BI(EN,J) - U1I * BR(EN,J)
         BR(EN,J) = U1 * BR(EN,J) + U1I * BI(EN,J)
         BI(EN,J) = XI
  270 CONTINUE
! C
      GO TO 70
! C     ********** SET ERROR -- BOTTOM SUBDIAGONAL ELEMENT HAS NOT
! C                BECOME NEGLIGIBLE AFTER 50 ITERATIONS **********
 1000 IERR = EN
! C     ********** SAVE EPSB FOR USE BY CQZVEC **********
 1001 IF (N  >  1) BR(N,1) = EPSB
      RETURN
! C     ********** LAST CARD OF CQZVAL **********
      END
      SUBROUTINE CQZVEC(NM,N,AR,AI,BR,BI,ALFR,ALFI,BETA,ZR,ZI)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)          
! C
      INTEGER EN
      REAL*8 AR(NM,N),AI(NM,N),BR(NM,N),BI(NM,N),ALFR(N),ALFI(N), &  
     &       BETA(N),ZR(NM,N),ZI(NM,N)
      COMPLEX*16 Z3
!
!
!
!
!
!     THIS SUBROUTINE IS A COMPLEX ANALOGUE OF THE FOURTH STEP OF THE
!     QZ ALGORITHM FOR SOLVING GENERALIZED MATRIX EIGENVALUE PROBLEMS,
!     SIAM J. NUMER. ANAL. 10, 241-256(1973) BY MOLER AND STEWART.
!
!     THIS SUBROUTINE ACCEPTS A PAIR OF COMPLEX MATRICES IN UPPER
!     TRIANGULAR FORM, WHERE ONE OF THEM FURTHER MUST HAVE REAL DIAGONAL
!     ELEMENTS.  IT COMPUTES THE EIGENVECTORS OF THE TRIANGULAR PROBLEM
!     AND TRANSFORMS THE RESULTS BACK TO THE ORIGINAL COORDINATE SYSTEM.
!     IT IS USUALLY PRECEDED BY  CQZHES  AND  CQZVAL.
!
!     ON INPUT-
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!          DIMENSION STATEMENT,
!
!        N IS THE ORDER OF THE MATRICES,
!
!        A=(AR,AI) CONTAINS A COMPLEX UPPER TRIANGULAR MATRIX,
!
!        B=(BR,BI) CONTAINS A COMPLEX UPPER TRIANGULAR MATRIX WITH REAL
!          DIAGONAL ELEMENTS.  IN ADDITION, LOCATION BR(N,1) CONTAINS
!          THE TOLERANCE QUANTITY (EPSB) COMPUTED AND SAVED IN  CQZVAL,
!
!        ALFR, ALFI, AND BETA ARE VECTORS WITH COMPONENTS WHOSE
!          RATIOS ((ALFR+I*ALFI)/BETA) ARE THE GENERALIZED
!          EIGENVALUES.  THEY ARE USUALLY OBTAINED FROM  CQZVAL,
!
!        Z=(ZR,ZI) CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
!          REDUCTIONS BY  CQZHES  AND  CQZVAL, IF PERFORMED.
!          IF THE EIGENVECTORS OF THE TRIANGULAR PROBLEM ARE
!          DESIRED, Z MUST CONTAIN THE IDENTITY MATRIX.
!
!     ON OUTPUT-
!
!        A IS UNALTERED,
!
!        B HAS BEEN DESTROYED,
!
!        ALFR, ALFI, AND BETA ARE UNALTERED,
!
!        Z CONTAINS THE EIGENVECTORS.  EACH EIGENVECTOR IS NORMALIZED
!          SO THAT THE MODULUS OF ITS LARGEST COMPONENT IS 1.0 .
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
!     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
!
!     ------------------------------------------------------------------
!
	DATA ZERO/0.D0/, ONE/1.D0/
	
      IF (N <= 1) GO TO 1001
      EPSB = BR(N,1)
!     ********** FOR EN=N STEP -1 UNTIL 2 DO -- **********
      DO 800 NN = 2, N
         EN = N + 2 - NN
         NA = EN - 1
         ALMR = ALFR(EN)
         ALMI = ALFI(EN)
         BETM = BETA(EN)
! C     ********** FOR I=EN-1 STEP -1 UNTIL 1 DO -- **********
         DO 700 II = 1, NA
            I = EN - II
            R = ZERO
            RI = ZERO
            M = I + 1
! C
            DO 610 J = M, EN
               T = BETM * AR(I,J) - ALMR * BR(I,J) + ALMI * BI(I,J)
               TI = BETM * AI(I,J) - ALMR * BI(I,J) - ALMI * BR(I,J)
               IF (J  ==  EN) GO TO 605
               XI = T * BI(J,EN) + TI * BR(J,EN)
               T = T * BR(J,EN) - TI * BI(J,EN)
               TI = XI
  605          R = R + T
               RI = RI + TI
  610       CONTINUE
! C
            T = ALMR * BETA(I) - BETM * ALFR(I)
            TI = ALMI * BETA(I) - BETM * ALFI(I)
            IF (T  ==  ZERO  .AND.  TI  ==  ZERO) T = EPSB
            Z3 = DCMPLX(R,RI) / DCMPLX(T,TI)
            BR(I,EN) = DBLE(Z3)
            BI(I,EN) = DIMAG(Z3)
  700    CONTINUE
! C
  800 CONTINUE
! C     ********** END BACK SUBSTITUTION.
! C                TRANSFORM TO ORIGINAL COORDINATE SYSTEM.
! C                FOR J=N STEP -1 UNTIL 2 DO -- **********
      DO 880 JJ = 2, N
         J = N + 2 - JJ
         M = J - 1
! C
         DO 880 I = 1, N
! C
            DO 860 K = 1, M
               ZR(I,J) = ZR(I,J) + ZR(I,K) * BR(K,J) - ZI(I,K) * BI(K,J)
               ZI(I,J) = ZI(I,J) + ZR(I,K) * BI(K,J) + ZI(I,K) * BR(K,J)
  860       CONTINUE
! C
  880 CONTINUE
! C     ********** NORMALIZE SO THAT MODULUS OF LARGEST
! C                COMPONENT OF EACH VECTOR IS 1 **********
      DO 950 J = 1, N
         T = ZERO
! C
         DO 930 I = 1, N
            R = CDABS(DCMPLX(ZR(I,J),ZI(I,J)))
            IF (R  >  T) T = R
  930    CONTINUE
! C
         DO 940 I = 1, N
            ZR(I,J) = ZR(I,J) / T
            ZI(I,J) = ZI(I,J) / T
  940    CONTINUE
! C
  950 CONTINUE
! C
 1001 RETURN
! C
      END




! this data is not used at all! though some day might be put to good use!!!
        BLOCK DATA
        REAL*8 FRSCAN(6),BETONLY
        COMMON /LADIOT/FRSCAN,BETONLY
        DATA FRSCAN/75.E-6,150.E-6,225.E-6,300.E-6,375.E-6,450.E-6/
        END


	SUBROUTINE CLOSURE(MASFLX,ICLOS,KE,NPTS,CUMAXI,SOL)
        use param_info, only : MP, MODE, KMODE, NPOST
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)

        COMPLEX*16 SOL(KE),CUMAXI,UMAXI
        COMPLEX*16 FLUX(MP)
        COMPLEX*16 EURONRM(KSTR,KCRF,4),EUROMAX(4)
        COMMON /MUSARAT/EURONRM,EUROMAX

	IF(ICLOS == 3 .AND. MASFLX /= 1)THEN
	CALL FLUXCAL(KE,NPTS,IP,SOL,FLUX)
	CALL MAXFLUX(IP,FLUX,UMAXI,YEXACT)
	EUROMAX(ICLOS)=UMAXI
	ELSE IF(ICLOS == 3 .AND. MASFLX == 1)THEN
	EUROMAX(ICLOS)=CUMAXI
	ELSE

        IP=1
	ISKIPPY=2*ICLOS-1
        DO I=1,NPTS-IL,IL
        FLUX(IP)=SOL(ISKIPPY+I)
        IP=IP+1
	end do
        CALL MAXFLUX(IP,FLUX,UMAXI,YEXACT)
	EUROMAX(ICLOS)=UMAXI
        END IF

	RETURN
	END

	SUBROUTINE CNORM(IL,NPTS,A,DCA)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (LK=9,KE=(MP-1)*LK)
	COMPLEX*16 A(*),DCA(*)
	COMPLEX*16 SDIV,SDIV2
        COMMON /TWOMODE/ITMAX,IUNOMAL,IFTX,ISELECT

	ASUM=0.D0
	BSUM=0.D0
	DO 10 I=IFTX,NPTS-IL,IL
	TEMP=CDABS(A(I))
	TEMP2=CDABS(DCA(I))
	IF(TEMP < ASUM)GOTO 66
	SDIV=A(I)
	ASUM=TEMP
66	CONTINUE
	IF(TEMP2 < BSUM)GOTO 10
	SDIV2=DCA(I)
	BSUM=TEMP2
10	CONTINUE
	SDIV=1.d0/SDIV
	SDIV2=1.d0/SDIV2
	DO I=1,NPTS
	A(I)=A(I)*SDIV
	DCA(I)=DCA(I)*SDIV2
	end do

	RETURN
	END



	SUBROUTINE CONSET
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	REAL*8 MEDGE
 	COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TO,T1,STOKES
	COMMON /SUTHER/SO,S1
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	PMEAN=1.D0/(GAMMA*Medge*Medge)
	ZAPGAM=(GAMMA-1.D0)*Medge*Medge*PRANDTL
	SIGMA=PRANDTL

	SIGHAT=SIGMA/GAMMA
	RHAT=(GAMMA-1.D0)*SIGHAT

	PINV=1.D0/PMEAN
	S1=T1/TEDGE
	SO=1.D0+S1
	RETURN
	END
	

		

        SUBROUTINE EMSMINT(XV,C,FOUT,IFAIL)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /ADDRESS/LO,NQ,NTOT
        REAL*8 C(4,NQ)
        FOUT=C(1,LO)+C(2,LO)*XV+C(3,LO)*XV*XV+C(4,LO)*XV*XV*XV
        RETURN
        END



	SUBROUTINE ESPLINT(NEQN,NQ,WRK,XP,F,COEFF,IFAIL)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NDIV=5)
	REAL*8 B(NDIV,NDIV),X(NDIV),D(NDIV)
	REAL*8, DIMENSION(*) :: XP, F, WRK
	REAL*8 COEFF(4,*)
	COMMON /ADDRESS/LO,MPOL,NTOT
	MPOL=NQ
	NTOT=NEQN
	NSTART=4
	NEND=NEQN-2

        DO 10 K=1,3
        IL=0
        DO I=1,5
	KIL=K+IL
        B(I,1)=1.D0
        B(I,2)=XP(KIL)
        B(I,3)=XP(KIL)*XP(KIL)
        B(I,4)=B(I,3)*XP(KIL)
        B(I,5)=B(I,4)*XP(KIL)
        D(I)=F(KIL)
        IL=IL+1
	END DO
        CALL SOLVE(B,D,X,NDIV,5)
        WRK(K)=X(2)+2.D0*X(3)*XP(K)+3.D0*X(4)*XP(K)*XP(K)+ &
     & 4.D0*X(5)*XP(K)**3
10	CONTINUE

	DO 20 K=NSTART,NEND
	IL=-2
	DO I=1,5
	KIL=K+IL
	B(I,1)=1.D0
	B(I,2)=XP(KIL)
	B(I,3)=XP(KIL)*XP(KIL)
        B(I,4)=B(I,3)*XP(KIL)
        B(I,5)=B(I,4)*XP(KIL)
	D(I)=F(KIL)
	IL=IL+1
	END DO
	CALL SOLVE(B,D,X,NDIV,5)
	WRK(K)=X(2)+2.D0*X(3)*XP(K)+3.D0*X(4)*XP(K)*XP(K)+ &
     & 4.D0*X(5)*XP(K)**3
20	CONTINUE
	K=NEND+1
	WRK(K)=X(2)+2.D0*X(3)*XP(K)+3.D0*X(4)*XP(K)*XP(K)+ &
     & 4.D0*X(5)*XP(K)**3
	K=K+1
	WRK(K)=X(2)+2.D0*X(3)*XP(K)+3.D0*X(4)*XP(K)*XP(K)+ &
     & 4.D0*X(5)*XP(K)**3

	DO 120 K=1,NEQN-1
	IL=0
	DO I=1,2
	KIL=K+IL
	B(I,1)=1.D0
	B(I,2)=XP(KIL)
	B(I,3)=XP(KIL)*XP(KIL)
	B(I,4)=B(I,3)*XP(KIL)
	D(I)=F(KIL)
	
	I3=I+2
	B(I3,1)=0.D0
	B(I3,2)=1.D0
	B(I3,3)=2.D0*XP(KIL)
	B(I3,4)=3.D0*XP(KIL)*XP(KIL)
	D(I3)=WRK(KIL)
	
	IL=IL+1
	END DO
	
!	IL=0
!	DO I=3,4
!	KIL=K+IL
!	B(I,1)=0.D0
!	B(I,2)=1.D0
!	B(I,3)=2.D0*XP(KIL)
!	B(I,4)=3.D0*XP(KIL)*XP(KIL)
!	D(I)=WRK(KIL)
!	IL=IL+1
!	END DO
	CALL SOLVE(B,D,X,NDIV,4)
	DO KOL=1,4
	COEFF(KOL,K)=X(KOL)
	END DO
120	CONTINUE
	IFAIL=0
	RETURN
	END

	SUBROUTINE ETASET(ND)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMMON /TEMESH/DY,YEND,YEDGE
	COMMON /TESTEP/HAFH,H2,HSQ12,H8
	COMMON /CONSTS/PP5,P12,P43,P23,P1IN,P2SQ,P3CU,P1INPP5
        COMMON /GROPT/KGRID
	KGRID=0
 	YEND=1.D0
	PND=ND

	H=1.D0/dfloat(ND-1)
	DY=H
	HAFH=0.5D0*DY
	H2=DY*DY
	HSQ12=12.D0*DY*DY
	H8=DY*8.D0

	PP5=0.5D0
	P12=1.D0/12.D0
	P43=4.D0/3.D0
	P23=2.D0/3.D0
	P1IN=1.D0/DY
	P2SQ=1.D0/(DY*DY)
	P3CU=1.D0/(DY*DY*DY)
	P1INPP5=P1IN*PP5

	RETURN
	END

        SUBROUTINE EXCNTL(IFILE,IFN)
        CHARACTER*8 QUERY
9999    WRITE(*,*)'Available commands in PSECNTL :'
        WRITE(*,*)' Yglobal, Ylocal, Nglobal, Nlocal, Itolray, Grid'
        WRITE(*,*)' Nitiray, Itolspc, Itolnon, Iraycnt, Irobust '
	WRITE(*,*) 'Nitmode, Nglbmd'
	WRITE(*,*)' Differ, Iasymp, Xstrech, Ythick, Inag, Iqzhi8'
        IACTUAL=19
        IMAX=19

        IF(IFILE == 0)RETURN
        CALL INFO(5,IFN,IACTUAL,IMAX,QUERY,IPARM)
        IF(IPARM == 1 .AND. QUERY == 'q')GOTO 9999
        RETURN
        END
        SUBROUTINE EXGAS(IFILE,IFN)
        CHARACTER*8 QUERY
9999    WRITE(*,*)'Available commands in PSEGAS :'
        WRITE(*,*)' Rgas, Gamma, Prandtl, Stokes, Isuther, Ichapmn'

        IACTUAL=4
        IMAX=6
        IF(IFILE == 0)RETURN
        CALL INFO(2,IFN,IACTUAL,IMAX,QUERY,IPARM)
        IF(IPARM == 1 .AND. QUERY == 'q')GOTO 9999
        RETURN
        END
        SUBROUTINE EXNONL(IFILE,IFN)
        CHARACTER*8 QUERY
9999    WRITE(*,*)'Available commands in PSENONL :'
        WRITE(*,*)' Ialfa, Ibeta, Amp, Iplock, Freq, Beta, Ifloq'
        WRITE(*,*)' Bc, Iadiab, Ithresh, Isymm, Iqe, Iprcent'
	WRITE(*,*)' Iob, Magnify, Nstat, ILoF'
	WRITE(*,*)'** Riblet model options : '
	WRITE(*,*)' Ieadsiw, Xformin, Xformax, Riblet_urms, Irib_urms'
     
        IACTUAL=22
        IMAX=22

        IF(IFILE == 0)RETURN
        CALL INFO(5,IFN,IACTUAL,IMAX,QUERY,IPARM)
        IF(IPARM == 1 .AND. QUERY == 'q')GOTO 9999
        RETURN
        END
        SUBROUTINE EXOUT(IFILE,IFN)
        CHARACTER*8 QUERY
9999    WRITE(*,*)'Available commands in PSEOUT :'
        WRITE(*,*)' Ieigl, Ieigs, Intval, Rcptur, Itspres, Iwvlnth '
        WRITE(*,*)' Nrmlocl, Nrmstrt, Nrmdim, Label, Iscreen, Iwvdump'
        WRITE(*,*)' Iqzfull, Ioutxc, Ioutsc, Rescale, Nrmcord, Natural'
	WRITE(*,*)' Ieuro, Ibae, ICoDS, I2ndary, Userenv'
        IACTUAL=22
        IMAX=23

        IF(IFILE == 0)RETURN
        CALL INFO(5,IFN,IACTUAL,IMAX,QUERY,IPARM)
        IF(IPARM == 1 .AND. QUERY == 'q')GOTO 9999
        RETURN
        END
	
	SUBROUTINE EXTRAP(IALC,IFREQ,ICR,ALPREV)
        use param_info, only : MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	LOGICAL YUP
	REAL*8 LNORM
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),F0X,S0X,ALPREV
	REAL*8 XCVAL(NPOST),RDVAL(NPOST),SDVAL(NPOST)
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IN
	COMMON /EISPC/ALFAZO
        COMMON /AWESOME/IOSE
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /SOMMER/LNORM
	COMMON /HOHOHO/IMULTF2, IROBUST
        IF(IMULTF2 == 0)GOTO 1111
        IF(IOSE == 0 .AND. IALC > 4)GOTO 1111
!	PRINT*,IOSE, IALC, IROBUST
        IF(IOSE == 1 .AND. IALC > IROBUST+1)GOTO 1111

        INQUIRE(FILE='PSEMULTF2.DAT',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 1111

        OPEN(3,FILE='PSEMULTF2.DAT',STATUS='OLD')
!        READ(3,*,END=1111,ERR=1111)
        DO I=1,IALC-2
        READ(3,*,END=1111,ERR=1111)
	END DO

        IF(ICR > 1 .AND. IFREQ == 1)THEN
        ALPREV=ALFAZO(IFREQ,ICR-1,IALC)
        ELSE IF(IFREQ > 1 .AND. ICR == 1)THEN
        ALPREV=ALFAZO(IFREQ-1,ICR,IALC)
        ELSE IF(IFREQ > 1 .AND. ICR > 1)THEN
        ALPREV=ALFAZO(IFREQ-1,ICR-1,IALC)
        ELSE
        READ(3,*,END=1111,ERR=1111)WN,WI
        ALPREV=DCMPLX(WI,WN)*LNORM
        END IF
        REWIND(3)
        CLOSE(3)
        RETURN
1111    CONTINUE
        IF(IALC == 2)THEN
        ALPREV=ALFAZO(IFREQ,ICR,IALC-1)
        ELSE IF(IALC == 3)THEN
        d1=SDVAL(IALC-1)-SDVAL(IALC-2)
        d0=SPOS-SDVAL(IALC-1)
        F0X=(ALFAZO(IFREQ,ICR,IALC-1)-ALFAZO(IFREQ,ICR,IALC-2))/d1
        ALPREV=ALFAZO(IFREQ,ICR,IALC-1)+d0*F0X

        ELSE IF(IALC == 4)THEN
!        ELSE IF(IALC > 3  .AND. IALC < 99999)THEN

        d0=SDVAL(IALC-1)-SDVAL(IALC-2)
        d1=SDVAL(IALC-2)-SDVAL(IALC-3)
        d2=SDVAL(IALC-1)-SDVAL(IALC-3)
        dd=SPOS-SDVAL(IALC-1)
        fxx=1.D0/(d0*d2)
        f0=(d0+d2)*fxx
        f1=-d2/(d0*d1)
        f2=d0/(d1*d2)
        s0=2.D0*fxx
        sx=s0/d1
        s1=-d2*sx
        s2=d0*sx

        F0X=f0*ALFAZO(IFREQ,ICR,IALC-1)+f1*ALFAZO(IFREQ,ICR,IALC-2)+ &
     & f2*ALFAZO(IFREQ,ICR,IALC-3)
        S0X=s0*ALFAZO(IFREQ,ICR,IALC-1)+s1*ALFAZO(IFREQ,ICR,IALC-2)+ &
     & s2*ALFAZO(IFREQ,ICR,IALC-3)
        ALPREV=ALFAZO(IFREQ,ICR,IALC-1)+dd*F0X+0.5D0*dd*dd*S0X
	
	ELSE
! this clearly not advantageous in real practical cases! TELFONA case!recollect
        d0=SDVAL(IALC-1)-SDVAL(IALC-2)
        d1=SDVAL(IALC-2)-SDVAL(IALC-3)
        d2=SDVAL(IALC-1)-SDVAL(IALC-3)
        d3=SDVAL(IALC-1)-SDVAL(IALC-4)
        d4=SDVAL(IALC-3)-SDVAL(IALC-4)
        dd=SPOS-SDVAL(IALC-1)
	
	ddd2=dd+d2
	ddd0=dd+d0
	ddd3=dd+d3
	d1d4=d1+d4
        sx=dd/(d1*d4*d1d4)
	
        f0=-ddd0*ddd2*ddd3*(d0-d3)/(d1d4*d0*d2*d3)
        f1=-sx*ddd2*ddd3*d4/d0
        f2=-sx*ddd0*ddd3*(d0-d3)/d2
        f3=-sx*ddd0*ddd2*d1/d3

       ALPREV=f0*ALFAZO(IFREQ,ICR,IALC-1)+f1*ALFAZO(IFREQ,ICR,IALC-2)+ &
     & f2*ALFAZO(IFREQ,ICR,IALC-3)+ f3*ALFAZO(IFREQ,ICR,IALC-4)
        END IF

	RETURN
	END SUBROUTINE EXTRAP	
	
	
	SUBROUTINE EXTRPRV(IALC,IFREQ,ICR,ALPREV)
        use param_info, only : MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	LOGICAL YUP
	REAL*8 LNORM
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),F0X,S0X,ALPREV
	REAL*8 XCVAL(NPOST),RDVAL(NPOST),SDVAL(NPOST)
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IN
	COMMON /EISPC/ALFAZO
        COMMON /AWESOME/IOSE
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /SOMMER/LNORM
	COMMON /HOHOHO/IMULTF2, IROBUST
        ALPREV=ALFAZO(IFREQ,ICR-1,IALC)
	RETURN
	END
	SUBROUTINE EXUSER(IFILE,IFN)
	CHARACTER*8 QUERY
9999	WRITE(*,*)'Available commands in PSEUSER :'
	WRITE(*,*)' Nstart, Nfinal, Nintr, Iumax, Icrsflo, Keu'
	WRITE(*,*)' Keuv, Keuvw, Iuherb, Masflx, Kemax, Iglb'
	WRITE(*,*)' Inonlin, Ylambda, Womg, Freqlin, Betalin'
	WRITE(*,*)' Nfswich, Iattach, Ivarip, Itse, Wavelin'
	WRITE(*,*)' Waveim, Iglbx, Nfactor, Itmax, Icurvi'
	WRITE(*,*)' Iwmax, Imultf2, Lookup, Iose, Ipse'
	WRITE(*,*)' Isim, Ibuoy, Ineutrl, Irecept, Irevflow'
	WRITE(*,*) 'Ipostenv, Xtransition, Xdlimit, Ioveride'
	WRITE(*,*)'** PPm-PSE3D commands : '
        WRITE(*,*)' Ido3d, Nzdir, Nzstart, Izero, Istrip, Fdxdz, Ipsez' 
        WRITE(*,*)' Iellipt, Iupwind, Icharbx, Iupsmrt, Iupimp, Iupedge'
	WRITE(*,*)' Isetup3d, Itransonic, IGLB3D, IEXTRAP3D'
	IACTUAL=58
	IMAX=58
 

	IF(IFILE == 0)RETURN
	CALL INFO(5,IFN,IACTUAL,IMAX,QUERY,IPARM)
	IF(IPARM == 1 .AND. QUERY == 'q')GOTO 9999
	RETURN
	END
 
        SUBROUTINE FILHAND(INDEX1,LABFRNT,STRTLAB,ENDLAB,PSELABL)         
        PARAMETER (ITRF=9,IMORE=ITRF+3,itrf2=39)
        CHARACTER*1 IAUTO(0:ITRF),N1                              
        CHARACTER*2 MAUTO(0:ITRF),M1, IAUTO2(0:ITRF2),n2                
        CHARACTER ENDLAB*(*),STRTLAB*(*)
        CHARACTER PSELABL*(*),LABFRNT*(*)
        DATA IAUTO/'0','1','2','3','4','5','6','7','8','9'/
        DATA IAUTO2/'10','11','12','13','14','15','16','17', &
     & '18','19', '20','21','22','23','24','25','26','27','28', &
     & '29', '30','31','32','33','34','35','36','37','38','39', &
     & '40','41','42','43','44','45','46','47','48','49'/
        DATA MAUTO/'m0','m1','m2','m3','m4','m5','m6','m7','m8','m9'/

        IF(INDEX1 <= 49)GOTO 9999                               
        IF(INDEX1 == 99)THEN                        
        PSELABL=LABFRNT//STRTLAB//ENDLAB
        ELSE                                               
        PSELABL=LABFRNT//ENDLAB                  
        END IF           
        RETURN                                      
        
9999    CONTINUE                                    
        IF(INDEX1 >= 0   .and. INDEX1  <= 9)THEN                                    
        N1=IAUTO(INDEX1)                                
        PSELABL=LABFRNT//STRTLAB//n1//ENDLAB
	else if(INDEX1 > 9   .and. INDEX1  <= 39) then                                   
        N2=IAUTO2(INDEX1-10)                                
        PSELABL=LABFRNT//STRTLAB//n2//ENDLAB
        ELSE
        M1=MAUTO(-INDEX1)
        PSELABL=LABFRNT//STRTLAB//m1//ENDLAB
        END IF                                         

        RETURN
        END
	SUBROUTINE FILOPEN(HLPNAM,IFILE,IFN)
        CHARACTER*80 DESTIN,SUBDIR,FILHLP
	CHARACTER HLPNAM*(*)
	LOGICAL YUP
	COMMON /HLPLOC/DESTIN,SUBDIR
        COMMON /HLPINT/IDEST,IDEST2
	IFN=81
        FILHLP=DESTIN(1:IDEST)//HLPNAM
        INQUIRE(FILE=FILHLP,EXIST=YUP)
        IF(YUP.EQV..FALSE.)THEN
        IFILE=0
        ELSE
        IFILE=1
        OPEN(IFN,FILE=FILHLP,STATUS='OLD')
        END IF
	RETURN
	END

	SUBROUTINE FLUXCAL(KE,NPTS,IP,SOL,FLUX)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        COMPLEX*16 SOL(*),RDENS
        COMPLEX*16 FLUX(*)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        REAL*8 US(MP),DU(MP),D2U(MP),UX(MP),DUX(MP),D2UX(MP)
        REAL*8 VS(MP)
        DOUBLE PRECISION TS(MP),D2T(MP),DT(MP),TX(MP),DTX(MP)
        DOUBLE PRECISION D2TX(MP)
        COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,VS
	COMMON /MALCROS/KEMAX,IQE,I100
        COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT
        COMMON /SURFACE/PWALL,PEDGE,PGRAD
        COMMON /IPRIMIT/IVARIP

        IF(KEMAX == 1)THEN
	IP=1
        DO I=1,NPTS-IL,IL
        FLUX(IP)=DSQRT(CDABS(SOL(I+1))**2.D0+ & 
     & CDABS(SOL(I+3))**2.D0)
	IP=IP+1
	end do
        ELSE

	IF(IVARIP == 0)THEN
        IP=1
        DO  I=1,NPTS-IL,IL
        TIN=1.D0/TS(IP)
	RODENT=PINV*PEDGE*TIN
        RDENS=(PINV*SOL(I)-RODENT*SOL(I+4))*TIN
        FLUX(IP)=ROV(IP)*SOL(I+1)+US(IP)*RDENS
        IP=IP+1
	end do
	ELSE
        IP=1
        DO  I=1,NPTS-IL,IL
        FLUX(IP)=ROV(IP)*SOL(I+1)+US(IP)*SOL(I)
        IP=IP+1
	end do
	END IF
	END IF
	RETURN
	END
        SUBROUTINE FRONT(IERR)
	use header_info
	use GPL_STUFF
        character*1 :: REPLY, YES, NO
        LOGICAL YUP
	COMMON /GUI/IGUI
	COMMON /ITOMMY/ITOM
	OPEN(2,FILE='pse.lp0',STATUS='UNKNOWN')
	IGUI=1
	WRITE(2,*)
	WRITE(2,*)
	CALL SET_SEED(ISEED)
	WRITE(*,*)
	WRITE(*,*)
	WRITE(*,*)' **************************************************'     
	WRITE(*,*)' ***                                            ***'
        WRITE(*,*)' ***     Compressible Non-Linear PSE Solver     ***'
	WRITE(*,'(1X,A4,6x,A,1x,A8,7X,A4)')' ***','CoPSE3d-LST version: ',&
     & version,'***'
	WRITE(*,*)' **************************************************'     
	CALL FLUSH(6)
	
!	PRINT*,' input ICR extration point'
!	READ*,ITOM
!	IF(ILCxx == 1)THEN
!	WRITE(*,'(2x,a)') &
!     & ' ** ERROR :: CoPSE exited due to incomplete data !!!'
!	WRITE(*,'(5x,a)') ' LC-error'
!     	IERR=ILCxx
!	end if
	IF(IERR == 1)RETURN
	
        INQUIRE(FILE='PSEREAD.NAM',EXIST=YUP)
        IF(YUP.EQV..FALSE.)GOTO 1111
        OPEN(UNIT=7,FILE='PSEREAD.NAM',STATUS='OLD')
        IERR=0

        RETURN
1111    WRITE(*,*)'      '                            
        yes='y'
        no='n'
	
        WRITE(*,*)' *** File PSEREAD.NAM Does Not Exist !!! ***'
        WRITE(*,*)' Create this file... This file contains the'
        WRITE(*,*)' basic commands for a successful CoPSE Run - '
        WRITE(*,*)' Refer to guidance notes : '
        WRITE(*,*)'    '
	PRINT*,' Basic File Format - '
 	PRINT*,' Line 1 : $PSEUSER ( main driver )' 
	PRINT*,'          namelist commands '
	PRINT*,'          $END'
	PRINT*,'          $PSENONL ( nonlinear scenario only )'
	PRINT*,'          namelist commands '
	PRINT*,'          $END'
	PRINT*,'          $PSECNTL ( internal tolerances )'
	PRINT*,'          namelist commands '
	PRINT*,'          $END'
	PRINT*,'          $PSEOUT  ( output of results )'
	PRINT*,'          namelist commands '
	PRINT*,'          $END'
	PRINT*,'          $PSEGAS ( gas properties )'
	PRINT*,'          namelist commands '
	PRINT*,'          $END'
	WRITE(*,*) CHAR(7)

	IF(IGUI == 0)THEN
        PRINT*,'          '
	write(*,'(a)',advance='no')  &
     & ' To create this type ? yes (y) or no (n) : '
        READ(*,'(A1)')REPLY
        IF(reply == yes)THEN
        OPEN(UNIT=7,FILE='PSEREAD.NAM',STATUS='new')
 	WRITE(7,'(a)')' $PSEUSER'
	WRITE(7,'(a)')' NSTART= ?' 
	WRITE(7,'(a)')' FREQLIN= ?'
	WRITE(7,'(a)')' $END'
	WRITE(7,'(a)')' $PSENONL '
	WRITE(7,'(a)')' $END'
	WRITE(7,'(a)')' $PSECNTL '
	WRITE(7,'(a)')' $END'
	WRITE(7,'(a)')' $PSEOUT  '
	WRITE(7,'(a)')' $END'
	WRITE(7,'(a)')' $PSEGAS '
	WRITE(7,'(a)')' $END'
	close(7)
        PRINT*,'          '
	PRINT*,' PSEREAD.NAM commands file created :'
	PRINT*,' ...now edit this file and set'
	PRINT*,'    parameters NSTART and FREQLIN'
        END IF
	CALL HELP(0)	
        END IF	
	
9999    IERR=1
        RETURN
	END


	SUBROUTINE GRIDIT(ND,DYP,WALSLOP)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        REAL*8, DIMENSION(MP) :: A1, A2, B1, SLOPE
        COMMON /STRECH/A1,A2,B1
        COMMON /SCALI/G,ULENTH
        COMMON /DERIV/SLOPE
        YO=0.D0
	ULGG=1.d0/(ULENTH*G)
        DO  I=1,ND
        SIGMA=(G-YO)*ULGG
        A1(I)=SIGMA*(G-YO)
        SLOPE(I)=A1(I)
        A2(I)=-2.D0*A1(I)*SIGMA
        B1(I)=A1(I)*A1(I)
        YO=YO+DYP
	end do
        WALSLOP=A1(1)
	RETURN
	END
        SUBROUTINE HELP(IFROM)
        PARAMETER (ITRF=9,IMORE=ITRF+3)
        character REPLY*1,YES*1,NO*1
        CHARACTER*80 DESTIN,SUBDIR
        CHARACTER*16 HLPNAM
        CHARACTER*78 POTEMTY
        COMMON /HLPLOC/DESTIN,SUBDIR
        COMMON /HLPINT/IDEST,IDEST2
        COMMON /CHAROMP/POTEMTY
        POTEMTY='                              '
        yes='y'
        no='n'

        IF(IFROM == 0)THEN
        DESTIN='./'
        SUBDIR='hlp/'
        IDEST=INDEX(DESTIN,' ')-1
        IDEST2=INDEX(SUBDIR,' ')-1

        PRINT*,'          '
	write(*,'(a)',advance='no')  &
     & ' Do you require further info ? yes (y) or no (n) : '
        READ(*,'(A1)')REPLY
        ELSE
        REPLY=yes
        END IF
        IF(reply == no)RETURN

        DESTIN=DESTIN(1:IDEST)//SUBDIR(1:IDEST2)
        IDEST=INDEX(DESTIN,' ')-1

8888    PRINT*,'          '
        WRITE(*,*)'       Namelist command descriptions '
        WRITE(*,*)' Choose by selecting the appropriate category'
        WRITE(*,*)'Type   '
        WRITE(*,*)'     (u) for PSEUSER namelist commands available or'
        WRITE(*,*)'     (n) for PSENONL '
        WRITE(*,*)'     (c) for PSECNTL '
        WRITE(*,*)'     (o) for PSEOUT '
        WRITE(*,*)'     (g) for PSEGAS '
        WRITE(*,*)'  ** (q) to Quit ** '
        PRINT*,'          '
        READ(*,'(A1)')REPLY
        IF(REPLY == 'u')THEN
        HLPNAM='pseuser.hlp'
        CALL FILOPEN(HLPNAM,IFILE,IFN)
        CALL EXUSER(IFILE,IFN)

        ELSE IF(REPLY == 'n')THEN
        HLPNAM='psenonl.hlp'
        CALL FILOPEN(HLPNAM,IFILE,IFN)
        CALL EXNONL(IFILE,IFN)

        ELSE IF(REPLY == 'c')THEN
        HLPNAM='psecntl.hlp'
        CALL FILOPEN(HLPNAM,IFILE,IFN)
        CALL EXCNTL(IFILE,IFN)

        ELSE IF(REPLY == 'o')THEN
        HLPNAM='pseout.hlp'
        CALL FILOPEN(HLPNAM,IFILE,IFN)
        CALL EXOUT(IFILE,IFN)

        ELSE IF(REPLY == 'g')THEN
        HLPNAM='psegas.hlp'
        CALL FILOPEN(HLPNAM,IFILE,IFN)
        CALL EXGAS(IFILE,IFN)

        ELSE
        RETURN
        END IF

        REWIND(IFN)
        CLOSE(IFN)

        PRINT*,'          '
	write(*,'(a)',advance='no')   &
     & ' Do you require further queries ? yes (y) or no (n) :'
        READ(*,'(A1)')REPLY
        IF(REPLY == yes)GOTO 8888

        RETURN
        END

	

	
        SUBROUTINE INFO(IJUMP,IFN,IACTUAL,IMAX,QUERY,IPARM)
        CHARACTER*8 QUERY,CF
	CHARACTER*78 clarify(6),POTEMTY
	CHARACTER*30 CFQ
	COMMON /CHAROMP/POTEMTY
	
	IPARM=0

9989	WRITE(*,*)
	WRITE(*,*)'Input first few letters for explanations... or'
	WRITE(*,*)'(q) returns to main menu'
	READ(*,'(A8)')QUERY
	IF(QUERY == 'q')RETURN

	INUMB=INDEX(QUERY,' ')-1

	REWIND(IFN)
	DO 20 I=1,IACTUAL
	DO 30 K=1,IJUMP
	READ(IFN,'(A78)')clarify(K)
30	CONTINUE
	CF=clarify(1)(1:INUMB)
	IF(CF /= QUERY(1:INUMB))GOTO 20
	GOTO 40
20	CONTINUE
        WRITE(*,*)'      No matches found in help data file!'
	WRITE(*,*)
        QUERY='q'
        IPARM=1
        RETURN
	
40	DO 70 I=1,IJUMP
	CFQ=clarify(I)(1:30)
	IF(CFQ == POTEMTY(1:30))GOTO 70
	WRITE(*,'(A78)')clarify(I)
70	CONTINUE
	GOTO 9989

	END

		
	
        SUBROUTINE INITLIZ_OPTIM
        use param_blprof
        use param_info, only : NPOST
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	REAL*8, DIMENSION(NPOST) :: XLOCAT, SLOCAT
	REAL*8, DIMENSION(33) :: BL
	REAL*8 :: Mo, Medge 
	INTEGER, DIMENSION(NPOST) :: IOUTSTUFF
	LOGICAL YUP
        CHARACTER*80 TITLE
	COMMON /IFLARE/IREVFLOW,IBACOMPAT

        INQUIRE(FILE='PSEMEAN.PRO',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 6776
        OPEN(8,FILE='PSEMEAN.PRO',STATUS='OLD')

	IERFIL=0
        KFL=0
        READ(8,'(A80)',ERR=30)TITLE
        READ(8,*,END=30,ERR=30)Uo
        READ(8,*,END=30,ERR=30)Nprof
	
	PRINT*,' IN PSEMEAN.PRO diagnostics...'
	
	DO KK=1,Nprof
        READ(8,*,END=80,ERR=70)IPROF,NPOINT,XLOCAT(KK),SLOCAT(KK)
        READ(8,*,END=80,ERR=70)xtop
	WRITE(101,'(I7,2E24.14)')KK,XLOCAT(KK),SLOCAT(KK)
        DO I=1,NPOINT
        READ(8,*,END=70,ERR=70)
     	END DO
	END DO
	
	DSMAX=1.E99
	DSMIN=-1.E10
	SUM=0.d0
	DO KK=2,Nprof
	DS=SLOCAT(KK)-SLOCAT(KK-1)
	SUM=SUM+DS
	
	IF(DS < DSMAX)THEN
	DSMAX=DS
	END IF
	
	IF(DS > DSMIN)THEN
	DSMIN=DS
	END IF
	
	WRITE(102,'(I7,2E24.14)')KK,XLOCAT(KK),DS
     	END DO
	PRINT*,DSMIN, DSMAX
	PRINT*,SLOCAT(Nprof)-SLOCAT(1),' << Srange'
	PRINT*,(SLOCAT(Nprof)-SLOCAT(1))/DSMIN, ' << Srange/dxmax'
	SUMAV=SUM/float(NPROF)
	PRINT*,SUMav,' averaged'
	PRINT*,2*(SLOCAT(Nprof)-SLOCAT(1))/(DSMAX+DSMIN),' << Srange/av'
	PRINT*,(SLOCAT(Nprof)-SLOCAT(1))/SUMav,' << av'
	PRINT*,NPROF
	
	PRINT*,' INPUT NPOINTS'
	READ*, NPTS
	DSL=(SLOCAT(Nprof)-SLOCAT(1))/dfloat(NPTS)
	
	SADD=0.d0
	XTOl=0.0d0
	IK=1
	DO KK=2,Nprof
	SADD=SADD+(SLOCAT(KK)-SLOCAT(KK-1))
	
	IF(SADD < DSL-XTOl)THEN
	ELSE
	IOUTSTUFF(IK)=KK
	SADD=0.d0
	IK=IK+1
	END IF
	END DO
	
	DO KK=1,IK-1
	WRITE(103,'(2I7,2E24.14)')KK,IOUTSTUFF(KK), &
     & XLOCAT(IOUTSTUFF(KK)),SLOCAT(IOUTSTUFF(KK))
	END DO
	
	REWIND(8)
	OPEN(105,FILE='PSEMEAN.PRO_DX',status='unknown')
        READ(8,'(A80)',ERR=30)TITLE
	WRITE(105,'(a)')TITLE
        READ(8,*,END=30,ERR=30)Uo,Mo,RGlobl,To,Chord,SWEEP
	WRITE(105,'(2x,6E24.14)')Uo,Mo,RGlobl,To,Chord,SWEEP
        READ(8,*,END=30,ERR=30)Nprof
	WRITE(105,'(2X,2I11)')IK-1, Nprof
	
	
	IK=1
	ISWITCHON=0
	Do KK=1,Nprof
	
	IF(IBACOMPAT == 1)THEN
        READ(8,*,END=80,ERR=70)IPROF,NPOINT,XPOS,SPOS,CK,CDKDS
     	tdisp=0.d0 ; tdispY=0.d0 ; tdispYY=0.d0 
	ELSE
        READ(8,*,END=80,ERR=70)IPROF,NPOINT,XPOS,SPOS,CK,CDKDS,tdisp, &
     & tdispY,tdispYY
	END IF
	
        READ(8,*,END=70,ERR=70)Uedge,Medge,Rlocal,Tedge,DELTA, &
     & PINFTY,PXGRAD
	IF(IOUTSTUFF(IK) == KK)THEN
	ISWITCHON=1
	WRITE(105,'(2I7,7E24.14)')IK,NPOINT,XPOS,SPOS,CK,CDKDS, &
     & tdisp, tdispY,tdispYY
	WRITE(105,'(2x,7E24.14)')Uedge,Medge,Rlocal,Tedge,DELTA, &
     & PINFTY,PXGRAD
     	END IF
	
	IF(ISWITCHON==1)THEN
	PRINT*,IOUTSTUFF(IK), KK
        DO  I=1,NPOINT
        READ(8,*,END=70,ERR=70)BL(1:23)
	WRITE(105,'(23E24.14)')BL(1:23)
	END DO
	ELSE
        DO  I=1,NPOINT
        READ(8,*,END=70,ERR=70)
	END DO
	END IF
	
	
	IF(ISWITCHON==1)THEN
	IK=IK+1
	ISWITCHON=0
	END IF
	
	
	END DO
     
	
	RETURN
30	PRINT*,' Error'
	RETURN
70	PRINT*,' Error-End'
	RETURN
80	PRINT*,' Error'
	RETURN
6776	PRINT*,' Error'
	RETURN
	END 
	
	

	SUBROUTINE INITL
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /K3D/IDO3D, IGLB3D, IEXTRAP3D, IGLB3D_EXIST
        IF(ICoDS >= 1)THEN
        CALL INTCoDS
        ELSE
        CALL INITLIZ
        END IF
        RETURN
        END
	
        SUBROUTINE INITLIZ
        use param_blprof
        use param_info, only : NPOST
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION Mo
	LOGICAL YUP
        CHARACTER*80 TITLE
	COMMON /PICTR/TITLE
        COMMON /SCLGLB/Uo,Mo,RGlobl,To,Chord
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /REGRID/KFL
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /FILINF/IERFIL
	COMMON /cylind/VISCOS,RINFTY,SWEPTY
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /MFDADIB/IADIAB
	COMMON /PISAVE/PI
	COMMON /PICOSV/SWPSCL,CLINE
	COMMON /TRANSLATE/SWPSIN, SWPCOS

        INQUIRE(FILE='PSEMEAN.PRO',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 6776
        OPEN(8,FILE='PSEMEAN.PRO',STATUS='OLD')

	IERFIL=0
        KFL=0
        READ(8,'(A80)',ERR=30)TITLE
        READ(8,*,END=30,ERR=30)Uo,Mo,RGlobl,To,Chord,SWEEP
        READ(8,*,END=30,ERR=30)Nprof

        SIGMA=SWEEP*PI/180.D0
        SWPSCL=DCOS(SIGMA)
	SWPCOS=SWPSCL
	SWPSIN=DSIN(SIGMA)
	CLINE=Chord
!	rescale everything normal to wing leading edge	

!       check whether in range to handle storage!
        IBUBBLE=NPOST-(NFINAL-NSTART+1)/(NINTR+1)

        IF(IBUBBLE >= 0)GOTO 1972
        PRINT*,' Error in mean boundary layer data input :  '
        PRINT*,' Too many profiles to handle in PSEMEAN.PRO '
        WRITE(*,2791)NPOST
        WRITE(*,2792)NPOST
2791    FORMAT(1X,' Maximum no. allowable : ',I5)
2792    FORMAT(1X,' OR ensure that (NFINAL-NSTART) < ',  &
     & I5,' in PSEREAD.NAM')
        IERFIL=1
        RETURN

1972    IKHAT=NSTART-1
        CALL MISJUMP(1,IKHAT)
        RETURN
30      PRINT*,' Error in mean boundary layer data input :  '
        PRINT*,' Check format of global properties in PSEMEAN.PRO '
        IERFIL=1
	RETURN
6776	PRINT*,'     '
	PRINT*,' File PSEMEAN.PRO ( mean boundary-layer properties ) '
	PRINT*,'               does not exist!!'
        IERFIL=1
	RETURN
        END



!      the _CoDS routines simply take data in Eurotrans format and
!	convert into form required for PSE wing-normal coordinates.
        SUBROUTINE INTCoDS
        use param_info, only : NPOST
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION Mo
	LOGICAL YUP
        CHARACTER*80 TITLE,TITLE2
	COMMON /PICTR/TITLE
        COMMON /SCLGLB/Uo,Mo,RGlobl,To,Chord
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /REGRID/KFL
        COMMON /INFORM/NSTART,NFINAL,NINTR
	COMMON /FILINF/IERFIL
	COMMON /PISAVE/PI
	COMMON /PICOSV/SWPSCL,CLINE
	COMMON /TRANSLATE/SWPSIN, SWPCOS
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TOO,T1,STOKES
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
        INQUIRE(FILE='PSECoDS.PRO',EXIST=YUP)

        IF(YUP.EQV..FALSE.)GOTO 6776
        OPEN(8,FILE='PSECoDS.PRO',STATUS='OLD')

	IERFIL=0
        KFL=0
        READ(8,'(A80)',ERR=30)TITLE
        READ(8,'(A80)',ERR=30)TITLE2
	READ(8,*,END=30,ERR=30)
        READ(8,*,END=30,ERR=30)ChordLf,SWPEFF,SWEEP,SWPTE
	IF(ICoDS == 1)THEN
	READ(8,*,END=30,ERR=30)
        READ(8,*,END=30,ERR=30)PRANDTL,GAMMA,RGAS
	end if

        SIGMA=SWEEP*PI/180.D0
        SWPSCL=DCOS(SIGMA)
	SWPCOS=SWPSCL
        SWPSIN=DSIN(SIGMA)
	Chord=ChordLf
	CLINE=Chord
!	rescale everything normal to wing leading edge	
!	though i dont think it matters! proofs in the pudding!

!       check whether in range to handle storage!
        IBUBBLE=NPOST-(NFINAL-NSTART+1)/(NINTR+1)

        IF(IBUBBLE >= 0)GOTO 1972
        PRINT*,'Error in mean boundary layer data input :  '
        PRINT*,'Too many profiles to handle in PSECoDS.PRO '
        WRITE(*,2791)NPOST
        WRITE(*,2792)NPOST
2791    FORMAT(1X,' Maximum no. allowable : ',I5)
2792    FORMAT(1X,' OR ensure that (NFINAL-NSTART) < ',  &
     & I5,' in PSEREAD.NAM')
        IERFIL=1
        RETURN

1972    IKHAT=NSTART-1
        CALL MISCoDS(1,IKHAT)
        RETURN
30      PRINT*,'Error in mean boundary layer data input :  '
        PRINT*,'Check format of global properties in PSECoDS.PRO '
        IERFIL=1
	RETURN
6776	PRINT*,'     '
	PRINT*,' File PSECoDS.PRO ( mean boundary-layer properties ) '
	PRINT*,'               does not exist!!'
        IERFIL=1
	RETURN
        END

        SUBROUTINE INTERP(X,QNETA,US,U1,U2,UX0,UX1,UX2,V0,WQS,  &
     & W1,W2,WX0,WX1,WX2,TQS,T1,T2,TX0,TX1,TX2,V1,V2)
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	REAL*8 QNETA(*)
        REAL*8 UC(4,NQ),DUC(4,NQ),U2C(4,NQ),VC(4,NQ)
        REAL*8 UXC(4,NQ),U1XC(4,NQ),U2XC(4,NQ)
	REAL*8 DVC(4,NQ),V2C(4,NQ)
        REAL*8 WC(4,NQ),DWC(4,NQ),W2C(4,NQ)
	REAL*8 WXC(4,NQ),W1XC(4,NQ),W2XC(4,NQ)
        REAL*8 TC(4,NQ),DTC(4,NQ),T2C(4,NQ)
	REAL*8 TXC(4,NQ),T1XC(4,NQ),T2XC(4,NQ)
        COMMON /FITT/UC,DUC,U2C,UXC,U1XC,U2XC,WC,DWC,W2C,WXC,  &
     & W1XC,W2XC,TC,DTC,T2C,TXC,T1XC,T2XC,VC,DVC,V2C
        COMMON /DECIDE/YEDGE,OUTER,RESHMA,BIBI

        IFAIL=0
        IF(X > OUTER)THEN
        U1=0.D0
        U2=0.D0
	V2=0.D0
        UX1=0.D0
        UX2=0.D0
	W1=0.D0
	W2=0.D0
	WX0=0.D0
	WX1=0.D0
	WX2=0.D0

        T1=0.D0
        T2=0.D0
        TX1=0.D0
        TX2=0.D0

	CALL LOCATE(OUTER,QNETA)
        CALL EMSMINT(OUTER,UC,US,IFAIL)
        CALL EMSMINT(OUTER,UXC,UX0,IFAIL)
        CALL EMSMINT(OUTER,WC,WQS,IFAIL)
        CALL EMSMINT(OUTER,TC,TQS,IFAIL)
        CALL EMSMINT(OUTER,TXC,TX0,IFAIL)
        V0=RESHMA*X+BIBI
        CALL EMSMINT(OUTER,DVC,V1,IFAIL)
        ELSE
	CALL LOCATE(X,QNETA)
        CALL EMSMINT(X,UC,US,IFAIL)
        CALL EMSMINT(X,DUC,U1,IFAIL)
        CALL EMSMINT(X,U2C,U2,IFAIL)
        CALL EMSMINT(X,UXC,UX0,IFAIL)
        CALL EMSMINT(X,U1XC,UX1,IFAIL)
        CALL EMSMINT(X,U2XC,UX2,IFAIL)
        CALL EMSMINT(X,VC,V0,IFAIL)
        CALL EMSMINT(X,DVC,V1,IFAIL)
        CALL EMSMINT(X,V2C,V2,IFAIL)
        CALL EMSMINT(X,WC,WQS,IFAIL)
        CALL EMSMINT(X,DWC,W1,IFAIL)
        CALL EMSMINT(X,W2C,W2,IFAIL)
        CALL EMSMINT(X,WXC,WX0,IFAIL)
        CALL EMSMINT(X,W1XC,WX1,IFAIL)
        CALL EMSMINT(X,W2XC,WX2,IFAIL)
        CALL EMSMINT(X,TC,TQS,IFAIL)
        CALL EMSMINT(X,DTC,T1,IFAIL)
        CALL EMSMINT(X,T2C,T2,IFAIL)
        CALL EMSMINT(X,TXC,TX0,IFAIL)
        CALL EMSMINT(X,T1XC,TX1,IFAIL)
        CALL EMSMINT(X,T2XC,TX2,IFAIL)
        END IF
        RETURN
        END


	
        SUBROUTINE SATURN(NQ,NNP,XX,XX2,ICAP,ubot,udel)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        REAL*8 XX(*),XX2(*)
        ICAP=NNP
! dtermine boundary-layer thickness.
	do kk=nnp,1,-1
        utop=dsqrt(XX(kk)**2.D0+XX2(kk)**2.D0)*ubot
	if(utop > UDEL)goto 345
	ICAP=kk
	RETURN
345	end do
        RETURN
        END

! chek this out!!!!!!! ICAP2 is not used...and yet it appears crucial!
        SUBROUTINE JUPITER(NQ,NNP,XX,XX2,ICAP)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        REAL*8 XX(*),XX2(*)
        COMMON /STRETCH/GRID
        ICAP=1

        ubot=1.D0/dsqrt(XX(NNP)**2.D0+XX2(NNP)**2.D0)
! find max...in case we have highly inlexional profiles where ue> 1!

	umax=0.
	do kk=1,nnp

        utop=dsqrt(XX(kk)**2.D0+XX2(kk)**2.D0)*ubot
	if(utop < umax)goto 345
	umax=utop
	ICAP2=kk
	UGRID=UMAX
345	end do

99      utop=dsqrt(XX(ICAP)**2.D0+XX2(ICAP)**2.D0)
        IF(utop*ubot/ugrid > GRID)GOTO 999
        ICAP=ICAP+1
        GOTO 99
999	CONTINUE
!	Print*,ICAP,XX(ICAP), ugrid, utop*ubot
!	READ*,icont
        RETURN
        END




	SUBROUTINE LMOVE(KE,SOL,NB,NPTS,FAR,CFAR,MFDCNT,IOKI) 
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMPLEX*16 SOL(*),UMAXI,CUMAXI	
        COMMON /SPRES/UMAXI,YEXACT
        COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB,IWSQ, &
     & IWMAX
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /UMAXDIR/ILoF
	IF(ILoF == 1)THEN
	CALL MAKETWO(MASFLX,KE,NPTS,NB,SOL)
	ELSE
	CALL MAKETWO_noLoF(MASFLX,KE,NPTS,NB,SOL)
	END IF
        SOL(NPTS+1)=0.D0
        SOL(NPTS+2)=0.D0
        SOL(NPTS+3)=DCMPLX(FAR,CFAR)
        SOL(NPTS+4)=0.D0
        SOL(NPTS+5)=0.D0

	IF(IEURO == 0 .OR. IOKI == 1)RETURN
	CUMAXI=UMAXI
        DO ICLOS=1,3
	CALL CLOSURE(MASFLX,ICLOS,KE,NPTS,CUMAXI,SOL)
	END DO

	RETURN
	END
        SUBROUTINE LOCATE(XV,XP)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /ADDRESS/LO,NQ,NTOT
        REAL*8 XP(*)
        IHI=NTOT
        LO=1
10      MID=(LO+IHI)/2
        IF(XV < XP(MID))THEN
        IHI=MID
        ELSE
        LO=MID
        END IF
        IF(IHI > LO+1)GOTO 10
        RETURN
        END
	
        SUBROUTINE MAKEONE(IIFX,NDIV,POUT,N,K,UMAXI,YEXACT)
        use param_info, only : MP, MODE, KMODE
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LS=5)
        COMPLEX*16 POUT(2,MP)
        COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMPLEX*16 UMAXI
        COMPLEX*16 FLUX(MP)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        REAL*8 US(MP),DU(MP),D2U(MP),UX(MP),DUX(MP),D2UX(MP)
        REAL*8 VS(MP)
        COMMON /HISTRY/PRV
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,VS
        COMMON /TWOMODE/ITMAX,IUNOMAL,IFTX,ISELECT
	COMMON /MALCROS/KEMAX,IQE,I100

	IF(IIFX == 1 .AND. KEMAX == 0)THEN
        DO I=1,NDIV
        FLUX(I)=ROV(I)*PRV(2,I,N,K)+US(I)*POUT(2,I)
	end do
	ELSE IF(KEMAX == 1)THEN
        DO  I=1,NDIV
        FLUX(I)=DSQRT(CDABS(PRV(2,I,N,K))**2.D0 + &
     & CDABS(PRV(4,I,N,K))**2.D0)
	end do
	ELSE
        DO  I=1,NDIV
        FLUX(I)=PRV(ISELECT,I,N,K)
	end do
	END IF

	CALL PARMAXT(NDIV,FLUX,UMAXI,YEXACT)

        DO  J=1,LS
        DO  I=1,NDIV
        PRV(J,I,N,K)=PRV(J,I,N,K)/UMAXI
	end do
	end do

	RETURN
	END
	
        SUBROUTINE MAKETWO(IIFX,KE,NPTS,NB,SOL)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        COMPLEX*16 SOL(*),UMAXI
        COMPLEX*16 FLUX(MP)
        COMMON /TWOMODE/ITMAX,IUNOMAL,IFTX,ISELECT
        COMMON /SPRES/UMAXI,YEXACT
	COMMON /MALCROS/KEMAX,IQE,I100
	COMMON /TRANSLATE/SWPSIN, SWPCOS

        IF(IIFX == 1 .OR. KEMAX == 1)THEN
	CALL FLUXCAL(KE,NPTS,IP,SOL,FLUX)
	ELSE
        IP=1
        DO  I=1,NPTS-IL,IL
        FLUX(IP)=SOL(ISELECT+I-1)*SWPCOS+SOL(ISELECT+I-1+2)*SWPSIN
        IP=IP+1
	END DO
	END IF

	CALL MAXFLUX(IP,FLUX,UMAXI,YEXACT)

	RETURN
	END SUBROUTINE MAKETWO        
	
	SUBROUTINE MAKETWO_noLoF(IIFX,KE,NPTS,NB,SOL)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        COMPLEX*16 SOL(*),UMAXI
        COMPLEX*16 FLUX(MP)
        COMMON /TWOMODE/ITMAX,IUNOMAL,IFTX,ISELECT
        COMMON /SPRES/UMAXI,YEXACT
	COMMON /MALCROS/KEMAX,IQE,I100
	COMMON /TRANSLATE/SWPSIN, SWPCOS


        IF(IIFX == 1 .OR. KEMAX == 1)THEN
	CALL FLUXCAL(KE,NPTS,IP,SOL,FLUX)
	ELSE
        IP=1
        DO  I=1,NPTS-IL,IL
        FLUX(IP)=SOL(ISELECT+I-1)
        IP=IP+1
	END DO
	END IF

	CALL MAXFLUX(IP,FLUX,UMAXI,YEXACT)

	RETURN
	END SUBROUTINE MAKETWO_noLoF
	
	
        SUBROUTINE MAKEURO(MASFLX,ICLOS,NDIV,POUT,N,K,UMAXI)
        use param_info, only : MP, MODE, KMODE
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LS=5)
        COMPLEX*16 POUT(2,MP)
        COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMPLEX*16 UMAXI
        COMPLEX*16 FLUX(MP)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        REAL*8, DIMENSION(MP) :: US,DU,D2U,UX,DUX,D2UX,VS
        COMMON /HISTRY/PRV
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,VS

        IF(ICLOS == 3 .AND. MASFLX /= 1)THEN
        DO  I=1,NDIV
        FLUX(I)=ROV(I)*PRV(2,I,N,K)+US(I)*POUT(2,I)
	end do
        CALL PARMAXT(NDIV,FLUX,UMAXI,YEXACT)
        ELSE IF(ICLOS == 3 .AND. MASFLX == 1)THEN
	UMAXI=DCMPLX(1.D0,0.D0)
	ELSE
        DO I=1,NDIV
        FLUX(I)=PRV(2*ICLOS,I,N,K)
	end do
        CALL PARMAXT(NDIV,FLUX,UMAXI,YEXACT)
        END IF

        RETURN
        END
	SUBROUTINE MAXFLUX(IP,FLUX,UMAXI,YEXACT)
        use param_info, only : MP, MODE, KMODE 
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NTQ=5,IL=5)
	REAL*8 C(NTQ),Y(NTQ),A(NTQ,NTQ),ANS(NTQ)
	COMPLEX*16 UMAXI
        COMPLEX*16 FLUX(*)
	COMMON /TEMESH/DY,YEND,YEDGE
	COMMON /SCALI/G,ULENTH

	VMAX=0.D0
	yo=dy
	DO  I=2,IP-1
	ULOCL=CDABS(FLUX(I))
	IF(ULOCL < VMAX)GOTO 104
	VMAX=ULOCL
	JFLAG=I
	YMAXI=yo
104	continue
        yo=yo+dy	
	end do

	Y(1)=ULENTH*(YmAxi-dy)/(G-(YmAxi-dy))
	Y(2)=ULENTH*YmAxi/(G-YmAxi)
	Y(3)=ULENTH*(YmAxi+dy)/(G-(YmAxi+dy))

	do i=1,3
        A(i,1)=y(i)*y(i)
	A(i,2)=y(i)
	A(i,3)=1.d0
	end do

	c(1)=CDABS(FLUX(JFLAG-1))
	c(2)=CDABS(FLUX(JFLAG))
	c(3)=CDABS(FLUX(JFLAG+1))

	CALL SOLVE(A,C,ANS,NTQ,3)
	yexAct=-Ans(2)/(2.*Ans(1))
	fvAl=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)
	c(1)=DBLE(FLUX(JFLAG-1))
	c(2)=DBLE(FLUX(JFLAG))
	c(3)=DBLE(FLUX(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
	fvr=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)

	c(1)=DIMAG(FLUX(JFLAG-1))
	c(2)=DIMAG(FLUX(JFLAG))
	c(3)=DIMAG(FLUX(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
	fvi=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)
        UMAXI=DCMPLX(fvr,fvi)
	RETURN
	END

	SUBROUTINE MAXEADS(ndiv,PUTEMP, umaxeads)
        use param_info, only : MP, MODE, KMODE 
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (NTQ=5,IL=5)
	REAL*8 C(NTQ),Y(NTQ),A(NTQ,NTQ),ANS(NTQ)
	COMPLEX*16 UMAXEADS
        COMPLEX*16 PUTEMP(*)
	COMMON /TEMESH/DY,YEND,YEDGE
	COMMON /SCALI/G,ULENTH

	VMAX=0.D0
	yo=dy
	DO  I=2,NDIV-1
	ULOCL=CDABS(PUTEMP(I))
	IF(ULOCL > VMAX)then
	VMAX=ULOCL
	JFLAG=I
	YMAXI=yo
	end if
        yo=yo+dy	
	end do

	Y(1)=ULENTH*(YmAxi-dy)/(G-(YmAxi-dy))
	Y(2)=ULENTH*YmAxi/(G-YmAxi)
	Y(3)=ULENTH*(YmAxi+dy)/(G-(YmAxi+dy))

	do i=1,3
        A(i,1)=y(i)*y(i)
	A(i,2)=y(i)
	A(i,3)=1.d0
	end do

	c(1)=CDABS(PUTEMP(JFLAG-1))
	c(2)=CDABS(PUTEMP(JFLAG))
	c(3)=CDABS(PUTEMP(JFLAG+1))

	CALL SOLVE(A,C,ANS,NTQ,3)
	yexAct=-Ans(2)/(2.*Ans(1))
	fvAl=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)
	c(1)=DBLE(PUTEMP(JFLAG-1))
	c(2)=DBLE(PUTEMP(JFLAG))
	c(3)=DBLE(PUTEMP(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
	fvr=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)

	c(1)=DIMAG(PUTEMP(JFLAG-1))
	c(2)=DIMAG(PUTEMP(JFLAG))
	c(3)=DIMAG(PUTEMP(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
	fvi=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)
        UMAXEADS=DCMPLX(fvr,fvi)
	RETURN
	END	
	

        SUBROUTINE MISCoDS(IBEG,IKHAT)
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /FILINF/IERFIL
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	IF(ICoDS == 1)THEN
	npre=2
	nextra=2
	else
	npre=1
	nextra=1
	end if
	
	IERFIL=0
        DO 10 IMISS=IBEG,IKHAT
	J=0
	IWHERE=1
	READ(8,*,END=40,ERR=40)
        READ(8,*,END=30,ERR=40)IPROF,NNP, xcv
	IWHERE=0
	IF(NNP+1 > NQ)GOTO 50
	DO IJK=1,4
        READ(8,*,END=40,ERR=40)
	end do
        DO  J=1,(NNP+1)*npre+nextra
        READ(8,*,END=40,ERR=40)
	end do
10      CONTINUE



	RETURN
!       useless dAtA reAd
30	WRITE(*,*)
	PRINT*,'No more mean boundary-layer data in PSECoDS.PRO :'
        WRITE(*,33)IPROF
33      FORMAT(2X,'Analysis aborted at profile ',I0)
	IERFIL=1
	RETURN
40	PRINT*,'Error in mean boundary-layer data input :'
	PRINT*,'Premature end of analysis - Check PSECoDS.PRO ' 
	WRITE(*,22)IPROF+IWHERE,J
22	FORMAT(1X,'Error in data at profile ',I0,' : Line ',I0)
	IERFIL=1
        RETURN
50	PRINT*,'No. of data points allowed to define the mean boundary-'
	WRITE(*,55)IPROF+IWHERE
	WRITE(*,56)NQ,NNP
55	FORMAT(1X,'layer profile at station ',I0,' has been exceeded')
56	FORMAT(1X,'No. of points must be < ',I0,' : You have Npoint = ',&
     & I0)
	IERFIL=1
        RETURN
        END
        SUBROUTINE MISJUMP(IBEG,IKHAT)
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /FILINF/IERFIL
	IERFIL=0
        DO 10 IMISS=IBEG,IKHAT
	J=0
	IWHERE=1
        READ(8,*,END=30,ERR=40)IPROF,NNP
!         READ(14,*,END=30,ERR=40)
	IWHERE=0
	IF(NNP > NQ)GOTO 50
        READ(8,*,END=40,ERR=40)
!         READ(14,*,END=30,ERR=40)
	J=0
        DO 20 J=1,NNP
        READ(8,*,END=40,ERR=40)
!        READ(8,*,END=40,ERR=40)
!         READ(14,*,END=30,ERR=40)
!         READ(14,*,END=30,ERR=40)
20      CONTINUE
10      CONTINUE
	RETURN
!        useless dAtA reAd
30	WRITE(*,*)
	PRINT*,'No more mean boundary-layer data in PSEMEAN.PRO :'
        WRITE(*,33)IPROF
33      FORMAT(2X,'Analysis aborted at profile ',I0)
	IERFIL=1
	RETURN
40	PRINT*,'Error in mean boundary-layer data input :'
	PRINT*,'Premature end of analysis - Check PSEMEAN.PRO ' 
	WRITE(*,22)IPROF+IWHERE,J
22	FORMAT(1X,'Error in data at profile ',I0,' : Line ',I0)
	IERFIL=1
        RETURN
50	PRINT*,'No. of data points allowed to define the mean boundary-'
	WRITE(*,55)IPROF+IWHERE
	WRITE(*,56)NQ,NNP
55	FORMAT(1X,'layer profile at station ',I0,' has been exceeded')
56	FORMAT(1X,'No. of points must be < ',I0,' : You have Npoint = ',&
     & I0)
	IERFIL=1
        RETURN
        END

        SUBROUTINE MISPRF(IBEG,IKHAT)
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /DIRFORM/IFORMAT,NONORTH
	IF(IFORMAT == 0)RETURN
        IF(ICoDS >= 1)THEN
        CALL MISCoDS(IBEG,IKHAT)
        ELSE
        CALL MISJUMP(IBEG,IKHAT)
        END IF
        RETURN
        END
        SUBROUTINE MLOCAL(IP,Medge,ULOCAL,TLOCAL)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)               
        DOUBLE PRECISION SMX(MP),Medge,MESQ,MAK
        COMMON /INHIBIT/SMX
        COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TO,T1,STOKES            
        COMMON /DISCR3D/xellips,ox

        MAK=MEDGE*ULOCAL/DSQRT(TLOCAL)                               
        IF(MAK <= 1.0)THEN
!         MESQ=MAK*MAK                                                 
!        SMX(IP)=GAMMA*MESQ/(1.D0+(GAMMA-1.D0)*MESQ)                  
        SMX(IP)=0.D0
	ox=xellips
        ELSE                            
        SMX(IP)=1.D0
	ox=0.D0
        END IF                                                       
        END                             



	SUBROUTINE NCALCUL(I,J,IALC,ENALL)
        use param_info, only : MP, MODE, KMODE, NPOST
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5,KE=IL*MP)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
        REAL*8 DXVAL(NPOST),ENALL(NPOST,4)
        COMMON /MOHD786/DXVAL
        COMMON /MUADZAM/EURALFA

! note that in cases where you begin in a unstable region
! probs with N calcs when the transients predict a return to (-ve)
!	N... then all suvbsequent calcs are set to 0.

	do ICLOS=1,4
	do IK=1,IALC
	ENALL(IK,ICLOS)=0.D0
	end do
	end do

	DO 10 ICLOS=1,4

	NOFF=0
	ENY=0.D0
	ENALL(1,ICLOS)=0.D0

	AO=DBLE(EURALFA(I,J,1,ICLOS))
	IF(AO > 0.D0)THEN
	ENALL(1,ICLOS)=0.D0
	NOFF=1
	END IF

	DO IK=2,IALC
	DX=DXVAL(IK)
	AN=DBLE(EURALFA(I,J,IK,ICLOS))
	IF(NOFF == 0)THEN
        IF(AO < 0.D0 .AND. AN < 0.D0)THEN
	ENALL(IK,ICLOS)=0.D0
        ELSE IF(AO < 0.D0 .AND. AN > 0.D0)THEN
        ENY=0.5D0*AN**2*DX/(AN-AO)
        ENALL(IK,ICLOS)=ENY
        NOFF=1
	END IF
	ELSE IF(NOFF == 1 .AND. ENALL(IK-1,ICLOS) >= 0.D0)THEN
        ENY=0.5D0*(AN+AO)*DX+ENY
	ENALL(IK,ICLOS)=ENY
	ELSE IF(NOFF == 1 .AND. ENALL(IK-1,ICLOS) < 0.D0)THEN
	ENY=0.D0
	NOFF=0
	ENALL(IK,ICLOS)=ENY
	END IF
	AO=AN
	END DO
10	CONTINUE

	RETURN
	END


	SUBROUTINE ORR(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,IGERR,VAL)
!        use param_info, only : MP, MODE, KMODE, NPOST
	use param_NUMBERCRUNCH
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 XCVAL(NPOST),DIMSCL(4)
        REAL*8 RDVAL(NPOST),SDVAL(NPOST)
	REAL*8 MEDGE,LNORM
	COMPLEX*16 XSOL(KE),OMEGA,ALFA,ABSQ
	COMPLEX*16 EIGEN
	COMPLEX*16 ALNEW,ALPREV
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 DCRHS(KE),RHS(KE)
	COMMON /SHAPE/RHS,DCRHS
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /EISPC/ALFAZO
	COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IN
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/FVALU
	COMMON /ROOTS/ALFA
	COMMON /NICEINF/RESIDU
	COMMON /SOMMER/LNORM
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /IPRIMIT/IVARIP
        COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR

	REQ=RESSCL

	CALL EXTRAP(IALC,IFREQ,ICR,ALPREV)

	OMEGA=DCMPLX(OMRANGE(IFREQ),0.D0)

        BETA=CRFBETA(ICR)
	BZERO=BETA
	BESQ=BZERO*BZERO

        IF(ITSPRES >= 1)GOTO 807

	IF(NOUTPUT == 1 .OR. NOUTPUT == 3)THEN
	WRITE(ITV,77)OMRANGE(IFREQ)/RLOCAL*1.D6
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,88)BZERO*LNORM*PZERO
	ELSE
	WRITE(ITV,787)OMRANGE(IFREQ)/RLOCAL*1.D6,BZERO*LNORM*PZERO
	END IF

	CALL FLUSH(ITV)
807	CONTINUE

	EIGEN=ALPREV/LNORM

        IF(ICURVI == 0 .AND. IVARIP == 1)THEN
        CALL SETRUVT(NDIV,IONEQ)
        ELSE IF(ICURVI == 1)THEN
!        CALL CURGRET(NDIV,IONEQ)
        CALL SETPAR(NDIV,IONEQ)
        ELSE
        CALL SETPAR(NDIV,IONEQ)
        END IF	
	
	DO KKK=1,IEQT
	RHS(KKK)=ORRRHS(IFREQ,ICR,KKK)
	DCRHS(KKK)=ORRDCRHS(IFREQ,ICR,KKK)
	end do

	CALL SETGLB(NDIV,IONEQ)
	CALL RAYLEI(IONEQ,NB,IEQT,XSOL,IGERR)
	
	DO KKK=1,IEQT
	ORRRHS(IFREQ,ICR,KKK)=RHS(KKK)
	ORRDCRHS(IFREQ,ICR,KKK)=DCRHS(KKK)
	end do
	
        IF(IGERR == 1)RETURN
	IF(IGERR == 2)THEN
	WRITE(ITV,*)
	WRITE(ITV,*)'Rayleigh Iteration not converged but may be local'
	WRITE(ITV,*)' non-convergence - hence continuing analysis'
	WRITE(ITV,*)
	CALL FLUSH(ITV)
	ELSE
	END IF
	ALNEW=ALFA
	ALPREV=ALNEW
	ALFAZO(IFREQ,ICR,IALC)=ALNEW*LNORM
	BZERO=BZERO*LNORM
	VAL=RESIDU

	RETURN
77	FORMAT(1X,'Computing for Frequency Parameter F*1.E6 = ',E12.6)
787	FORMAT(1X,'Computing for F*1.E6 = ',E12.6, &
     & ' & Beta =',E12.6)
88	FORMAT(1X,'Computing for Crossflow Wavenumber = ',E12.6)
	END
        SUBROUTINE ORRQZX(NDIV,IL,IEQT,ITRASH,IGERR)
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	PARAMETER (LS=5,KS=MP*LS)
        CHARACTER*80 TITLE
	CHARACTER*58 PSELABL
	COMPLEX*16 XSOL(KE),OMEGA,ALFA
	COMPLEX*16 EIGEN,UMAXI,ABSQ,CUMAXI
	COMPLEX*16 EKE(MP),ENTEMP
	COMPLEX*16 POUT(2,MP)
	COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),ENERGY(KSTR,KCRF)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),DIMSCL(4)
	REAL*8 CRFBETA(KCRF)
        INTEGER JOUTSC(2)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        COMPLEX*16 EURONRM(KSTR,KCRF,4),EUROMAX(4)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
        COMMON /MUADZAM/EURALFA     
        COMMON /MUSARAT/EURONRM,EUROMAX
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /PICTR/TITLE
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /FRVARI/OMEGA,RE
	COMMON /ROOTS/ALFA
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /SCALI/G,ULENTH
	COMMON /EISPC/ALFAZO
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /KERES/ENERGY
	COMMON /HISTRY/PRV
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB,IWSQ, &
     & IWMAX
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
        COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /MULTF2/ISMOOTH
	ISMOOTH=1

	NB=NDIV-1
	REQ=RESSCL
        sqfAct=dsqrt(2.D0)
	WRITE(ITV,*)
	IF(IGLOB /= 1)GOTO 123
	WRITE(ITV,*)
        WRITE(ITV,*)'     Global method in x called !!! '
	WRITE(ITV,*)
	CALL FLUSH(ITV)
	IF(IQZFULL == 1)THEN
	OPEN(19,FILE='PSESPEC.FULL',STATUS='UNKNOWN')
	ELSE
	END IF
	OPEN(15,FILE='PSESPEC.DAT',STATUS='UNKNOWN')
	IA=1
	IB=0
	DO 101 I=1,1
	ICROSS=1
	OMEGA=DCMPLX(OMRANGE(I),0.D0)	
	BETA=CRFBETA(ICROSS)
	BZERO=BETA
	BESQ=BZERO*BZERO

	IF(ICURVI == 0 .AND. IVARIP == 1)THEN
!        CALL SETRUVT(NDIV,IL)
	CALL SETPAR(NDIV,IL)
	ELSE IF(ICURVI == 1)THEN
	CALL SETPAR(NDIV,IL)
	ELSE 
	CALL SETPAR(NDIV,IL)
	END IF

	CALL QZCALLSET(NDIV,IL)
	
	IF(IQZFULL == 1)THEN
	WRITE(19,676)DBLE(OMEGA),BZERO*REQ
	ELSE
	END IF

	WRITE(15,676)DBLE(OMEGA),BZERO*REQ
	WRITE(ITV,676)DBLE(OMEGA),BZERO*REQ
	WRITE(ITV,*)
	WRITE(ITV,*)'   Full Eigenvalue Spectrum in PSESPEC.DAT '
	WRITE(ITV,*)'          Most Probable Roots are ... '
	CALL FLUSH(ITV)
	CALL QZALG(IA,IB,IEQT,I,IL,0)
	WRITE(ITV,*)
	WRITE(ITV,*)
	CALL FLUSH(ITV)
101	CONTINUE
676	FORMAT(2x,'Computing for Omega = ',E16.9,' : BETA = ',E16.9)

	RETURN
123	CONTINUE
	END
	SUBROUTINE OSE(I3DNEQ,IGERR)
        use param_info, only : MODE, KMODE, NPOST
	use param_FILEADD
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	COMPLEX*16 OMEGA,ALNEW,ALPREV,ALERROR
	REAL*8 MEDGE,MO,DIMSCL(4),RES(KSTR,KCRF)
	COMPLEX*16 QZLIN(KSTR),ALFAZO(KSTR,KCRF,NPOST)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 ENMETH(KSTR,KCRF,NPOST),XCVAL(NPOST),RDVAL(NPOST)
	INTEGER NFFLAG(KSTR,KCRF),ISWITCH(KSTR,KCRF)
        INTEGER IPREMAT(KSTR,KCRF),MATURE(KSTR,KCRF)
	INTEGER IERPOST(KSTR,KCRF),IERSWCH(KSTR,KCRF)
	INTEGER IERALFA(KSTR,KCRF)
	REAL*8 WAVBETA(KCRF),SDVAL(NPOST)
        REAL*8 PHIN(NPOST)
        COMMON /RELVECT/PHIN
        COMMON /RELATIV/PHINVSC
	COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /EISPC/ALFAZO
	COMMON /QCNTRL/ND
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /FRVARI/OMEGA,RE
        COMMON /SPFLG/ISPC,IOUT,IHOW,IPAR
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
	COMMON /POSSSI/IPROF
	COMMON /STEPNG/XNEW
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /PICOSV/SWPSCL,CLINE
	COMMON /PRFIL/DXSTEP,DXINV
	COMMON /AMPLIT/NFACT,NFFLAG
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/F
	COMMON /FILINF/IERFIL
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ISAVEY/ISWITCH
	COMMON /PISAVE/PI
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
        COMMON /NCURVES/INEUTRL
		
	IFIRST=0
	IERFIL=0
	NFLAG=0
	IALC=1

	IOFF=0
!	ioff is marker for whether Any nfActors Are non-zero... not
!	reAlly being used At present!!!! should use this As indICAtot
!	to user that no nfActors Are nonzero

	NDIV=ND
	CALL ETASET(NDIV)

        CALL INITL

        IF(IERFIL == 1)RETURN

	CALL PROFRD(NDIV,NSTART,1)

	IF(IERFIL == 1)RETURN

	DO  ICR=1,NCRBET
	DO  IJK=1,NFREQ
	NFFLAG(IJK,ICR)=0
	ISWITCH(IJK,ICR)=0
	IERSWCH(IJK,ICR)=0
	IPREMAT(IJK,ICR)=0
	ENMETH(IJK,ICR,1)=0.D0
	IERALFA(IJK,ICR)=1
	end do
	end do

	RELOC=RLOCAL
	REPRV=RLOCAL
	REXPRV=REX
        XACT=SPOS
	XPREV=XPOS
	XSCALE=XNEW

	IF(ISCREEN == 0)THEN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.dmp',PSELABL)
	OPEN(ITV,FILE=PSELABL,STATUS='UNKNOWN')
	ELSE
	END IF

        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,110)IPROF,XPOS,RELOC
        WRITE(2,110)IPROF,XPOS,RELOC
        WRITE(ITV,440)
	IF(NOUTPUT == 1)THEN
        WRITE(ITV,311)0.001D0*FRENCY(1),CRFBETA(1)
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,411)0.001D0*FRENCY(1)
311     FORMAT(1X,'Frequency =',E12.6,'KHz ;', & 
     & 1X,'Crossflow Wave No. =',E12.6)
411     FORMAT(1X,'Frequency =',E12.6,'KHz')
	ELSE IF(NOUTPUT == 3)THEN
	WRITE(ITV,511)CRFBETA(1)
511     FORMAT(1X,'Crossflow Wave No. =',E12.6)
	ELSE
	END IF
	CALL FLUSH(ITV)

	XCVAL(1)=XPOS
	RDVAL(1)=RELOC
	SDVAL(1)=SPOS
        PHIN(1)=PHINVSC

220     FORMAT(5x,'  RDelta = ',E14.7)
110     FORMAT(1X,'OSE begins at station ',I0,' : x/c = ',F11.6, &
     & 1X,'; RDelta = ',E12.6)
440     FORMAT(1X,69('-'))

	IONEQ=9
	NB=NDIV-1
	IEQT=NB*IONEQ
        CALL AP3D2(NDIV,IEQT,IONEQ)
	
        IF(INEUTRL == 1 .AND. IGLOB == 0)THEN
        CALL NEUTIT(NDIV,IONEQ,IEQT,0,IGERR)
	stop
        else	
	CALL PARLIN(NDIV,IONEQ,IEQT,0,IGERR)
	END IF
	IEQT=NB*IONEQ
        CALL AP3D2(NDIV,IEQT,IONEQ)

        IF(IGERR == 1)RETURN
	IF(IGLOB == 1)RETURN

	IF(NSTART+NINTR+1 > NFINAL)RETURN

	IHOW=IOUT

        CALL MISPRF(1,NINTR)

        IF(IERFIL == 1)RETURN
        INP=NSTART+NINTR+1

	DO 10 IN=INP,NFINAL,NINTR+1
	CALL PROFRD(NDIV,IN,1)

	IF(IERFIL == 1)GOTO 7671
	RE=RLOCAL
211     FORMAT(3X,'OSE at station ',I0,' : x/c = ',F12.6, &
     & 2X,'RDelta = ',E12.6)

	IF(ITSPRES == 0)THEN
        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,440)
        WRITE(ITV,211)IPROF,XPOS,RLOCAL
        WRITE(ITV,440)
	CALL FLUSH(ITV)
	ELSE
	END IF

	DX=XNEW-XSCALE
	DXSTEP=DX
	DXINV=1.D0/DX

	IALC=IALC+1
	XCVAL(IALC)=XPOS
	RDVAL(IALC)=RLOCAL
	SDVAL(IALC)=SPOS
        PHIN(IALC)=PHINVSC

	DO 101 ICR=1,NCRBET
	CRFBETA(ICR)=WAVBETA(ICR)*DELZER
	DO 101 IFREQ=1,NFREQ

	IF(IERSWCH(IFREQ,ICR) == 1)GOTO 247

	IF(ISWITCH(IFREQ,ICR) == 1 .AND. NFFLAG(IFREQ,ICR) == 0)THEN
	ENMETH(IFREQ,ICR,IALC)=0.D0
	ELSE
	END IF
	IF(ISWITCH(IFREQ,ICR) == 1 .AND. NFSWICH == 0)GOTO 247

        IF(ITSPRES >= 1)GOTO 507
        WRITE(ITV,*)
507	CONTINUE

	OMRANGE(IFREQ)=2.D0*PI*FRENCY(IFREQ)*DELZER/UEDGE
	CALL ORR(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,ierr,RESIDU)
!	test to see whether nonconvergence if Any is serious or not
	ALERROR=ALFAZO(IFREQ,ICR,IALC)
	CALL EROS(IERR,RESIDU,ALERROR,IDECID)
	IF(IDECID == 1)THEN
	IERSWCH(IFREQ,ICR)=1
	IERPOST(IFREQ,ICR)=IPROF
	IERALFA(IFREQ,ICR)=IALC-1
	ELSE
	IERALFA(IFREQ,ICR)=IALC
	END IF

	if(ierr == 9099)GOTO 7671
	RES(IFREQ,ICR)=RESIDU

        IF(ITSPRES == 0)THEN
        WRITE(ITV,*)
	ELSE
	END IF


	IF(IERR > 0)GOTO 11

        IF(NFACT /= 1)GOTO 11 
	AN=DBLE(ALFAZO(IFREQ,ICR,IALC))
	AO=DBLE(ALFAZO(IFREQ,ICR,IALC-1))
        IF(NFFLAG(IFREQ,ICR) == 1)GOTO 22
	IF(AN < 0.D0 .AND. AO < 0.D0)THEN
	ENMETH(IFREQ,ICR,IALC)=0.D0
	ELSE IF(AO < 0.D0 .AND. AN > 0.D0)THEN
	GNBIT=0.5D0*AN**2*DX/(AN-AO)
	ENMETH(IFREQ,ICR,IALC)=GNBIT
	NFFLAG(IFREQ,ICR)=1
	IOFF=1	
	ELSE
	IPREMAT(IFREQ,ICR)=1
	MATURE(IFREQ,ICR)=IPROF
	ENMETH(IFREQ,ICR,IALC)=0.5D0*(AN+AO)*DX
	NFFLAG(IFREQ,ICR)=1
	IOFF=1	
	END IF
	GOTO 11
22      GNBIT=ENMETH(IFREQ,ICR,IALC-1)+0.5D0*(AN+AO)*DX
	ENMETH(IFREQ,ICR,IALC)=GNBIT
	IF(GNBIT >= 0.D0)GOTO 11

	NFFLAG(IFREQ,ICR)=0   
	ISWITCH(IFREQ,ICR)=1

11	CONTINUE

247	CONTINUE

101	CONTINUE

	IF(ITSPRES >= 1)GOTO 7011
	IF(NOUTPUT >= 2)THEN
	WRITE(ITV,440)
	ELSE
	END IF
7011	CALL RESLIN(1,IALC,DX)
	CALL FLUSH(ITV)
	IF(ITSPRES >= 1)GOTO 711
	IF(NOUTPUT >= 2)THEN
	WRITE(ITV,440)
	ELSE
	END IF
711	CONTINUE

279     FORMAT(2X,2F10.5,2x,f12.7,2x,f15.8,3X,F12.6)

	DXNF=SPOS-XACT
	DXREAL=XPOS-XPREV
	DELRED=RLOCAL-REPRV
	DXRX=REX-REXPRV

	ihow=ihow+1

	XACT=SPOS
	XPREV=XPOS
	XSCALE=XNEW
	REPRV=RLOCAL
	REXPRV=REX

        CALL MISPRF(1,NINTR)

        IF(IERFIL == 1)GOTO 7671

!	test to see whether Any waves Are worth computing....
	IADD=0
	DO ICR=1,NCRBET
	DO IFREQ=1,NFREQ
	IADD=IADD+IERSWCH(IFREQ,ICR)
	END DO
	END DO

	IF(IADD < NCRBET+NFREQ-1)GOTO 1066
	WRITE(ITV,*)
	WRITE(ITV,*)' All waves show signs of non-convergence'
	WRITE(ITV,*)' abandoning analysis....'
	CALL FLUSH(ITV)
	GOTO 7671

1066	CONTINUE

!	Check to see whether All n-fActors Are < 0
	IF(IOFF == 0)GOTO 10

	IPAK=0
	IPRE=0
	DO ICR=1,NCRBET
	DO IFREQ=1,NFREQ
	IPAK=IPAK+NFFLAG(IFREQ,ICR)   
	IPRE=IPRE+IPREMAT(IFREQ,ICR)
	END DO
	END DO

        IF(IPAK > 0 .OR. NFSWICH == 1)GOTO 10
	WRITE(ITV,*)
	WRITE(ITV,*)' All N-Factors are < 0.0 '
	CALL FLUSH(ITV)
	GOTO 7671


10	continue

7671	CONTINUE

!	nonconvergence story.....
	IF(IADD == 0)GOTO 7666
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.noconvrg',PSELABL)
	OPEN(17,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(17,70111)
	WRITE(17,70112)
	do 55055 ICR=1,NCRBET
	do 55055 IFREQ=1,NFREQ
	IF(IERSWCH(IFREQ,ICR) == 0)GOTO 55055
	WRITE(17,70113)IERPOST(IFREQ,ICR),FRENCY(IFREQ)*0.001,WAVBETA(ICR)
55055	CONTINUE
70113	FORMAT(I8,4X,E16.6,6X,E16.6)
70112	FORMAT(2X,' Profile ',4X,' Frequency (KHz)',2X, &
     & ' Crossflow Wave No. (/m)')
70111	FORMAT(1X,'The following waves had convergence problems...')

7666	CONTINUE

!	premature start story....
	IF(IPRE == 0)GOTO 6667
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.redo',PSELABL)
	OPEN(17,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(17,60111)
	WRITE(17,60112)
	do 65055 ICR=1,NCRBET
	do 65055 IFREQ=1,NFREQ
	IF(IPREMAT(IFREQ,ICR) == 0)GOTO 65055
	WRITE(17,60113)MATURE(IFREQ,ICR),FRENCY(IFREQ)*0.001,WAVBETA(ICR)
65055	CONTINUE
60113	FORMAT(I8,4X,E16.6,6X,E16.6)
60112	FORMAT(2X,' Profile ',4X,' Frequency (KHz)',2X, &
     & ' Crossflow Wave No. (/m)')
60111	FORMAT(1X,'The following waves need to be redone at earlier', &
     & 1X,'stations...')


6667	CONTINUE

        IF(NFACT == 0)RETURN 

        IF(IBAE == 1)THEN
        CALL ENBAE2(IALC,ENMETH,IERALFA)
        END IF

        IF(IEURO == 1)THEN
        CALL ENEURO2(IALC,ENMETH,IERALFA)
	END IF

        IF(NATURAL == 1)CALL ENPRINT(IALC,ENMETH)

	RETURN
	END
	
	
	SUBROUTINE ERRINFO(ICOUNT,IGERR,NABAND,ICHOICE)
	COMPLEX*16 ALFA
	COMMON /ROOTS/ALFA
	
	WRITE(*,*)' '
        PRINT*,' no convergence...'
	IF(DABS(DBLE(ALFA)) < 1.E-12)THEN
	PRINT*,' I suspect that tolerance criteria is set too high!'
	WRITE(*,11)DBLE(ALFA)
11	format(2x,'alpha_i has converged to ',f18.15,' ! or ... ')
	END IF
        IF(IGERR > 0)THEN
        PRINT*,' Possible grid resolution problems (Rayleigh failure).'
        ELSE IF(ICOUNT > NABAND .AND. ICHOICE == 1)THEN
        PRINT*,' Possibly in area of parameter space where no neutral'
	PRINT*,' points for all Re'
	else
        PRINT*,' Possibly in area of parameter space where no neutral'
	PRINT*,' points for all Omega'
        end if
	
	RETURN
	END


	SUBROUTINE OMBIT(IL,NB,NDIV,IEQT,NABAND,OMZERO, &
     & TOLCGS,PERTURB,DFREQ,IERR)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (a-h,o-z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        PARAMETER (LS=5,KS=MP*LS)
        COMPLEX*16 XSOL(KE),eigen
	COMPLEX*16 ALPLUS,ALMINUS,ALMID,OMEGA,ALFA
	COMMON /ROOTS/ALFA
	COMMON /FRVARI/OMEGA,RE
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
        COMMON /INUTTY/wrscale
	COMMON /SXXXXX/ISIM,Ibuoy
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
	WRITE(*,112)
112	format(6x,' Omega',13x,'Re ',14x,' Alpha_r',10x,' Alpha_i')
107	format(2x,' frequency = ',g18.8,' Hz; Re = ',g18.8) 
	
        ICOUNT=1
	OMSTRT=DBLE(OMEGA)
	RESTRT=RE
10017   CONTINUE

        ALMID=ALFA
        REZERO=RE
        OMSTEP=PERTURB*RE
        RE=REZERO+OMSTEP
        EIGEN=ALFA

        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALPLUS=ALFA
        RE=REZERO-OMSTEP
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALMINUS=ALFA

        DOMBOT=0.5D0*DBLE(ALPLUS-ALMINUS)/OMSTEP
        DNEW=-DBLE(ALMID)/DOMBOT
        RE=REZERO+DNEW

        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
	IF(IGERR > 0)GOTO 1696
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA)

        DRES=DBLE(ALFA)
        IF(ICOUNT > NABAND)GOTO 1696
        ICOUNT=ICOUNT+1
        IF(DABS(DRES) > TOLCGS)GOTO 10017
	IF(ISIM == 1)THEN
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA),XPOS
	else
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA)
	end if
	RETURN
1696    CONTINUE
	IERR=-1
	WRITE(*,*)' '
	PRINT*,' no convergence...'
	IF(IGERR > 0)THEN
	PRINT*,' Possible grid resolution problems (Rayleigh failure).'
	PRINT*,' or may be a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	ELSE IF(DABS(DBLE(ALFA)) < 1.E-12)THEN
	PRINT*,' I suspect that tolerance criteria is set too high!'
	WRITE(*,11)DBLE(ALFA)
11	format(2x,'alpha_i has converged to ',f18.15,' ! ')
	ELSE
	PRINT*,' Possibly a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	end if
	WRITE(*,*)' '
	PRINT*,' *** Parameters at non-convergence :'
	WRITE(*,'(2x,A,E15.9,a,E15.9)')' Frequency = ',OMSTRT*wrscale,&
     & ' Hz; Re = ', RESTRT
106     FORMAT(9E18.8)
	RETURN
	END
	
 

	SUBROUTINE RERANGE(INPSTAT,IL,NB,NDIV,IEQT,NABAND,OMZERO, &
     & TOLCGS,PERTURB,DFREQ,IERR,ITRIP)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (a-h,o-z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        PARAMETER (LS=5,KS=MP*LS)
	REAL*8 :: Medge
        COMPLEX*16 XSOL(KE)
	COMPLEX*16 ALPLUS,ALMINUS,ALMID,OMEGA,ALFA,eigen
	COMMON /ROOTS/ALFA
	COMMON /FRVARI/OMEGA,RE
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
        COMMON /INUTTY/wrscale
	COMMON /SXXXXX/ISIM,Ibuoy
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /SCLLOC/Uedge,Medge,RLOCAL,Tedge,Delzer
        COMMON /PISAVE/PI

     	sconst=2.d0*PI*Delzer
	WRITE(*,112)
112	format(6x,' Omega',13x,'Re ',14x,' Alpha_r',10x,' Alpha_i')
	
        ICOUNT=1
	OMSTRT=DBLE(OMEGA)
	RESTRT=RE
11107   CONTINUE

        ALMID=ALFA
        OMZERO=DBLE(OMEGA)
        OMSTEP=PERTURB*OMZERO
        OMEGA=DCMPLX(OMZERO+OMSTEP,0.D0)
        EIGEN=ALFA

        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

        ALPLUS=ALFA

        OMEGA=DCMPLX(OMZERO-OMSTEP,0.D0)
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALMINUS=ALFA

        DOMBOT=0.5D0*DBLE(ALPLUS-ALMINUS)/OMSTEP
        DNEW=-DBLE(ALMID)/DOMBOT
        OMEGA=DCMPLX(OMZERO+DNEW,0.D0)

        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
	IF(IGERR > 0)GOTO 1678
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA)
        DRES=DBLE(ALFA)
        IF(ICOUNT > NABAND)GOTO 1678
        ICOUNT=ICOUNT+1
        IF(DABS(DRES) > TOLCGS)GOTO 11107

	IF(ISIM == 1)THEN
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA),XPOS, & 
     & Uedge*DBLE(OMEGA)/sconst
	else
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA), & 
     & Uedge*DBLE(OMEGA)/sconst
	end if

	RETURN

1678    CONTINUE
	IERR=-1
	WRITE(*,*)' '
	PRINT*,' no convergence...'
	IF(IGERR > 0)THEN
	PRINT*,' Possible grid resolution problems (Rayleigh failure).'
	PRINT*,' or may be a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	ELSE IF(DABS(DBLE(ALFA)) < 1.E-12)THEN
	PRINT*,' I suspect that tolerance criteria is set too high!'
	WRITE(*,11)DBLE(ALFA)
11	format(2x,'alpha_i has converged to ',f18.15,' ! ')
	ELSE
	PRINT*,' Possibly a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	end if
	WRITE(*,*)' '
	PRINT*,' *** Parameters at non-convergence :'
	WRITE(*,'(2x,A,E15.9,a,E15.9,A,I0)')' Frequency = ', &
     & OMSTRT*wrscale,' Hz; Re = ', RESTRT, ' ;  NSTATION : ', INPSTAT
107	format(2x,' frequency = ',g18.8,' Hz; Re = ',g18.8) 
106     FORMAT(19E18.8)
	RETURN
	END SUBROUTINE RERANGE
	
	
	

	SUBROUTINE BRANGE(INPSTAT,IL,NB,NDIV,IEQT,NABAND,OMZERO, &
     & TOLCGS,PERTURB,DFREQ,IERR,ITRIP)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (a-h,o-z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        PARAMETER (LS=5,KS=MP*LS)
        COMPLEX*16 XSOL(KE)
	COMPLEX*16 ALPLUS,ALMINUS,ALMID,OMEGA,ALFA,eigen,ABSQ
	DOUBLE PRECISION Mo, LNORM, Medge 
	COMMON /ROOTS/ALFA
	COMMON /FRVARI/OMEGA,RE
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
        COMMON /INUTTY/wrscale
	COMMON /SXXXXX/ISIM,Ibuoy
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /SOMMER/LNORM
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
!        PZERO=CHORD/DZERO
	PDIMIT=1.d0/Delzer

	WRITE(*,112)
112	format(6x,' Beta',13x,'Re ',14x,' Alpha_r',10x,' Alpha_i')
	
        ICOUNT=1
	OMSTRT=BZERO
	RESTRT=RE
11107   CONTINUE

        ALMID=ALFA
        OMZERO=BZERO
        OMSTEP=PERTURB*OMZERO
        BZERO=OMZERO+OMSTEP
	BESQ=BZERO*BZERO
	
        EIGEN=ALFA

        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

        ALPLUS=ALFA

        BZERO=OMZERO-OMSTEP
	BESQ=BZERO*BZERO
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALMINUS=ALFA

        DOMBOT=0.5D0*DBLE(ALPLUS-ALMINUS)/OMSTEP
        DNEW=-DBLE(ALMID)/DOMBOT
        BZERO=OMZERO+DNEW
	BESQ=BZERO*BZERO

        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
	IF(IGERR > 0)GOTO 1678
        WRITE(*,'(9E18.8)')BZERO,RE,DIMAG(ALFA),DBLE(ALFA)
        DRES=DBLE(ALFA)
        IF(ICOUNT > NABAND)GOTO 1678
        ICOUNT=ICOUNT+1
        IF(DABS(DRES) > TOLCGS)GOTO 11107

	IF(ISIM == 1)THEN
        WRITE(37,'(9E18.8)')BZERO*PDIMIT,RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA),XPOS
	ELSE
        WRITE(37,'(9E18.8)')BZERO*PDIMIT,RE,DIMAG(ALFA),DBLE(ALFA), &
     & 1.E6*DBLE(OMEGA)/Re, DBLE(OMEGA)/DIMAG(ALFA)
	end if

	RETURN

1678    CONTINUE
	IERR=-1
	WRITE(*,*)' '
	PRINT*,' no convergence...'
	IF(IGERR > 0)THEN
	PRINT*,' Possible grid resolution problems (Rayleigh failure).'
	PRINT*,' or may be a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	ELSE IF(DABS(DBLE(ALFA)) < 1.E-12)THEN
	PRINT*,' I suspect that tolerance criteria is set too high!'
	WRITE(*,11)DBLE(ALFA)
11	format(2x,'alpha_i has converged to ',f18.15,' ! ')
	ELSE
	PRINT*,' Possibly a turning point in neutral curve.'
	PRINT*,' Start calcs again at non-convergence point by a change '
	PRINT*,' in Frequency and neutral curve computation strategy.'
	PRINT*,' NB : You will over-write .neutral file - so alter name'
	end if
	WRITE(*,*)' '
	PRINT*,' *** Parameters at non-convergence :'
	WRITE(*,'(2x,A,E15.9,a,E15.9, A,I0)')' Frequency = ', &
     & OMSTRT*wrscale,' Hz; Re = ', RESTRT, ' ; NSTATION : ', INPSTAT
107	format(2x,' frequency = ',g18.8,' Hz; Re = ',g18.8) 
106     FORMAT(9E18.8)
	RETURN
	END SUBROUTINE BRANGE	


	SUBROUTINE NEUTIT(NDIV,IL,IEQT,ITRASH,IGERR)
	    use cmdInput ! Use the input command options
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	PARAMETER (LS=5,KS=MP*LS)
        character REPLY*1,YES*1,NO*1
        COMPLEX*16 ALPLUS,ALMINUS,ALMID
	complex*16 alact,omold
        CHARACTER*80 TITLE
	CHARACTER*58 PSELABL
	COMPLEX*16 XSOL(KE),OMEGA,ALFA
	COMPLEX*16 EIGEN,UMAXI,ABSQ,CUMAXI
	COMPLEX*16 EKE(MP),ENTEMP
	COMPLEX*16 POUT(2,MP)
	COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),ENERGY(KSTR,KCRF)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),DIMSCL(4)
	REAL*8 CRFBETA(KCRF),Medge
        INTEGER JOUTSC(2)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        COMPLEX*16 EURONRM(KSTR,KCRF,4),EUROMAX(4)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
		real*8 sign
        COMMON /SCLLOC/Uedge,Medge,RLOCAL,Tedge,Delzer
        COMMON /MUADZAM/EURALFA     
        COMMON /MUSARAT/EURONRM,EUROMAX
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /PICTR/TITLE
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /FRVARI/OMEGA,RE
	COMMON /ROOTS/ALFA
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /SCALI/G,ULENTH
	COMMON /EISPC/ALFAZO
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /KERES/ENERGY
	COMMON /HISTRY/PRV
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB,IWSQ, &
     & IWMAX
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
        COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /PHOPTIM/FOME,RMR0
	COMMON /NCURVES/INEUTRL
	COMMON /SXXXXX/ISIM,Ibuoy
        COMMON /FILINF/IERFIL
        COMMON /PISAVE/PI
	IERFIL=0

        yes='y'
        no='n'
! NOTE the below stuff option does not have a path...
! as INEUTRL=9 in pseread.nam, will never bring it in here! note!
! 12/feb/2015   shahid
	IF(INEUTRL == 9)THEN     
	write(*,'(a)',advance='no')  &
     & ' Change Reynolds number ? yes (y) no (n) :'
        READ(*,'(A1)')REPLY
        IF(reply == yes)THEN
	PRINT*,' Current Re is : ', Re
	write(*,'(a)',advance='no')' Input New Re : '
	READ(*,*)RE
	END IF
	END IF


	NB=NDIV-1
	REQ=RESSCL
        sqfAct=dsqrt(2.D0)
	WRITE(ITV,*)

        ICROSS=1
        IFQ=1

	EIGEN=QZLIN(IFQ)

	IX=IFQ-MODE-1
	IY=ICROSS-KMODE-1

	OMEGA=DCMPLX(OMRANGE(IFQ),0.D0)

	BETA=CRFBETA(ICROSS)
        BZERO=BETA
        BESQ=BZERO*BZERO

	IF(ITSPRES >= 1)GOTO 807
	IF(NOUTPUT == 1 .OR. NOUTPUT == 3)THEN
	WRITE(ITV,77)DBLE(OMEGA/RE)*1.E6
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,88)BZERO*PZERO
	ELSE
	WRITE(ITV,787)DBLE(OMEGA/RE)*1.E6,BZERO*PZERO
	END IF
77	FORMAT(1X,'Computing for Frequency Parameter F*1.E6 = ',E12.6)
88	FORMAT(1X,'Computing for Crossflow Wavenumber = ',E12.6)
787	FORMAT(1X,'Computing for F*1.E6 = ',E12.6,   &
     & ' & Beta =',E12.6)

807	CONTINUE

	CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
	
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

        IF(IGERR == 1)THEN
	PRINT*,' no convergence! '
	RETURN
	end if

        CALL FILHAND(999,LABEL(1:ICHART),'Q','.neutral',PSELABL)
        OPEN(37,FILE=PSELABL,STATUS='UNKNOWN')

	WRITE(*,*)
	WRITE(*,*)' Ok, got a start point to initiate calcs - '
! ok now search for neutral points...
	WRITE(*,*)

	PRINT*,'Select from following ::: '
	PRINT*,' ** Neutral points : '
        PRINT*,'     Iterate on Re, for fixed Omega,Beta    (1) or'
        PRINT*,'     Iterate on Omega, for Re Range         (2) or'
        PRINT*,'     Iterate on Re, for Omega Range         (3) or'
        PRINT*,'     Iterate on Beta, for Re Range          (4) or'
        PRINT*,'     Iterate on Omega only at Fixed Re,Beta (5) or'
	PRINT*,' ** Scan ranges : '
        PRINT*,'     Scan Omega range at Fixed Beta,Re      (6) or'
        PRINT*,'     Scan Beta range at Fixed Omega,Re      (7) or'
		PRINT*,'     Scan Beta Omega range at Fixed Re      (8) '
        
		!READ*,ICHOICE
		ICHOICE = option
		write(*,*) "Input option is: ", ICHOICE
		
!        WRITE(*,'(a)',advance='no') &
!     & 'Input Tolerance criteria in alpha_i (1.E-7 ok!) : '
	IF(ICHOICE < 6)THEN
	WRITE(*,*) 
	IF(ICHOICE < 5)THEN
	IF(ISIM == 1)THEN
        WRITE(*,*)'*** Mean profile assumption : Non-Similar ***'
	ELSE
        WRITE(*,*)'*** Mean profile assumption : Self-Similar ***'
	END IF
	WRITE(*,*) 
	END IF
	
	IF(ICHOICE == 4)THEN
		IF(ISIM == 0)THEN
	WRITE(37,1012)
1012	format(6x,' Beta',13x,'Re ',14x,' Alpha_r',10x,' Alpha_i', &
     & 9x,'1.E6xOmega/Re',9x,'Phase C')
		ELSE
	WRITE(37,'(6x,A,13x,A,14x,A,10x,A,9x,A,9x,A,9X,A)')' Beta', &
     & 'Re ',' Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C','Xpos'
		END IF
	ELSE
		IF(ISIM == 0)THEN
	WRITE(37,112)
		ELSE
		IF(ICHOICE == 2)THEN
	WRITE(37,'(6x,A,13x,A,14x,A,10x,A,9x,A,9x,A,9X,A,14X,A)')&
     & ' Omega', 'Re ',' Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C','Xpos','Freq_dim'
     		ELSE
	WRITE(37,'(6x,A,13x,A,14x,A,10x,A,9x,A,9x,A,9X,A)')' Omega', &
     & 'Re ',' Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C','Xpos'
		END IF
		END IF
	END IF
	
        write(*,'(a)',advance='no') &
     & 'Input Tolerance criteria in alpha_i (1.E-7 ok!) : ' 
        READ(*,*)TOLCGS
	ELSE
	TOLCGS=1.d-9
	END IF
!	WRITE(*,'(a)',advance='no')  &
!     & 'Input fraction to perturb iterant (0.001 ok!) : '
!        READ(*,*)Perturb
        PERTURB=0.0005

	NABAND=9

        IF(ICHOICE == 1)THEN	
	
	IF(ISIM == 1)THEN
        WRITE(*,*)
	WRITE(*,*)' *** Warning :: This option does NOT navigate'
	WRITE(*,*)'     MEAN-Profile data. If MEAN-Profile '
	WRITE(*,*)'     data is non-similar, use selection 2 or 4 above.'
        WRITE(*,*)
	WRITE(*,*)' * Assuming you are aware of this...proceeding ! '
	END IF

	WRITE(*,*)
	IF(ITSPRES > 0)THEN
	WRITE(*,112)
	END IF
	REOLDY=RE
112	format(6x,' Omega',13x,'Re ',14x,' Alpha_r',10x,' Alpha_i', &
     & 9x,'1.E6xOmega/Re',9x,'Phase C')
	ICOUNT=0
11007   CONTINUE
        ALMID=ALFA
        REZERO=RE
        OMSTEP=PERTURB*RE
        RE=REZERO+OMSTEP
        EIGEN=ALFA

        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALPLUS=ALFA

        RE=REZERO-OMSTEP
        EIGEN=ALMID
	CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALMINUS=ALFA

        DOMBOT=0.5D0*DBLE(ALPLUS-ALMINUS)/OMSTEP
        DNEW=-DBLE(ALMID)/DOMBOT
        RE=REZERO+DNEW

        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA)

	ICOUNT=ICOUNT+1
	IF(ICOUNT > NABAND .OR. IGERR > 0)THEN
	CALL ERRINFO(ICOUNT,IGERR,NABAND,ICHOICE)
	STOP
	ELSE

        DRES=DBLE(ALFA)
        IF(DABS(DRES) > TOLCGS)GOTO 11007
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA)
	END IF
     	WRITE(*,*)' '
     	WRITE(*,'(2x,A)')' Neutral point found :: '
	WRITE(*,'(2x,A,G15.6)')' Starting Re was = ', REOLDY
	WRITE(*,112)
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA)

! choice 2.. now...
        ELSE IF(ICHOICE == 2)THEN
        WRITE(*,'(A,G15.7)')'Present Re is :: ',RE
	IF(ISIM == 0)THEN
	write(*,'(a)',advance='no')  &
     & 'INPUT RE final and no. of Re_increments : '
        READ(*,*)FFinal,NFSTEPS
	ELSE
	write(*,'(a)',advance='no')  &
     & 'INPUT no. of boundary-layer profiles to compute for : '
        READ(*,*)NFSTEPS
	FFINAL=1000000.d0
	END IF
        FREQFIN=FFinal
	OM0=DBLE(OMEGA)
	NABAND=9
	
	IF(NFSTEPS < 2)THEN
	NFSTEPS=2
	END IF

        DFREQ=(FREQFIN-RE)/float(NFSTEPS-1)

	INP=NSTART

        DO IJK=1,NFSTEPS
	IF(ITSPRES == 0)THEN
        WRITE(*,*)'********************************'
	END IF
        WRITE(*,434)IJK,NFSTEPS,RE
434     FORMAT(2X,'Doing for ',I0,' of ',I0,' Re = ',E12.6)
	IF(ITSPRES == 0)THEN
        WRITE(*,*)
	END IF

	OM0=DBLE(OMEGA)
	RE0=RE
	CALL RERANGE(INP,IL,NB,NDIV,IEQT,NABAND,OMZERO,TOLCGS, &
     & PERTURB,DFREQ,IERR,ITRIP)

        IF(IERR == -1)GOTO 1678

	IF(ISIM == 1)THEN
	CALL PROFRD(NDIV,INP,1)
	IF(IERFIL == 1)GOTO 1678
	RE=RLOCAL
	INP=INP+1
	ELSE
        RE=RE+DFREQ
	END IF

        ALFA=ALFA+0.0001
	END DO

1678    CONTINUE

        ELSE IF(ICHOICE == 3)THEN
	
	IF(ISIM == 1)THEN
        WRITE(*,*)
	WRITE(*,*)' *** Warning :: This option does NOT navigate'
	WRITE(*,*)'     MEAN-Profile data. If MEAN-Profile '
	WRITE(*,*)'     data is non-similar, use selection 2 or 4 above.'
        WRITE(*,*)
	WRITE(*,*)' * Assuming you are aware of this...proceeding ! '
        WRITE(*,*)
	END IF
	
	WRITE(*,555)DBLE(OMEGA),(DBLE(OMEGA)/Re)*1.E6
555	format(2x,'Present Omega is : ',g13.5, &
     &  ' or 1.E6*Omega/Re = ',g14.6)
	write(*,'(a)',advance='no') &
     & 'INPUT OM_final and No. of Steps/Increments : '
        READ(*,*)FFinal,NFSTEPS
        FREQFIN=FFinal
	NABAND=9
	
	IF(NFSTEPS < 2)THEN
	NFSTEPS=2
	END IF

        DFREQ=(FREQFIN-DBLE(OMEGA))/float(NFSTEPS-1)

        DO IJK=1,NFSTEPS
	IF(ITSPRES == 0)THEN
        WRITE(*,*)'********************************'
	END IF
        WRITE(*,4347)IJK,NFSTEPS,DBLE(OMEGA)
4347     FORMAT(2X,'Doing for ',I0,' of ',I0,' Omega = ',E12.6)
	IF(ITSPRES == 0)THEN
        WRITE(*,*)
	END IF

	CALL OMBIT(IL,NB,NDIV,IEQT,NABAND,OMZERO,TOLCGS, &
     & PERTURB,DFREQ,IERR)
        ALFA=ALFA+0.0001
        OMEGA=OMEGA+DFREQ
	IF(IERR == -1)THEN
	PRINT*,' no convergence in Newton iterations...'
	end if
        IF(IERR == -1)GOTO 1686
	end do

1686    CONTINUE

        ELSE IF(ICHOICE == 4)THEN

        WRITE(*,'(A,G15.7)')'Present Re is :: ',RE
	IF(ISIM == 0)THEN
	write(*,'(a)',advance='no')  &
     & 'INPUT RE final and no. of Re_increments : '
        READ(*,*)FFinal,NFSTEPS
	ELSE
	write(*,'(a)',advance='no')  &
     & 'INPUT no. of boundary-layer profiles to compute for : '
        READ(*,*)NFSTEPS
	FFINAL=1000000.d0
	END IF
        FREQFIN=FFinal
	OM0=BZERO
	NABAND=9
	
	IF(NFSTEPS < 2)THEN
	NFSTEPS=2
	END IF

        DFREQ=(FREQFIN-RE)/float(NFSTEPS-1)

	INP=NSTART

        DO IJK=1,NFSTEPS
	IF(ITSPRES == 0)THEN
        WRITE(*,*)'********************************'
	END IF
        WRITE(*,434)IJK,NFSTEPS,RE
	IF(ITSPRES == 0)THEN
        WRITE(*,*)
	END IF

	OM0=BZERO
	RE0=RE
	CALL BRANGE(INP,IL,NB,NDIV,IEQT,NABAND,OMZERO,TOLCGS, &
     & PERTURB,DFREQ,IERR,ITRIP)

        IF(IERR == -1)GOTO 16781

	IF(ISIM == 1)THEN
	CALL PROFRD(NDIV,INP,1)
	IF(IERFIL == 1)GOTO 16781
	RE=RLOCAL
	INP=INP+1
	print*,'******RE=',Re
	ELSE
        RE=RE+DFREQ
	END IF

        ALFA=ALFA+0.0001

	END DO
16781   CONTINUE	
	
	

        ELSE IF(ICHOICE == 5)THEN

	OMOLDY=DBLE(OMEGA)
	ICOUNT=0
	WRITE(*,*)
	IF(ItSpres > 0)THEN
	WRITE(*,112)
	END IF

10007   CONTINUE
        ALMID=ALFA
        OMZERO=DBLE(OMEGA)
        OMSTEP=PERTURB*OMZERO
        OMEGA=DCMPLX(OMZERO+OMSTEP,0.D0)
        EIGEN=ALFA

        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALPLUS=ALFA

        OMEGA=DCMPLX(OMZERO-OMSTEP,0.D0)
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        ALMINUS=ALFA
        DOMBOT=0.5D0*DBLE(ALPLUS-ALMINUS)/OMSTEP
        DNEW=-DBLE(ALMID)/DOMBOT
        OMEGA=DCMPLX(OMZERO+DNEW,0.D0)

        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA)

	ICOUNT=ICOUNT+1
	IF(ICOUNT > NABAND .OR. IGERR > 0)THEN
	CALL ERRINFO(ICOUNT,IGERR,NABAND,-1)
	STOP
	ELSE
        DRES=DBLE(ALFA)
        IF(DABS(DRES) > TOLCGS)GOTO 10007
	END IF

        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA)
     	WRITE(*,*)' '
     	WRITE(*,'(2x,A)')' Neutral point found :: '
	WRITE(*,'(2x,A,G15.6)')' Starting Omega was = ', OMOLDY
	WRITE(*,112)
        WRITE(*,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA)
     
        ELSE IF(ICHOICE == 6)THEN
	
     	sconst=2.d0*PI*Delzer
	
	WRITE(*,555)DBLE(OMEGA),(DBLE(OMEGA)/Re)*1.E6
	write(*,'(a)',advance='no') &
     & 'INPUT OM_final and No. of Steps/Increments : '
        !READ(*,*)FFinal,NFSTEPS
		
		FFinal  = finalValue1 ! Use input from calling
		NFSTEPS = numSteps1
		
		write(*,*)
		write(*,*) "Input final value is:    ", FFinal
		write(*,*) "Input number of step is: ", NFSTEPS
		
		
        FREQFIN=FFinal
	NABAND=9

        DFREQ=(FREQFIN-DBLE(OMEGA))/float(NFSTEPS-1)
	
     
	WRITE(37,'(6x,A,13x,A,14x,A,10x,A,9x,A,7x,A,8X,A, 7x,a)')&
     & ' Omega', 'Re ',' Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C',' Cg_group_Vel','Freq_dim(Hz)'

        DO IJK=1,NFSTEPS
        WRITE(*,*)'********************************'
        WRITE(*,4347)IJK,NFSTEPS,DBLE(OMEGA)
        WRITE(*,*)
        EIGEN=ALFA
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

        IF(IGERR == 1)THEN
	PRINT*,' no convergence! Stopping computations '
	RETURN
	end if

        EIGEN=ALFA

! domega/dalfa calc....simple at prsent...
	omold=OMEGA
	ALMID=ALFA

	OMEGA=omold+DFREQ*PERTURB
        EIGEN=ALFA
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
	alplus=ALFA
	OMEGA=omold-DFREQ*PERTURB
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

	group=DFREQ*PERTURB*2.d0/DIMAG(alplus-ALFA)
!

	OMEGA=omold
	ALFA=ALMID
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA),group, &
     & Uedge*DBLE(OMEGA)/sconst
     
        OMEGA=OMEGA+DFREQ
	end do

		
	ELSE IF(ICHOICE == 7)THEN
606	format(2x,'Present Beta is : ',g13.5, &
     &  ' and 1.E6*Omega/Re = ',g14.6)
4307    FORMAT(2X,'Doing for ',I0,' of ',I0,' Beta = ',g12.5)
        WRITE(*,*)
! now doing the beta choice...	
	BETA=CRFBETA(ICROSS)
        BZERO=BETA
        BESQ=BZERO*BZERO

	WRITE(*,606)BETA,(DBLE(OMEGA)/Re)*1.E6
	write(*,'(a)',advance='no') &
     & 'INPUT Beta_final and No. of Steps/Increments : '
        !READ(*,*)FFinal,NFSTEPS
		FFinal  = finalValue1 ! Use input from calling
		NFSTEPS = numSteps1
		
		write(*,*)
		write(*,*) "Input final value is:    ", FFinal
		write(*,*) "Input number of step is: ", NFSTEPS
		
        FREQFIN=FFinal
	NABAND=9
	
	IF(NFSTEPS < 2)THEN
	NFSTEPS=2
	END IF

        DFREQ=(FREQFIN-BETA)/float(NFSTEPS-1)
	
	WRITE(37,'(7x,A,13x,A,14x,A,10x,A,9x,A,7x,A,9x,a,7x,a,13x,a)')&
     & 'Omega','Re','  Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C',' Cg_group_Vel','Beta','Beta_dim/m'
	

        DO IJK=1,NFSTEPS
        WRITE(*,*)'********************************'
        WRITE(*,4307)IJK,NFSTEPS,BETA
        WRITE(*,*)
        EIGEN=ALFA
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

        IF(IGERR == 1)THEN
	PRINT*,' no convergence! Stopping computations '
	RETURN
	end if

        EIGEN=ALFA

! domega/dalfa calc....simple at prsent...
	omold=BETA
	ALMID=ALFA

	BETA=omold+DFREQ*PERTURB
        EIGEN=ALFA
	
        BZERO=BETA
        BESQ=BZERO*BZERO
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
	alplus=ALFA
	BETA=omold-DFREQ*PERTURB
	BZERO=BETA
        BESQ=BZERO*BZERO
	
        EIGEN=ALMID
        CALL SETPAR(NDIV,IL)
!	CALL SETRUVT(NDIV,IL)
        CALL SETGLB(NDIV,IL)
        CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

	group=DFREQ*PERTURB*2.d0/DIMAG(alplus-ALFA)
!
	BETA=omold
	ALFA=ALMID
        WRITE(37,106)DBLE(OMEGA),RE,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA),group,BETA, &
     & BETA/Delzer
        BETA=BETA+DFREQ
	end do

	ELSE
! Chince 8, omega and beta range, modified from 6, Ganlin
	sconst=2.d0*PI*Delzer
		
	BETA=CRFBETA(ICROSS)
        BZERO=BETA
        BESQ=BZERO*BZERO
		
	WRITE(*,555)DBLE(OMEGA),(DBLE(OMEGA)/Re)*1.E6
	WRITE(*,606)BETA,(DBLE(OMEGA)/Re)*1.E6
		
	write(*,'(a)',advance='no') &
     & 'INPUT OM_final and No. of Steps/Increments : '
        !READ(*,*)FFinal,NFSTEPS
		FFinal  = finalValue1 ! Use input from calling
		NFSTEPS = numSteps1
		
		write(*,*)
		write(*,*) "Input final value for frequency is:    ", FFinal
		write(*,*) "Input number of step for frequency is: ", NFSTEPS
		
        FREQFIN=FFinal
	
	IF(NFSTEPS < 2)THEN
	NFSTEPS=2
	END IF
		
	write(*,'(a)',advance='no') &
     & 'INPUT Beta_final and No. of Steps/Increments : '
	    !READ(*,*)BFinal,NBSTEPS
	    BFinal  = finalValue2 ! Use input from calling
		NBSTEPS = numSteps2
	
		write(*,*)
		write(*,*) "Input final value for wavenumber is:    ", BFinal
		write(*,*) "Input number of step for wavenumber is: ", NBSTEPS
	
        BETAFIN=BFinal	
	
	IF(NBSTEPS < 2)THEN
	NBSTEPS=2
	END IF
		
	NABAND=9

        DFREQ=(FREQFIN-DBLE(OMEGA))/float(NFSTEPS-1)
	DBETA=(BETAFIN-DBLE(BETA)) /float(NBSTEPS-1)
	

	WRITE(37,'(7x,A,13x,A,14x,A,10x,A,9x,A,7x,A,9x,a,7x,a)')&
     & 'Omega','Beta','Alpha_r',' Alpha_i', &
     & '1.E6xOmega/Re','Phase C','Freq_dim(Hz)','Beta_dim/m'

		
! Scan beta and omega
	sign = 1.0		
	DO IJK_B = 1,NBSTEPS     ! beta loop
! Initialize omega
	DO IJK_F=1,NFSTEPS	 ! omega loop
	WRITE(*,*)'********************************'
	WRITE(*,4307)IJK_B,NBSTEPS,BETA
	WRITE(*,4347)IJK_F,NFSTEPS,DBLE(OMEGA)
	
	! For the given beta and omega, compute
	EIGEN=ALFA
	CALL SETPAR(NDIV,IL)
	CALL SETGLB(NDIV,IL)
	CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
    
	IF(IGERR == 1) THEN
	PRINT*,' no convergence! Stopping computations '
	RETURN
	END IF
    
	EIGEN=ALFA
    
	omold=OMEGA
	ALMID=ALFA
	OMEGA=omold
	ALFA=ALMID
	WRITE(37,106)DBLE(OMEGA),BETA,DIMAG(ALFA),DBLE(ALFA), &
     & DBLE(OMEGA)*1.E6/RE,DBLE(OMEGA)/DIMAG(ALFA), &
     & Uedge*DBLE(OMEGA)/sconst,BETA/Delzer
				
! Update omega
	OMEGA=OMEGA+DFREQ*sign
	END DO            ! end of omega loop
			
			
! move back a step in omega
	sign = -1.0 * sign
	OMEGA=OMEGA+DFREQ*sign  
			
			
! beta computation
	EIGEN=ALFA
	CALL SETPAR(NDIV,IL)
	CALL SETGLB(NDIV,IL)
	CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)

	IF(IGERR == 1)THEN
	PRINT*,' no convergence! Stopping computations '
	RETURN
	end if

	EIGEN=ALFA

	omold=BETA
	ALMID=ALFA
	BETA=omold
	ALFA=ALMID
	
! Update beta
	BETA=BETA+DBETA
			
	END DO                      ! end of beta loop
!-------------------End of choices ------------------------------
			
 
	END IF
	
106     FORMAT(16E18.8)

	RETURN

	END


	
	
	
	
	SUBROUTINE OSEGLBX(I3DNEQ,IGERR,IGLBX)
        use param_info, only : MODE, KMODE, NPOST
	use param_FILEADD
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	COMPLEX*16 OMEGA,ALNEW,ALPREV,ALERROR
	REAL*8 MEDGE,MO,DIMSCL(4),RES(KSTR,KCRF)
	COMPLEX*16 QZLIN(KSTR),ALFAZO(KSTR,KCRF,NPOST)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 ENMETH(KSTR,KCRF,NPOST),XCVAL(NPOST),RDVAL(NPOST)
	INTEGER NFFLAG(KSTR,KCRF),ISWITCH(KSTR,KCRF)
        INTEGER IPREMAT(KSTR,KCRF),MATURE(KSTR,KCRF)
	INTEGER IERPOST(KSTR,KCRF),IERSWCH(KSTR,KCRF)
	INTEGER IERALFA(KSTR,KCRF)
	REAL*8 WAVBETA(KCRF),SDVAL(NPOST)
        REAL*8 PHIN(NPOST)
	COMMON /HOHOHO/IMULTF2, IROBUST
        COMMON /RELVECT/PHIN
        COMMON /RELATIV/PHINVSC
	COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /EISPC/ALFAZO
	COMMON /PICOSV/SWPSCL,CLINE
	COMMON /QCNTRL/ND
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /FRVARI/OMEGA,RE
        COMMON /SPFLG/ISPC,IOUT,IHOW,IPAR
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
	COMMON /POSSSI/IPROF
	COMMON /STEPNG/XNEW
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /PRFIL/DXSTEP,DXINV
	COMMON /AMPLIT/NFACT,NFFLAG
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/F
	COMMON /FILINF/IERFIL
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ISAVEY/ISWITCH
	COMMON /PISAVE/PI
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	IFIRST=0
	IERFIL=0
	NFLAG=0
	IALC=1

	IOFF=0
!	ioff is marker for whether Any nfActors Are non-zero... not
!	reAlly being used At present!!!! should use this As indICAtot
!	to user that no nfActors Are nonzero

	NDIV=ND
	CALL ETASET(NDIV)

	CALL INITL
	IF(IERFIL == 1)RETURN
	CALL PROFRD(NDIV,NSTART,1)
	IF(IERFIL == 1)RETURN

	DO ICR=1,NCRBET
	DO IJK=1,NFREQ
	NFFLAG(IJK,ICR)=0
	ISWITCH(IJK,ICR)=0
	IERSWCH(IJK,ICR)=0
	IPREMAT(IJK,ICR)=0
	ENMETH(IJK,ICR,1)=0.D0
	IERALFA(IJK,ICR)=1
	end do
	end do

	RELOC=RLOCAL
	REPRV=RLOCAL
	REXPRV=REX
        XACT=SPOS
	XPREV=XPOS
	XSCALE=XNEW

	IF(ISCREEN == 0)THEN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.dmp',PSELABL)
	OPEN(ITV,FILE=PSELABL,STATUS='UNKNOWN')
	ELSE
	END IF

        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,110)IPROF,XPOS/SWPSCL,RELOC
        WRITE(2,110)IPROF,XPOS/SWPSCL,RELOC
        WRITE(ITV,440)
	IF(NOUTPUT == 1)THEN
        WRITE(ITV,311)0.001D0*FRENCY(1),CRFBETA(1)
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,411)0.001D0*FRENCY(1)
311     FORMAT(1X,'Frequency =',E12.6,'KHz ;', &
     & 1X,'Crossflow Wave No. =',E12.6)
411     FORMAT(1X,'Frequency =',E12.6,'KHz')
	ELSE IF(NOUTPUT == 3)THEN
	WRITE(ITV,511)CRFBETA(1)
511     FORMAT(1X,'Crossflow Wave No. =',E12.6)
	ELSE
	END IF
	CALL FLUSH(ITV)

	XCVAL(1)=XPOS
	RDVAL(1)=RELOC
	SDVAL(1)=SPOS
        PHIN(1)=PHINVSC

220     FORMAT(5x,'  RDelta = ',E14.7)
110     FORMAT(1X,'OSE begins at station ',I0,' : x/c = ',F11.6, &
     & 1X,'; RDelta = ',E12.6)
440     FORMAT(1X,69('-'))

	IONEQ=9
	NB=NDIV-1
	IEQT=NB*IONEQ
        CALL AP3D2(NDIV,IEQT,IONEQ)
	CALL PARLIN(NDIV,IONEQ,IEQT,0,IGERR)

        IF(IGERR == 1)RETURN
        IF(IGLOB == 1 .AND. IGLBX == 0)RETURN

	NFINAL=NSTART+(NINTR+1)*1

	IF(NSTART+NINTR+1 > NFINAL)RETURN

	IHOW=IOUT
        CALL MISPRF(1,NINTR)
        IF(IERFIL == 1)RETURN
        INP=NSTART+NINTR+1
	
	NFINAL=NFINAL+IROBUST*(NINTR+1)
	

	DO 10 IN=INP,NFINAL,NINTR+1
	CALL PROFRD(NDIV,IN,1)
	IF(IERFIL == 1)GOTO 7671
	RE=RLOCAL
211     FORMAT(3X,'OSE at station ',I0,' : x/c = ',F12.6, &
     & 1X,': RDelta = ',E12.6)

	IF(ITSPRES == 0)THEN
        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,440)
        WRITE(ITV,211)IPROF,XPOS/SWPSCL,RLOCAL
        WRITE(ITV,440)
	CALL FLUSH(ITV)
	ELSE
	END IF

	DO ICR=1,1
	CRFBETA(ICR)=Chord*WAVBETA(ICR)*DELZER/Chord
	DO IFREQ=1,1
	OMRANGE(IFREQ)=2.D0*PI*FRENCY(IFREQ)*DELZER/UEDGE
	CALL ORRQZX(NDIV,IONEQ,IEQT,0,IGERR)
	end do
	end do


279     FORMAT(2X,2F10.5,2x,f12.7,2x,f15.8,3X,F12.6)

	ihow=ihow+1

	CALL MISPRF(1,NINTR)
        IF(IERFIL == 1)GOTO 7671

10      CONTINUE
7671    CONTINUE
        CLOSE(9)

	RETURN
	END
	
	SUBROUTINE QZCALLSET(NDIV,IL)
	COMMON /ALEENAQZ/IQZHI8
	IF(IQZHI8 == 0 )THEN
!	PRINT*,' enter old  (0) or new (1)'
!	READ*,ikop
!	IF(ikop == 0)THEN
        CALL SETGLB(NDIV,IL)
!		ELSE
!	PRINT*,' calling 4-2th exact..'
! THIS needs checking again 4th-order..! nov. 2018 
!         CALL SETGLB_QZ4(NDIV,IL)
!	 END IF
	ELSE
! THIS appears to give good results! nov. 2018 (8th-order accurate)
	WRITE(*,'(2x,a)')'Global QZ call with 8th-order FDs ...'
        CALL SETGLB_QZ(NDIV,IL)
        END IF
	RETURN
	END SUBROUTINE QZCALLSET 
	
        SUBROUTINE PARLIN(NDIV,IL,IEQT,ITRASH,IGERR)
	use param_NUMBERCRUNCH
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	PARAMETER (LS=5,KS=MP*LS)
        CHARACTER*80 TITLE
	CHARACTER*58 PSELABL
	COMPLEX*16 XSOL(KE),OMEGA,ALFA
	COMPLEX*16 EIGEN,UMAXI,ABSQ,CUMAXI
	COMPLEX*16 EKE(MP),ENTEMP
	COMPLEX*16 POUT(2,MP)
	COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),ENERGY(KSTR,KCRF)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
        COMPLEX*16 QZLIN(KSTR)
	REAL*8 MZERO,DIMSCL(4)
        INTEGER JOUTSC(2)
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
        COMPLEX*16 EURONRM(KSTR,KCRF,4),EUROMAX(4)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
	COMPLEX*16 DCRHS(KE),RHS(KE)
	COMMON /SHAPE/RHS,DCRHS
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
        COMMON /MUADZAM/EURALFA     
        COMMON /MUSARAT/EURONRM,EUROMAX
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
        COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /PICTR/TITLE
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /FRVARI/OMEGA,RE
	COMMON /ROOTS/ALFA
        COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /SCALI/G,ULENTH
	COMMON /EISPC/ALFAZO
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /KERES/ENERGY
	COMMON /HISTRY/PRV
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB,IWSQ, &
     & IWMAX
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
        COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /DIRFORM/IFORMAT,NONORTH
	COMMON /YBUMPY/tdisp,tdispY,tdispYY     
	COMMON /KEMALIK/WEUE,UEBL

	IFORMAT=1

	NB=NDIV-1
	REQ=RESSCL
        sqfAct=dsqrt(2.D0)
	WRITE(ITV,*)
	IF(IGLOB /= 1)GOTO 123
	WRITE(ITV,*)
	WRITE(ITV,*)'             Global Method Called !!! '
	WRITE(ITV,*)
	IF(IQZFULL == 1)THEN
	OPEN(19,FILE='PSESPEC.FULL',STATUS='UNKNOWN')
	ELSE
	END IF
	OPEN(15,FILE='PSESPEC.DAT',STATUS='UNKNOWN')
	IA=1
	IB=0
	DO 101 I=1,NFREQ
	ICROSS=1
	OMEGA=DCMPLX(OMRANGE(I),0.D0)	
	BETA=CRFBETA(ICROSS)
	BZERO=BETA
	BESQ=BZERO*BZERO

	IF(ICURVI == 0 .AND. IVARIP == 1)THEN
        CALL SETRUVT(NDIV,IL)
!	CALL SETPAR(NDIV,IL)
	ELSE IF(ICURVI == 1)THEN
!	CALL CURGRET(NDIV,IL)
	CALL SETPAR(NDIV,IL)
	ELSE 
	CALL SETPAR(NDIV,IL)
	END IF

	CALL QZCALLSET(NDIV,IL)
	
	IF(IQZFULL == 1)THEN
	WRITE(19,'(2x,A,E16.9,A,E16.9,A,E16.9)') &
     & 'Computing for F = ',DBLE(OMEGA)*1.E6/RE, &
     & ' : Omega = ',DBLE(OMEGA),' : BETA = ',BZERO*REQ
	ELSE
	END IF
	
	WRITE(15,'(2x,A,E16.9,A,E16.9,A,E16.9)') &
     & 'Computing for F = ',DBLE(OMEGA)*1.E6/RE, &
     & ' : Omega = ',DBLE(OMEGA),' : BETA = ',BZERO*REQ
	WRITE(ITV,'(2x,A,E16.9,A,E16.9,A,E16.9)') &
     & 'Computing for F = ',DBLE(OMEGA)*1.E6/RE, &
     & ' : Omega = ',DBLE(OMEGA),' : BETA = ',BZERO*REQ
   
     	IF(abs(WEUE) > 1.E-28)THEN
	WRITE(ITV,'(2x,A,E16.9,A,E16.9,A,E16.9)') &
     & '   Malik_span F = ',DBLE(OMEGA)*1.E6/(RE*WEUE*WEUE), &
     & ' : Omega = ',DBLE(OMEGA),' : BETA = ',BZERO*REQ
     	END IF
	
	WRITE(ITV,*)
	WRITE(ITV,*)'   Full Eigenvalue Spectrum in PSESPEC.DAT '
	WRITE(ITV,*)'          Most Probable Roots are ... '
	CALL FLUSH(ITV)
	CALL QZALG(IA,IB,IEQT,I,IL,0)
	WRITE(ITV,*)
	WRITE(ITV,*)
	CALL FLUSH(ITV)
101	CONTINUE

	RETURN
123	CONTINUE

	IF(IEIG == 1)THEN
	CALL FILHAND(999,LABEL(1:ICHART),'Q','.local',PSELABL)
	OPEN(11,FILE=PSELABL,STATUS='UNKNOWN')
	ELSE
	END IF
	
	DO IEQ=1,LS
	DO M=-MODE,MODE
	DO ML=-KMODE,KMODE
	PRV(IEQ,1:NDIV,M,ML)=0.D0
	end do
	end do
	end do

        DO 20 ICROSS=1,NCRBET
        DO 20 IFQ=1,NFREQ
	IF(ICROSS == 1)THEN
	EIGEN=QZLIN(IFQ)
	ELSE
	EIGEN=ALFAZO(IFQ,ICROSS-1,1)
!	EIGEN=QZLIN(IFQ)
	END IF

	IX=IFQ-MODE-1
	IY=ICROSS-KMODE-1

	OMEGA=DCMPLX(OMRANGE(IFQ),0.D0)

	BETA=CRFBETA(ICROSS)
        BZERO=BETA
        BESQ=BZERO*BZERO

	IF(ITSPRES >= 1)GOTO 807
	IF(NOUTPUT == 1 .OR. NOUTPUT == 3)THEN
	WRITE(ITV,77)DBLE(OMEGA/RE)*1.E6
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,88)BZERO*PZERO
	ELSE
	WRITE(ITV,787)DBLE(OMEGA/RE)*1.E6,BZERO*PZERO
	END IF
77	FORMAT(1X,'Computing for Frequency Parameter F*1.E6 = ',E12.6)
88	FORMAT(1X,'Computing for Crossflow Wavenumber = ',E12.6)
787	FORMAT(1X,'Computing for F*1.E6 = ',E12.6, &
     & ' & Beta =',E12.6)

807	CONTINUE

        IF(ICURVI == 0 .AND. IVARIP == 1)THEN
        CALL SETRUVT(NDIV,IL)
        ELSE IF(ICURVI == 1)THEN
!        CALL CURGRET(NDIV,IL)
        CALL SETPAR(NDIV,IL)
        ELSE
        CALL SETPAR(NDIV,IL)
        END IF

        CALL SETGLB(NDIV,IL)
	CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        IF(ITSPRES >= 1)GOTO 507
	WRITE(ITV,*)
507	CONTINUE
	IF(IGERR == 1)RETURN
        IF(IGERR == 2)THEN
	WRITE(ITV,*)
        WRITE(ITV,*)'Initial conditions have not converged to requested'
        WRITE(ITV,*)'accuracy - but continuing analysis '
        ELSE
        END IF
	
	OPEN(63,file='PSE3d.multf',status='unknown')
	WRITE(63,'(I9,2E24.14)')NSTART, PZERO, dzero
	WRITE(63,'(10E24.14)')OMEGA,DIMAG(ALFA), &
     & DBLE(ALFA), BZERO
	WRITE(63,'(10E24.14)')XPOS,SPOS,OMEGA,DIMAG(ALFA)*PZERO, &
     & DBLE(ALFA)*PZERO, BZERO
	
	DO KKK=1,IEQT
	ORRRHS(IFQ,ICROSS,KKK)=RHS(KKK)
	ORRDCRHS(IFQ,ICROSS,KKK)=DCRHS(KKK)
	end do

        J=1
        DO I=1,NDIV-1
        PRV(1,I,IX,IY)=XSOL(J)
        PRV(2,I,IX,IY)=XSOL(J+1)
        PRV(3,I,IX,IY)=XSOL(J+3)
        PRV(4,I,IX,IY)=XSOL(J+5)
        PRV(5,I,IX,IY)=XSOL(J+7)
        J=J+IL
	end do

        PRV(1:LS,NDIV,IX,IY)=0.D0

        CALL PDCALC(NDIV,POUT,IX,IY)
	CALL MAKEONE(MASFLX,NDIV,POUT,IX,IY,UMAXI,YMAXI)

	ALFAZO(IFQ,ICROSS,1)=ALFA


	IF(ITRASH == 1 .AND. IEURO == 1)THEN

	DO ICLOS=1,3
	CALL MAKEURO(MASFLX,ICLOS,NDIV,POUT,IX,IY,CUMAXI)
	EURONRM(IFQ,ICROSS,ICLOS)=CUMAXI
	EURALFA(IFQ,ICROSS,1,ICLOS)=ALFA
	end do

        DO I=1,NDIV
        EKE(I)=(CDABS(PRV(3,I,IX,IY))**2+CDABS(PRV(2,I,IX,IY))**2+ &
     & CDABS(PRV(4,I,IX,IY))**2)*ROV(I)
	end do
        EKE(NDIV)=0.0
        CALL SIMPSN(NDIV,EKE,ENTEMP)
        EURONRM(IFQ,ICROSS,4)=ENTEMP
	EURALFA(IFQ,ICROSS,1,4)=ALFA

	ELSE
	END IF

6605	CONTINUE

	IF(IENRGY /= 1)GOTO 2023
!	computing u-integrAl for normAlisAtion condition	
	do I=1,ndiv
	EKE(I)=(IUSQ*PRV(3,I,IX,IY)**2+PRV(2,I,IX,IY)**2+ &
     & IWSQ*PRV(4,I,IX,IY)**2)*ROV(I)
	end do
	EKE(NDIV)=0.0
        CALL SIMPSN(NDIV,EKE,ENTEMP)
        ENERGY(IFQ,ICROSS)=ENTEMP

2023	CONTINUE

	if (ieig /= 1) goto 1963
	WRITE(11,535)IPROF,XPOS,RE*RESCALE
535	FORMAT(' IProf =',i0,' : x/c =',f12.6,' : Rdelta =',f12.2)
	WRITE(11,44)DIMAG(ALFAZO(IFQ,ICROSS,1)*REQ), &
     & DBLE(ALFAZO(IFQ,ICROSS,1)*REQ)
	WRITE(11,43)RE*RESCALE,DBLE(OMEGA),BZERO*REQ
	WRITE(11,46)

	YO=0.D0
!	REQ2=1000.d0*DZERO
	REQ2=1.d0
	DO  I=1,NDIV
	YNETA=ULENTH*YO/(G-YO)
	YNETA=(YNETA+tdisp)/REQ
!	YNETA=(YNETA+tdisp)*REQ2
!        WRITE(11,333)YNETA,POUT(1,I),
!     & (PRV(KOP,I,IX,IY),KOP=2,5),POUT(2,I)
        WRITE(11,333)YNETA,cdabs(POUT(1,I)), &
     & (cdabs(PRV(KOP,I,IX,IY)),KOP=2,5),cdabs(POUT(2,I))

	YO=YO+DY
	end do
	WRITE(11,*)
1963	CONTINUE

20	CONTINUE

	CALL RESLIN(0,1, 0.d0)
	CALL FLUSH(6)

1000	RETURN
333     FORMAT(14E14.6)
46	FORMAT(8X,'Y',12X,'P',12X,'U_Vel',9X,'V_Vel',9X,'W_Vel', &
     & 10X,'T',10X,'Density')
44	FORMAT(2X,'Wave No. = ',E12.6,'  : Growth Rate = ',E13.7)
43      FORMAT(2x,'Eigen functions : Re = ',F10.2,' : Omega = ', &
     & E12.6,' : BETA = ',E12.6)

	END
        SUBROUTINE PARMAXT(NDIV,FLUX,UMAXI,YEXACT)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LS=5)
        PARAMETER (NTQ=5)
	REAL*8 C(NTQ),Y(NTQ),A(NTQ,NTQ),ANS(NTQ)
        COMPLEX*16 UMAXI
	COMPLEX*16 FLUX(*)
        COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /SCALI/G,ULENTH

        VMAX=0.D0
        YPOS=DY
        DO  I=2,NDIV-1
        ULOCL=CDABS(FLUX(I))
        IF(ULOCL < VMAX)GOTO 99
        VMAX=ULOCL
        JFLAG=I
        YMAXI=YPOS
99      CONTINUE
        YPOS=YPOS+DY
	end do

        Y(1)=ULENTH*(YmAxi-dy)/(G-(YmAxi-dy))
        Y(2)=ULENTH*YmAxi/(G-YmAxi)
        Y(3)=ULENTH*(YmAxi+dy)/(G-(YmAxi+dy))

        do  i=1,3
        A(i,1)=y(i)*y(i)
        A(i,2)=y(i)
        A(i,3)=1.d0
	end do

        c(1)=CDABS(FLUX(JFLAG-1))
        c(2)=CDABS(FLUX(JFLAG))
        c(3)=CDABS(FLUX(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
        yexAct=-Ans(2)/(2.*Ans(1))
        fvAl=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)

        c(1)=DBLE(FLUX(JFLAG-1))
        c(2)=DBLE(FLUX(JFLAG))
        c(3)=DBLE(FLUX(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
        fvr=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)

        c(1)=DIMAG(FLUX(JFLAG-1))
        c(2)=DIMAG(FLUX(JFLAG))
        c(3)=DIMAG(FLUX(JFLAG+1))
	CALL SOLVE(A,C,ANS,NTQ,3)
        fvi=Ans(1)*yexAct*yexAct+Ans(2)*yexAct+Ans(3)
        UMAXI=DCMPLX(fvr,fvi)

        RETURN
        END
        SUBROUTINE PDCALC(NDIV,POUT,N,K)
        use param_info, only : MP, MODE, KMODE
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LS=5)
        DOUBLE PRECISION, DIMENSION(MP) :: TS,D2T,DT,TX,DTX,D2TX
        COMPLEX*16 POUT(2,MP)                                          
        COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMMON /HISTRY/PRV                                           
        COMMON /IPRIMIT/IVARIP
        COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX                        
        COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT                 
        COMMON /SURFACE/PWALL,PEDGE,PGRAD

        IF(IVARIP == 0)THEN                                          

        DO NM=1,NDIV          
        TIN=1.D0/TS(NM)                                              
        RO=PINV*TIN*PEDGE                                           

        POUT(1,NM)=PRV(1,NM,N,K) 
        POUT(2,NM)=(PINV*PRV(1,NM,N,K)-RO*PRV(5,NM,N,K))*TIN    
	end do                                      

        ELSE                                                         

        DO  NM=1,NDIV
        TQ=TS(NM)
        RO=PINV*PEDGE/TQ

        POUT(2,NM)=PRV(1,NM,N,K)
        POUT(1,NM)=(PRV(1,NM,N,K)*TQ+RO*PRV(5,NM,N,K))/PINV
	end do
        END IF

        RETURN
        END
	

        SUBROUTINE SIMPSN(NP,GF,ENERGY) 
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)      
        COMPLEX*16 GF(*),ENERGY
        COMMON /SCALI/B,G
        COMMON /TEMESH/DY,YEND,YEDGE
        YO=0.D0
	GB=G*B
        DO I=1,NP
        GF(I)=GF(I)*GB/(B-YO)**2
        YO=YO+DY
	end do                                  
        ENERGY=0.D0
        DO I=1,NP-2,2
        ENERGY=ENERGY+GF(I)+4.D0*GF(I+1)+GF(I+2)
	end do                                  
	ENERGY=ENERGY*DY/3.D0
        RETURN
        END	
	
	SUBROUTINE PHABINS(I3DNEQ,IGERR)
        use param_info, only : MODE, KMODE, NPOST
	use param_FILEADD
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	COMPLEX*16 OMEGA,ALNEW,ALPREV
	REAL*8 MEDGE,MO,DIMSCL(4)
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	REAL*8 ENMETH(KSTR,KCRF,NPOST)
	INTEGER, DIMENSION(KSTR,KCRF) :: NFFLAG,ISWITCH,IPREMAT,IERSWCH
	REAL*8, DIMENSION(KCRF) :: WAVBETA,CRFBETA
	REAL*8, DIMENSION(NPOST) :: SDVAL, XCVAL, RDVAL
        REAL*8, DIMENSION(KSTR) :: STRALFA, WAVALFA, STRALIM,WAVALIM, &
     & FRENCY,OMRANGE
	COMPLEX*16, DIMENSION(KSTR) :: WAVLOCL, QZLIN
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /PHCUTTY/WAVLOCL
	COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /EISPC/ALFAZO
	COMMON /QCNTRL/ND
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /FRVARI/OMEGA,RE
        COMMON /SPFLG/ISPC,IOUT,IHOW,IPAR
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /STEPNG/XNEW
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /PRFIL/DXSTEP,DXINV
	COMMON /AMPLIT/NFACT,NFFLAG
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/F
	COMMON /FILINF/IERFIL
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ISAVEY/ISWITCH
	COMMON /POLISH/IOMSW
	IOMSW=1
	IFIRST=0
	IERFIL=0
	NFLAG=0
	IALC=1

	IOFF=0
!	ioff is marker for whether Any nfActors Are non-zero... not
!	reAlly being used At present!!!! should use this As indICAtot
!	to user that no nfActors Are nonzero

	NDIV=ND
	CALL ETASET(NDIV)

	CALL INITL
	IF(IERFIL == 1)RETURN
	CALL PROFRD(NDIV,NSTART,1)
	IF(IERFIL == 1)RETURN

	DO ICR=1,NCRBET
	DO IJK=1,NWAVE
	NFFLAG(IJK,ICR)=0
	ISWITCH(IJK,ICR)=0
	IERSWCH(IJK,ICR)=0
	IPREMAT(IJK,ICR)=0
	ENMETH(IJK,ICR,1)=0.D0
	end do
	end do

	RELOC=RLOCAL
	REPRV=RLOCAL
	REXPRV=REX
        XACT=SPOS
	XPREV=XPOS
	XSCALE=XNEW

	IF(ISCREEN == 0)THEN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.dmp',PSELABL)
	OPEN(ITV,FILE=PSELABL,STATUS='UNKNOWN')
	ELSE
	END IF

        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,110)IPROF,XPOS,RELOC

        WRITE(2,110)IPROF,XPOS,RELOC
        WRITE(ITV,440)
	IF(NOUTPUT == 1)THEN
        WRITE(ITV,311)WAVALFA(1),WAVALIM(1),CRFBETA(1)
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,411)WAVALFA(1),WAVALIM(1)
311     FORMAT(1X,'Streamwise Wave No. =',E12.6,'+i',E12.6, &
     & 1X,'Crossflow Wave No. =',E12.6)
411     FORMAT(1X,'Streamwise Wave No. =',E12.6,'+i',E12.6)
	ELSE IF(NOUTPUT == 3)THEN
	WRITE(ITV,511)CRFBETA(1)
511     FORMAT(1X,'Crossflow Wave No. =',E12.6)
	ELSE
	END IF

	XCVAL(1)=XPOS
	RDVAL(1)=RELOC
	SDVAL(1)=SPOS
220     FORMAT(5x,'  RDelta = ',E14.7)
110     FORMAT(1X,'TSE begins at station ',I0,' : x/c =', &
     & F10.5,1X,'; RDelta = ',E12.6)

440     FORMAT(1X,66('-'))

	IONEQ=I3DNEQ
	NB=NDIV-1
	IEQT=NB*IONEQ
        CALL AP3D2(NDIV,IEQT,IONEQ)
	CALL PHLIN(NDIV,IONEQ,IEQT,IGERR)
	IEQT=NB*IONEQ
        CALL AP3D2(NDIV,IEQT,IONEQ)

        IF(IGERR == 1)RETURN
	IF(IGLBX == 1)GOTO 13272
	IF(IGLOB == 1)RETURN
13272	CONTINUE

	IF(NSTART+NINTR+1 > NFINAL)RETURN

	IHOW=IOUT
        CALL MISPRF(1,NINTR)
        IF(IERFIL == 1)RETURN
        INP=NSTART+NINTR+1

	DO 10 IN=INP,NFINAL,NINTR+1
	CALL PROFRD(NDIV,IN,1)
	IF(IERFIL == 1)GOTO 7671
	RE=RLOCAL
211     FORMAT(3X,'TSE at station ',I0,' : x/c =',F10.5, &
     & 2X,'RDelta = ',E12.6)

	IF(ITSPRES == 0)THEN
        WRITE(ITV,*)
        WRITE(ITV,*)
        WRITE(ITV,440)
        WRITE(ITV,211)IPROF,XPOS,RLOCAL
        WRITE(ITV,440)
	ELSE
	END IF

        IF(IGLBX == 1)THEN
        WRITE(19,211)IPROF,XPOS,RLOCAL
        ELSE
        END IF

	

	DX=XNEW-XSCALE
	DXSTEP=DX
	DXINV=1.D0/DX

	IALC=IALC+1
	XCVAL(IALC)=XPOS
	RDVAL(IALC)=RLOCAL
	SDVAL(IALC)=SPOS

	DO 101 ICR=1,NCRBET
	CRFBETA(ICR)=WAVBETA(ICR)*DELZER/Chord
	DO 101 IFREQ=1,NWAVE

        IF(ITSPRES == 0 .AND. IGLBX == 0)THEN
        WRITE(ITV,*)
	ELSE
	END IF

	WAVLOCL(IFREQ)=DCMPLX(WAVALIM(IFREQ),WAVALFA(IFREQ))*DELZER/ &
     & Chord

	IF(IGLBX == 1)THEN
	CALL PHORRQZ(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,ierr,RESIDU)
	ELSE
	CALL PHORR(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,ierr,RESIDU)
	END IF

        IF(ITSPRES == 0 .AND. IGLBX == 0)THEN
        WRITE(ITV,*)
	ELSE
	END IF

	IF(IERR > 0)RETURN

101	CONTINUE

	IF(IGLBX == 0)THEN

	IF(ITSPRES >= 1)GOTO 7011
	IF(NOUTPUT >= 2)THEN
	WRITE(ITV,440)
	ELSE
	END IF
7011	CALL PHRES(1,IALC)
	IF(ITSPRES >= 1)GOTO 711
	IF(NOUTPUT >= 2)THEN
	WRITE(ITV,440)
	ELSE
	END IF
711	CONTINUE

	ELSE
	END IF

279     FORMAT(2X,2F10.5,2x,f12.7,2x,f15.8,3X,F12.6)

	DXNF=SPOS-XACT
	DXREAL=XPOS-XPREV
	DELRED=RLOCAL-REPRV
	DXRX=REX-REXPRV

	ihow=ihow+1

	XACT=SPOS
	XPREV=XPOS
	XSCALE=XNEW
	REPRV=RLOCAL
	REXPRV=REX

	CALL MISPRF(1,NINTR)
10      continue
7671    CONTINUE

	RETURN
	END
	SUBROUTINE PHCOEFF(NDIV,LK)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9)
	COMPLEX*16 SIBETA,SIOM,CROSS
	COMPLEX*16 SING,OMEGA,ABSQ
        COMPLEX*16 ALFA,ALSQ,ALF3,AL4
        COMPLEX*16, DIMENSION(IL,IL,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION, DIMENSION(MP) :: A1, A2, B1
	DOUBLE PRECISION, DIMENSION(MP) :: US,V,D2U,DU,UX,DUX,D2UX, &
     & TS, D2T, DT, TX, DTX, D2TX, WS, D2W, DW, WX, DWX, D2WX
	COMMON /ROOL2/ALFA,ALSQ,ALF3,AL4
	COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /DICTAT/Y,YA,Z,ZA
	COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,V
	COMMON /SWPPRF/WS,DW,D2W,WX,DWX,D2WX
	COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /FRVARI/OMEGA,RE
	COMMON /ZCLEAR/SING
	COMMON /STRECH/A1,A2,B1
	COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
	COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT
	COMMON /ACTBIT/YLAMBDA,WOMG
        COMMON /SWEPTIN/DUBLU
	COMMON /SURFACE/PWALL,PEDGE,PGRAD
	COMMON /IPRIMIT/IVARIP

	CAPGAM=ZAPGAM

        DUBLU=WS(NDIV)

	SIBETA=SING*BZERO
	SIOM=SING

	TEMPER=TS(1)
	TIN=1.D0/TEMPER
	CALL VISCOS(TEMPER,TIN,FM,FMT,FMTT)
	PINKY=1.D0/(A1(1)*PEDGE)

	DO 10 NM=1,NDIV
	D1NM=1.D0/B1(NM)
	TEMPER=TS(NM)

	CROSS=ALFA*US(NM)+SIBETA*WS(NM)
	TIN=1.D0/TEMPER

	CALL VISCOS(TEMPER,TIN,FM,FMT,FMTT)
	RODENT=PINV*TIN*PEDGE
	CURLYR=RE*RODENT/FM
	SIG=CURLYR*SIGMA
	EMM=CAPGAM*RE/FM
	YMUo=FMT*DT(NM)/FM

	IF(NM == 1)THEN
	Z(1,1,1)=(SIBETA*DW(1)+ALFA*DU(1))/PEDGE
	ZA(1,1,1)=-SIOM*(DT(NM)*TIN+YMUo)/PEDGE

	Z(1,3,1)=-ABSQ*RMINV-YMUo*DT(1)*TIN-TIN*D2T(1)
        ZA(1,3,1)=-CURLYR*SIOM*RMINV

	Y(1,1,1)=A1(NM)*RE*RMINV/FM  
	YA(1,1,1)=-A1(NM)*SIOM/PEDGE  
	Y(1,2,1)=A1(NM)*ALFA*RMINV
	Y(1,4,1)=A1(NM)*SIBETA*RMINV
	YA(1,5,1)=A1(NM)*SIOM*TIN

	Z(3,1,1)=-WOMG
	Z(3,3,1)=1.D0
	Y(3,2,1)=-YLAMBDA*A1(1)

	ELSE

	Z(1,1,NM)=CROSS*D1NM/PEDGE
	ZA(1,1,NM)=-SIOM*D1NM/PEDGE
	Z(1,2,NM)=ALFA*D1NM
        Y(1,3,NM)=A1(NM)*D1NM
	Z(1,3,NM)=-DT(NM)*TIN*D1NM
	Z(1,4,NM)=SIBETA*D1NM
	Z(1,5,NM)=-CROSS*TIN*D1NM
	ZA(1,5,NM)=SIOM*TIN*D1NM

        Z(2,1,NM)=-ALFA*RE*D1NM/FM
        Y(2,2,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        Z(2,2,NM)=(ALSQ*RM-BESQ-CURLYR*CROSS)*D1NM
        ZA(2,2,NM)=CURLYR*SIOM*D1NM
        Y(2,3,NM)=ALFA*RS*A1(NM)*D1NM
        Z(2,3,NM)=(ALFA*YMUo-CURLYR*DU(NM))*D1NM
	Z(2,4,NM)=ALFA*SIBETA*RS*D1NM
        Y(2,5,NM)=A1(NM)*FMT*DU(NM)*D1NM/FM
	Z(2,5,NM)=(FMT*D2U(NM)+FMTT*DT(NM)*DU(NM))*D1NM/FM


        Y(3,1,NM)=-A1(NM)*RE*RMINV*D1NM/FM
        Y(3,2,NM)=A1(NM)*ALFA*RS*RMINV*D1NM
        Z(3,2,NM)=ALFA*YLMU*RMINV*YMUo*D1NM
        Y(3,3,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        Z(3,3,NM)=(ABSQ-CURLYR*CROSS)*RMINV*D1NM
        ZA(3,3,NM)=CURLYR*SIOM*RMINV*D1NM
        Z(3,4,NM)=YLMU*RMINV*SIBETA*YMUo*D1NM
        Y(3,4,NM)=A1(NM)*RS*RMINV*SIBETA*D1NM
        Z(3,5,NM)=(SIBETA*DW(NM)+ALFA*DU(NM))*FMT*RMINV*D1NM/FM

	Z(4,1,NM)=-SIBETA*RE*D1NM/FM
	Z(4,2,NM)=ALFA*SIBETA*RS*D1NM
        Y(4,3,NM)=A1(NM)*RS*SIBETA*D1NM
	Z(4,3,NM)=(SIBETA*YMUo-CURLYR*DW(NM))*D1NM
        Y(4,4,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        Z(4,4,NM)=(ALSQ-BESQ*RM-CURLYR*CROSS)*D1NM
        ZA(4,4,NM)=CURLYR*SIOM*D1NM
        Y(4,5,NM)=A1(NM)*FMT*DW(NM)*D1NM/FM
	Z(4,5,NM)=(FMT*D2W(NM)+FMTT*DT(NM)*DW(NM))*D1NM/FM


        Y(5,5,NM)=(A2(NM)+2.D0*A1(NM)*YMUo)*D1NM
	CONST=CAPGAM*(DU(NM)*DU(NM)+DW(NM)*DW(NM))+D2T(NM)
        CONST=(FMTT*DT(NM)*DT(NM)+FMT*CONST)/FM
        Z(5,5,NM)=(CONST+ABSQ-SIG*CROSS)*D1NM
        ZA(5,5,NM)=SIG*SIOM*D1NM
        Z(5,1,NM)=EMM*CROSS*D1NM
        ZA(5,1,NM)=-EMM*SIOM*D1NM
        Y(5,2,NM)=2.D0*A1(NM)*CAPGAM*DU(NM)*D1NM
        Z(5,3,NM)=(2.D0*CAPGAM*(SIBETA*DW(NM)*ALFA*DU(NM))- &
     & DT(NM)*SIG)*D1NM
        Y(5,4,NM)=2.D0*A1(NM)*CAPGAM*DW(NM)*D1NM

	END IF
	
10	CONTINUE

	RETURN              
	END
        SUBROUTINE PHLIN(NDIV,IL,IEQT,IGERR)
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (LK=9,KE=(MP-1)*LK,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	PARAMETER (LS=5,KS=MP*LS)
        CHARACTER*80 TITLE
	CHARACTER*58 PSELABL
	COMPLEX*16 XSOL(KE)
	COMPLEX*16 POUT(2,MP)
	COMPLEX*16 PRV(LS,MP,-MODE:MODE,-KMODE:KMODE)
        COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),ENERGY(KSTR,KCRF)
	COMPLEX*16 OMALF,ALSQ,ALF3,AL4, ABSQ
	COMPLEX*16 EIGEN,UMAXI,OMEGA,ALFA
	REAL*8 DIMSCL(4)
	REAL*8 CRFBETA(KCRF)
        REAL*8 Medge,MZERO
	REAL*8, DIMENSION(KSTR) :: STRALFA,STRALIM,WAVALFA,FRENCY, &
     & WAVALIM, OMRANGE
        INTEGER JOUTSC(2)
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
        COMMON /IPHIWAV/NWAVE,IGLBX
        COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /PICTR/TITLE
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /TEMESH/DY,YEND,YEDGE
        COMMON /FRVARI/OMEGA,RE
	COMMON /ROOTS/OMALF
	COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /SCALI/G,ULENTH
	COMMON /EISPC/ALFAZO
        COMMON /WHATP/IEIG,IRQL,IRAYCNT
        COMMON /WHREL/FACTOR
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
	COMMON /KERES/ENERGY
	COMMON /HISTRY/PRV
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /ROOL2/ALFA,ALSQ,ALF3,AL4
        COMMON /STRTGY/IUMAX,MASFLX,IENRGY,IUSQ,MHERB,IUHERB, &
     & IWSQ,IWMAX
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
	COMMON /PISAVE/PI
	common /TBSAL/salmon
	
	salmon=2.d0*PI*DZERO/UZERO

	NB=NDIV-1
	REQ=RESSCL
        sqfAct=dsqrt(2.D0)
	WRITE(ITV,*)
	IF(IGLOB /= 1)GOTO 123
	OPEN(19,FILE='PSESPEC.FULL',STATUS='UNKNOWN')
	WRITE(ITV,*)
	IF(IGLOB == 1 .AND. IGLBX == 0)THEN
	WRITE(ITV,*)'           Global Method Called !!! '
	ELSE
	WRITE(ITV,*)'         Global Method In X Called !!! '
70001	FORMAT(9X,'s/c',13X,'R',15X,'Alfa_R',5x,'Alfa_I',12x,'Beta')
70002	FORMAT(9X,'x/c',13X,'R',15X,'Alfa_R',5x,'Alfa_I',12x,'Beta')
	END IF
	WRITE(ITV,*)
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,70001)
        WRITE(19,70001)
        ELSE
        WRITE(ITV,70002)
        WRITE(19,70002)
        END IF
	CALL FLUSH(ITV)

	ZETA=SPOS*JOUTSC(1)+XPOS*JOUTSC(2)

	IA=1
	IB=0
	DO 101 I=1,NWAVE
	ICROSS=1
	BETA=CRFBETA(ICROSS)
	ALFA=DCMPLX(STRALIM(I),STRALFA(I))

	BZERO=BETA
	BESQ=BZERO*BZERO
	ALSQ=ALFA*ALFA
	ABSQ=ALSQ-BESQ

        CALL PHCOEFF(NDIV,IL)

	CALL QZCALLSET(NDIV,IL)
	
	WRITE(ITV,6768)ZETA,RLocal,DIMAG(ALFA),DBLE(ALFA),BZERO
	WRITE(19,6768)ZETA,RLocal,DIMAG(ALFA),DBLE(ALFA),BZERO
	CALL FLUSH(ITV)
	CALL PHQZALG(IA,IB,IEQT,I,IL,0)
	CALL FLUSH(ITV)

101	CONTINUE
676	FORMAT(2x,'Computing for ALFA = ',E16.9,'+i', &
     & E16.9,' : BETA = ',E12.6)
6768	FORMAT(1X,6E16.7)

	RETURN
123	CONTINUE

	IF(IEIG == 1)THEN
	CALL FILHAND(999,LABEL(1:ICHART),'Q','.local',PSELABL)
	OPEN(11,FILE=PSELABL,STATUS='UNKNOWN')
	ELSE
	END IF


	DO 5501 IEQ=1,LS
	DO 5501 M=-MODE,MODE
	DO 5501 ML=-KMODE,KMODE
	PRV(IEQ,1:NDIV,M,ML)=0.D0
5501	CONTINUE

        DO 20 ICROSS=1,NCRBET
        DO 20 IFQ=1,NWAVE
	IF(ICROSS == 1)THEN
	EIGEN=QZLIN(IFQ)
	ELSE
	EIGEN=ALFAZO(IFQ,ICROSS-1,1)
!	EIGEN=QZLIN(IFQ)
	END IF

	IX=IFQ-MODE-1
	IY=ICROSS-KMODE-1

	ALFA=DCMPLX(STRALIM(IFQ),STRALFA(IFQ))
	BETA=CRFBETA(ICROSS)
        BZERO=BETA
        BESQ=BZERO*BZERO

	ALSQ=ALFA*ALFA
	ABSQ=ALSQ-BESQ

	IF(ITSPRES >= 1)GOTO 807
	IF(NOUTPUT == 1 .OR. NOUTPUT == 3)THEN
	WRITE(ITV,77)DIMAG(ALFA),DBLE(ALFA)
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,88)BZERO
	ELSE
	WRITE(ITV,787)DIMAG(ALFA),DBLE(ALFA),BZERO
	END IF
77	FORMAT(1X,'Computing for Streamwise Wavenumber = ', &
     & E12.6,'+i',E12.6)
88	FORMAT(1X,'Computing for Crossflow Wavenumber = ',E12.6)
787	FORMAT(1X,'Computing for Alfa = ',E12.6,'+i',E12.6, &
     & ' & Beta =',E12.6)

807	CONTINUE

	CALL FLUSH(ITV)
        CALL PHCOEFF(NDIV,IL)
        CALL SETGLB(NDIV,IL)
	CALL RAYLEI(IL,NB,IEQT,XSOL,IGERR)
        IF(ITSPRES >= 1)GOTO 507
	WRITE(ITV,*)
	CALL FLUSH(ITV)
507	CONTINUE
	IF(IGERR == 1)RETURN
        IF(IGERR == 2)THEN
	WRITE(ITV,*)
        WRITE(ITV,*)'Initial conditions have not converged to requested'
        WRITE(ITV,*)'accuracy - but continuing analysis '
        ELSE
        END IF

	J=1
        DO 50 I=1,NDIV-1
        PRV(1,I,IX,IY)=XSOL(J)
        PRV(2,I,IX,IY)=XSOL(J+1)
        PRV(3,I,IX,IY)=XSOL(J+2)
        PRV(4,I,IX,IY)=XSOL(J+3)
        PRV(5,I,IX,IY)=XSOL(J+4)
	J=J+IL
50      CONTINUE
        PRV(1:5,NDIV,IX,IY)=0.D0

!	CALL PDCALC(NDIV,POUT,IX,IY)
!        CALL MAKEONE(MASFLX,NDIV,POUT,IX,IY,UMAXI,YMAXI)

	ALFAZO(IFQ,ICROSS,1)=OMALF

6605	CONTINUE

	if (ieig /= 1) goto 1963
	WRITE(11,535)IPROF,XPOS,RE*RESCALE
535	FORMAT(' IProf =',i0,' : x/c =',f8.4,' : Rdelta =',f10.2)
	WRITE(11,44)DIMAG(ALFA)*REQ,BZERO*REQ
	WRITE(11,43)RE*RESCALE,ALFAZO(IFQ,ICROSS,1)*REQ
	WRITE(11,46)
46	FORMAT(8X,'Y',12X,'P',12X,'U_Vel',9X,'V_Vel',9X,'W_Vel', &
     & 10X,'T',10X,'Density')
44	FORMAT(2X,'Streamwise Wave No. = ',E12.6, &
     & ' Crossflow Wave No. =', E12.6)
43      FORMAT(2x,'Eigen functions : Re = ',F10.2,' : Omega_R = ', &
     & E12.6,' : Omega_I = ',E12.6)

	YO=0.D0
	DO I=1,NDIV
	YNETA=ULENTH*YO/(G-YO)
!	YNETA=YNETA/REQ
!	YNETA=YNETA/REQ
        WRITE(11,333)YNETA,(CDABS(PRV(KOP,I,IX,IY)),KOP=1,LS)
	YO=YO+DY
	END DO
	WRITE(11,*)
333     FORMAT(7E14.6)
1963	CONTINUE

20	CONTINUE

	CALL PHRES(0,1)

1000	RETURN

	END
	SUBROUTINE PHORR(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,IGERR,VAL)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 XCVAL(NPOST),DIMSCL(4)
        REAL*8 RDVAL(NPOST),SDVAL(NPOST)
	REAL*8 MEDGE,LNORM
	COMPLEX*16 XSOL(KE),OMEGA,ALFA
	COMPLEX*16 EIGEN
	COMPLEX*16 ALNEW,ALPREV
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 OMALF,ALSQ,ALF3,AL4,ABSQ
        REAL*8 STRALFA(KSTR),WAVALFA(KSTR)
        REAL*8 STRALIM(KSTR),WAVALIM(KSTR)
	COMPLEX*16 WAVLOCL(KSTR)
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /PHCUTTY/WAVLOCL
	COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /ROOL2/ALFA,ALSQ,ALF3,AL4
        COMMON /EISPC/ALFAZO
	COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IN
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/FVALU
	COMMON /ROOTS/OMALF
	COMMON /NICEINF/RESIDU
	COMMON /SOMMER/LNORM
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /IPRIMIT/IVARIP
	REQ=RESSCL

	CALL EXTRAP(IALC,IFREQ,ICR,ALPREV)

	OMEGA=0.D0

        BETA=CRFBETA(ICR)
	BZERO=BETA
	BESQ=BZERO*BZERO

        ALFA=WAVLOCL(IFREQ)

        ALSQ=ALFA*ALFA
        ABSQ=ALSQ-BESQ


        IF(ITSPRES >= 1)GOTO 807

	IF(NOUTPUT == 1 .OR. NOUTPUT == 3)THEN
	WRITE(ITV,77)DIMAG(ALFA)*LNORM*PZERO, &
     & DBLE(ALFA)*LNORM*PZERO
	ELSE IF(NOUTPUT == 2)THEN
	WRITE(ITV,88)BZERO*LNORM*PZERO
	ELSE
	WRITE(ITV,787)DIMAG(ALFA)*LNORM*PZERO, &
     & DBLE(ALFA)*LNORM*PZERO,BZERO*LNORM*PZERO
	END IF

807	CONTINUE

	EIGEN=ALPREV/LNORM

        CALL PHCOEFF(NDIV,IONEQ)
	CALL SETGLB(NDIV,IONEQ)
	CALL RAYLEI(IONEQ,NB,IEQT,XSOL,IGERR)
        IF(IGERR == 1)RETURN
	IF(IGERR == 2)THEN
	WRITE(ITV,*)
	WRITE(ITV,*)'Rayleigh Iteration not converged but may be local'
	WRITE(ITV,*)' non-convergence - hence continuing analysis'
	WRITE(ITV,*)
	ELSE
	END IF

	ALNEW=OMALF
	ALPREV=ALNEW
	ALFAZO(IFREQ,ICR,IALC)=ALNEW*LNORM
	VAL=RESIDU

	RETURN
77	FORMAT(1X,'Computing for Alfa = ',E12.6,'+i',E12.6)
787	FORMAT(1X,'Computing for Alfa = ',E12.6,'+i',E12.6, &
     & ' & Beta =',E12.6)
88	FORMAT(1X,'Computing for Crossflow Wavenumber = ',E12.6)
	END
	SUBROUTINE PHORRQZ(NB,IEQT,IONEQ,IALC,IFREQ,ICR,NDIV,DX,IGERR,VAL)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KGP=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 DIMSCL(4)
	REAL*8 XCVAL(NPOST),RDVAL(NPOST)
        REAL*8 SDVAL(NPOST)
	REAL*8 MEDGE,LNORM, MZERO
	COMPLEX*16 OMEGA,ALFA
	COMPLEX*16 EIGEN
	COMPLEX*16 ALNEW,ALPREV
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	COMPLEX*16 QZLIN(KSTR)
	COMPLEX*16 OMALF,ALSQ,ALF3,AL4,ABSQ
        REAL*8 STRALFA(KSTR),WAVALFA(KSTR)
        REAL*8 STRALIM(KSTR),WAVALIM(KSTR)
	COMPLEX*16 WAVLOCL(KSTR)
	INTEGER JOUTSC(2)
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
	COMMON /JOVIAN/JOUTSC,RESCALE
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /PHCUTTY/WAVLOCL
	COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /ROOL2/ALFA,ALSQ,ALF3,AL4
        COMMON /EISPC/ALFAZO
	COMMON /GLOBAL/EIGEN,TOLR,TPSL,TPSM,TOLNLR
	COMMON /RIGHT/ALNEW,ALPREV
	COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IN
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
        COMMON /LAYREL/FVALU
	COMMON /ROOTS/OMALF
	COMMON /NICEINF/RESIDU
	COMMON /SOMMER/LNORM
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /IPRIMIT/IVARIP
	COMMON /PISAVE/PI
	common /TBSAL/salmon
	REQ=RESSCL
	
	salmon=2.d0*PI*DZERO/UZERO

	OMEGA=0.D0

        BETA=CRFBETA(ICR)
	BZERO=BETA
	BESQ=BZERO*BZERO

        ALFA=WAVLOCL(IFREQ)
        ALSQ=ALFA*ALFA
        ABSQ=ALSQ-BESQ

	ZETA=SPOS*JOUTSC(1)+XPOS*JOUTSC(2)

        IF(JOUTSC(1) == 1)THEN
!        WRITE(ITV,70001)
        WRITE(19,70001)
        ELSE
!        WRITE(ITV,70002)
        WRITE(19,70002)
        END IF

        ZETA=SPOS*JOUTSC(1)+XPOS*JOUTSC(2)
        WRITE(ITV,6768)ZETA,RDVAL(IALC),DIMAG(ALFA),DBLE(ALFA), &
     & BZERO
        WRITE(19,6768)ZETA,RDVAL(IALC),DIMAG(ALFA),DBLE(ALFA), &
     & BZERO

	EIGEN=ALPREV/LNORM

        CALL PHCOEFF(NDIV,IONEQ)
	
	CALL QZCALLSET(NDIV,IL)
	
	CALL PHQZALG(1,0,IEQT,IFREQ,IONEQ,0)

        IF(IGERR == 1)RETURN

	RETURN
6768    FORMAT(1X,6E16.7)
70001   FORMAT(9X,'s/c',13X,'R',15X,'Alfa_R',5x,'Alfa_I',12x,'Beta')
70002   FORMAT(9X,'x/c',13X,'R',15X,'Alfa_R',5x,'Alfa_I',12x,'Beta')

	END
	SUBROUTINE PHQZALG(IA,IB,NEQ,IFREQ,IL,IBOX)
        use param_info, only : MP, MPG
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	LOGICAL MATV
	PARAMETER (LK=9,KE=(MP-1)*LK,KPG=LK*5)
	PARAMETER (KEG=(MPG-1)*LK,N=KEG+2)
	CHARACTER*80 TITLE
	COMPLEX*16, DIMENSION(KE,KPG) :: A, B
	COMPLEX*16, DIMENSION(N,N) :: AQZ, BQZ
	COMPLEX*16 EIGEN(KEG),OMEGA,ABSQ
	COMPLEX*16 ALFA,ALSQ,ALF3,AL4
	DOUBLE PRECISION, DIMENSION(N) :: ALFI, ALFR, BETA
	DOUBLE PRECISION, DIMENSION(N,N) :: BI,BR,VI,VR,AI,AR
	INTEGER ITER(N)
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
	COMMON /GLPROP/B,A
	COMMON /GLPQZ/BQZ,AQZ
	COMMON /FRVARI/OMEGA,RE
	COMMON /PICTR/TITLE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /SWEPTIN/DUBLU
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /ROOL2/ALFA,ALSQ,ALF3,AL4
	common /TBSAL/salmon
	DTOLS=1.D-12

	DO I=1,NEQ
	DO J=1,NEQ
	AR(I,J)=0.D0 ; AI(I,J)=0.D0 ; BR(I,J)=0.D0 ; BI(I,J)=0.D0
	END DO
	END DO
	
	IF(IQZ == 0)THEN
	DO I=1,NEQ
	DO J=1,IL*5
	JMV=J+IT(I)
	AR(I,JMV)=DBLE(A(I,J))
	AI(I,JMV)=DIMAG(A(I,J))
	BR(I,JMV)=-DBLE(B(I,J))
	BI(I,JMV)=-DIMAG(B(I,J))
	END DO
	END DO
	ELSE
	DO I=1,NEQ
	DO J=1,NEQ
	AR(I,J)=DBLE(AQZ(I,J))
	AI(I,J)=DIMAG(AQZ(I,J))
	BR(I,J)=-DBLE(BQZ(I,J))
	BI(I,J)=-DIMAG(BQZ(I,J))
	END DO
	END DO
	END IF
 
	K=NEQ
	EPSI=0.D0
	MATV=.FALSE.
	IFAIL=1
        CALL QZLIB(K,N,AR,AI,BR,BI,EPSI,ALFR,ALFI, &
     &                 BETA,MATV,VR,VI,ITER,IFAIL)

	WRITE(19,'(A80)')TITLE
	WRITE(19,23)

	ICount=1
	DO 40 I=1,K
	IF(DABS(BETA(I)) > DTOLS)THEN
	ALFR(I)=ALFR(I)/BETA(I)
	ALFI(I)=ALFI(I)/BETA(I)
	ELSE
	END IF
 
	IF(DABS(ALFR(I)) > DTOLS .AND. DABS(ALFI(I)) > DTOLS)THEN
	EIGEN(I)=1.D0/DCMPLX(ALFR(I),ALFI(I))
	VAL=DBLE(EIGEN(I))
	V2=DIMAG(EIGEN(I))

	WRITE(19,99994)ICOUNT,EIGEN(I),EIGEN(I)/salmon, &
     & EIGEN(I)/dimag(ALFA)
	ICount=ICount+1

	ELSE
	END IF
40	CONTINUE

	RETURN
99996	FORMAT(2x,'ERROR IN QZLIB IFAIL = ',I0)
23	FORMAT(' ID_AUTO',4X,'Omega_R',8X,'Omega_I')
99994	FORMAT(i5,9E18.7)
	END
	SUBROUTINE PHRES(IFIRST,IALC)
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 XCVAL(NPOST),RDVAL(NPOST)
        REAL*8 SDVAL(NPOST)
	REAL*8 Medge,Mo,DIMSCL(4)
	COMPLEX*16 QZLIN(KSTR),ALFAZO(KSTR,KCRF,NPOST)
	COMPLEX*16 ABSQ
	COMPLEX*16 WAVLOCL(KSTR)
	DOUBLE PRECISION WAVBETA(KCRF)
	INTEGER JOUTSC(2)
        REAL*8 STRALFA(KSTR),WAVALFA(KSTR)
        REAL*8 STRALIM(KSTR),WAVALIM(KSTR)
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /PHCUTTY/WAVLOCL
        COMMON /IPHIWAV/NWAVE,IGLBX
        COMMON /QZBETA/WAVBETA
        COMMON /QZXXXX/NCROSS
        COMMON /SCLLOC/Uedge,Medge,RLOCAL,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /EISPC/ALFAZO
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /PISAVE/PI
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
        COMMON /LAYOUT/IDIV,ItSpres,IGLOO
        COMMON /ROBETA/BZERO,BESQ,ABSQ

	SQFACT=DSQRT(2.D0)
	REQ=RESSCL
	ALIEN=RESCALE*RLOCAL
	ZETA=SPOS*JOUTSC(1)+XPOS*JOUTSC(2)

	ITU=80
	ITW=90

	IF(NOUTPUT /= 1)GOTO 1111
        IF(IFIRST /= 0)GOTO 1101
        WRITE(ITV,73)WAVALFA(1),WAVALIM(1),WAVBETA(1)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,22)
        ELSE
        WRITE(ITV,221)
        END IF

        WRITE(ITV,33)ZETA,ALIEN,ALFAZO(1,1,1)*REQ, &
     & ALFAZO(1,1,1)*REQ/WAVALFA(1) 
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.omega',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
        WRITE(ITU,73)WAVALFA(1),WAVALIM(1),WAVBETA(1)
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,22)
        ELSE
        WRITE(ITU,221)
        END IF

        WRITE(ITU,33)ZETA,ALIEN,ALFAZO(1,1,1)*REQ

	RETURN
1101	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,22)
        ELSE
        WRITE(ITV,221)
        END IF

	ELSE
	END IF
	IF(IWVDUMP == 0)RETURN
        WRITE(ITV,33)ZETA,ALIEN,ALFAZO(1,1,IALC)*REQ
        WRITE(ITU,33)ZETA,ALIEN,ALFAZO(1,1,IALC)*REQ
	RETURN

1111	IF(NOUTPUT /= 2)GOTO 2222
        IF(IFIRST /= 0)GOTO 2202
	WRITE(ITV,74)WAVALFA(1),WAVALIM(1)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,55)
        ELSE
        WRITE(ITV,551)
        END IF

	WRITE(ITV,66)(WAVBETA(ICR),ICR=1,NCRBET)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.omega_I',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.omega_R',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,74)WAVALFA(1)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,55)
        ELSE
        WRITE(ITU,551)
        END IF

	WRITE(ITU,66)(WAVBETA(ICR),ICR=1,NCRBET)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	WRITE(ITW,75)WAVALFA(1),WAVALIM(1)
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,55)
        ELSE
        WRITE(ITW,551)
        END IF
	WRITE(ITW,66)(WAVBETA(ICR),ICR=1,NCRBET)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	RETURN
2202	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,55)
        ELSE
        WRITE(ITV,551)
        END IF
	WRITE(ITV,66)(WAVBETA(ICR),ICR=1,NCRBET)
	ELSE
	END IF
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	IF(IWVDUMP == 0)RETURN
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	RETURN

2222	IF(NOUTPUT /= 3)GOTO 3333
        IF(IFIRST /= 0)GOTO 3303
	WRITE(ITV,78)WAVBETA(1)


        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,77)
        ELSE
        WRITE(ITV,771)
        END IF
	WRITE(ITV,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITV,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.omega_I',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.omega_R',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(1)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,77)
        ELSE
        WRITE(ITU,771)
        END IF
	WRITE(ITU,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITU,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	WRITE(ITW,79)WAVBETA(1)
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,77)
        ELSE
        WRITE(ITW,771)
        END IF
	WRITE(ITW,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITW,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	RETURN
3303	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,77)
        ELSE
        WRITE(ITV,771)
        END IF
	WRITE(ITV,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITV,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
	ELSE
	END IF
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	IF(IWVDUMP == 0)RETURN
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	RETURN

3333	CONTINUE

        IF(IFIRST /= 0)GOTO 4404
	DO 10 ICR=1,NCRBET
	INFT=ICR
	IF(ICR == 1)THEN
	WRITE(ITV,78)WAVBETA(ICR)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,77)
        ELSE
        WRITE(ITV,771)
        END IF
	WRITE(ITV,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITV,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	ELSE
	END IF

	IF(IWVDUMP == 0)GOTO 10
        CALL FILHAND(INFT,LABEL(1:ICHART),'B','.omega_I',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(INFT,LABEL(1:ICHART),'B','.omega_R',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(ICR)


        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,77)
        ELSE
        WRITE(ITU,771)
        END IF
	WRITE(ITU,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITU,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	WRITE(ITW,79)WAVBETA(ICR)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,77)
        ELSE
        WRITE(ITW,771)
        END IF
	WRITE(ITW,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITW,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)

	ITU=ITU+1
	ITW=ITW+1
10	CONTINUE
	RETURN

4404	CONTINUE
	DO 20 ICR=1,NCRBET
	IF(ITSPRES == 0 .AND. ICR == 1)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,77)
        ELSE
        WRITE(ITV,771)
        END IF
	WRITE(ITV,66)(WAVALFA(ICFQ),ICFQ=1,NWAVE)
	WRITE(ITV,66)(WAVALIM(ICFQ),ICFQ=1,NWAVE)
	ELSE
	END IF
	IF(ICR == 1)THEN
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	ELSE
	END IF
	IF(IWVDUMP == 0)GOTO 20
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NWAVE)
	ITU=ITU+1
	ITW=ITW+1
20	CONTINUE
	RETURN

11      FORMAT(3X,'Amplification Rates ...')
73	FORMAT(3X,'Alfa =',E12.6,'+i',E12.6,'; Beta =',E12.6)
74      FORMAT(3X,'Amplification Rates for Alfa =',E12.6,'+i',E12.6)
75      FORMAT(3X,'Omega_R for Alfa =',E12.6,'+i',E12.6)
78      FORMAT(3X,'Omega_I for Beta =',E12.6)
79      FORMAT(3X,'Omega_R for Beta =',E12.6)
121     FORMAT(3X,'Omega_R ...')
44      FORMAT(F7.4,E12.5,10E13.5)
55	FORMAT(3X,'s/c',6X,'RDelta',21X,'Crossflow Wavenumbers ')
551	FORMAT(3X,'x/c',6X,'RDelta',21X,'Crossflow Wavenumbers ')
66	FORMAT(19X,10E13.5)
771	FORMAT(3X,'x/c',6X,'RDelta',21X,'Alfa Range ')
221	FORMAT(5X,'x/c',8X,'RDelta',9X,'Omega_R',6X, & 
     & '      Omega_I    ', 7x,' Cr ',11x,'Ci')
77	FORMAT(3X,'s/c',6X,'RDelta',21X,'Alfa Range ')
22	FORMAT(5X,'s/c',8X,'RDelta',9X,'Omega_R',6X, & 
     & '      Omega_I    ', 7x,'Cr ',11x,'Ci')
33      FORMAT(F9.5,2X,E13.6,3x,E14.8,5X,E14.8,2X,2F14.9)
	END
	
	
        SUBROUTINE PRFCoDS(ND)
        use param_info, only : MP, MODE, KMODE
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        COMPLEX*16 OMEGA,HAFOMG
	COMPLEX*16 QZLIN(KSTR)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 DIMSCL(4)
        REAL*8 QU(NQ),QV(NQ),QD2U(NQ),QDU(NQ),QUX(NQ),QDUX(NQ)
        REAL*8 QD2UX(NQ),QNETA(NQ)
	REAL*8 QWS(NQ),QD2W(NQ),QDW(NQ),QWX(NQ),QDWX(NQ),QD2WX(NQ)
	REAL*8 QTS(NQ),QD2T(NQ),QDT(NQ),QTX(NQ),QDTX(NQ),QD2TX(NQ)
        REAL*8 U(MP),D2U(MP),DU(MP),UX(MP),DUX(MP)
        REAL*8 D2UX(MP),V(MP)
	REAL*8 QVY(NQ),QVYY(NQ)
	REAL*8 DV(MP),D2V(MP)
	REAL*8 WS(MP),D2W(MP),DW(MP),WX(MP),DWX(MP)
	REAL*8 D2WX(MP)
	REAL*8 TS(MP),D2T(MP),DT(MP),TX(MP),DTX(MP)
        REAL*8 D2TX(MP)
	REAL*8, DIMENSION(MP) :: XKI, XKIS
        REAL*8 Medge,MZERO,LNORM,MO
	REAL*8 WAVBETA(KCRF),WAVALFA(KSTR),STRALFA(KSTR),WAVALIM(KSTR)
	REAL*8 JFCFRS(6),STRALIM(KSTR)
	REAL*8 FRSCAN(6)
	INTEGER JOUTSC(2)
	EXTERNAL UMUVISC

	common /nleuro/phizero,uz1,vz1
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /A1AUTO/QNETA
        COMMON /QZBETA/WAVBETA
        COMMON /QZXXXX/NCROSS
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /JOVIAN/JOUTSC,RESCALE
        COMMON /NICEINF/RESIDU
        COMMON /NICINTG/ICNORM
	COMMON /SOMMER/LNORM
        COMMON /GARTEUR/UNORM
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /SWPPRF/WS,DW,D2W,WX,DWX,D2WX
        COMMON /VELPRF/U,DU,D2U,UX,DUX,D2UX,V
        COMMON /VELVVV/DV,D2V
	COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /SCALI/G,ULENTH
        COMMON /SIMPL/DY,YO,YSTART
        COMMON /IMPLINT/NDIV
        COMMON /TEMESH/DYP,YEND,YEDGE
        COMMON /GROPT/KGRID
        COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /SURFACE/PWALL,PEDGE,PGRAD
	COMMON /SURF3Z/PGRADZ
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
        COMMON /STEPNG/XNEW
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /RQINFO/BETAINF
        COMMON /NEPAL/HAFOMG
        COMMON /FRVARI/OMEGA,RZERO
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
	COMMON /FILINF/IERFIL
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /PHASIT/ITWO,KCARRY
	COMMON /TRPOINT/WALSLOP,WALBSIC,CRRESC
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PISAVE/PI
	COMMON /LADIOT/FRSCAN,BETONLY
	COMMON /cylind/VISCOS,RINFTY,SWEPTY
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TSUO,TSU1,STOKES
        COMMON /BODYCUR/XKI,XKIS,ASK,ASDKDS
	COMMON /BODYSAV/CURRIE
	COMMON /PICOSV/SWPSCL,CLINE
        COMMON /NONFREQ/IFRSCAL       
	COMMON /KEMALIK/WEUE,UEBL
	COMMON /MUSS3D/TNORM
	COMMON /I2NDBIT/ROEDGE,RONORM
        COMMON /INUTTY/wrscale
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	IERFIL=0
	IWHERE=1
	I=0
	READ(8,*,END=80,ERR=80)
        READ(8,*,END=70,ERR=70)IPROF,NPOINT,XPOS,xdum,SPOS,CK
	NPOINT=NPOINT+1
	CDKDS=0.D0
	CK=-CK
	IWHERE=0
	IF(NPOINT > NQ)GOTO 50
	READ(8,*,END=70,ERR=70)
	READ(8,*,END=70,ERR=70)Rlocal,Medge,phitdum,Uedge
	READ(8,*,END=70,ERR=70)
	READ(8,*,END=70,ERR=70)DELTA,roedge,Tedge,viscose
	READ(8,*,END=70,ERR=70)

	PINFTY=1.D0/(GAMMA*Medge*Medge)
	WSPAN=Uedge
	WMACH=Medge
	RBAR=Rlocal
	WDELTA=DELTA
	DELZER=DELTA
!
!	in PSEMEAN.PRO this is scaled with Chord
!	CURLO=DELTA/CHORD
	CURLO=DELTA
!   skip bit if pse analysis required ***************************
        IF(KGRID == 1)GOTO 999
        UZERO=Uedge
        MZERO=Medge
        RZERO=Rlocal
	VISCOS=viscose
        TZERO=Tedge
        DZERO=DELTA
!	in PSEMEAN.PRO this is scaled with Chord
!	CURRIE=DZERO/CHORD
	CURRIE=DZERO
	
        PZERO=CHORD/DZERO
        wr=2.D0*PI*FREQ*DZERO/UZERO
	BETAINF=BETA
        OMEGA=DCMPLX(WR,0.D0)
!	HAFOMG=OMEGA/IFRSCAL
	HAFOMG=OMEGA

!	icods compatible
!	eurotrans folk like things dimensional ...
!	BETA=BETA/PZERO
	BETA=CHORD*BETA/PZERO


!       setting nondimensional suction control parameters...
	DENSITY=roedge

        DZclev= 6.125D-04
        UZclev= 14.72155580333787

	IF(IVARIP == 0)THEN
	PFACTOR=1.D0
	ELSE
	PFACTOR=1.D0/(GAMMA*MZERO*MZERO)
	END IF

        WOMG=WOMG*UZERO*DENSITY*PFACTOR
	YLAMBDA=YLAMBDA/DZERO
        wrscale=UZERO/(2.D0*PI*DZERO)

        DO 909 I=1,NFREQ
	OMRANGE(I)=2.D0*PI*FRENCY(I)*DZERO/UZERO
909     CONTINUE

	CALL CONSET


	NCRBET=NCROSS
!	icods compatible
!	eurotrans folk like things dimensional and in line of flight...
        DO 202 I=1,NCROSS
	CRFBETA(I)=CHORD*WAVBETA(I)/PZERO
202     CONTINUE
	BETONLY=CRFBETA(1)

        NSTRALF=NWAVE
        DO 20002 I=1,NSTRALF
        STRALFA(I)=CHORD*WAVALFA(I)/PZERO
        STRALIM(I)=CHORD*WAVALIM(I)/PZERO
20002   CONTINUE


999     CONTINUE
!   skip bit if pse analysis required ***************************

	ASK=CK*CURRIE
	ASDKDS=CDKDS*CURRIE*CURRIE

        REX=RLOCAL*SPOS*CHORD/DELTA
        LNORM=DZERO/DELTA
        UNORM=UEDGE/UZERO
	TNORM=TEDGE/TZERO
	
        XNEW=SPOS*PZERO
	RONORM=ROEDGE/DENSITY

	PNORM=RONORM*UNORM*UNORM

!	dimensionalising lenth scale for output of results
	RESSCL=(DIMSCL(1)*PZERO+DIMSCL(2)/LNORM+ &
     & DIMSCL(3)+DIMSCL(4)/DZERO)*RESCALE


	IF(ICNORM == 0)GOTO 7766

!  read the local basic state
	I=0
        DO 10 I=1,NPOINT
        READ(8,*,END=70,ERR=70)nrub,QNETA(I),QU(I),QDU(I),QD2U(I), &
     & QWS(I),QDW(I),QD2W(I),QTS(I),QDT(I),QD2T(I)
        QNETA(I)=QNETA(I)/LNORM
        QU(I)=QU(I)*UNORM
        QDU(I)=QDU(I)*UNORM*LNORM
        QD2U(I)=QD2U(I)*UNORM*LNORM*LNORM
        QWS(I)=QWS(I)*UNORM
        QDW(I)=QDW(I)*UNORM*LNORM
        QD2W(I)=QD2W(I)*UNORM*LNORM*LNORM
        QTS(I)=QTS(I)*TNORM
        QDT(I)=QDT(I)*TNORM*LNORM
        QD2T(I)=QD2T(I)*TNORM*LNORM*LNORM
10      CONTINUE

! now read the non-parallel basic state
	READ(8,*,END=40,ERR=40)
        I=0
        DO 1001 I=1,NPOINT
        READ(8,*,END=40,ERR=40)nrub,xdum,QV(I),QVY(I),QVYY(I),QUX(I), &
     & QDUX(I),xdum,xdum,xdum,QWX(I),QTX(I),xdum,QDTX(I)

        CALL MLOCAL(I,Medge,QU(I),QTS(I))

        QUX(I)=QUX(I)*UNORM/PZERO
        QDUX(I)=QDUX(I)*UNORM*LNORM/PZERO
        QD2UX(I)=0.D0
        QV(I)=QV(I)*UNORM
        QVY(I)=QVY(I)*UNORM*LNORM
        QVYY(I)=QVYY(I)*UNORM*LNORM*LNORM
        QWX(I)=QWX(I)*UNORM/PZERO
        QDWX(I)=0.D0
        QD2WX(I)=0.D0
        QTX(I)=QTX(I)*TNORM/PZERO
        QDTX(I)=QDTX(I)*TNORM*LNORM/PZERO
        QD2TX(I)=0.D0

1001    CONTINUE

!	rescale edge pesure and gradient....
	PEDGE=PINFTY*PNORM
	PXGRAD=-QUX(NPOINT)*PZERO/UNORM
	PGRAD=PXGRAD*PNORM/PZERO
	PZGRAD=0.D0

	GOTO 6677

7766    I=0
! force to re-strech again

!	rescale edge pesure and gradient....
	PEDGE=PINFTY
	PGRAD=0.D0
	PGRADZ=0.D0
	ASK=CK*CURLO
	ASDKDS=CDKDS*CURLO*CURLO
	CALL CONSET

	DO 103 I=1,NPOINT
        READ(8,*,END=70,ERR=70)nrub,QNETA(I),QU(I),QDU(I),QD2U(I), &
     & QWS(I),QDW(I),QD2W(I),QTS(I),QDT(I),QD2T(I)
103     CONTINUE
        DO  I=1,NPOINT+1
	IF(ICoDS == 1)THEN
	READ(8,*,END=40,ERR=40)
	end if
        QUX(I)=0.D0
        QDUX(I)=0.D0
        QD2UX(I)=0.D0
        QV(I)=0.D0
        QVY(I)=0.D0
        QVYY(I)=0.D0
        QWX(I)=0.D0
        QDWX(I)=0.D0
        QD2WX(I)=0.D0
        QTX(I)=0.D0
        QDTX(I)=0.D0
        QD2TX(I)=0.D0
	end do

6677    CONTINUE

	WALBSIC=QDU(1)/(UNORM*LNORM)
	HEATSIC=QDT(1)/(TNORM*LNORM)
	CRRESC=1.D0/(UNORM*LNORM)

	uz1 = qdu(1)
	vz1 = qdw(1)
	phizero = datan2(qws(npoint),qu(npoint))
	WEUE=QWS(NPOINT)/UNORM
	UEBL=QU(NPOINT)/UNORM


        NNP=NPOINT
	CALL SPLINE(NNP,QNETA,QU,QDU,QD2U,QUX,QDUX,QD2UX,QV, &
     & QWS,QDW,QD2W,QWX,QDWX,QD2WX,QTS,QDT,QD2T,QTX,QDTX,QD2TX, &
     & QVY,QVYY)
        YO=0.D0
        DO 20 I=1,ND
        YINT=ULENTH*YO/(G-YO)
        CALL INTERP(YINT,QNETA,US,U1,U2,UX0,UX1,UX2,V0,WQS, &
     & W1,W2,WX0,WX1,WX2,TQS,T1,T2,TX0,TX1,TX2,V1,V2)

        U(I)=US
        DU(I)=U1
        D2U(I)=U2
        UX(I)=UX0
        DUX(I)=UX1
        D2UX(I)=0.D0
        V(I)=V0
        DV(I)=V1
        D2V(I)=V2
        WS(I)=WQS
        DW(I)=W1
        D2W(I)=W2
        WX(I)=WX0
        DWX(I)=0.D0
        D2WX(I)=0.D0

        TS(I)=TQS
        DT(I)=T1
        D2T(I)=T2
        TX(I)=TX0
        DTX(I)=TX1
        D2TX(I)=0.D0

	XKI(I)=1.D0/(1.D0-YINT*ASK)
	XKIS(I)=YINT*ASDKDS*XKI(I)*XKI(I)

        YO=YO+DYP
20      CONTINUE

        IF(KGRID == 1 .AND. ICNORM /= 0)RETURN
	CALL GRIDIT(ND,DYP,WALSLOP)
        KGRID=1
        RETURN

80	WRITE(*,*)
        PRINT*,'No more mean boundary layer data in PSECoDS.PRO :'
	WRITE(*,33)IPROF
33	FORMAT(2X,'Analysis aborted at profile ',I0)
        IERFIL=1
	RETURN
70      PRINT*,'Error in local boundary layer data input :'
        PRINT*,'Premature end of analysis - Check PSECoDS.PRO ' 
        WRITE(*,22)IPROF+IWHERE,I
        IERFIL=1
	RETURN
40      PRINT*,'Error in non-local boundary layer data input :'
        PRINT*,'Premature end of analysis - Check PSECoDS.PRO ' 
        WRITE(*,22)IPROF+IWHERE,I
        IERFIL=1
	RETURN
22      FORMAT(1X,'Error in data at profile ',I0,' : Line ',I0)

50      PRINT*,'No. of data points allowed to define the Mean Boundary'
        WRITE(*,55)IPROF+IWHERE
        WRITE(*,56)NQ,NPOINT
55      FORMAT(1X,'Layer profile at station ',I0,' has been exceeded')
56      FORMAT(1X,'No. of points must be < ',I0, &
     & ' : you have npoint = ',I0)
        IERFIL=1
        RETURN
        END

        SUBROUTINE ENBAE2(IALC,ENMETH,IERALFA)
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5,KE=IL*MP)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        CHARACTER*58 PSELABL
        CHARACTER*80 TITLE
	REAL*8 DIMSCL(4)
        REAL*8 XCVAL(NPOST),RDVAL(NPOST),SDVAL(NPOST)
        REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
        COMPLEX*16 QZLIN(KSTR)
        REAL*8  WAVBETA(KCRF)
	REAL*8 PHIN(NPOST)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
	REAL*8 DXVAL(NPOST)
        REAL*8 ENMETH(KSTR,KCRF,NPOST)
        COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	INTEGER IERALFA(KSTR,KCRF)
        COMMON /EISPC/ALFAZO
	COMMON /PICTR/TITLE
	COMMON /MOHD786/DXVAL
        COMMON /MUADZAM/EURALFA
	COMMON /RELVECT/PHIN
        COMMON /PISAVE/PI
        COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
        COMMON /POSITN/XPOS,SPOS,PZERO,REX

	DEGREES=180.D0/PI

        CALL FILHAND(999,LABEL(1:ICHART),'Q','.BAe',PSELABL)
        OPEN(26,FILE=PSELABL,STATUS='UNKNOWN')
	DO 10 J=1,NCRBET
	DO 10 I=1,NFREQ
        WRITE(26,101)FRENCY(I),WAVBETA(J),LABEL(1:ICHART)
	WRITE(26,102)J,IERALFA(I,J),8
	WRITE(26,103)
	DO 20 K=1,IERALFA(I,J)

        PHI=PHIN(K)*DEGREES
	PSI=DATAN(CRFBETA(J)/DIMAG(ALFAZO(I,J,K)))*DEGREES-PHI

        WRITE(26,123)XCVAL(K),SDVAL(K),FRENCY(I),WAVBETA(J),PSI,PHI, & 
     & DBLE(ALFAZO(I,J,K)*RESSCL),ENMETH(I,J,K)

20	CONTINUE
10	CONTINUE


123	FORMAT(14(1x,g13.6))
101     FORMAT(F7.1,' -freq',3X,'b=',F6.1,2X,'TEST case :',A21 )
102	FORMAT(3I9)
103     FORMAT(7X,'x/c',11X,'s/c',6X,'freq_(Hz)',6X,'beta_(/m)', & 
     & 9X,'psi',11X,'phi',11x,'sigma',9x,'N')

        RETURN
        END  

	

        SUBROUTINE PRFCoPS(ND)
        use param_info, only : MP, MODE, KMODE
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        COMPLEX*16 OMEGA,HAFOMG
	COMPLEX*16 QZLIN(KSTR)
	REAL*8, DIMENSION(KSTR) :: FRENCY,OMRANGE,STRALIM,WAVALFA, &
     & STRALFA,WAVALIM
	REAL*8, DIMENSION(KCRF) :: WAVBETA, CRFBETA
	REAL*8 DIMSCL(4)
        REAL*8, DIMENSION(NQ) :: QU,QV,QD2U,QDU,QUX,QDUX, &
     & QD2UX,QNETA,QVY,QVYY,QWS,QD2W,QDW,QWX,QDWX,QD2WX,QTS,QD2T,QDT, &
     & QTX,QDTX,QD2TX
        REAL*8, DIMENSION(MP) :: U, D2U, DU, UX, DUX, &
     & D2UX,V,DV,D2V,WS,D2W,DW,WX,DWX,D2WX,TS,D2T,DT,TX,DTX,D2TX
	REAL*8, DIMENSION(MP) :: XKI, XKIS, XFLO
        REAL*8 Medge,MZERO,LNORM,MO
	REAL*8 FRSCAN(6),JFCFRS(6)
	INTEGER JOUTSC(2)
	EXTERNAL UMUVISC
	COMMON /IFLARE/IREVFLOW,IBACOMPAT
	COMMON /FLAREAPPX/XFLO
        COMMON /INUTTY/wrscale
	common /nleuro/phizero,uz1,vz1
        COMMON /PHWAVEN/WAVALFA,STRALFA,WAVALIM,STRALIM
        COMMON /IPHIWAV/NWAVE,IGLBX
	COMMON /A1AUTO/QNETA
        COMMON /QZBETA/WAVBETA
        COMMON /QZXXXX/NCROSS
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /JOVIAN/JOUTSC,RESCALE
        COMMON /NICEINF/RESIDU
        COMMON /NICINTG/ICNORM
	COMMON /SOMMER/LNORM
        COMMON /GARTEUR/UNORM
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /SWPPRF/WS,DW,D2W,WX,DWX,D2WX
        COMMON /VELPRF/U,DU,D2U,UX,DUX,D2UX,V
        COMMON /VELVVV/DV,D2V
	COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /SCALI/G,ULENTH
        COMMON /SIMPL/DY,YO,YSTART
        COMMON /IMPLINT/NDIV
        COMMON /TEMESH/DYP,YEND,YEDGE
        COMMON /GROPT/KGRID
        COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /SURFACE/PWALL,PEDGE,PGRAD
	COMMON /SURF3Z/PGRADZ
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
        COMMON /STEPNG/XNEW
        COMMON /INFORM/NSTART,NFINAL,NINTR
        COMMON /RQFORM/FREQ,BETA
	COMMON /RQINFO/BETAINF
        COMMON /NEPAL/HAFOMG
        COMMON /FRVARI/OMEGA,RZERO
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
	COMMON /FILINF/IERFIL
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /PHASIT/ITWO,KCARRY
	COMMON /TRPOINT/WALSLOP,WALBSIC,CRRESC
	COMMON /ACTBIT/YLAMBDA,WOMG
	COMMON /PISAVE/PI
	COMMON /LADIOT/FRSCAN,BETONLY
	COMMON /cylind/VISCOS,RINFTY,SWEPTY
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IPRIMIT/IVARIP
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TSUO,TSU1,STOKES
        COMMON /BODYCUR/XKI,XKIS,ASK,ASDKDS
	COMMON /BODYSAV/CURRIE
	COMMON /PICOSV/SWPSCL,CLINE
        COMMON /NONFREQ/IFRSCAL       
	COMMON /KEMALIK/WEUE,UEBL
	COMMON /MUSS3D/TNORM
	COMMON /I2NDBIT/ROEDGE,RONORM
	COMMON /FSU/FoSQ
	COMMON /SXXXXX/ISIM,Ibuoy
	COMMON /YBUMPY/tdisp,tdispY,tdispYY
!	DATA FRSCAN/75.E-6,150.E-6,225.E-6,300.E-6,375.E-6,450.E-6/

	IERFIL=0
	IWHERE=1
	I=0
!	IBACOMPAT=1
	IF(IBACOMPAT == 1)THEN
        READ(8,*,END=80,ERR=70)IPROF,NPOINT,XPOS,SPOS,CK,CDKDS
     	tdisp=0.d0 ; tdispY=0.d0 ; tdispYY=0.d0 
	ELSE
        READ(8,*,END=80,ERR=70)IPROF,NPOINT,XPOS,SPOS,CK,CDKDS,tdisp, &
     & tdispY,tdispYY
	END IF
	IWHERE=0
! shahid temporary!
	CDKDS=0.D0
	IF(NPOINT > NQ)GOTO 50
        READ(8,*,END=70,ERR=70)Uedge,Medge,Rlocal,Tedge,DELTA, &
     & PINFTY,PXGRAD
	FoSQL=1.D66
! shahid finally for 3d this is nonzero! ie. pgradz
	PZGRAD=0.D0
	WSPAN=Uedge
	WMACH=Medge
	RBAR=Rlocal
	WDELTA=DELTA
	DELZER=DELTA
	CURLO=DELTA/CHORD
	FoSQ=FoSQL
!   skip bit if pse analysis required ***************************
        IF(KGRID == 1)GOTO 999
        UZERO=Uedge
        MZERO=Medge
        RZERO=Rlocal
	VISCOS=Uedge*DELTA/Rlocal
        TZERO=Tedge
        DZERO=DELTA
	CURRIE=DZERO/CHORD
	
        PZERO=CHORD/DZERO
        wr=2.D0*PI*FREQ*DZERO/UZERO
	BETAINF=BETA
        OMEGA=DCMPLX(WR,0.D0)
!	HAFOMG=OMEGA/IFRSCAL
	HAFOMG=OMEGA

!	icods compatible
!	eurotrans folk like things dimensional ...
!	BETA=BETA/PZERO
	BETA=CHORD*BETA/PZERO


!       setting nondimensional suction control parameters...
        DENSITY=UMUVISC(TEDGE)*RLOCAL/(UEDGE*DZERO)

        DZclev= 6.125D-04
        UZclev= 14.72155580333787

	IF(IVARIP == 0)THEN
	PFACTOR=1.D0
	ELSE
	PFACTOR=1.D0/(GAMMA*MZERO*MZERO)
	END IF

        WOMG=WOMG*UZERO*DENSITY*PFACTOR
	YLAMBDA=YLAMBDA/DZERO
        wrscale=UZERO/(2.D0*PI*DZERO)

        DO I=1,NFREQ
	OMRANGE(I)=2.D0*PI*FRENCY(I)*DZERO/UZERO
	END DO

	CALL CONSET


	DO I=1,6
	JFCFRS(I)=FRSCAN(I)*UZERO*RZERO/(2.D0*PI*DZERO)
	FRSCAN(I)=FRSCAN(I)*RZERO
	END DO

	NCRBET=NCROSS
!	icods compatible
!	eurotrans folk like things dimensional 
        DO I=1,NCROSS
	CRFBETA(I)=CHORD*WAVBETA(I)/PZERO
	END DO
	BETONLY=CRFBETA(1)

        NSTRALF=NWAVE
        DO I=1,NSTRALF
        STRALFA(I)=WAVALFA(I)
        STRALIM(I)=WAVALIM(I)
	END DO


999     CONTINUE
!   skip bit if pse analysis required ***************************

	ASK=CK*CURRIE
	ASDKDS=CDKDS*CURRIE*CURRIE

        REX=RLOCAL*SPOS*CHORD/DELTA
        LNORM=DZERO/DELTA
        UNORM=UEDGE/UZERO
	TNORM=TEDGE/TZERO
	
	tdisp=tdisp*PZERO
	tdispYY= tdispYY/PZERO  ! emma: h2Bit may needs a scaling with delta and chord.
        XNEW=SPOS*PZERO

        ROEDGE=UMUVISC(TEDGE)*RLOCAL/(UEDGE*DELTA)
	RONORM=ROEDGE/DENSITY

	PNORM=RONORM*UNORM*UNORM

!	dimensionalising lenth scale for output of results
	RESSCL=(DIMSCL(1)*PZERO/SWPSCL+DIMSCL(2)/LNORM+ &
     & DIMSCL(3)+DIMSCL(4)/DZERO)*RESCALE

	IF(ICNORM == 0)GOTO 7766
!	rescale edge pesure and gradient....
	PEDGE=PINFTY*PNORM
	PGRAD=PXGRAD*PNORM/PZERO
	PGRADZ=PZGRAD*PNORM/PZERO

	FoSQ=FoSQL/LNORM
	

	I=0
        DO 10 I=1,NPOINT
        READ(8,*,END=70,ERR=70)QNETA(I),QU(I),QDU(I),QD2U(I),QUX(I), &
     & QDUX(I),QD2UX(I),QV(I),QWS(I),QDW(I),QD2W(I),QWX(I), &
     & QDWX(I),QD2WX(I),QTS(I),QDT(I),QD2T(I),QTX(I),QDTX(I),QD2TX(I), &
     & QVY(I),QVYY(I)
     
	CALL MLOCAL(I,Medge,QU(I),QTS(I))

        QNETA(I)=QNETA(I)/LNORM
        QU(I)=QU(I)*UNORM
        QDU(I)=QDU(I)*UNORM*LNORM
        QD2U(I)=QD2U(I)*UNORM*LNORM*LNORM
        QUX(I)=QUX(I)*UNORM/PZERO
        QDUX(I)=QDUX(I)*UNORM*LNORM/PZERO
        QD2UX(I)=QD2UX(I)*UNORM*LNORM*LNORM/PZERO
        QV(I)=QV(I)*UNORM
        QVY(I)=QVY(I)*UNORM*LNORM
        QVYY(I)=QVYY(I)*UNORM*LNORM*LNORM
        QWS(I)=QWS(I)*UNORM
        QDW(I)=QDW(I)*UNORM*LNORM
        QD2W(I)=QD2W(I)*UNORM*LNORM*LNORM
        QWX(I)=QWX(I)*UNORM/PZERO
        QDWX(I)=QDWX(I)*UNORM*LNORM/PZERO
        QD2WX(I)=QD2WX(I)*UNORM*LNORM*LNORM/PZERO

        QTS(I)=QTS(I)*TNORM
        QDT(I)=QDT(I)*TNORM*LNORM
        QD2T(I)=QD2T(I)*TNORM*LNORM*LNORM
        QTX(I)=QTX(I)*TNORM/PZERO
        QDTX(I)=QDTX(I)*TNORM*LNORM/PZERO
        QD2TX(I)=QD2TX(I)*TNORM*LNORM*LNORM/PZERO
	
10      CONTINUE


	GOTO 6677

7766    I=0
!	rescale edge pesure and gradient....
	PEDGE=PINFTY
	PGRAD=PXGRAD
	PGRADZ=PZGRAD
	ASK=CK*CURLO
	ASDKDS=CDKDS*CURLO*CURLO
	CALL CONSET

	DO I=1,NPOINT
        READ(8,*,END=70,ERR=70)QNETA(I),QU(I),QDU(I),QD2U(I),QUX(I), &
     & QDUX(I),QD2UX(I),QV(I),QWS(I),QDW(I),QD2W(I), &
     & QWX(I), &
     & QDWX(I),QD2WX(I),QTS(I),QDT(I),QD2T(I),QTX(I),QDTX(I), &
     & QD2TX(I),QVY(I),QVYY(I)
     	END DO

6677    CONTINUE

	WALBSIC=QDU(1)/(UNORM*LNORM)
	HEATSIC=QDT(1)/(TNORM*LNORM)
	CRRESC=1.D0/(UNORM*LNORM)

	uz1 = qdu(1)
	vz1 = qdw(1)
	phizero = datan2(qws(npoint),qu(npoint))
	WEUE=QWS(NPOINT)/UNORM
	UEBL=1.D0
	
        NNP=NPOINT
	CALL SPLINE(NNP,QNETA,QU,QDU,QD2U,QUX,QDUX,QD2UX,QV, &
     & QWS,QDW,QD2W,QWX,QDWX,QD2WX,QTS,QDT,QD2T,QTX,QDTX,QD2TX, &
     & QVY,QVYY)
        YO=0.D0
        DO 20 I=1,ND
        YINT=ULENTH*YO/(G-YO)
        CALL INTERP(YINT,QNETA,US,U1,U2,UX0,UX1,UX2,V0,WQS, &
     & W1,W2,WX0,WX1,WX2,TQS,T1,T2,TX0,TX1,TX2,V1,V2)

        U(I)=US
        DU(I)=U1
        D2U(I)=U2
        UX(I)=UX0
        DUX(I)=UX1
        D2UX(I)=UX2
        V(I)=V0
        DV(I)=V1
        D2V(I)=V2
        WS(I)=WQS
        DW(I)=W1
        D2W(I)=W2
        WX(I)=WX0
        DWX(I)=WX1
        D2WX(I)=WX2

        TS(I)=TQS
        DT(I)=T1
        D2T(I)=T2
        TX(I)=TX0
        DTX(I)=TX1
        D2TX(I)=TX2
	
	IF(IREVFLOW == 1)THEN
		IF(US < 0.d0)THEN
	XFLO(I)=0.d0
		ELSE
	XFLO(I)=1.d0
		END IF
	ELSE
	XFLO(I)=1.d0
	END IF

	XKI(I)=1.D0/(1.D0-YINT*ASK)
	XKIS(I)=YINT*ASDKDS*XKI(I)*XKI(I)

        YO=YO+DYP
20      CONTINUE

        IF(KGRID == 1 .AND. ICNORM /= 0)RETURN
        CALL GRIDIT(ND,DYP,WALSLOP)
        KGRID=1
        RETURN

80	WRITE(*,*)
        PRINT*,'No more mean boundary layer data in PSEMEAN.PRO :'
	WRITE(*,33)IPROF
33	FORMAT(2X,'Analysis aborted at profile ',I0)
        IERFIL=1
	RETURN
70      PRINT*,'Error in mean boundary layer data input :'
        PRINT*,'Premature end of analysis - Check PSEMEAN.PRO ' 
        WRITE(*,22)IPROF+IWHERE,I
22      FORMAT(1X,'Error in data at profile ',I0,' : Line ',I0)
        IERFIL=1
	RETURN

50      PRINT*,'No. of data points allowed to define the Mean Boundary'
        WRITE(*,55)IPROF+IWHERE
        WRITE(*,56)NQ,NPOINT
55      FORMAT(1X,'Layer profile at station ',I0,' has been exceeded')
56      FORMAT(1X,'No. of points must be < ',I0, &
     & ' : you have npoint = ',I0)
        IERFIL=1
        RETURN
        END

	SUBROUTINE PROFRD(NDIV,NSTART,NZ)
	COMMON /EUTRANS/IEURO,IBAE,NATURAL,ICoDS,I2NDARY
	COMMON /K3D/IDO3D, IGLB3D, IEXTRAP3D, IGLB3D_EXIST
        IF(ICoDS >= 1)THEN
        CALL PRFCoDS(NDIV)
        ELSE
        CALL PRFCoPS(NDIV)
        END IF
        RETURN
        END



	SUBROUTINE QZALG(IA,IB,NEQ,IFREQ,IL,IBOX)
        use param_info, only : MP, MPG
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	LOGICAL MATV
	PARAMETER (LK=9,KE=(MP-1)*LK,KPG=LK*5)
	PARAMETER (KEG=(MPG-1)*LK,N=KEG+2)
	CHARACTER*80 TITLE
	COMPLEX*16 A(KE,KPG),B(KE,KPG)
	DOUBLE PRECISION ALFA(KE),GROWTH(KE)
	COMPLEX*16 EIGEN(KEG),OMEGA,ABSQ,qtop
	DOUBLE PRECISION AI(N,N),ALFI(N),ALFR(N),AR(N,N),BETA(N)
	DOUBLE PRECISION BI(N,N),BR(N,N),VI(N,N),VR(N,N)
	COMPLEX*16 AQZ(N,N),BQZ(N,N)
	INTEGER ITER(N)
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
	COMMON /GLPROP/B,A
	COMMON /GLPQZ/BQZ,AQZ
	COMMON /FRVARI/OMEGA,RE
	COMMON /PICTR/TITLE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /BRPOINT/UQZ,WQZ
	COMMON /DISCRET/Xreal,Ximag,xUPPER1,yUPPER1,xUPPER2, &
     & yUPPER2,xbufu,xbufl,xblankt,xac
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /NAGBIT/INAG,IWVLNTHx
	DTOLS=1.D-12
	IBRANCH=1
	ICONJ=0

	IENGIN=1
! ok first evalute the branch points... to identify the continuous spectrum
	IF(INAG == 1)THEN
        CALL BRANCH_NAG
	ELSE
        CALL BRANCH_CUTS
	end if

	qtop=(OMEGA-BZERO*WQZ)/UQZ
	if(DBLE(OMEGA-BZERO*WQZ) < 0.D0) ICONJ=1

	XCUT=DBLE(OMEGA)*.95
	XMAX=XCUT*15.

	IMOTHER=1
	IF(IMOTHER == 0)THEN

        XBELOW=(DBLE(OMEGA)-BZERO*WQZ)/UQZ
	XBELOW=XBELOW-0.002*DABS(XBELOW)
	XMAXU=14.D0*DABS(XBELOW)
		IF(DABS(BZERO) < 1.D-2)THEN
		XMAX=1.5D0*XCUT
		ELSE
		XMAX=DABS(BZERO)*0.95
		XBELOW=XBELOW-15.*DABS(XBELOW)
		END IF

	ELSE
	XCUT=DBLE(OMEGA)*.95
	xeps=0.16

        XBELOW=Xreal
! here subtract off from origin!
! will be better
! these are used for icode =1 to blank out modes which lie
! near the acoustic branch.
	xblankt=xblankt*xeps
	YACOUST=DABS(YUPPER1)*0.95
	IF(ICODE == 1 .AND. IBRANCH == 1)THEN
	YFIN=4.D0*YACOUST
	YOO=-YFIN
	YD=(YFIN-YOO)*0.1
	DO kk=1,11
!	WRITE(98,'(5E16.7)')YOO, xac+xblankt, xac-xblankt
	YOO=YOO+YD
	end do
!	CLOSE(98)
	END IF

	XSONICU=xbufu*0.98
	XSONICL=Xbufl*0.98
	yblank=dabs(XSONICU-XSONICL)*1.D0
	yblank=0.4

        XBELOW=Xreal-0.002*DABS(XREAL)
        XMAXU=16.D0*DABS(XBELOW)
	IF(XMAXU < 0.04)THEN
	IF(DABS(BZERO) < 0.02)THEN
	XMAXU=DBLE(OMEGA)*4.
	else 
	XMAXU=DABS(BZERO)*4.
	end if
	end if
        IF(DABS(BZERO*WQZ) < 1.D-2)THEN
        XMAX=3.5D0*DABS(XCUT)
        ELSE
	XMAX=DABS(BZERO)*4.5
	XBELOW=XBELOW-20.*DABS(XBELOW)
        END IF
	END IF

	DO I=1,NEQ
	DO J=1,NEQ
	AR(I,J)=0.D0 ; AI(I,J)=0.D0 ; BR(I,J)=0.D0 ; BI(I,J)=0.D0
	end do
	end do
	
 
	IF(IQZ == 0)THEN
	DO I=1,NEQ
	DO J=1,IL*5
	JMV=J+IT(I)
	AR(I,JMV)=DBLE(A(I,J))
	AI(I,JMV)=DIMAG(A(I,J))
	BR(I,JMV)=-DBLE(B(I,J))
	BI(I,JMV)=-DIMAG(B(I,J))
	end do
	end do

	ELSE
	DO I=1,NEQ
	DO J=1,NEQ
	AR(I,J)=DBLE(AQZ(I,J))
	AI(I,J)=DIMAG(AQZ(I,J))
	BR(I,J)=-DBLE(BQZ(I,J))
	BI(I,J)=-DIMAG(BQZ(I,J))
	end do
	end do

	END IF

 
	K=NEQ
	EPSI=0.D0
	MATV=.FALSE.
        CALL QZLIB(K,N,AR,AI,BR,BI,EPSI,ALFR,ALFI, &
     &                 BETA,MATV,VR,VI,ITER,IFAIL)

	IF(IBOX == 1)GOTO 11011
	IF(IQZFULL == 1)THEN
	WRITE(19,'(A80)')TITLE
	WRITE(19,23)
	ELSE
	END IF
	WRITE(15,'(A80)')TITLE
	WRITE(15,23)
11011	CONTINUE

	ICount=1
	DO 40 I=1,K
	IF(DABS(BETA(I)) > DTOLS)THEN
	ALFR(I)=ALFR(I)/BETA(I)
	ALFI(I)=ALFI(I)/BETA(I)
	ELSE
	END IF
 
	IF(DABS(ALFR(I)) > DTOLS .OR. DABS(ALFI(I)) > DTOLS)THEN
	EIGEN(I)=1.D0/DCMPLX(ALFR(I),ALFI(I))
	VAL=DBLE(EIGEN(I))
	V2=DIMAG(EIGEN(I))

	IF(IQZFULL == 1)THEN
	IF(ICONJ == 0)THEN
	WRITE(19,99994)ICOUNT,DIMAG(EIGEN(I)),DBLE(EIGEN(I)), &
     & qtop/DCMPLX(DIMAG(EIGEN(I)),-DBLE(EIGEN(I)))
	ELSE
	WRITE(19,99994)ICOUNT,DIMAG(EIGEN(I)),DBLE(EIGEN(I)), &
     & DCONJG(qtop/DCMPLX(DIMAG(EIGEN(I)),-DBLE(EIGEN(I))))
	END IF
	ELSE
	END IF


	IF(IMOTHER == 0)THEN

	IF(V2 < XBELOW .OR. V2 > XMAXU) GOTO 111
	IF(DABS(VAL) > XMAX)GOTO 111
	IF(VAL > 0.875D0*DABS(V2))GOTO 111

	ALFA(ICOUNT)=DIMAG(EIGEN(I))
	GROWTH(ICOUNT)=DBLE(EIGEN(I))
	IF(IBOX == 1)GOTO 11012
	WRITE(15,99994)ICOUNT,DIMAG(EIGEN(I)),DBLE(EIGEN(I))
11012	CONTINUE

	ICount=ICount+1
111	CONTINUE

! case select IMOTHER
	ELSE
!	PRINT*,XBELOW, XMAXU, XMAX,' KKKKK'
        IF(V2 < XBELOW .OR. V2 > XMAXU) GOTO 101
        IF(DABS(VAL) > XMAX)GOTO 101
	
	IF(IENGIN == 1)THEN
	IF(VAL > 0.09999  .and. DABS(BZERO) > 0.04 )GOTO 101
	end if

! first blank out points lying along acoustic branch cuts

	IF(ICODE == 1)THEN
	IF(DABS(VAL) >= YACOUST)THEN
	IF(V2 <= xblankt+xac .AND. V2 >= xac-xblankt)GOTO 101 
	end if

	ELSE IF(ICODE == 2)THEN
	IF(V2 >= XSONICU .AND. DABS(VAL) <= yblank*DABS(V2))GOTO 101 
	IF(V2 <= XSONICL .AND. DABS(VAL) <= yblank*DABS(V2))GOTO 101 
	END IF
191	CONTINUE

        ALFA(ICOUNT)=DIMAG(EIGEN(I))
        GROWTH(ICOUNT)=DBLE(EIGEN(I))
        IF(IBOX == 1)GOTO 11013
        WRITE(15,99994)ICOUNT,DIMAG(EIGEN(I)),DBLE(EIGEN(I))
11013   CONTINUE

        ICount=ICount+1
101     CONTINUE

	END IF


	ELSE
	END IF
40	CONTINUE
	IF(IMOTHER == 0)THEN
	CALL SEARCH(IA,IB,ALFA,GROWTH,ICOUNT-1,IFREQ,IBOX)
	ELSE
	CALL SEARCH2(IA,IB,ALFA,GROWTH,ICOUNT-1,IFREQ,IBOX)
	END IF
	CALL P_SPEC(XMAX)
	RETURN
99996	FORMAT(2x,'ERROR IN QZLIB IFAIL = ',I0)
23	FORMAT(' ID_AUTO',4X,'Wave_No.',8X,'Growth_Rate')
99994	FORMAT(i5,4E19.8)
	END SUBROUTINE QZALG




	SUBROUTINE RAYLEI(IL,NB,NPTS,XSOL,IGERR)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (ISPARSE=5)
	PARAMETER (LK=9,KE=(MP-1)*LK,KGP=LK*ISPARSE)
	COMPLEX*16 A(KE,KGP),UB(KE,KGP),B(KE,KGP)
	COMPLEX*16 CMU,CML,EIGEN,DCSUM
	COMPLEX*16 SMUP,SMLO,SMALIK,SMCHEK
	COMPLEX*16 DCXSOL(KE)
	COMPLEX*16 DCRHS(KE),RHS(KE)
	COMPLEX*16 XSOL(*)
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
	COMMON /SHAPE/RHS,DCRHS
        COMMON /ILUILU/MPDDE,MEQ,M3,MFOR
	COMMON /ROOTS/SMALIK
	COMMON /GLPROP/A,B
        COMMON /WHATP/IEIG,IRAY,IRAYCNT
        COMMON /WHREL/FACTOR
	COMMON /GLOBAL/EIGEN,DTOL,TPSL,TPSM,TOLNLR
	COMMON /SPFLG/ISPC,IOUT,IHOW,IPAR
        COMMON /LAYOUT/IDIV,ItSpres,IFIRST
	COMMON /NICEINF/RESIDU
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /POLISH/IOMSW
        COMMON /NCURVES/INEUTRL

	M2S=IL+1
	IGERR=0
	DO I=1,IL
	DO J=1,M3
	UB(I,J)=A(I,J)+EIGEN*B(I,J)
	end do
	end do
	
	DO I=M2S,MEQ
	DO J=1,MFOR
	UB(I,J)=A(I,J)+EIGEN*B(I,J)
	end do
	end do
 
	I=MEQ
	ILISP=IL*ISPARSE
	DO 80 IP=3,NB-2
	DO KPO=1,IL
	I=I+1
	DO J=1,ILISP
	UB(I,J)=A(I,J)+EIGEN*B(I,J)
	end do
	end do
80	CONTINUE

	NBUT1=(NB-2)*IL+1
	DO  I=NBUT1,NPTS-IL
	DO  J=1,MFOR
	UB(I,J)=A(I,J)+EIGEN*B(I,J)
	end do
	end do
 
	NBIT=(NB-1)*IL+1
	DO  I=NBIT,NPTS
	DO  J=1,M3
	UB(I,J)=A(I,J)+EIGEN*B(I,J)
	end do
	end do
	CALL LOWUPP(NB,NPTS,IL,UB)
	IFLAG=1
	MSM=0
	SMCHEK=EIGEN
	IF(ITSPRES >= 1)GOTO 198
	IF(IPAR /= 0)GOTO 198
	IF(IOMSW == 1)THEN
	WRITE(ITV,241)
	ELSE
	WRITE(ITV,24)
	END IF
24	FORMAT(3x,' Nit',6x,'Wave-No.',6x,'Growth Rate',7x,'Residual')
241	FORMAT(3x,' Nit',6x,'Omega_R ',6x,'  Omega_I',7x,'Residual')
23	FORMAT(i6,2F16.8,E16.4)
198	MSM=MSM+1
 
	CALL LUSOLV(NPTS,UB,1,2,RHS,XSOL)
	CALL LUSOLT(NPTS,UB,2,1,DCRHS,DCXSOL)
	CALL CNORM(IL,NPTS,XSOL,DCXSOL)
!  ****************************
 
	SMUP=0.D0 ; SMLO=0.D0
 	DO 200 I=1,IL
	CMU=0.D0 ; CML=0.D0; DCSUM=0.D0
	IEX=IT(I)
 
	DO  J=1,M3
	JP=J+IEX
	CMU=CMU+A(I,J)*XSOL(JP)
	CML=CML-B(I,J)*XSOL(JP)
	DCSUM=DCSUM-B(JP,I-IT(JP))*DCXSOL(JP)
	end do
 
	RHS(I)=CMU
	DCRHS(I)=DCSUM
	SMUP=SMUP+CMU*DCXSOL(I)
	SMLO=SMLO+CML*DCXSOL(I)
200	CONTINUE
!
 	DO 2005 I=M2S,MEQ
	CMU=0.D0 ; CML=0.D0; DCSUM=0.D0
	IEX=IT(I)
 
	DO J=1,MFOR
	JP=J+IEX
	CMU=CMU+A(I,J)*XSOL(JP)
	CML=CML-B(I,J)*XSOL(JP)
	DCSUM=DCSUM-B(JP,I-IT(JP))*DCXSOL(JP)
	end do
 
	RHS(I)=CMU
	DCRHS(I)=DCSUM
	SMUP=SMUP+CMU*DCXSOL(I)
	SMLO=SMLO+CML*DCXSOL(I)
2005	CONTINUE

	DO 230 I=MEQ+1,NPTS-MEQ
	CMU=0.D0 ; CML=0.D0; DCSUM=0.D0
	IEX=IT(I)
 
	DO J=1,IL*ISPARSE
	JP=J+IEX
	CMU=CMU+A(I,J)*XSOL(JP)
	CML=CML-B(I,J)*XSOL(JP)
	DCSUM=DCSUM-B(JP,I-IT(JP))*DCXSOL(JP)
	end do
 
	RHS(I)=CMU
	DCRHS(I)=DCSUM
	SMUP=SMUP+CMU*DCXSOL(I)
	SMLO=SMLO+CML*DCXSOL(I)
230	CONTINUE

	DO 260 I=NBUT1,NBIT-1
	CMU=0.D0 ; CML=0.D0; DCSUM=0.D0
	IEX=IT(I)
	
	DO  J=1,MFOR
	JP=J+IEX
	CMU=CMU+A(I,J)*XSOL(JP)
	CML=CML-B(I,J)*XSOL(JP)
	DCSUM=DCSUM-B(JP,I-IT(JP))*DCXSOL(JP)
	end do
 
	RHS(I)=CMU
	DCRHS(I)=DCSUM
	SMUP=SMUP+CMU*DCXSOL(I)
	SMLO=SMLO+CML*DCXSOL(I)
260	CONTINUE
 
	DO 9000 I=NBIT,NPTS
	CMU=0.D0 ; CML=0.D0; DCSUM=0.D0
	IEX=IT(I)
	DO  J=1,M3
	JP=J+IEX
	CMU=CMU+A(I,J)*XSOL(JP)
	CML=CML-B(I,J)*XSOL(JP)
	DCSUM=DCSUM-B(JP,I-IT(JP))*DCXSOL(JP)
	end do
	RHS(I)=CMU
	DCRHS(I)=DCSUM
	SMUP=SMUP+CMU*DCXSOL(I)
	SMLO=SMLO+CML*DCXSOL(I)
9000	CONTINUE
	SMALIK=SMUP/SMLO
	SZEROx=DABS(DBLE(SMALIK))
        IF(INEUTRL == 1 .OR. SZEROx < 1.D-5)THEN
        VALU=CDABS((SMALIK-SMCHEK)/SMCHEK)
        ELSE
	VALU=DABS((DBLE(SMALIK)-DBLE(SMCHEK))/DBLE(SMCHEK))
	end if
!	VALU=CDABS((SMALIK-SMCHEK)/SMCHEK)
	SMCHEK=SMALIK
	RESIDU=VALU
	IF(VALU < 1.D5)GOTO 1984
	IGERR=1
	GOTO 1050
1984	CONTINUE
	IF(IPAR /= 0)GOTO 5678
	IF(ITSPRES >= 1)GOTO 5678
	IF(IOMSW == 1)THEN
 	WRITE(ITV,23)MSM,DBLE(SMCHEK),DIMAG(SMCHEK),VALU
	ELSE
 	WRITE(ITV,23)MSM,DIMAG(SMCHEK),DBLE(SMCHEK),VALU
	END IF
5678	CONTINUE
	IF(MSM > IRAY)GOTO 7777
	IF(VALU > DTOL)GOTO 198
	RETURN
7777	WRITE(ITV,*)
	IGERR=2
	IF(IRAYCNT == 1)RETURN
	WRITE(ITV,*)' Tolerance in Rayleigh Iteration too large - '
	WRITE(ITV,13)VALU
13	FORMAT('Analysis abandoned : Residual =',E14.6) 
	IGERR=1
	RETURN
1050	PRINT*,'      '
	PRINT*,'Residual in Rayleigh too large !!! - Have You Used'
        PRINT*,'the Correct Initial Estimate for the Eigenvalue ?'
        PRINT*,'Try QZ-Algorithm for the Initial Estimate'
	PRINT*,'If This Does Not Work Either Refine Grid In QZ or'
        PRINT*,'Inspect Full Eigenvalue Spectrum of Unit 15 & Check' 
	PRINT*,'Whether the Eigenvalue Search Routine hAs Selected'
	PRINT*,'the Correct Discrete Root'
	WRITE(*,*)
	RETURN
	END


        SUBROUTINE ROTATE(NQ,NPT,THETA,U,W)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        REAL*8 U(*),W(*)
	DO I=1,NPT
	xNU=U(I)*DCOS(THETA)+W(I)*DSIN(THETA)
	xNW=U(I)*DSIN(THETA)-W(I)*DCOS(THETA)
	U(I)=xNU
	W(I)=xNW
	end do
	RETURN
        END



	SUBROUTINE SETPAR(NDIV,LK)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9)
	COMPLEX*16 SIBETA,SIOM,CROSS
	COMPLEX*16 SING,OMEGA,ABSQ
        COMPLEX*16, DIMENSION(IL,IL,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION US(MP),V(MP),D2U(MP),DU(MP),UX(MP),DUX(MP)
	DOUBLE PRECISION D2UX(MP)
	DOUBLE PRECISION, DIMENSION(MP) :: A1, A2, B1
	DOUBLE PRECISION TS(MP),D2T(MP),DT(MP),TX(MP),DTX(MP),D2TX(MP)
	DOUBLE PRECISION WS(MP),D2W(MP),DW(MP),WX(MP),DWX(MP),D2WX(MP)
	COMPLEX*16 PC1,PC2,PC3,UBIT,WBIT,TBIT
        REAL*8, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
	COMMON /DICTAT/Y,YA,Z,ZA
        COMMON /ABIDA/ROV,UMAT,ROYX,ROY
	COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,V
	COMMON /SWPPRF/WS,DW,D2W,WX,DWX,D2WX
	COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /FRVARI/OMEGA,RE
	COMMON /ZCLEAR/SING
	COMMON /STRECH/A1,A2,B1
	COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
	COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT
	COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /ACTBIT/YLAMBDA,WOMG
        COMMON /SWEPTIN/DUBLU
	COMMON /SURFACE/PWALL,PEDGE,PGRAD
	COMMON /IPRIMIT/IVARIP
	COMMON /FSU/FoSQ
	COMMON /SXXXXX/ISIM,Ibuoy

	CAPGAM=ZAPGAM

        DUBLU=WS(NDIV)

	SIBETA=SING*BZERO
	SIOM=SING*OMEGA

	TEMPER=TS(1)
	TIN=1.D0/TEMPER
	CALL VISCOS(TEMPER,TIN,FM,FMT,FMTT)
	PINKY=1.D0/(A1(1)*PEDGE)

	PC1=DU(1)*PINKY
	PC2=(SIBETA*DW(1)-SIOM*DT(1)*TIN*(1.D0+FMT*TS(1)/FM))*PINKY
	PC3=RE/(FM*RM)-SIOM/PEDGE
	UBIT=1.D0/RM
	WBIT=SIBETA/RM
	TBIT=SIOM*TIN

	DO 10 NM=1,NDIV
	D1NM=1.D0/B1(NM)
	TEMPER=TS(NM)

	CROSS=SIBETA*WS(NM)-SIOM
	TIN=1.D0/TEMPER

	CALL VISCOS(TEMPER,TIN,FM,FMT,FMTT)

	RODENT=PINV*TIN*PEDGE
        ROV(NM)=RODENT
        UMAT(NM)=1.D0/FM
        ROY(NM)=-DT(NM)*TIN*RODENT
        ROYX(NM)=0.D0

	CURLYR=RE*RODENT/FM
	SIG=CURLYR*SIGMA
	EMM=CAPGAM*RE/FM

! note dpy ne. 0 for buyoyancy....
	DPY=0.d0/PEDGE
	FGBIT=Ibuoy*RE*RODENT/(FM*FoSQ*RM)

	IF(NM /= 1)GOTO 7777
	Z(1,1,1)=(SIBETA*DW(1)-SIOM*DT(1)*TIN*(1.D0+FMT*TS(1)/FM))/PEDGE
	ZA(1,1,1)=DU(1)/PEDGE
	Z(1,4,1)=(BESQ-CURLYR*SIOM)*RMINV- & 
     & DT(1)*FMT*DT(1)*TIN/FM-TIN*D2T(1)
	ZA(1,5,1)=-RMINV

	Y(1,1,1)=PC3*A1(1)
	YA(1,2,1)=UBIT*A1(1)
	Y(1,6,1)=WBIT*A1(NM)
	Y(1,8,1)=TBIT*A1(NM)

	Z(4,1,1)=-WOMG
	Z(4,4,1)=1.D0
	Y(4,2,1)=-YLAMBDA*A1(1)
	GOTO 10

7777	CONTINUE

        Y(1,4,NM)=A1(NM)*D1NM
	Z(1,1,NM)=CROSS*D1NM/PEDGE
	ZA(1,1,NM)=US(NM)*D1NM/PEDGE
	ZA(1,2,NM)=D1NM
	Z(1,4,NM)=-DT(NM)*TIN*D1NM
	Z(1,6,NM)=SIBETA*D1NM
	Z(1,8,NM)=-CROSS*TIN*D1NM
	ZA(1,8,NM)=-US(NM)*TIN*D1NM

        Y(2,2,NM)=(A2(NM)+A1(NM)*FMT*DT(NM)/FM)*D1NM
        YA(2,4,NM)=RS*A1(NM)*D1NM
        Y(2,8,NM)=A1(NM)*FMT*DU(NM)/FM*D1NM
        ZA(2,1,NM)=-RE/FM*D1NM
        Z(2,2,NM)=(-BESQ-CURLYR*CROSS)*D1NM
        ZA(2,2,NM)=-CURLYR*US(NM)*D1NM
        ZA(2,3,NM)=RM*D1NM
        Z(2,4,NM)=-CURLYR*DU(NM)*D1NM
        ZA(2,4,NM)=FMT*DT(NM)/FM*D1NM
	ZA(2,6,NM)=SIBETA*RS*D1NM
	Z(2,8,NM)=(FMT*D2U(NM)+FMTT*DT(NM)*DU(NM))*D1NM/FM
	Z(3,3,NM)=1.D0
	ZA(3,2,NM)=-1.D0


        Y(4,1,NM)=-A1(NM)*RE*RMINV*D1NM/FM
        YA(4,2,NM)=A1(NM)*RS*RMINV*D1NM
        Y(4,4,NM)=(A2(NM)+A1(NM)*FMT*DT(NM)/FM)*D1NM
        Y(4,6,NM)=A1(NM)*RS*RMINV*SIBETA*D1NM
        ZA(4,2,NM)=YLMU*RMINV*FMT*DT(NM)*D1NM/FM
        Z(4,4,NM)=(-BESQ-CURLYR*CROSS)*RMINV*D1NM
        ZA(4,4,NM)=-CURLYR*US(NM)*RMINV*D1NM
        ZA(4,5,NM)=RMINV*D1NM
        Z(4,6,NM)=YLMU*RMINV*SIBETA*FMT*DT(NM)*D1NM/FM
        Z(4,8,NM)=RMINV*SIBETA*FMT*DW(NM)*D1NM/FM
        ZA(4,8,NM)=RMINV*FMT*DU(NM)*D1NM/FM
	Z(5,5,NM)=1.D0
	ZA(5,4,NM)=-1.D0

        Y(6,4,NM)=A1(NM)*RS*SIBETA*D1NM
        Y(6,6,NM)=(A2(NM)+A1(NM)*FMT*DT(NM)/FM)*D1NM
        Y(6,8,NM)=A1(NM)*FMT*DW(NM)*D1NM/FM
	Z(6,1,NM)=-SIBETA*RE*D1NM/FM
	ZA(6,2,NM)=SIBETA*RS*D1NM
	Z(6,4,NM)=(SIBETA*FMT*DT(NM)/FM-CURLYR*DW(NM))*D1NM
        Z(6,6,NM)=(-BESQ*RM-CURLYR*CROSS)*D1NM
        ZA(6,6,NM)=-CURLYR*US(NM)*D1NM
	ZA(6,7,NM)=D1NM
	Z(6,8,NM)=(FMT*D2W(NM)+FMTT*DT(NM)*DW(NM))*D1NM/FM
	Z(7,7,NM)=1.D0
	ZA(7,6,NM)=-1.D0

        Y(8,8,NM)=(A2(NM)+2.D0*A1(NM)*FMT*DT(NM)/FM)*D1NM
        Y(8,2,NM)=2.D0*A1(NM)*CAPGAM*DU(NM)*D1NM
        Y(8,6,NM)=2.D0*A1(NM)*CAPGAM*DW(NM)*D1NM
        Z(8,4,NM)=(2.D0*CAPGAM*SIBETA*DW(NM)-DT(NM)*SIG)*D1NM
        ZA(8,4,NM)=2.D0*CAPGAM*DU(NM)*D1NM
        Z(8,1,NM)=EMM*CROSS*D1NM
        ZA(8,1,NM)=EMM*US(NM)*D1NM
	CONST=CAPGAM*(DU(NM)*DU(NM)+DW(NM)*DW(NM))+D2T(NM)
        CONST=(FMTT*DT(NM)*DT(NM)+FMT*CONST)/FM
        Z(8,8,NM)=(CONST-BESQ-SIG*CROSS)*D1NM
        ZA(8,8,NM)=-SIG*US(NM)*D1NM
        ZA(8,9,NM)=D1NM
	Z(9,9,NM)=1.D0
	ZA(9,8,NM)=-1.D0
	
10	CONTINUE

	RETURN              
	END



	SUBROUTINE SETRUVT(NDIV,LK)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9)
	COMPLEX*16 SIBETA,SIOM,CROSS
	COMPLEX*16 SING,OMEGA,ABSQ
        COMPLEX*16, DIMENSION(IL,IL,MP) :: Y,YA,Z,ZA
	DOUBLE PRECISION US(MP),V(MP),D2U(MP),DU(MP),UX(MP),DUX(MP)
	DOUBLE PRECISION D2UX(MP)
	DOUBLE PRECISION, DIMENSION(MP) :: A1, A2, B1
	DOUBLE PRECISION TS(MP),D2T(MP),DT(MP),TX(MP),DTX(MP),D2TX(MP)
	DOUBLE PRECISION WS(MP),D2W(MP),DW(MP),WX(MP),DWX(MP),D2WX(MP)
	COMPLEX*16 PC2
        DOUBLE PRECISION, DIMENSION(MP) :: ROV, UMAT, ROYX, ROY
	COMMON /ABIDA/ROV,UMAT,ROYX,ROY
	COMMON /DICTAT/Y,YA,Z,ZA
	COMMON /VELPRF/US,DU,D2U,UX,DUX,D2UX,V
	COMMON /SWPPRF/WS,DW,D2W,WX,DWX,D2WX
	COMMON /TEMPPRF/TS,DT,D2T,TX,DTX,D2TX
        COMMON /FRVARI/OMEGA,RE
	COMMON /ZCLEAR/SING
	COMMON /STRECH/A1,A2,B1
	COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
 	COMMON /CONLOC/PINV,ZAPGAM,SIGMA,SIGHAT,RHAT
	COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /ACTBIT/YLAMBDA,WOMG
        COMMON /SWEPTIN/DUBLU
	COMMON /SURFACE/PWALL,PEDGE,PGRAD
	COMMON /IPRIMIT/IVARIP

	CAPGAM=ZAPGAM

        DUBLU=WS(NDIV)

	SIBETA=SING*BZERO
	SIOM=SING*OMEGA

	DO 10 NM=1,NDIV
	D1NM=1.D0/B1(NM)
	TEMPER=TS(NM)

	CROSS=SIBETA*WS(NM)-SIOM
	TIN=1.D0/TEMPER

	CALL VISCOS(TEMPER,TIN,FM,FMT,FMTT)
	RODENT=PINV*TIN*PEDGE
	RINKY=1.D0/RODENT
	CURLYR=RE*RODENT/FM
	SIG=CURLYR*SIGMA
	SIGAM=CURLYR*SIGHAT
	EMM=CAPGAM*RE/FM
	EMMGAM=RHAT*RE/FM

        ROV(NM)=RODENT
        UMAT(NM)=1.D0/FM
        ROY(NM)=-DT(NM)*TIN*RODENT
        ROYX(NM)=0.D0

	FMPIN=1.D0/(PINV*FM)

	ROCON=RE*RODENT*FMPIN
	ROREM=RMINV*DT(NM)*ROCON

	YMUo=FMT*DT(NM)/FM

	IF(NM == 1)THEN

	PC2=SIBETA*DW(1)-SIOM*(2.D0*DT(1)*TIN+YMUo)+ROREM
	Z(1,1,1)=PC2*RINKY
	ZA(1,1,1)=DU(1)*RINKY

	Z(1,4,1)=(BESQ-CURLYR*SIOM)*RMINV-TIN*(D2T(1)+DT(1)*YMUo)
	ZA(1,5,1)=-RMINV

	Z(1,8,1)=-ROREM*TIN-FMT*RMINV*SIBETA*DW(1)/FM
	ZA(1,8,1)=-FMT*RMINV*DU(1)/FM	

	Y(1,1,1)=(TEMPER*RMINV*ROCON-SIOM)*RINKY*A1(1)
	YA(1,2,1)=RMINV*A1(1)
	Y(1,6,1)=SIBETA*RMINV*A1(1)
	Y(1,8,1)=RMINV*ROCON*A1(1)

	Z(4,1,1)=-WOMG*TEMPER
	Z(4,4,1)=1.D0
	Z(4,8,1)=-WOMG*RODENT
	Y(4,2,1)=-YLAMBDA*A1(1)

	ELSE

        Y(1,4,NM)=A1(NM)*D1NM
	Z(1,1,NM)=CROSS*D1NM*RINKY
	ZA(1,1,NM)=US(NM)*D1NM*RINKY
	ZA(1,2,NM)=D1NM
	Z(1,4,NM)=-DT(NM)*TIN*D1NM
	Z(1,6,NM)=SIBETA*D1NM

        Y(2,2,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        YA(2,4,NM)=RS*A1(NM)*D1NM
        Y(2,8,NM)=A1(NM)*FMT*DU(NM)/FM*D1NM
        ZA(2,1,NM)=-RE*TEMPER*FMPIN*D1NM
        Z(2,2,NM)=(-BESQ-CURLYR*CROSS)*D1NM
        ZA(2,2,NM)=-CURLYR*US(NM)*D1NM
        ZA(2,3,NM)=RM*D1NM
        Z(2,4,NM)=-CURLYR*DU(NM)*D1NM
        ZA(2,4,NM)=YMUo*D1NM
	ZA(2,6,NM)=SIBETA*RS*D1NM
	Z(2,8,NM)=(FMT*D2U(NM)+FMTT*DT(NM)*DU(NM))*D1NM/FM
	ZA(2,8,NM)=-ROCON*D1NM
	Z(3,3,NM)=1.D0
	ZA(3,2,NM)=-1.D0


        Y(4,1,NM)=-A1(NM)*RE*TEMPER*RMINV*FMPIN*D1NM
        YA(4,2,NM)=A1(NM)*RS*RMINV*D1NM
        Y(4,4,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        Y(4,6,NM)=A1(NM)*RS*RMINV*SIBETA*D1NM
        ZA(4,2,NM)=YLMU*RMINV*YMUo*D1NM
        Z(4,4,NM)=(-BESQ-CURLYR*CROSS)*RMINV*D1NM
        ZA(4,4,NM)=-CURLYR*US(NM)*RMINV*D1NM
        ZA(4,5,NM)=RMINV*D1NM
        Z(4,6,NM)=YLMU*RMINV*SIBETA*YMUo*D1NM
        Z(4,8,NM)=(RMINV*SIBETA*FMT*DW(NM)/FM+ROREM*TIN)*D1NM
        ZA(4,8,NM)=RMINV*FMT*DU(NM)*D1NM/FM
        Z(4,1,NM)=-RE*DT(NM)*RMINV*FMPIN*D1NM
        Y(4,8,NM)=-A1(NM)*RMINV*ROCON*D1NM
	Z(5,5,NM)=1.D0
	ZA(5,4,NM)=-1.D0

        Y(6,4,NM)=A1(NM)*RS*SIBETA*D1NM
        Y(6,6,NM)=(A2(NM)+A1(NM)*YMUo)*D1NM
        Y(6,8,NM)=A1(NM)*FMT*DW(NM)*D1NM/FM
	Z(6,1,NM)=-SIBETA*RE*TEMPER*FMPIN*D1NM
	ZA(6,2,NM)=SIBETA*RS*D1NM
	Z(6,4,NM)=(SIBETA*YMUo-CURLYR*DW(NM))*D1NM
        Z(6,6,NM)=(-BESQ*RM-CURLYR*CROSS)*D1NM
        ZA(6,6,NM)=-CURLYR*US(NM)*D1NM
	ZA(6,7,NM)=D1NM
	Z(6,8,NM)=((FMT*D2W(NM)+FMTT*DT(NM)*DW(NM))/FM- &
     & SIBETA*ROCON)*D1NM
	Z(7,7,NM)=1.D0
	ZA(7,6,NM)=-1.D0

        Y(8,8,NM)=(A2(NM)+2.D0*A1(NM)*YMUo)*D1NM
        Y(8,2,NM)=2.D0*A1(NM)*CAPGAM*DU(NM)*D1NM
        Y(8,6,NM)=2.D0*A1(NM)*CAPGAM*DW(NM)*D1NM
        Z(8,4,NM)=(2.D0*CAPGAM*SIBETA*DW(NM)-DT(NM)*SIG)*D1NM
        ZA(8,4,NM)=2.D0*CAPGAM*DU(NM)*D1NM
        Z(8,1,NM)=EMMGAM*TEMPER*CROSS*D1NM
        ZA(8,1,NM)=EMMGAM*TEMPER*US(NM)*D1NM
	CONST=CAPGAM*(DU(NM)*DU(NM)+DW(NM)*DW(NM))+D2T(NM)
        CONST=(FMTT*DT(NM)*DT(NM)+FMT*CONST)/FM
        Z(8,8,NM)=(CONST-BESQ-SIGAM*CROSS)*D1NM
        ZA(8,8,NM)=-SIGAM*US(NM)*D1NM
        ZA(8,9,NM)=D1NM
	Z(9,9,NM)=1.D0
	ZA(9,8,NM)=-1.D0
	
	END IF

10	CONTINUE

	RETURN              
	END
	
        SUBROUTINE SIMP2(NP,GF,BOT,ENERGY)
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DOUBLE PRECISION BOT(*)
        COMPLEX*16 GF(*),ENERGY               
        COMMON /SCALI/B,G                                              
        COMMON /TEMESH/DY,YEND,YEDGE
        YO=0.D0 
	GB=G*B                                                    
        DO I=1,NP          
	GB2=GB/(B-YO)**2
        GF(I)=GF(I)*GB2                                    
        BOT(I)=BOT(I)*GB2 
        YO=YO+DY
	end do                        
        BELL=0.D0                                                    
        ENERGY=0.D0                                     
        DO I=1,NP-2,2         
        ENERGY=ENERGY+GF(I)+4.D0*GF(I+1)+GF(I+2)
        BELL=BELL+BOT(I)+4.D0*BOT(I+1)+BOT(I+2)
	end do                                           
        ENERGY=ENERGY/BELL       
        RETURN                                                  
        END 

	
			
		
	SUBROUTINE SPLINE(NNP,X,U,DU,D2U,UX,DUX,D2UX,V, & 
     & QWS,QDW,QD2W,QWX,QDWX,QD2WX,QTS,QDT,QD2T,QTX,QDTX,QD2TX, & 
     & QVY,QVYY)
        use param_blprof
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        REAL*8, DIMENSION(*) :: X,DU,U, D2U,UX,DUX,D2UX,V,QVY,QVYY, &
     & QWS,QD2W,QDW,QWX,QDWX,QD2WX,QTS,QD2T,QDT,QTX,QDTX,QD2TX
	REAL*8 WRK(NQ),DVC(4,NQ),V2C(4,NQ)
        REAL*8 UC(4,NQ),DUC(4,NQ),U2C(4,NQ),VC(4,NQ)
	REAL*8 UXC(4,NQ),U1XC(4,NQ),U2XC(4,NQ)
        REAL*8 WC(4,NQ),DWC(4,NQ),W2C(4,NQ)
	REAL*8 WXC(4,NQ),W1XC(4,NQ),W2XC(4,NQ)
        REAL*8 TC(4,NQ),DTC(4,NQ),T2C(4,NQ)
	REAL*8 TXC(4,NQ),T1XC(4,NQ),T2XC(4,NQ)
        COMMON /FITT/UC,DUC,U2C,UXC,U1XC,U2XC,WC,DWC,W2C,WXC, & 
     & W1XC,W2XC,TC,DTC,T2C,TXC,T1XC,T2XC,VC,DVC,V2C
        COMMON /SCALI/G,ULENTH
        COMMON /DECIDE/YEDGE,OUTER,RESHMA,BIBI
	COMMON /STRETCH/GRID
        COMMON /REGRID/KFL
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /VENUS/IATTACH
	COMMON /RELATIV/PHINVSC
        COMMON /NICINTG/ICNORM
	COMMON /BRPOINT/UQZ,WQZ
	COMMON /flexib/xstrech,udel
        OUTER=X(NNP)

        IF(KFL == 1)GOTO 9673
	
	QT=DSQRT(U(NNP)**2+QWS(NNP)**2)
	QTINV=1.d0/QT

	CALL SATURN(NQ,NNP,U,QWS,ICAP,QTINV,udel)
        YEDGE=X(ICAP)*YEDGE
		
        IF(IATTACH == 0)GOTO 9673

        THETA=DASIN(QWS(NNP)/QT)

        CALL ROTATE(NQ,NNP,THETA,U,QWS)
        CALL ROTATE(NQ,NNP,THETA,DU,QDW)
        CALL ROTATE(NQ,NNP,THETA,D2U,QD2W)
9673	CONTINUE

        IF(YEDGE > X(NNP))THEN
        NPT=NNP+1
        X(NPT)=YEDGE+.01D0*OUTER
        U(NPT)=U(NPT-1)
        DU(NPT)=0.D0
        D2U(NPT)=0.D0
        DUX(NPT)=0.D0
        D2UX(NPT)=0.D0
        UX(NPT)=UX(NPT-1)
	RESHMA=(V(NNP)-V(NNP-1))/(X(NNP)-X(NNP-1))
	BIBI=V(NNP)-RESHMA*X(NNP)
        V(NPT)=RESHMA*X(NPT)+BIBI
        QVY(NPT)=QVY(NPT-1)
        QVYY(NPT)=0.D0
	QWS(NPT)=QWS(NPT-1)
        QDW(NPT)=0.D0
        QD2W(NPT)=0.D0
        QWX(NPT)=0.D0
        QDWX(NPT)=0.D0
        QD2WX(NPT)=0.D0
        QTS(NPT)=QTS(NPT-1)
        QDT(NPT)=0.D0
        QD2T(NPT)=0.D0
        QTX(NPT)=QTX(NPT-1)
        QDTX(NPT)=0.D0
        QD2TX(NPT)=0.D0
        ELSE
        NPT=NNP
        END IF

! needed for QZ-search routine all as edge velocities not scaled with
! Ue! i.e Eurotrans notation upsets my conventions!
	UQZ=U(NPT)
	WQZ=QWS(NPT)


	PHINVSC=DATAN(QWS(NPT)/U(NPT))

        IF(KFL == 1 .AND. ICNORM /= 0)GOTO 673
!       MESH REFINEMENT PARAMETER
	IF(GRID < 1.D0)THEN
        I=NNP
	CALL JUPITER(NQ,NNP,U,QWS,I)
        ULENTH=2.D0*X(I)*xstrech
        ELSE
        ULENTH=2.D0*X(NNP)*grid
        END IF
        G=1.D0+ULENTH/YEDGE
673     CONTINUE

        IFAIL=0
	CALL ESPLINT(NPT,NQ,WRK,X,U,UC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,DU,DUC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,D2U,U2C,IFAIL)

	CALL ESPLINT(NPT,NQ,WRK,X,QVY,DVC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QVYY,V2C,IFAIL)

	CALL ESPLINT(NPT,NQ,WRK,X,UX,UXC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,DUX,U1XC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,D2UX,U2XC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,V,VC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QWS,WC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QDW,DWC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QD2W,W2C,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QWX,WXC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QDWX,W1XC,IFAIL)
	CALL ESPLINT(NPT,NQ,WRK,X,QD2WX,W2XC,IFAIL)

	CALL ESPLINT(NPT,NQ,WRK,X,QTS,TC,IFAIL)
        CALL ESPLINT(NPT,NQ,WRK,X,QDT,DTC,IFAIL)
        CALL ESPLINT(NPT,NQ,WRK,X,QD2T,T2C,IFAIL)
        CALL ESPLINT(NPT,NQ,WRK,X,QTX,TXC,IFAIL)
        CALL ESPLINT(NPT,NQ,WRK,X,QDTX,T1XC,IFAIL)
        CALL ESPLINT(NPT,NQ,WRK,X,QD2TX,T2XC,IFAIL)

        KFL=1
        RETURN
        END
	

	
        DOUBLE PRECISION FUNCTION TENPOW(IDIGIT)
        TENPOW=9.999D0**(IDIGIT)
        TENPOW=1.D0/TENPOW
        RETURN
        END
	DOUBLE PRECISION FUNCTION UMUVISC(TEDGE)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TO,T1,STOKES
        UMUVISC=UMUO*(TEDGE/TO)**1.5D0*(TO+T1)/(TEDGE+T1)
	RETURN
	END
	

	SUBROUTINE VISCOS(T,TDER,FO,F1,F2)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMMON /SUTHER/SO,S1
	BOT=1.D0/(T+S1)
	FO=SO*BOT*T**1.5D0
	F1=FO*(1.5D0*TDER-BOT)
	F2=0.75D0*FO*TDER*TDER-2.D0*F1*BOT
	RETURN
	END
	SUBROUTINE VISIMP(T,TDER,FO,F1,F2)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMMON /TRATUR/TOMEG,TM1,TM2
	FO=T**TOMEG
	F1=TOMEG*T**TM1
	F2=TOMEG*TM1*T**TM2
	RETURN
	END
	               
	SUBROUTINE ZEROAL(I3DNEQ)
        use param_info, only : MP, MODE, KMODE, NPOST
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KPG=IL*5)
	PARAMETER (KCRF=2*KMODE+1,KSTR=2*MODE+1)
	PARAMETER (ISEC=5,KEG=ISEC*MP)
	PARAMETER (ICOEFF=10)
	COMPLEX*16 CAPGAM(-MODE:MODE,-KMODE:KMODE,2)
	COMPLEX*16 SHAHID(0:MODE,-KMODE:KMODE)
        COMPLEX*16 STORE(0:MODE,-KMODE:KMODE,3)
	DOUBLE PRECISION, DIMENSION(IL) :: AI, BC
	DOUBLE PRECISION AMP(0:MODE,-KMODE:KMODE)
	COMPLEX*16 GAMMA(0:MODE,-KMODE:KMODE,2)
	COMPLEX*16, DIMENSION(KE) :: RHS, DCRHS
	COMPLEX*16, DIMENSION(KE,KPG) :: A, C
        COMPLEX*16, DIMENSION(IL,IL,MP) :: Y,YA,Z,ZA
        COMPLEX*16 QUY(MP,-MODE:MODE,-KMODE:KMODE,ISEC)
        COMPLEX*16 QUYY(MP,-MODE:MODE,-KMODE:KMODE,ISEC)
	COMPLEX*16 PHI(KEG,-MODE:MODE,-KMODE:KMODE)
	COMPLEX*16 RHSNLR(ISEC,MP)
	COMPLEX*16, DIMENSION(KCRF) :: ALRECRD, BARECRD
        COMPLEX*16 SOLD(ISEC,MP,-KMODE:KMODE)
        REAL*8, DIMENSION(0:ICOEFF) :: B0, C0, D0
	COMPLEX*16 SN(ISEC,MP,-KMODE:KMODE)
	REAL*8, DIMENSION(NPOST,KCRF) :: H1list, H2list
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
        COMPLEX*16 PZX(NPOST,KCRF),BAZO(KCRF,NPOST)
        COMMON /DUCK/BAZO,PZX
	COMMON /EISPC/ALFAZO
        COMMON /HLIST/H1list,H2list
	COMMON /EXIST3D/SN
	COMMON /BCOEFFS/B0,C0,D0
	COMMON /ANOTHER/SOLD
	COMMON /CRITERA/ALRECRD,BARECRD
	COMMON /MRCNON/RHSNLR
	COMMON /CLEVER/PHI
        COMMON /HIROSE/QUY,QUYY
	COMMON /DICTAT/Y,YA,Z,ZA
	COMMON /GLPROP/A,C
	COMMON /SHAPE/RHS,DCRHS
	COMMON /MCHGAM/CAPGAM,GAMMA
        COMMON /FOURIR/AMP
        COMMON /FOURINT/MFUR,KFUR,IFL
        COMMON /APPROX/SHAHID,STORE
        COMMON /APPINT/IFCNT
	COMMON /EQTYPE/AI,BC
	COMMON /PISAVE/PI
        COMMON /MULTF2/ISMOOTH
        COMMON /NOLOGER/VZEROIT
        COMMON /TRYIT/MAGNIFY,NSTAT,IDONEIT,IMANYS

        ISMOOTH=0

        VZEROIT=1.E-37
        IMANYS=0
	
	DO IZ=1,KCRF
	DO IX=1,NPOST
	ALFAZO(1:KSTR,IZ,IX)=0.d0
	PZX(IX,IZ)=0.d0
	BAZO(IZ,IX)=0.d0
	END DO
	ALRECRD(IZ)=0.D0
	BARECRD(IZ)=0.D0
	END DO
	
	DO I=0,ICOEFF
	B0(I)=0.D0
	C0(I)=0.D0
	D0(I)=0.D0
	END DO


        DO I=1,KE
	RHS(I)=1.D0
	DCRHS(I)=.5D0
	DO J=1,KPG
	A(I,J)=0.D0
	C(I,J)=0.D0
	END DO
	END DO

	DO 1020 IE=1,ISEC
	DO 1020 M=-MODE,MODE
	DO 1020 K=-KMODE,KMODE
	DO 1020 I=1,MP
        QUY(I,M,K,IE)=0.D0
        QUYY(I,M,K,IE)=0.D0
1020	CONTINUE

	DO 1220 M=-MODE,MODE
	DO 1220 K=-KMODE,KMODE
        PHI(1:KEG,m,k)=0.D0
1220	CONTINUE
	
	DO 1221 IE=1,ISEC
        DO 1221 K=-KMODE,KMODE
        DO 1221 IK=1,MP
	SN(IE,IK,K)=0.D0
	SOLD(IE,IK,K)=0.D0
1221	CONTINUE

	DO 102 J=1,IL
	DO 102 K=1,IL
	DO 102 I=1,MP
	Y(J,K,I)=0.D0
	YA(J,K,I)=0.D0
	Z(J,K,I)=0.D0
	ZA(J,K,I)=0.D0
102	CONTINUE

	DO IE=1,ISEC
        RHSNLR(IE,1:MP)=0.D0
	END DO

	DO 60 I=0,MODE
	DO 60 J=-KMODE,KMODE
	SHAHID(I,J)=DCMPLX(1.D0,0.D0)
	STORE(I,J,1:3)=0.D0
	GAMMA(I,J,1:2)=0.D0
60	CONTINUE

        DO 606 I=-MODE,MODE
        DO 606 J=-KMODE,KMODE
        CAPGAM(I,J,1)=DCMPLX(1.D0,0.D0)
        CAPGAM(I,J,2)=DCMPLX(1.D0,0.D0)
606     CONTINUE

!        CALL NIDENT(MFUR,KFUR)

	IF(I3DNEQ == 5)THEN
        AI(1)=0.D0
        AI(2)=1.D0
        AI(3)=1.D0
        AI(4)=1.D0
        AI(5)=1.D0
	ELSE
        AI(1)=0.D0
        AI(2)=1.D0
        AI(3)=0.D0
        AI(4)=1.D0
        AI(5)=0.D0
        AI(6)=1.D0
        AI(7)=0.D0
        AI(8)=1.D0
        AI(9)=0.D0
	END IF

	RETURN
	END
 


        SUBROUTINE ENEURO2(IALC,ENMETH,IERALFA)
	use header_info
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KE=IL*MP,KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
        CHARACTER*58 PSELABL
	REAL*8 DIMSCL(4),MZERO,Mo
        REAL*8 XCVAL(NPOST),RDVAL(NPOST),SDVAL(NPOST)
        REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
        COMPLEX*16 QZLIN(KSTR)
        REAL*8  WAVBETA(KCRF)
	REAL*8 PHIN(NPOST)
        COMPLEX*16 EURALFA(KSTR,KCRF,NPOST,4)
	REAL*8 DXVAL(NPOST)
        REAL*8 ENMETH(KSTR,KCRF,NPOST)
        COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST)
	INTEGER IERALFA(KSTR,KCRF)
        CHARACTER*80 TITLE
	COMMON /PICTR/TITLE
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
        COMMON /EISPC/ALFAZO
	COMMON /MOHD786/DXVAL
        COMMON /MUADZAM/EURALFA
	COMMON /RELVECT/PHIN
        COMMON /PISAVE/PI
        COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /RESDIM/DIMSCL,RESSCL
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /IBODYC/ICURVI,ICURVV,IC2UBAR

	DEGREES=180.D0/PI

        CALL FILHAND(999,LABEL(1:ICHART),'Q','.euro',PSELABL)
        OPEN(29,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(29,101) TITLE
	WRITE(29,'(A,A8,A)')'# Calculations performed with CoPSE3d V', &
     & VERSION,' :Imperial College London'
	DO 10 J=1,NCRBET
	DO 10 I=1,NFREQ
	IF(ICURVI == 0)THEN
	WRITE(29,104)WAVBETA(J), FRENCY(I)
	else
	WRITE(29,105)WAVBETA(J), FRENCY(I)
	end if
	WRITE(29,103)
	DO 20 K=1,IERALFA(I,J)

	PHI=PHIN(K)*DEGREES
!	PSI=DATAN(WAVBETA(J)/(DIMAG(ALFAZO(I,J,K))*PZERO))*DEGREES-PHI
	PSI=DATAN(CRFBETA(J)/DIMAG(ALFAZO(I,J,K)))*DEGREES-PHI

        WRITE(29,123)XCVAL(K),SDVAL(K),FRENCY(I), & 
     & WAVBETA(J),PSI,PHI, & 
     & DBLE(ALFAZO(I,J,K)/DZERO),ENMETH(I,J,K)

20	CONTINUE
10	CONTINUE


123	FORMAT(14(1x,g13.6))
101	FORMAT('# Test case :',A)
103	FORMAT('#',6X,'x/c ',9X,' s/c',6X,'freq',8X,'beta_(/m)', & 
     & 9X,'psi',9X,'phi',11x,'sig',11x,'N')
104	format('# Compressible linear stability theory LST,' & 
     & ' Curvature terms: OFF;', & 
     & 1x,'Spanwise wavenumber/m= ',g11.6,' Frequency =',g11.6,' Hz')
105	format('# Compressible linear stability theory LST,' & 
     & ' Curvature terms: ON;', & 
     & 1x,'Spanwise wavenumber/m = ',g11.6,' Frequency =',g11.6,'Hz')
     
        RETURN
        END                           


	SUBROUTINE RESLIN(IFIRST,IALC, DXTOM)
        use param_info, only : MP, MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=5)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
        PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8, DIMENSION(NPOST):: XCVAL,RDVAL,SDVAL,PHIN,DXVAL
	REAL*8 Medge,Mo,DIMSCL(4), MZERO
	COMPLEX*16 QZLIN(KSTR),ALFAZO(KSTR,KCRF,NPOST)
	COMPLEX*16 OMEGA, ABSQ
	INTEGER JOUTSC(2)
	DOUBLE PRECISION WAVBETA(KCRF)
        COMMON /FRVARI/OMEGA,RE
        COMMON /PISAVE/PI
        COMMON /QZBETA/WAVBETA
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZXXXX/NCROSS
        COMMON /SCLLOC/Uedge,Medge,RLOCAL,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /EISPC/ALFAZO
	COMMON /RESDIM/DIMSCL,RESSCL
	COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
	COMMON /LAYOUT/IDIV,ItSpres,IGLOO
	COMMON /RELVECT/PHIN
        COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /NAGBIT/INAG,IWVLNTH
        COMMON /REKEEP/UZERO,MZERO,TZERO,DZERO,DENSITY
	common /HENRIQUE/CAPGAMX, pwallmax, pumaxamp

	DEGREES=180.D0/PI
        PHI=PHIN(IALC)*DEGREES

	SQFACT=DSQRT(2.D0)
	REQ=RESSCL
	ALIEN=RESCALE*RLOCAL
	ZETA=SPOS*JOUTSC(1)+XPOS*JOUTSC(2)

! so as to convert wavelenths into real dimension numbers in mmitres
! but note can not yet handle errors in IWVLNTH=1 switch...
! i.e if wavenumber =0! effectively or set to zero due to non-cgs.	
	wmm=2.d0*PI*1000.d0*REQ*DZERO	
!	wmm=2.d0*PI*1000.d0

	ITU=80
	ITW=90

	IF(NOUTPUT /= 1)GOTO 1111
        IF(IFIRST /= 0)GOTO 1101
        WRITE(ITV,73)0.001D0*FRENCY(1),WAVBETA(1)

	IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,2120)
	ELSE
        WRITE(ITV,220)
	END IF

	ZEROWAV=DIMAG(ALFAZO(1,1,1))
	IF(DABS(ZEROWAV) < 1.D-9)RETURN
	wavemm=wmm/(DIMAG(ALFAZO(1,1,1)*REQ))
        PSI=DATAN(BZERO/DIMAG(ALFAZO(1,1,1)))*DEGREES-PHI
	totwav=dsqrt(BZERO**2+DIMAG(ALFAZO(1,1,1))**2)
        WRITE(ITV,33)ZETA,ALIEN,DIMAG(ALFAZO(1,1,1)*REQ), &
     & DBLE(ALFAZO(1,1,1)*REQ),PSI
! wmm/(DIMAG(ALFAZO(1,1,1)*REQ))
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.wngrwth',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
        WRITE(ITU,73)0.001D0*FRENCY(1),WAVBETA(1)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,212)
	ELSE
        WRITE(ITU,22)
	END IF
	wavemm=wmm/(DIMAG(ALFAZO(1,1,1)*REQ))
        PSI=DATAN(BZERO/DIMAG(ALFAZO(1,1,1)))*DEGREES-PHI
        WRITE(ITU,33)ZETA,ALIEN,DIMAG(ALFAZO(1,1,1)*REQ), &
     & DBLE(ALFAZO(1,1,1)*REQ),PSI,wmm/(DIMAG(ALFAZO(1,1,1)*REQ)), &
     & wmm/(totwav*REQ), 1.d0/DIMAG(ALFAZO(1,1,1)), DXTOM, &
     & dble(omega/DIMAG(ALFAZO(1,1,1))), &
     & DXTOM*DIMAG(ALFAZO(1,1,1)),DXTOM*DZERO*1000.d0  

	RETURN
1101	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,2120)
        ELSE
        WRITE(ITV,220)
        END IF

	ELSE
	END IF

	ZEROWAV=DIMAG(ALFAZO(1,1,IALC))
	IF(DABS(ZEROWAV) < 1.D-9)RETURN
        PSI=DATAN(BZERO/DIMAG(ALFAZO(1,1,IALC)))*DEGREES-PHI
	totwav=dsqrt(BZERO**2+DIMAG(ALFAZO(1,1,IALC))**2)

        WRITE(ITV,33)ZETA,ALIEN,DIMAG(ALFAZO(1,1,IALC)*REQ), &
     & DBLE(ALFAZO(1,1,IALC)*REQ),PSI

	IF(IWVDUMP == 0)RETURN
	wavemm=wmm/(DIMAG(ALFAZO(1,1,IALC)*REQ))
	waverat=wavemm/(DXTOM*DZERO*1000.d0)
        PSI=DATAN(BZERO/DIMAG(ALFAZO(1,1,IALC)))*DEGREES-PHI
        WRITE(ITU,33)ZETA,ALIEN,DIMAG(ALFAZO(1,1,IALC)*REQ), &
     & DBLE(ALFAZO(1,1,IALC)*REQ),PSI,wmm/(DIMAG(ALFAZO(1,1,IALC)*REQ)), & 
     & wmm/(totwav*REQ), 1.d0/DIMAG(ALFAZO(1,1,IALC)), DXTOM, &
     & dble(omega/DIMAG(ALFAZO(1,1,IALC))), &
     & DXTOM*DIMAG(ALFAZO(1,1,IALC)),DXTOM*DZERO*1000.d0,waverat, &
     & pwallmax*CAPGAMX, pumaxamp*CAPGAMX  
     
	RETURN

1111	IF(NOUTPUT /= 2)GOTO 2222
        IF(IFIRST /= 0)GOTO 2202
	WRITE(ITV,74)FRENCY(1)*0.001D0

	IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,515)
	ELSE
        WRITE(ITV,55)
	END IF

	WRITE(ITV,66)(WAVBETA(ICR),ICR=1,NCRBET)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.wn',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.grwth',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,74)FRENCY(1)*0.001D0
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,515)
	ELSE
        WRITE(ITU,55)
	END IF
	WRITE(ITU,66)(WAVBETA(ICR),ICR=1,NCRBET)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	WRITE(ITW,75)FRENCY(1)*0.001D0
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,515)
	ELSE
        WRITE(ITW,55)
	END IF
	WRITE(ITW,66)(WAVBETA(ICR),ICR=1,NCRBET)
	
	IF(IWVLNTH == 1)THEN
        WRITE(ITW,44)ZETA,ALIEN,(wmm/(DIMAG(ALFAZO(1,ICR,IALC)*REQ)), &
     & ICR=1,NCRBET)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
        end if
	
	RETURN
2202	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,515)
        ELSE
        WRITE(ITV,55)
        END IF
	WRITE(ITV,66)(WAVBETA(ICR),ICR=1,NCRBET)
	ELSE
	END IF
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
	IF(IWVDUMP == 0)RETURN
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
     
     	IF(IWVLNTH == 1)THEN
        WRITE(ITW,44)ZETA,ALIEN,(wmm/(DIMAG(ALFAZO(1,ICR,IALC)*REQ)), &
     & ICR=1,NCRBET)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(1,ICR,IALC)*REQ), &
     & ICR=1,NCRBET)
        END IF
     
     
	RETURN

2222	IF(NOUTPUT /= 3)GOTO 3333
        IF(IFIRST /= 0)GOTO 3303
	WRITE(ITV,78)WAVBETA(1)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,717)
	ELSE
        WRITE(ITV,77)
	END IF
	WRITE(ITV,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	IF(IWVDUMP == 0)RETURN
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.wn',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.grwth',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(1)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,717)
	ELSE
        WRITE(ITU,77)
	END IF
	WRITE(ITU,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	WRITE(ITW,79)WAVBETA(1)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,717)
	ELSE
        WRITE(ITW,77)
	END IF
	WRITE(ITW,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
	
	IF(IWVLNTH == 1)THEN
        WRITE(ITW,44)ZETA,ALIEN,(wmm/(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ)), &
     & ICFQ=1,NFREQ)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
        end if
	
	RETURN
3303	CONTINUE
	IF(ITSPRES == 0)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,717)
        ELSE
        WRITE(ITV,77)
        END IF
	WRITE(ITV,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
	ELSE
	END IF
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	IF(IWVDUMP == 0)RETURN
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
     
        IF(IWVLNTH == 1)THEN
	WRITE(ITW,44)ZETA,ALIEN,(wmm/(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ)), &
     & ICFQ=1,NFREQ)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,1,IALC)*REQ), &
     & ICFQ=1,NFREQ)
        end if
     
	RETURN

3333	CONTINUE

        IF(IFIRST /= 0)GOTO 4404
	DO 10 ICR=1,NCRBET
	INFT=ICR
	IF(ICR == 1)THEN
	WRITE(ITV,78)WAVBETA(ICR)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,717)
        ELSE
        WRITE(ITV,77)
        END IF
	WRITE(ITV,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	ELSE
	END IF

	IF(IWVDUMP == 0)GOTO 10
        CALL FILHAND(INFT,LABEL(1:ICHART),'B','.wn',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(INFT,LABEL(1:ICHART),'B','.grwth',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(ICR)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,717)
	ELSE
        WRITE(ITU,77)
	END IF
	WRITE(ITU,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	WRITE(ITW,79)WAVBETA(ICR)
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,717)
	ELSE
        WRITE(ITW,77)
	END IF
	WRITE(ITW,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
     
     	IF(IWVLNTH == 1)THEN
	WRITE(ITW,44)ZETA,ALIEN, &
     & (wmm/(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ)), &
     & ICFQ=1,NFREQ)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
        end if

	ITU=ITU+1
	ITW=ITW+1
10	CONTINUE
	RETURN

4404	CONTINUE
	DO 20 ICR=1,NCRBET
	IF(ITSPRES == 0 .AND. ICR == 1)THEN

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITV,717)
        ELSE
        WRITE(ITV,77)
        END IF
	WRITE(ITV,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
	ELSE
	END IF
	IF(ICR == 1)THEN
        WRITE(ITV,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	ELSE
	END IF
	IF(IWVDUMP == 0)GOTO 20
        WRITE(ITU,44)ZETA,ALIEN,(DBLE(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
     
	IF(IWVLNTH == 1)THEN
	WRITE(ITW,44)ZETA,ALIEN, &
     & (wmm/(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ)), &
     & ICFQ=1,NFREQ)
	else
        WRITE(ITW,44)ZETA,ALIEN,(DIMAG(ALFAZO(ICFQ,ICR,IALC)*REQ), &
     & ICFQ=1,NFREQ)
	end if
	
	ITU=ITU+1
	ITW=ITW+1
20	CONTINUE
	RETURN

11      FORMAT(2X,'Amplification Rates ...')
73	FORMAT(5X,'Frequency = ',E12.6,'KHz ; Beta = ',E12.6,'/m')
74      FORMAT(2X,'Amplification Rates for Frequency = ',E12.6,'KHz')
75      FORMAT(5X,'Wave Numbers for Frequency = ',E12.6,'KHz')
78      FORMAT(2X,'Amplification Rates for Beta = ',E12.6,'/m' )
79      FORMAT(5X,'Wave Numbers for Beta (/m) = ',E12.6)
121     FORMAT(5X,'Wavenumbers (/m) ...')
44      FORMAT(F9.6,E12.5,96E13.5)
55	FORMAT(3X,' x/c',6X,'RDelta',9X,'Crossflow Wavenumbers...')
515	FORMAT(3X,' s/c',6X,'RDelta',9X,'Crossflow Wavenumbers...')
66	FORMAT(21X,96E13.5)
77	FORMAT(3X,' x/c',6X,'RDelta',9X,'Frequency Range (KHz)...')
717	FORMAT(3X,' s/c',6X,'RDelta',9X,'Frequency Range (KHz)...')
22	FORMAT(5X,' x/c',7X,'RDelta',10X,'Wave No.',6X, &
     & 'Amplification Rate',4X,'Orientation (deg)', &
     & 5x,'Wavelength (mm)',3x,'abs. Wavelength (mm)',2x, &
     & '1/alpha            dx               c_phase           ',&
     & 'dx*alpha',10x,'dx(mm)',10x,'Lambda/dx',10x,'pwall',10x,'umax')
212	FORMAT(3X,' s/c',7X,'RDelta',10X,'Wave No.',6X, &
     & 'Amplification Rate',4X,'Orientation (deg)', &
     & 5x,'Wavelength (mm)',3x,'abs. Wavelength (mm)',2x, &
     & '1/alpha            dx               c_phase           ', &
     & 'dx*alpha',10x,'dx(mm)',10x,'Lambda/Dx',10x,'pwall',10x,'umax')
     
220	FORMAT(5X,' x/c',7X,'RDelta',10X,'Wave No.',6X, &
     & 'Amplification Rate',4X,'Orientation (deg)')
2120	FORMAT(3X,' s/c',7X,'RDelta',10X,'Wave No.',6X, &
     & 'Amplification Rate',4X,'Orientation (deg)')
33      FORMAT(F9.5,2X,E13.6,3x,E14.7,5X,E14.7,6X,F12.4, 9x, 16g18.6)
	END
	

        SUBROUTINE ENPRINT(IALC,ENMETH)
        use param_info, only : MODE, KMODE, NPOST
	use param_FILEADD
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	CHARACTER*58 PSELABL
	COMPLEX*16 QZLIN(KSTR)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
	REAL*8 XCVAL(NPOST),RDVAL(NPOST),SDVAL(NPOST), PHIN(NPOST)
	REAL*8 Medge,Mo
	REAL*8 WAVBETA(KCRF)	
	COMPLEX*16 ALFAZO(KSTR,KCRF,NPOST),ABSQ
        REAL*8 ENMETH(KSTR,KCRF,NPOST)        
	INTEGER JOUTSC(2)          
	COMMON /ROBETA/BZERO,BESQ,ABSQ
	COMMON /EISPC/ALFAZO
        COMMON /QZBETA/WAVBETA
        COMMON /QZXXXX/NCROSS
        COMMON /SCLLOC/Uedge,Medge,RLOCAL,Tedge,Delzer
	COMMON /SCLGLB/Uo,Mo,RGlobl,To,CHORD
	COMMON /SCLINT/NPROF,NZPROF
        COMMON /POSITN/XPOS,SPOS,PZERO,REX
        COMMON /POSSSI/IPROF
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /JOVIAN/JOUTSC,RESCALE
	COMMON /IGLISH/XCVAL,RDVAL,SDVAL
	COMMON /PISAVE/PI	
	COMMON /RELVECT/PHIN
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH

	SQFACT=DSQRT(2.D0)
	REQ=RESCALE
	
	DEGREES=180.D0/PI
	

	IZERO=1

	ITU=30
	ITW=40

	IF(NOUTPUT /= 1)GOTO 1111
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.n',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
        WRITE(ITU,73)0.001D0*FRENCY(1),WAVBETA(1)

        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,22)
        ELSE
        WRITE(ITU,221)
        END IF
	DO 1101 I=IZERO,IALC
        ALIEN=RESCALE*RDVAL(I)         
	PHI=PHIN(I)*DEGREES
	PSI=DATAN(BZERO/DIMAG(ALFAZO(1,1,I)))*DEGREES-PHI
        ZHETA=SDVAL(I)*JOUTSC(1)+XCVAL(I)*JOUTSC(2)
        WRITE(ITU,33)ZHETA,ALIEN,ENMETH(1,1,I), &
     & 0.001D0*FRENCY(1),WAVBETA(1),PSI
1101	CONTINUE
	RETURN

1111	IF(NOUTPUT /= 2)GOTO 2222
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.n',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.env',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,74)FRENCY(1)*0.001D0
	WRITE(ITW,74)FRENCY(1)*0.001D0
	
	Ndiagnostic=NCRBET
	
	IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,55)
        WRITE(ITW,755)Ndiagnostic
        ELSE
        WRITE(ITU,551)
        WRITE(ITW,7551)Ndiagnostic
        END IF

	WRITE(ITU,66)(WAVBETA(ICR),ICR=1,NCRBET)

	DO I=IZERO,IALC
        ALIEN=RESCALE*RDVAL(I)
        ZHETA=SDVAL(I)*JOUTSC(1)+XCVAL(I)*JOUTSC(2)
        WRITE(ITU,44)ZHETA,ALIEN,(ENMETH(1,ICR,I),ICR=1,NCRBET)

	EMAX=ENMETH(1,1,I)
	BETAMAX=WAVBETA(1)
	PHI=PHIN(I)*DEGREES
	PSI=DATAN(CRFBETA(1)/DIMAG(ALFAZO(1,1,I)))*DEGREES-PHI
	EMAX=ENMETH(1,ICR,I)-0.1
	DO 565 ICR=1,NCRBET
	ECOMP=ENMETH(1,ICR,I)
	IF(ECOMP < EMAX)GOTO 565
	EMAX=ECOMP
	BETAMAX=WAVBETA(ICR)
	PSI=DATAN(CRFBETA(ICR)/DIMAG(ALFAZO(1,ICR,I)))*DEGREES-PHI
565	CONTINUE

	IF(EMAX >= 1.D-9)THEN
        WRITE(ITW,33)ZHETA,ALIEN,EMAX,FRENCY(1)*0.001D0,BETAMAX,PSI
	ELSE
        WRITE(ITW,33)ZHETA,ALIEN,0.d0,FRENCY(1)*0.001D0,BETAMAX,PSI
	END IF

	end do
	
	RETURN

2222	IF(NOUTPUT /= 3)GOTO 3333
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.n',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
        CALL FILHAND(999,LABEL(1:ICHART),'Q','.env',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(1)
	WRITE(ITW,78)WAVBETA(1)
	
	Ndiagnostic=NFREQ
	
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,77)
	WRITE(ITW,755)Ndiagnostic
        ELSE
        WRITE(ITU,771)
	WRITE(ITW,7551)Ndiagnostic
        END IF
	WRITE(ITU,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
	DO 3303 I=IZERO,IALC
        ALIEN=RESCALE*RDVAL(I)
        ZHETA=SDVAL(I)*JOUTSC(1)+XCVAL(I)*JOUTSC(2)
        WRITE(ITU,44)ZHETA,ALIEN,(ENMETH(ICFQ,1,I),ICFQ=1,NFREQ)

        EMAX=ENMETH(1,1,I)
	FREQMAX=0.001*FRENCY(1)
	PHI=PHIN(I)*DEGREES
	PSI=DATAN(CRFBETA(1)/DIMAG(ALFAZO(1,1,I)))*DEGREES-PHI
        DO 566 ICFQ=2,NFREQ
        ECOMP=ENMETH(ICFQ,1,I)
        IF(ECOMP < EMAX)GOTO 566
        EMAX=ECOMP
        FREQMAX=0.001*FRENCY(ICFQ)
	PSI=DATAN(CRFBETA(1)/DIMAG(ALFAZO(ICFQ,1,I)))*DEGREES-PHI
566	CONTINUE

	IF(EMAX >= 1.D-9)THEN
        WRITE(ITW,33)ZHETA,ALIEN,EMAX,FREQMAX,WAVBETA(1),PSI
	ELSE
        WRITE(ITW,33)ZHETA,ALIEN,0.d0,FREQMAX,WAVBETA(1),PSI
	END IF

3303	CONTINUE
	RETURN

3333	CONTINUE

	DO 10 ICR=1,NCRBET
	INFT=ICR
        CALL FILHAND(INFT,LABEL(1:ICHART),'B','.n',PSELABL)
	OPEN(ITU,FILE=PSELABL,STATUS='UNKNOWN')
	WRITE(ITU,78)WAVBETA(ICR)
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITU,77)
        ELSE
        WRITE(ITU,771)
        END IF

	WRITE(ITU,66)(0.001*FRENCY(ICFQ),ICFQ=1,NFREQ)
	DO 4404 I=IZERO,IALC
        ALIEN=RESCALE*RDVAL(I)
        ZHETA=SDVAL(I)*JOUTSC(1)+XCVAL(I)*JOUTSC(2)
        WRITE(ITU,44)ZHETA,ALIEN,(ENMETH(ICFQ,ICR,I),ICFQ=1,NFREQ)
4404	CONTINUE
	ITU=ITU+1
10	CONTINUE

        CALL FILHAND(999,LABEL(1:ICHART),'Q','.env',PSELABL)
	OPEN(ITW,FILE=PSELABL,STATUS='UNKNOWN')
	Ndiagnostic=NCRBET*NFREQ
        WRITE(ITW,976)Ndiagnostic
	
        IF(JOUTSC(1) == 1)THEN
        WRITE(ITW,755)Ndiagnostic
        ELSE
        WRITE(ITW,7551)Ndiagnostic
        END IF


	DO 4474 I=IZERO,IALC
        ALIEN=RESCALE*RDVAL(I)
        ZHETA=SDVAL(I)*JOUTSC(1)+XCVAL(I)*JOUTSC(2)
        EMAX=ENMETH(1,1,I)
	FREQMAX=0.001*FRENCY(1)
	BETAMAX=WAVBETA(1)	
	PHI=PHIN(I)*DEGREES
	PSI=DATAN(CRFBETA(1)/DIMAG(ALFAZO(1,1,I)))*DEGREES-PHI
	DO ICR=1,NCRBET
        DO 5661 ICFQ=1,NFREQ
        ECOMP=ENMETH(ICFQ,ICR,I)
        IF(ECOMP < EMAX)GOTO 5661
        EMAX=ECOMP
        FREQMAX=0.001*FRENCY(ICFQ)
	BETAMAX=WAVBETA(ICR)
	PSI=DATAN(CRFBETA(ICR)/DIMAG(ALFAZO(ICFQ,ICR,I)))*DEGREES-PHI
5661	CONTINUE
	end do

	IF(EMAX >= 1.D-9)THEN
        WRITE(ITW,33)ZHETA,ALIEN,EMAX,FREQMAX,BETAMAX,PSI
	ELSE
        WRITE(ITW,33)ZHETA,ALIEN,0.d0,FREQMAX,BETAMAX,PSI
	END IF

4474	CONTINUE


	RETURN

11      FORMAT(3X,'N-Factors ...')
73	FORMAT(3X,'Frequency = ',E12.6,'KHz ; Beta = ',E12.6,'/m')
74      FORMAT(3X,'N Factors for Frequency = ',E12.6,'KHz')
78      FORMAT(3X,'N Factors for Beta = ',E12.6,'/m')
44	FORMAT(F14.9,E12.5,96E13.5)
55	FORMAT(5X,' s/c',9X,'RDelta',4X,'Crossflow Wavenumbers (/m)...')
551	FORMAT(5X,' x/c',9X,'RDelta',4X,'Crossflow Wavenumbers (/m)...')
77	FORMAT(5X,' s/c',9X,'RDelta',4X,'Frequency Range (KHz)...', &
     & 2x,I0)
771	FORMAT(5X,' x/c',9X,'RDelta',4X,'Frequency Range (KHz)...', &
     & 2x,I0)
755	FORMAT(5X,' s/c',11X,'RDelta',8X,'Ln(A/Ao)_Max',8X, &
     & 'Freq (kHz)',8x,'Beta_Max(/m)',9x,'PSI',2x,I0)
7551	FORMAT(5X,' x/c',11X,'RDelta',8X,'Ln(A/Ao)_Max',8X, &
     & 'Freq (kHz)',8x,'Beta_Max(/m)',9x,'PSI',2x,I0)
976	FORMAT(3X,'N-Factor Envelope over Frequency & Beta Range...', &
     & 2x,I0)

66	FORMAT(26X,96E13.5)

22	FORMAT(5X,' s/c',12X,'RDelta',8X,'Ln(A/Ao)_Max',8X, &
     & 'Freq (kHz)',8x,'Beta_Max(/m)',9x,'PSI')
221	FORMAT(5X,' x/c',12X,'RDelta',8X,'Ln(A/Ao)_Max',8X, &
     & 'Freq (kHz)',8x,'Beta_Max(/m)',9x,'PSI')
33      FORMAT(F14.10,2X,E13.6,3x,E14.7,5X,E14.7,5X,E14.7,5X,2F9.4)

	END


	SUBROUTINE SEARCH(IA,IB,ALFA,GROWTH,NSPEC,IFREQ,IBOX)
        use param_info, only :  MODE, KMODE, MPG
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MPG-1)*IL)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	DOUBLE PRECISION ALFA(*),GROWTH(*)
	COMPLEX*16 OMEGA,ABSQ
	COMPLEX*16 QZLIN(KSTR)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
 	COMMON /QZMETH/INGUESS,IGLB
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
        COMMON /BLACKBX/QZWAVE,QZAMPL,QZINDS
        COMMON /BLACINT/IFIND
	COMMON /MULTF2/ISMOOTH
	COMMON /BRPOINT/UQZ,WQZ
	COMMON /PPM_PSE3D/xAct,yAct


	IFIND=0
	IF(IBOX == 1)GOTO 11011
	IF(INGUESS == 1)THEN
	OPEN(13,FILE='PSEIGEN.DAT',STATUS='UNKNOWN')
	WRITE(ITV,*)
	WRITE(ITV,33)
	WRITE(ITV,55)
	ELSE
	OPEN(12,FILE='PSEMULTF.DAT',STATUS='UNKNOWN')
	IF(ISMOOTH == 1)OPEN(9,FILE='PSEMULTF2.DAT', &
     & STATUS='UNKNOWN')
	WRITE(ITV,*)
	WRITE(ITV,733)
	WRITE(ITV,755)
	END IF
11011	CONTINUE

33	FORMAT(7X,'Mode',9X,'Wave No.') 
55	FORMAT(7X,'----',3x,'---------------------')
733      FORMAT(12X,'Wave No.',10X,'Frequency')
755      FORMAT(7X,'------------------',2X,'-------------')

! trying to find the point to use as reference point...

	X=(DBLE(OMEGA)-BZERO*WQZ)/UQZ

	IDENT=999	
	IDENTP=999	
	XR=ALFA(1)
	YR=GROWTH(1)

	xabuv=0.05D0
	xmore=0.02D0

	IF(X > 0.0)THEN
	XO=X*(1.D0+xmore)
	ELSE
	XO=X*(1.D0-xmore)
	END IF

	YO=DABS(xabuv*X)

	INSPEC=0
! check to see if anything worth investigating....< 5 usualy means
! some unusual parameter regime/dodgy bl profiles or very stable regime.
	IF(NSPEC < 5)THEN
	IF(INGUESS == 1)THEN
	WRITE(ITV,40057)
	ELSE
	WRITE(ITV,4057)
	END IF
	INSPEC=-1
	END IF

	slop=0.
	do 207 I=1,NSPEC
	xr=ALFA(I)
	yr=GROWTH(I)
	slope=dabs((yr-yo)/(xr-xo))
111	format(5g19.6)
	IF(SLOPE < SLOP)goto 207
	xAct=xr
	yACT=yr
	SLOP=SLOPE
        ldent=i
207	CONTINUE

	xref=xr
	yref=yr

        slop=-99.

! msm
	xabuv=0.02D0
	xmore=0.025D0

! cuttoff for too stable eigenvalues!
	ytomuch=-0.095
	ytxmuch=ytomuch-0.04

        IDENT=999
        IDENTP=999

        IF(X > 0.0)THEN
        XO=Xref*(1.D0+xmore)
        ELSE
        XO=Xref*(1.D0-xmore)
        END IF

! new stuff 
	IF(dabs(yref) > 0.0001)THEN
        YO=yref+xabuv*dabs(yref)
	ELSE
        YO=xabuv*DABS(X)
	END IF

        XRES=XO*(1.D0-xmore)
        XRESUP=XO*(1.D0+xmore)

        do 20 I=1,NSPEC
	IF(I == ldent)GOTO 20
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yr-yo)/(xr-xo)
        IF(xr <= XRES .OR. XR < XO)GOTO 20
        IF(SLOPE < SLOP .OR. DABS(SLOPE) > 1.25D0)goto 20
	IF(yr < ytomuch)GOTO 20
        xAct=xr
        yACT=yr
        ident=i
        SLOP=SLOPE
20      CONTINUE

! msms

	IF(IDENT /= 999)GOTO 66066
	TWODOK=BZERO
	IF(DABS(TWODOK) < 1.D-3)GOTO 66066

	slop=99.

	IF(X > 0.0)THEN
	XO=Xref*(1.D0-xmore)
	ELSE
	XO=Xref*(1.D0+xmore)
	END IF

        do 205 I=1,NSPEC
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yo-yr)/(xo-xr)
	IF(I == ldent)GOTO 205
        IF(xr >= XRESUP .OR. XR > XO)GOTO 205
	IF(SLOPE > SLOP)goto 205
	IF(SLOPE < -1.25)goto 205
	IF(yr < ytomuch)GOTO 205
        xAct=xr
        yACT=yr
        ident=i
        SLOP=SLOPE
205      CONTINUE

	IDENTP=IDENT
	IF(IDENT /= 999)THEN
	IDENTP=-19
	END IF

66066	CONTINUE
	IF(IDENT == 999)THEN
	IF(INSPEC == -1)WRITE(ITV,4056)
	IF(INSPEC == 0)WRITE(ITV,456)
	WRITE(ITV,457)
	END IF

	IF(IDENT == 999)GOTO 1963
	IF(IBOX == 1)THEN
	QZWAVE=xAct
	QZAMPL=yAct
	QZINDS=SLOP
	ELSE IF(IBOX == 0 .AND. INGUESS == 1)THEN
	WRITE(ITV,44)IA,IB,xAct,yAct
	WRITE(13,11)IA,IB,xAct,yAct
	ELSE 
        WRITE(ITV,744)xAct,yAct,FRENCY(IFREQ)
	IF(ISMOOTH == 0)THEN
	WRITE(12,711)xAct,yAct,FRENCY(IFREQ)
	ELSE
        WRITE(9,711)xAct,yAct
	END IF
	END IF

711     FORMAT(2g19.7,E19.9)
744     FORMAT(5X,2F10.6,E16.8)
11	FORMAT(2I4,2E19.9)
44	FORMAT(5X,2I3,2F12.7)

1963	CONTINUE
	IF(IDENTP == -19)GOTO 55055


        IF(X > 0.0)THEN
        XO=Xref*(1.D0+xmore)
        ELSE
        XO=Xref*(1.D0-xmore)
        END IF


	IF(IDENT == 1)THEN
	IDENT2=2
	xAct2=ALFA(2)
	yAct2=GROWTH(2)
	slop=(yAct2-yo)/(xAct2-xo)
	IST=3
	ELSE
	IDENT2=1
	xAct2=ALFA(1)
	yAct2=GROWTH(1)
	slop=(yAct2-yo)/(xAct2-xo)
	IST=2
	END IF
	IMESS=0
	do 30 I=IST,NSPEC
	xr=ALFA(I)
	yr=GROWTH(I)
	slope=(yr-yo)/(xr-xo)
        IF(xr <= XRES .OR. XR < XO)GOTO 30
	IF(I == LDENT)GOTO 30
	IF(I == IDENT)GOTO 30
	IF(SLOPE < SLOP .OR. DABS(SLOPE) > 2.5D0)goto 30
	IF(yr < ytxmuch)GOTO 30
	xAct2=xr
	yACT2=yr
	ident2=i
	IFIND=1
	SLOP=SLOPE
30	CONTINUE

	GOTO 77077

55055	CONTINUE

        IF(X > 0.0)THEN
        XO=Xref*(1.D0-xmore)
        ELSE
        XO=Xref*(1.D0+xmore)
        END IF

        IF(IDENT == 1)THEN
        IDENT2=2
        xAct2=ALFA(2)
        yAct2=GROWTH(2)
        slop=(yo-yAct2)/(xo-xAct2)
        IST=3
        ELSE
        IDENT2=1
        xAct2=ALFA(1)
        yAct2=GROWTH(1)
        slop=(yo-yAct2)/(xo-xAct2)
        IST=2
        END IF
        IMESS=0
        do 3011 I=IST,NSPEC
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yo-yr)/(xo-xr)
        IF(xr >= XRESUP .OR. xr > XO)GOTO 3011
	IF(I == ldent)GOTO 3011
        IF(I == IDENT)GOTO 3011
	IF(SLOPE > SLOP)goto 3011
	IF(SLOPE < -2.5)goto 3011
	IF(yr < ytxmuch)GOTO 3011
        xAct2=xr
        yACT2=yr
        ident2=i
        IFIND=1
        SLOP=SLOPE
3011    CONTINUE

77077	CONTINUE

	IF(IBOX == 1)RETURN
	IQFIND=0
	IF(DABS(xAct2) < 1.D-6 .AND. dabs(yAct2) < 1.D-6)THEN
	WRITE(ITV,701)
	IQFIND=1
	END IF
	IF(IQFIND == 1)RETURN

	IF(INGUESS == 1)THEN
	WRITE(ITV,44)IA,IB,xAct2,yAct2
!	WRITE(12,22)IA,IB,xAct2,yAct2
	ELSE
        WRITE(ITV,744)xAct2,yAct2,FRENCY(IFREQ)
	END IF


456	FORMAT(2X,' No discrete eigenvalues found - You may be ')
4056	FORMAT(2X,' No discrete eigenvalues found - You really are ')
457	FORMAT(2X,' in a highly stable region. Try following :')
4057	format(2x,' ** Warning : you are probably in a very ', &
     & 'stable region **',/, &
     & '   I would not trust the result in PSEMULTF if any !!!',/, &
     & '   Either change Beta/Frequency combination or NSTART ')
40057	format(2x,' ** Warning : you are probably in a very ', &
     & 'stable region **',/, &
     & '   I would not trust the result in PSEIGEN if any !!!',/, &
     & '   Either change Beta/Frequency combination or NSTART ')
701	FORMAT(2X,'         No alternatives found !!! ')

22	FORMAT(2I3,2F10.6,F8.4,I0,'  -  A l t e r n a t i v e  -')
	RETURN
	END


! unfinished .. to further refine eiegenvalue search routine using
! least squares centered on xref,yref point!
! this gives an outer bound of numerical continuous spectrum
        SUBROUTINE SDERROR(xref,yref,ALFA,GROWTH,NSPEC,slop)
        use param_info, only : MPG
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=9,KE=(MPG-1)*IL)
	PARAMETER (NDIV=KE)
        DOUBLE PRECISION ALFA(*),GROWTH(*)
        REAL*8 XC(NDIV),YC(NDIV)
        REAL*8 X(NDIV)
!	xfit(a,b,c,xx)=a*xx*xx+b*xx+c
	slop=-99
	IF(NSPEC < 5)RETURN
	xfract=0.2*dabs(xref)
	xlo=xref-xfract
	xup=xref+xfract

	SPERFCT=1.D0
	xmax=xref
	DO I=1,NSPEC
	IB=1
	DO 20 J=1,NSPEC

! simply used xmax in plotting of ls-fitt
	if(DBLE(ALFA(J)) > xmax)THEN
	xmax=DBLE(ALFA(J))
	end if

	XDUB=DBLE(ALFA(J))
	IF(XDUB > xup .OR. XDUB < xlo)GOTO 20
	IF(I == J)GOTO 20
	XC(IB)=ALFA(J)
	YC(IB)=GROWTH(J)
	IB=IB+1
20	CONTINUE

	CALL LEASTSQ(IB-1,XC,YC,X,KE,xref,yref,serr)

! this selects eigenvalue furthest away from optimal curve fit,
! hence the most likely candidate!
	IF(serr < SPERFCT)THEN
	SPERFCT=serr
	slop=x(2)+2.D0*x(3)*ALFA(I)
!	aselect=ALFA(I)
!	gselect=GROWTH(I)
	ELSE
	END IF

	end do

	ndiff=200
	xstep=(xup-xlo)/(ndiff-1.D0)

!	xx=xlo
!	DO kk=1,ndiff
!	WRITE(89,'(2E19.7)')xx,xfit(x(3),x(2),x(1),xx)	
!	xx=xx+xstep
!	end do

! slope scale

! reset normally in low res qz results.
	slop=DABS(slop*0.8)
	IF(dabs(slop) < 1.D0)THEN
	slop=2.25
	ELSE
	END IF
!	WRITE(88,'(2E19.7)')aselect, gselect

	RETURN
	END

! least squares quadratic fit constarined at one end to
! satisfy xref, yref. Second-order fit.. when have time
! to n-thorder fit. Simple
        SUBROUTINE LEASTSQ(NPT,XC,YC,X,KE,xref,yref,serr)
        use param_info, only : MPG
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=9,NDIV=(MPG-1)*IL)
        REAL*8 XC(*),YC(*)
        REAL*8 B(NDIV,NDIV),X(NDIV),D(NDIV)
        DO 10 I=1,3
        D(I)=0.D0
        DO 10 J=1,3
        B(I,J)=0.D0
10      CONTINUE

        I=1
        B(I,1)=xref**(I-1)
        B(I,2)=xref**I
        B(I,3)=xref**(I+1)
        D(I)=yref

        DO 20 I=2,3
        DO 20 J=1,NPT
        B(I,1)=B(I,1)+XC(J)**(I-1)
        B(I,2)=B(I,2)+XC(J)**I
        B(I,3)=B(I,3)+XC(J)**(I+1)
        D(I)=D(I)+YC(J)*XC(J)**(I-1)
20      CONTINUE


        CALL SOLVE(B,D,X,NDIV,3)

	serr=0.d0
	do I=1,npt
	serr=serr+(x(1)+x(2)*XC(i)+x(3)*XC(i)*XC(i)-YC(i))**2.D0
	end do	
	serr=1.D0-dsqrt(serr)

        RETURN
        END

	
        SUBROUTINE LOWUPP(NB,NPTS,NEQ,A)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (ISPARSE=5)
	PARAMETER (IL=9,KE=(MP-1)*IL,KGP=ISPARSE*IL)
	COMPLEX*16 A(KE,KGP),DIGN(KE,2)
	COMPLEX*16 SUM, AINV
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
        COMMON /SINDING/DIGN
        COMMON /ILUILU/MPDDE,MEQ,M3,MFOR
	MBLOK=NEQ
	LI=1
	AINV=1.d0/A(LI,LI)
	DO J=2,M3
	A(J,LI)=A(J,LI)*AINV
	end do
	
	DO 30 I=2,MBLOK
	IM1=I-1
	IP1=I+1
	DO J=I,M3
	SUM=0.D0
	DO K=1,IM1
	SUM=SUM+A(I,K)*A(K,J)
	end do
	A(I,J)=A(I,J)-SUM
	end do
	AINV=1.d0/A(I,I)
	DO J=IP1,M3
	SUM=0.D0
	DO K=1,IM1
	SUM=SUM+A(J,K)*A(K,I)
	end do
        A(J,I)=(A(J,I)-SUM)*AINV
	end do
30	CONTINUE

	MBLOK=MBLOK+NEQ
	DO 80 I=MBLOK-MPDDE,MBLOK
	ITI=IT(I)
	IM1=I-1
	IP1=I+1
	DO J=I,MFOR
	SUM=0.D0
	DO K=IT(J)+1,IM1
	SUM=SUM+A(I,K-ITI)*A(K,J-IT(K))
	end do
        JS=J-ITI
	A(I,JS)=A(I,JS)-SUM
	end do
	AINV=1.d0/A(I,I-ITI)
	DO J=IP1,MFOR
	ITJ=IT(J)
	SUM=0.D0
	DO K=ITJ+1,IM1
	SUM=SUM+A(J,K-ITJ)*A(K,I-IT(K))
	end do
        IS=I-ITJ
	A(J,IS)=(A(J,IS)-SUM)*AINV
	end do
80	CONTINUE
 
	DO 9999 NIJ=3,NB-2
	MBLOK=MBLOK+NEQ
	MEND=MBLOK+MEQ
	DO 130 I=MBLOK-MPDDE,MBLOK
	ITI=IT(I)
	IM1=I-1
	IP1=I+1
	DO J=I,MEND
	SUM=0.D0
	DO K=IT(J)+1,IM1
	SUM=SUM+A(I,K-ITI)*A(K,J-IT(K))
	end do
        JS=J-ITI
	A(I,JS)=A(I,JS)-SUM
	end do
	AINV=1.d0/A(I,I-ITI)
	DO J=IP1,MEND
	ITJ=IT(J)
	SUM=0.D0
	DO K=ITJ+1,IM1
	SUM=SUM+A(J,K-ITJ)*A(K,I-IT(K))
	end do
        IS=I-ITJ
        A(J,IS)=(A(J,IS)-SUM)*AINV
	end do
	
130	CONTINUE
9999    CONTINUE

	MBLOK=MBLOK+NEQ
	DO 180 I=MBLOK-MPDDE,MBLOK
	ITI=IT(I)
	IM1=I-1
	IP1=I+1
	DO J=I,NPTS
	SUM=0.D0
	DO K=IT(J)+1,IM1
	SUM=SUM+A(I,K-ITI)*A(K,J-IT(K))
	end do
        JS=J-ITI
	A(I,JS)=A(I,JS)-SUM
	end do
	AINV=1.d0/A(I,I-ITI)
	DO J=IP1,NPTS
	ITJ=IT(J)
	SUM=0.D0
	DO K=ITJ+1,IM1
	SUM=SUM+A(J,K-ITJ)*A(K,I-IT(K))
	end do
        IS=I-ITJ
        A(J,IS)=(A(J,IS)-SUM)*AINV
	end do
180	CONTINUE
 
	MBLOK=MBLOK+NEQ
	DO 230 I=MBLOK-MPDDE,NPTS
	ITI=IT(I)
	IM1=I-1
	IP1=I+1
	DO J=I,NPTS
	SUM=0.D0
	DO K=IT(J)+1,IM1
	SUM=SUM+A(I,K-ITI)*A(K,J-IT(K))
	end do
        JS=J-ITI
	A(I,JS)=A(I,JS)-SUM
	end do
	AINV=1.d0/A(I,I-ITI)
	DO J=IP1,NPTS
	ITJ=IT(J)
	SUM=0.D0
	DO K=ITJ+1,IM1
	SUM=SUM+A(J,K-ITJ)*A(K,I-IT(K))
	end do
        IS=I-ITJ
        A(J,IS)=(A(J,IS)-SUM)*AINV
	end do
230	CONTINUE
 
	DO I=1,NPTS
	DIGN(I,2)=A(I,I-IT(I))
	end do
	
	RETURN
	END




	
	SUBROUTINE LUSOLV(NPTS,UB,IF1,IF2,C,X)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KR=IL*5)
	COMPLEX*16 UB(KE,KR),ZTEMP(KE),DIAG(KE,2)
	COMPLEX*16 X(*),C(*)
	COMPLEX*16 SUM
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
        COMMON /SINDING/DIAG
	ZTEMP(1)=C(1)
	DO 20 I=2,NPTS
	ITI=IT(I)
	SUM=C(I)
	KST=ITI+1
	DO J=KST,I-1
	SUM=SUM-UB(I,J-ITI)*ZTEMP(J)
	END DO
	ZTEMP(I)=SUM
20	CONTINUE
	
	X(NPTS)=ZTEMP(NPTS)/DIAG(NPTS,IF2)
	K=2
	DO 40 I=NPTS-1,1,-1
	SUM=ZTEMP(I)
	ITI=IT(I)
	DO J=I+1,NPTS-IT(K)
	SUM=SUM-UB(I,J-ITI)*X(J)
	END DO
	K=K+1
	X(I)=SUM/DIAG(I,IF2)
40	CONTINUE
	RETURN
	END
	
	SUBROUTINE LUSOLT(NPTS,UB,IF1,IF2,C,X)
        use param_info, only : MP
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MP-1)*IL,KR=IL*5)
	COMPLEX*16 UB(KE,KR),ZTEMP(KE),DIAG(KE,2)
	COMPLEX*16 X(*),C(*)
	COMPLEX*16 SUM
	INTEGER IT(KE)
        COMMON /SINAP3D/IT
        COMMON /SINDING/DIAG

	ZTEMP(1)=C(1)/DIAG(1,IF1)
	DO 20 I=2,NPTS
	SUM=C(I)
	KST=IT(I)+1
	DO J=KST,I-1
	SUM=SUM-UB(J,I-IT(J))*ZTEMP(J)
	END DO
	ZTEMP(I)=SUM/DIAG(I,IF1)
20	CONTINUE
	
	X(NPTS)=ZTEMP(NPTS)
	K=2
	DO 40 I=NPTS-1,1,-1
	SUM=ZTEMP(I)
	DO J=I+1,NPTS-IT(K)
	SUM=SUM-UB(J,I-IT(J))*X(J)
	END DO
	K=K+1
	X(I)=SUM
40	CONTINUE
	RETURN
	END


        SUBROUTINE AP3D2(NDIV,NPOINTS,NEQ)
        use param_info, only : MP
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (ISPARSE=5)                 
        PARAMETER (IL=9,KE=(MP-1)*IL,KGP=ISPARSE*IL)
        COMPLEX*16 DIGN(KE,2)
        INTEGER IT(KE)                                                 
        COMMON /SINAP3D/IT
        COMMON /SINDING/DIGN                                         
        COMMON /ILUILU/MPDDE,MEQ,M3,MFOR

        MBEG=(ISPARSE+1)/2                                           
        MEQ=NEQ*(ISPARSE-1)/2           
        M3=NEQ*MBEG              
        MFOR=NEQ*(ISPARSE+3)/2                                       
        MPDDE=NEQ-1                                     

        IT(1:M3)=0

        MFLAG=M3+1                                              
        DO I=(ISPARSE+3)/2,NDIV                    
        IMS=(I-MBEG)*NEQ            
        DO J=1,NEQ                                                
        IT(MFLAG)=IMS                            
        MFLAG=MFLAG+1
	END DO 
	END DO

        DIGN(1:NPOINTS,1)=1.D0 
        RETURN
        END
	SUBROUTINE SOLVE(B,D,X,KDIV,N)
        use param_info, only : MPG
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        PARAMETER (IL=9,ILOCL=(MPG-1)*IL)
	REAL*8 A(ILOCL,ILOCL),B(KDIV,KDIV),X(KDIV),C(ILOCL)
	REAL*8 DUM(ILOCL),D(KDIV)
	INTEGER IT(ILOCL)
	DO I=1,N
	C(I)=D(I)
	A(I,1:N)=B(I,1:N)
	end do
	DO I=1,N
        IT(I)=I
	end do
        L=N-1
        DO 10 IC=1,L
        ICODE=IC
        JC=IC
        CHECK=DABS(A(IC,IC))
        DO  I=IC,N
        DO 20 J=IC,N
        CQ=DABS(A(I,J))
        IF(CQ < CHECK)GOTO 20
        JC=J
        ICODE=I
        CHECK=CQ
20      CONTINUE
	end do
        DO J=IC,N
        TEMP=A(ICODE,J)
        A(ICODE,J)=A(IC,J)
        A(IC,J)=TEMP
	end do
        CT=C(ICODE)
        C(ICODE)=C(IC)
        C(IC)=CT
        DO  I=1,N
        TEMP=A(I,JC)
        A(I,JC)=A(I,IC)
        A(I,IC)=TEMP
	end do
        ICT=IT(JC)
        IT(JC)=IT(IC)
        IT(IC)=ICT
        MR=IC+1
        DO  I=MR,N
        Z=A(I,IC)/A(IC,IC)
        DO  J=IC,N
        A(I,J)=A(IC,J)*Z-A(I,J)
	end do
        C(I)=C(IC)*Z-C(I)
	end do
10      CONTINUE
        DUM(N)=C(N)/A(N,N)
        DO  I=L,1,-1
        SUM=0.0D0
        K=I+1
        DO  J=K,N
        SUM=SUM+A(I,J)*DUM(J)
	end do
        DUM(I)=(C(I)-SUM)/A(I,I)
	end do
        X(IT(1:N))=DUM(1:N)
	RETURN
        END

        SUBROUTINE BRANCH_CUTS
        use param_info, only : MPG
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MPG-1)*IL)
	PARAMETER (MAXN=5)
	COMPLEX*16 COEFF(0:MAXN), ROOTS(MAXN)
	COMPLEX*16 RCRIT(MAXN),hcoeff(0:MAXN)
	COMPLEX*16 hROOTS(MAXN)

        COMPLEX*16 OMEGA,ABSQ,SING
        COMPLEX*16 b1,b2,b3,b4,scale
	COMPLEX*16 AA,BB,CC,R1,R2,teta,tetore
	COMPLEX*16 R1x, R2x
	complex*16 cbit,tetasq
	COMPLEX*16 RV1,RV2,T1,T2
	REAL*8 Medge,MSQ,kappa,MSQUSQ
        COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /BRPOINT/UQZ,WQZ
        COMMON /PISAVE/PI
        COMMON /ZCLEAR/SING
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TSO,TS1,STOKES
        COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
	COMMON /SPECTRA/RCRIT
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /FSU/FoSQ
	COMMON /SXXXXX/ISIM,Ibuoy

        nsteps=5450
        etaz=.32
        deta=etaz/(nsteps-1.d0)
        eta=0.d0

	etahi=30.d0

	hid=etahi/(nsteps-1.d0)

	MSQ=Medge*Medge
	MSQUSQ=MSQ*UQZ*UQZ

	his=etahi
!        do i=1,nsteps
        do i=1,1
        etasq=eta*eta

	AA=1.D0
	BB=SING*RE*UQZ
	CC=BESQ+etasq*RE*RE+SING*RE*(BZERO*WQZ-DBLE(OMEGA))
	CALL CQUAD(AA,BB,CC,R1,R2)


! some definitions
	pe=etasq*RE
	
	fpe=dsqrt(pe)
	pbsq=pe+BESQ

	teta=BZERO*WQZ-OMEGA
	tetasq=teta*teta
	tetore=teta/RE
	kappa=GAMMA+PRANDTL*RM
	RMG=RM*GAMMA
	REPRANT=RE*PRANDTL
	cbit=1.D0+SING*RMG*MSQ*tetore

	COEFF(0)=-SING*MSQ*RMG*UQZ
	COEFF(1)=-SING*MSQ*RMG*teta+RE*(MSQUSQ*kappa-1.D0)
	COEFF(2)=-SING*UQZ*(RE*REPRANT+MSQ*(2.D0*RMG*pbsq+ &
     & RE*RE*(2.D0*SING*tetore*kappa-PRANDTL*UQZ*UQZ)))
	COEFF(3)=MSQ*RE*kappa*(UQZ*UQZ*BESQ+tetasq)+ &
     & SING*RE*REPRANT*teta*(3.D0*MSQUSQ-1.D0)+ &
     & RE*pe*MSQUSQ*kappa-2.D0*RE*pbsq*cbit
	COEFF(4)=-SING*UQZ*(MSQ*(RMG*pbsq*pbsq+ &
     & 2.D0*SING*RE*teta*(pe*GAMMA+BESQ*kappa))+ &
     & REPRANT*RE*(pbsq+MSQ*(2.D0*SING*RM*tetore*pe- &
     & 3.D0*tetasq)))
	COEFF(5)=RE*pbsq*(MSQ*tetasq*kappa-pbsq*cbit)- &
     & SING*RE*teta*REPRANT*(pbsq-MSQ*tetasq)

	cvb=MSQ*RE*fpe*Ibuoy/FoSQ
     	COEFF(3)=COEFF(3)+GAMMA*cvb
     	COEFF(4)=COEFF(4)+SING*RE*PRANDTL*UQZ*cvb
	COEFF(5)=COEFF(5)+(teta*PRANDTL*SING*RE+GAMMA*pbsq)*cvb

!        CALL NAGROOT(5,MAXN,COEFF,ROOTS)

	IF(I == 1)THEN
	DO J=1,5
!	RCRIT(J)=ROOTS(J)
	end do

	RV1=R1
	RV2=R2

! Hi Re acoustic branch points
	ISIDE=0
	txc=dsqrt(etasq*RE)*MSQ/FoSQ
	BB=(MSQ*UQZ*UQZ-1.D0)*(etasq*RE+BESQ-txc)+MSQ*teta*teta 
	IF(DBLE(BB) < 0.D0)ISIDE=1
	BB=CDSQRT(BB) 
	T1=(-MSQ* UQZ*teta+BB)/(MSQ*UQZ*UQZ-1.D0)
	T2=(-MSQ* UQZ*teta-BB)/(MSQ*UQZ*UQZ-1.D0)

	END IF

	BB=CDSQRT((MSQ*UQZ*UQZ-1.D0)*(etasq*RE+BESQ-txc)+MSQ*teta*teta) 
	R1=(-MSQ* UQZ*teta+BB)/(MSQ*UQZ*UQZ-1.D0)
	R2=(-MSQ* UQZ*teta-BB)/(MSQ*UQZ*UQZ-1.D0)

! Incompressible limit branch points/curves

	b1=SING*(BESQ+eta*RE)

	BB=CDSQRT(RE*PRANDTL*(4.D0*SING*teta+RE*UQZ*UQZ*PRANDTL)+ &
     & 4.D0*(BESQ+eta*RE))
	b2=SING*0.5D0*(-RE*UQZ*PRANDTL+BB)
	b3=SING*0.5D0*(-RE*UQZ*PRANDTL-BB)
!        WRITE(98,121)eta, dconjg(R1),dconjg(R2),dconjg(b1),dconjg(-b1),
!     & dconjg(b2),dconjg(b3),T1,T2
!        WRITE(98,121)eta, (RCRIT(J),J=1,5)

        eta=eta+deta
	his=his-hid
        end do



	RCRIT(1)=T1
	RCRIT(2)=T2
	RCRIT(3)=dconjg(b1)
	RCRIT(4)=dconjg(-b1)
	RCRIT(5)=dconjg(RV1)

	CALL SORT(RCRIT,MAXN,5,RV1,RV2,T1,T2,ISIDE)

121     format(30E19.8)

        RETURN
        END



        SUBROUTINE BRANCH_NAG
        use param_info, only : MPG
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MPG-1)*IL)
	PARAMETER (MAXN=5)
	COMPLEX*16 COEFF(0:MAXN), ROOTS(MAXN)
	COMPLEX*16 RCRIT(MAXN),hcoeff(0:MAXN)
	COMPLEX*16 hROOTS(MAXN)

        COMPLEX*16 OMEGA,ABSQ,SING
        COMPLEX*16 b1,b2,b3,b4,scale
	COMPLEX*16 AA,BB,CC,R1,R2,teta,tetore
	COMPLEX*16 R1x, R2x
	complex*16 cbit,tetasq
	COMPLEX*16 RV1,RV2,T1,T2
	REAL*8 Medge,MSQ,kappa,MSQUSQ
        COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /BRPOINT/UQZ,WQZ
        COMMON /PISAVE/PI
        COMMON /ZCLEAR/SING
	COMMON /GASPRP/RGAS,GAMMA,PRANDTL,UMUO,TSO,TS1,STOKES
        COMMON /ZSTOKE/YLMU,RS,RM,RMINV,REM
	COMMON /SPECTRA/RCRIT
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /FSU/FoSQ
	COMMON /SXXXXX/ISIM,Ibuoy

! note call to co2aaf...requires poly in foloowing form:
! eg: COEFF(0) * Y**2 + COEFF(1) Y + COEFF(2)
!	NN=2
!	COEFF(0)=1.
!	COEFF(1)=-1.
!	COEFF(2)=-2.

        nsteps=1250
        etaz=.32
        deta=etaz/(nsteps-1.d0)
        eta=0.d0

	etahi=30.d0

	hid=etahi/(nsteps-1.d0)

!	PRINT*,BZERO, Medge, ' beta, w/u, Me'
!	PRINT*,UQZ,WQZ,' : u w'
	MSQ=Medge*Medge
	MSQUSQ=MSQ*UQZ*UQZ

	his=etahi
        do i=1,nsteps
!        do i=1,1
        etasq=eta*eta

	hetasq=his*his

	AA=1.D0
	BB=SING*RE*UQZ
	CC=BESQ+etasq*RE*RE+SING*RE*(BZERO*WQZ-DBLE(OMEGA))
	CALL CQUAD(AA,BB,CC,R1,R2)

        IF(IBRANCH == 1)WRITE(99,121)eta, dconjg(R1),dconjg(R2)

	AA=1.D0
	BB=SING*RE*UQZ
	CC=BESQ+hetasq*RE+SING*RE*(BZERO*WQZ-DBLE(OMEGA))
	CALL CQUAD(AA,BB,CC,R1x,R2x)

        IF(IBRANCH == 1)WRITE(89,121)eta, dconjg(R1x),dconjg(R2x)


! some definitions
	pe=etasq*RE
	
	fpe=dsqrt(pe)
	pbsq=pe+BESQ

	hpe=hetasq*RE
	hfpe=dsqrt(hpe)
	hpbsq=hpe+BESQ

	teta=BZERO*WQZ-OMEGA
	tetasq=teta*teta
	tetore=teta/RE
	kappa=GAMMA+PRANDTL*RM
	RMG=RM*GAMMA
	REPRANT=RE*PRANDTL
	cbit=1.D0+SING*RMG*MSQ*tetore

	COEFF(0)=-SING*MSQ*RMG*UQZ
	COEFF(1)=-SING*MSQ*RMG*teta+RE*(MSQUSQ*kappa-1.D0)
	COEFF(2)=-SING*UQZ*(RE*REPRANT+MSQ*(2.D0*RMG*pbsq+ &
     & RE*RE*(2.D0*SING*tetore*kappa-PRANDTL*UQZ*UQZ)))
	COEFF(3)=MSQ*RE*kappa*(UQZ*UQZ*BESQ+tetasq)+ &
     & SING*RE*REPRANT*teta*(3.D0*MSQUSQ-1.D0)+ &
     & RE*pe*MSQUSQ*kappa-2.D0*RE*pbsq*cbit
	COEFF(4)=-SING*UQZ*(MSQ*(RMG*pbsq*pbsq+ &
     & 2.D0*SING*RE*teta*(pe*GAMMA+BESQ*kappa))+ &
     & REPRANT*RE*(pbsq+MSQ*(2.D0*SING*RM*tetore*pe- &
     & 3.D0*tetasq)))
	COEFF(5)=RE*pbsq*(MSQ*tetasq*kappa-pbsq*cbit)- &
     & SING*RE*teta*REPRANT*(pbsq-MSQ*tetasq)

	cvb=MSQ*RE*fpe*Ibuoy/FoSQ
     	COEFF(3)=COEFF(3)+GAMMA*cvb
     	COEFF(4)=COEFF(4)+SING*RE*PRANDTL*UQZ*cvb
	COEFF(5)=COEFF(5)+(teta*PRANDTL*SING*RE+GAMMA*pbsq)*cvb


!        CALL NAGROOT(5,MAXN,COEFF,ROOTS)
        CALL LAG_ROOT(5,MAXN,COEFF,ROOTS)

	hCOEFF(0)=-SING*MSQ*RMG*UQZ
	hCOEFF(1)=-SING*MSQ*RMG*teta+RE*(MSQUSQ*kappa-1.D0)
	hCOEFF(2)=-SING*UQZ*(RE*REPRANT+MSQ*(2.D0*RMG*hpbsq+ &
     & RE*RE*(2.D0*SING*tetore*kappa-PRANDTL*UQZ*UQZ)))
	hCOEFF(3)=MSQ*RE*kappa*(UQZ*UQZ*BESQ+tetasq)+ &
     & SING*RE*REPRANT*teta*(3.D0*MSQUSQ-1.D0)+ &
     & RE*hpe*MSQUSQ*kappa-2.D0*RE*hpbsq*cbit
	hCOEFF(4)=-SING*UQZ*(MSQ*(RMG*hpbsq*hpbsq+ &
     & 2.D0*SING*RE*teta*(hpe*GAMMA+BESQ*kappa))+ &
     & REPRANT*RE*(hpbsq+MSQ*(2.D0*SING*RM*tetore*hpe- &
     & 3.D0*tetasq)))
	hCOEFF(5)=RE*hpbsq*(MSQ*tetasq*kappa-hpbsq*cbit)- &
     & SING*RE*teta*REPRANT*(hpbsq-MSQ*tetasq)

	cvb=MSQ*RE*hfpe/FoSQ
     	hCOEFF(3)=hCOEFF(3)+GAMMA*cvb
     	hCOEFF(4)=hCOEFF(4)+SING*RE*PRANDTL*UQZ*cvb
	hCOEFF(5)=hCOEFF(5)+(teta*PRANDTL*SING*RE+GAMMA*hpbsq)*cvb

!        CALL NAGROOT(5,MAXN,hCOEFF,hROOTS)
        CALL LAG_ROOT(5,MAXN,hCOEFF,hROOTS)


	IF(IBRANCH == 1)THEN
!        WRITE(97,121)eta, (dconjg(ROOTS(J)),j=1,5)
!        WRITE(87,121)hpe, (dconjg(hROOTS(J)),j=1,5)
	END IF

	IF(I == 1)THEN
	DO J=1,5
	RCRIT(J)=ROOTS(J)
	end do

	RV1=R1
	RV2=R2

! Hi Re acoustic branch points
	ISIDE=0
	txc=dsqrt(etasq*RE)*MSQ/FoSQ
	BB=(MSQ*UQZ*UQZ-1.D0)*(etasq*RE+BESQ-txc)+MSQ*teta*teta 
	IF(DBLE(BB) < 0.D0)ISIDE=1
	BB=CDSQRT(BB) 
	T1=(-MSQ* UQZ*teta+BB)/(MSQ*UQZ*UQZ-1.D0)
	T2=(-MSQ* UQZ*teta-BB)/(MSQ*UQZ*UQZ-1.D0)

	END IF

	BB=CDSQRT((MSQ*UQZ*UQZ-1.D0)*(etasq*RE+BESQ-txc)+MSQ*teta*teta) 
	R1=(-MSQ* UQZ*teta+BB)/(MSQ*UQZ*UQZ-1.D0)
	R2=(-MSQ* UQZ*teta-BB)/(MSQ*UQZ*UQZ-1.D0)

! Incompressible limit branch points/curves

	b1=SING*(BESQ+eta*RE)

	BB=CDSQRT(RE*PRANDTL*(4.D0*SING*teta+RE*UQZ*UQZ*PRANDTL)+ &
     & 4.D0*(BESQ+eta*RE))
	b2=SING*0.5D0*(-RE*UQZ*PRANDTL+BB)
	b3=SING*0.5D0*(-RE*UQZ*PRANDTL-BB)
!        WRITE(98,121)eta, dconjg(R1),dconjg(R2),dconjg(b1),dconjg(-b1),
!     & dconjg(b2),dconjg(b3),T1,T2
!         WRITE(98,121)eta, (RCRIT(J),J=1,5)

        eta=eta+deta
	his=his-hid
        end do

	CALL SORT(RCRIT,MAXN,5,RV1,RV2,T1,T2,ISIDE)

	IF(IBRANCH == 1)THEN
!	WRITE(93,'(2E19.7)')(RCRIT(J),j=1,4)
	CLOSE(93)
	CLOSE(97)
	CLOSE(99)
	END IF
121     format(30E19.8)

        RETURN
        END



	SUBROUTINE CQUAD(AA,BB,CC,R1,R2)
	IMPLICIT REAL*8 (a-h,o-z)
	COMPLEX*16 AA,BB,CC,R1,R2
	COMPLEX*16 BSQ
	BSQ=CDSQRT(BB*BB-4.D0*AA*CC)
	R1=(-BB+BSQ)*0.5D0/AA
	R2=(-BB-BSQ)*0.5D0/AA
	RETURN
	END



      	SUBROUTINE LAG_ROOT(N,MAXN,COEFF,ROOTS)
      	PARAMETER        (NIN=5,NOUT=6)
        parameter	(idegree=5) 
      	PARAMETER       (MAXDEG=10)
      
 	LOGICAL first_3, xpolish
      	PARAMETER        (first_3= .FALSE., xpolish= .FALSE. )
      	complex*16 roots(*)                     ! intent(inout) 
      	complex*16 poly(idegree+1)       
      	COMPLEX*16 COEFF(0:MAXN) 
	
      	IF (N > 0  .AND.  N <= idegree) THEN
	do I=0,N
	POLY(N+1-I)=COEFF(I)
	END DO
	call cmplx_roots_5(roots, first_3, poly, xpolish)
	
      	ELSE
         WRITE (NOUT,*) 'N is out of range'
	STOP
      	END IF
	RETURN
99999 FORMAT (1X,A,I0)
      	END SUBROUTINE LAG_ROOT

! nolonger NAGROOT required, kept in Just in case, as a check on Lag_root accuracy!
! april 2021, M. S. Mughal
      	SUBROUTINE NAGROOT(N,MAXN,COEFF,ROOTS)
      	PARAMETER        (NIN=5,NOUT=6)
      	PARAMETER        (MAXDEG=10)
      	LOGICAL          SCALE
      	PARAMETER        (SCALE=.TRUE.)
      
 	LOGICAL first_3, xpolish
        parameter	(idegree=5) ! specifying this speeds up optimization a lit
      	PARAMETER        (first_3=.FALSE., xpolish=.FALSE. )
      	double complex ap_roots(idegree)                     ! intent(inout) 
      	double complex ap_poly(idegree+1)       
      
      
      	DOUBLE PRECISION A(2,0:MAXDEG), W(4*MAXDEG+4), Z(2,MAXDEG)
      	COMPLEX*16 COEFF(0:MAXN), ROOTS(MAXN)
!      EXTERNAL         C02AFF
      	IF (N > 0  .AND.  N <= MAXDEG) THEN
	do I=0,N
	A(1,I)=DBLE(COEFF(I))
	A(2,I)=DIMAG(COEFF(I))
	ap_POLY(N+1-I)=COEFF(I)
	end do
	call cmplx_roots_5(ap_roots, first_3, ap_poly, xpolish)

         IFAIL = 0
!         CALL C02AFF(A,N,SCALE,Z,W,IFAIL)
         DO I = 1, N
!	ROOTS(I)=DCMPLX(Z(1,I),Z(2,I))
	END DO
	
	DO KK=1,N
!	WRITE(158,'(8E25.13)')ROOTS(KK), ap_ROOTS(KK)
	ROOTS(KK)=ap_ROOTS(KK)
	END DO
	
      	ELSE
         WRITE (NOUT,*) 'N is out of range'
	STOP
      	END IF
	RETURN
99999 FORMAT (1X,A,I0)
      END

	SUBROUTINE P_SPEC(XMAX)	
	use param_NUMBERCRUNCH
	IMPLICIT DOUBLE PRECISION (a-h,o-z)
	PARAMETER (KSTR=2*MODE+1,KCRF=2*KMODE+1)
	COMPLEX*16 ABSQ,OMEGA
	DOUBLE PRECISION Medge
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
        COMPLEX*16 QZLIN(KSTR)
        COMMON /FRVARI/OMEGA,RE
        COMMON /SCLLOC/Uedge,Medge,Rlocal,Tedge,Delzer
	COMMON /PLOTXY/xpltmin,xpltmax
	COMMON /DISCRET/Xreal,Ximag,xUPPER1,yUPPER1,xUPPER2, &
     & yUPPER2,xbufu,xbufl,xblankt,xac
	COMMON /PLUTO/I3DSTAB,IQZFULL
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA

	yscale=1.
	XMAX=XMAX*yscale
	XMAX2=XMAX

	IF(BZERO > XMAX .AND. ICODE == 1)THEN
	XMAX=1.25D0*BZERO
	XMAX2=XMAX*0.7
	END IF
	
	IGRID=1
	ISKIP=3
	ISYMB=6
	ISYMBF=0
	xsymb=0.045
	xsymb2=0.06
	xchar=0.09
	adash1=0.25
	adash2=0.05
	bdash1=0.05
	bdash2=0.035
	xl=5.25
	yl=5.
	xpos=1.75
	ypos=3.75
	OPEN(99,file='cspec.cmds',status='unknown')
	WRITE(99,'(A,F5.3)')'character ',xchar
	WRITE(99,'(A,F8.6,A,F8.5,A,G11.5)') &
     & 'title \ita\ Eigenvalue spectra : \omega\ = ', &
     & OMRANGE(1),' ; \beta\ = ',CRFBETA(1),' ; Re =',Re	
	IF(IBRANCH == 1)THEN
	WRITE(99,'(A,F4.2,2G13.4)')'xlim ',xl,xpltmin,xpltmax
	WRITE(99,'(A,F4.2,2G13.4)')'ylim ',yl,-XMAX, XMAX2
	ELSE
	WRITE(99,'(A,F4.2,2G13.4)')'xlim ',xl
	WRITE(99,'(A,F4.2,2G13.4)')'ylim ',yl
	END IF
	WRITE(99,'(A)')'xlabel \ita\ \alpha\ \sub{r}'
	WRITE(99,'(A)')'ylabel \ita\ \alpha\ \sub{i}'
	WRITE(99,'(A)')'output plotxy.ps'
	IF(IGRID == 1)THEN
	WRITE(99,'(A)')'frame grid'
	ELSE
	WRITE(99,'(A)')'frame grid'
	END IF

	WRITE(99,'(A)')'colour red'
	WRITE(99,'(A)')'file PSESPEC.DAT'
	WRITE(99,'(A,I0)')'skip ',iskip
	WRITE(99,'(A,I0,1x,f3.2)')'symbol ',ISYMBF,xsymb
	WRITE(99,'(A)')'mode 20 2 3'
	WRITE(99,'(A)')'read'
	
	IF(IQZFULL == 1)THEN
	WRITE(99,*)
	WRITE(99,'(A)')'colour black'
	WRITE(99,'(A)')'file PSESPEC.FULL'
	WRITE(99,'(A,I0)')'skip ',iskip
	WRITE(99,'(A,I0,1x,f3.2)')'symbol ',ISYMB,xsymb2
	WRITE(99,'(A)')'mode 20 2 3'
	WRITE(99,'(A)')'read'
	WRITE(99,*)
	end if
	
	IF(IBRANCH == 1)THEN
	do k=2,4,2
	WRITE(99,'(A)')'file fort.99'
	WRITE(99,'(A)')'symbol -1'
	WRITE(99,'(A,f3.2,1x,f3.2)')'dash ',adash1,adash2 
	WRITE(99,'(A,2I3)')'mode 20 ',k,k+1
	WRITE(99,'(A)')'read'
	end do
	
	IF(ICODE == 1)THEN
	WRITE(99,*)
	
	do k=2,3
	WRITE(99,'(A)')'file fort.98'
	WRITE(99,'(A,f3.2,1x,f3.2)')'dash ',bdash1,bdash2 
	WRITE(99,'(A,2I3)')'mode 20 ',k,1
	WRITE(99,'(A)')'read'
	end do
	END IF
	
	WRITE(99,*)
		
	ISYMB2=9
	xsyms=0.005
	do k=2,10,2
	WRITE(99,'(A)')'file fort.97'
	WRITE(99,'(A,f3.2,1x,f3.2)')'dash ',0.,0. 
	WRITE(99,'(A,I0,1x,f4.3)')'symbol ',ISYMB2,xsyms
	WRITE(99,'(A,2I3)')'mode 20 ',k,k+1
	WRITE(99,'(A)')'read'
	end do
	
	WRITE(99,*)
	
	WRITE(99,'(A)')'file fort.91'
	WRITE(99,'(A,f3.2,1x,f3.2)')'dash ',0.,0. 
	WRITE(99,'(A,I0,1x,f4.3)')'symbol ',ISYMB2,0.08
	WRITE(99,'(A,2I3)')'mode 20 ',2,3
	WRITE(99,'(A)')'read'
		
	WRITE(99,*)
	do k=2,4,2
	WRITE(99,'(A)')'file fort.94'
	WRITE(99,'(A,f3.2,1x,f3.2)')'dash ',0.,0. 
	WRITE(99,'(A,I0,1x,f4.3)')'symbol ',3,0.15
	WRITE(99,'(A,2I3)')'mode 20 ',k-1,k
	WRITE(99,'(A)')'read'
	end do	
	
	END IF


	WRITE(99,'(A)')'colour black'
	WRITE(99,'(A,1x,2F5.2)')'plot ',xpos,ypos
	WRITE(99,'(A)')'stop'
	CLOSE(99)
	
	RETURN
	END




	SUBROUTINE SEARCH2(IA,IB,ALFA,GROWTH,NSPEC,IFREQ,IBOX)
        use param_info, only : MPG, MODE, KMODE
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	PARAMETER (IL=9,KE=(MPG-1)*IL)
        PARAMETER (NLOTS=MODE,NBETA=KMODE)
	PARAMETER (KSTR=2*NLOTS+1,KCRF=2*NBETA+1)
	DOUBLE PRECISION ALFA(*),GROWTH(*)
	COMPLEX*16 OMEGA,ABSQ
	COMPLEX*16 QZLIN(KSTR)
	REAL*8 FRENCY(KSTR),OMRANGE(KSTR),CRFBETA(KCRF)
 	COMMON /QZMETH/INGUESS,IGLB
        COMMON /QZRANGE/QZLIN,FRENCY,OMRANGE,CRFBETA
        COMMON /QZINTEG/IGLOB,NFREQ,NCRBET
	COMMON /FRVARI/OMEGA,RE
        COMMON /ROBETA/BZERO,BESQ,ABSQ
        COMMON /SCREENP/NOUTPUT,ITV,ISCREEN,IWVDUMP,NFSWICH
        COMMON /BLACKBX/QZWAVE,QZAMPL,QZINDS
        COMMON /BLACINT/IFIND
	COMMON /MULTF2/ISMOOTH
	COMMON /BRPOINT/UQZ,WQZ
	COMMON /DISCRET/Xreal,Ximag,xUPPER1,yUPPER1,xUPPER2, &
     & yUPPER2,xbufu,xbufl,xblankt,xac
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /PPM_PSE3D/xAct,yAct

	IENGQZ=1


	IFIND=0
	IF(IBOX == 1)GOTO 11011
	IF(INGUESS == 1)THEN
	OPEN(13,FILE='PSEIGEN.DAT',STATUS='UNKNOWN')
	WRITE(ITV,*)
	WRITE(ITV,33)
	WRITE(ITV,55)
	ELSE
	OPEN(12,FILE='PSEMULTF.DAT',STATUS='UNKNOWN')
	IF(ISMOOTH == 1)OPEN(9,FILE='PSEMULTF2.DAT', &
     & STATUS='UNKNOWN')
	WRITE(ITV,*)
	WRITE(ITV,733)
	WRITE(ITV,755)
	END IF
11011	CONTINUE

33	FORMAT(7X,'Mode',9X,'Wave No.') 
55	FORMAT(7X,'----',3x,'---------------------')
733      FORMAT(12X,'Wave No.',10X,'Frequency')
755      FORMAT(7X,'------------------',2X,'-------------')

! trying to find the point to use as reference point...

	X=Xreal
! ok now evalute the branch points... to identify the continuous spectrum


	IDENT=999	
	IDENTP=999	

	xabuv=0.05D0
	xmore=0.02D0

	INSPEC=0
! check to see if anything worth investigating....< 5 usualy means
! some unusual parameter regime/dodgy bl profiles or very stable regime.
	IF(NSPEC < 5)THEN
	IF(INGUESS == 1)THEN
	WRITE(ITV,40057)
	ELSE
	WRITE(ITV,4057)
	END IF
	INSPEC=-1
	END IF

	slop=0.
	do 207 I=1,NSPEC
	xr=ALFA(I)
	yr=GROWTH(I)
	slope=dabs((yr-yo)/(xr-xo))
	IF(SLOPE < SLOP)goto 207
	xAct=xr
	yACT=yr
	SLOP=SLOPE
        ldent=i
207	CONTINUE

	xo=Xreal
	yo=Ximag


111	format(5g19.6)

	xref=Xreal
	yref=Ximag

	IF(IENGQZ == 1)THEN
	xengcut=.20D0
	xengmor=-0.5
	ELSE
	xengcut=1.D0
	xengmor=-0.5
	END IF

! this attemts to blank out modes too close to the viscous continuous
! spectrum curve..
        CALL SDERROR(xref,yref,ALFA,GROWTH,NSPEC,SGRADM)
! cuttoff if discrete modes are too close to continous spectra.
	SGRADL=0.8*SGRADM
	SUPPR=1.25

        slop=-99.

! msm
	xabuv=0.015D0
	xmore=0.025D0

! cuttoff for too stable eigenvalues!

	ytomuch=Ximag-xengcut
	ytxmuch=ytomuch-xengmor


        IDENT=999
        IDENTP=999

        IF(X > 0.0)THEN
        XO=Xref*(1.D0+xmore)
        ELSE
        XO=Xref*(1.D0-xmore)
        END IF

! new stuff 
	IF(dabs(yref) > 0.00001)THEN
        YO=yref+xabuv*dabs(yref)
	ELSE
        YO=xabuv*DABS(X)
	END IF

        XRES=XO*(1.D0-xmore)
        XRESUP=XO*(1.D0+xmore)

        do 20 I=1,NSPEC
	IF(I == ldent)GOTO 20
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yr-yo)/(xr-xo)
        IF(xr <= XRES .OR. XR < XO)GOTO 20
        IF(SLOPE < SLOP .OR. DABS(SLOPE) > SGRADL)goto 20
        IF(SLOPE > SUPPR)goto 20
	IF(yr < ytomuch)GOTO 20

        xAct=xr
        yACT=yr
        ident=i
        SLOP=SLOPE
20      CONTINUE

! msms

	IF(IDENT /= 999)GOTO 66066
	TWODOK=BZERO*WQZ
	IF(DABS(TWODOK) < 1.D-3)GOTO 66066

	slop=99.

	IF(X > 0.0)THEN
	XO=Xref*(1.D0-xmore)
	ELSE
	XO=Xref*(1.D0+xmore)
	END IF

        do 205 I=1,NSPEC
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yo-yr)/(xo-xr)
	IF(I == ldent)GOTO 205
        IF(xr >= XRESUP .OR. XR > XO)GOTO 205
	IF(SLOPE > SLOP)goto 205
	IF(SLOPE < -SGRADL)goto 205
        IF(SLOPE < -SUPPR)goto 205
	IF(yr < ytomuch)GOTO 205
        xAct=xr
        yACT=yr
        ident=i
        SLOP=SLOPE
205      CONTINUE

	IDENTP=IDENT
	IF(IDENT /= 999)THEN
	IDENTP=-19
	END IF

66066	CONTINUE
	IF(IDENT == 999)THEN
	IF(INSPEC == -1)WRITE(ITV,4056)
	IF(INSPEC == 0)WRITE(ITV,456)
	WRITE(ITV,457)
	END IF

	IF(IDENT == 999)GOTO 1963
	IF(IBOX == 1)THEN
	QZWAVE=xAct
	QZAMPL=yAct
	QZINDS=SLOP
	ELSE IF(IBOX == 0 .AND. INGUESS == 1)THEN
	WRITE(ITV,44)IA,IB,xAct,yAct
	WRITE(13,11)IA,IB,xAct,yAct
	ELSE 
        WRITE(ITV,744)xAct,yAct,FRENCY(IFREQ)
	IF(ISMOOTH == 0)THEN
	WRITE(12,711)xAct,yAct,FRENCY(IFREQ)
	ELSE
        WRITE(9,711)xAct,yAct
	END IF
	END IF

711     FORMAT(2g19.7,E19.9)
744     FORMAT(5X,2F10.6,E16.8)
11	FORMAT(2I4,2E19.9)
44	FORMAT(5X,2I3,2F12.7)

1963	CONTINUE
	IF(IDENTP == -19)GOTO 55055


        IF(X > 0.0)THEN
        XO=Xref*(1.D0+xmore)
        ELSE
        XO=Xref*(1.D0-xmore)
        END IF


	IF(IDENT == 1)THEN
	IDENT2=2
	xAct2=ALFA(2)
	yAct2=GROWTH(2)
	slop=(yAct2-yo)/(xAct2-xo)
	IST=3
	ELSE
	IDENT2=1
	xAct2=ALFA(1)
	yAct2=GROWTH(1)
	slop=(yAct2-yo)/(xAct2-xo)
	IST=2
	END IF
	IMESS=0
	do 30 I=IST,NSPEC
	xr=ALFA(I)
	yr=GROWTH(I)
	slope=(yr-yo)/(xr-xo)
        IF(xr <= XRES .OR. XR < XO)GOTO 30
	IF(I == LDENT)GOTO 30
	IF(I == IDENT)GOTO 30
	IF(SLOPE < SLOP .OR. DABS(SLOPE) > SGRADM)goto 30
	IF(yr < ytxmuch)GOTO 30
	xAct2=xr
	yACT2=yr
	ident2=i
	IFIND=1
	SLOP=SLOPE
30	CONTINUE

	GOTO 77077

55055	CONTINUE

        IF(X > 0.0)THEN
        XO=Xref*(1.D0-xmore)
        ELSE
        XO=Xref*(1.D0+xmore)
        END IF

        IF(IDENT == 1)THEN
        IDENT2=2
        xAct2=ALFA(2)
        yAct2=GROWTH(2)
        slop=(yo-yAct2)/(xo-xAct2)
        IST=3
        ELSE
        IDENT2=1
        xAct2=ALFA(1)
        yAct2=GROWTH(1)
        slop=(yo-yAct2)/(xo-xAct2)
        IST=2
        END IF
        IMESS=0
        do 3011 I=IST,NSPEC
        xr=ALFA(I)
        yr=GROWTH(I)
        slope=(yo-yr)/(xo-xr)
        IF(xr >= XRESUP .OR. xr > XO)GOTO 3011
	IF(I == ldent)GOTO 3011
        IF(I == IDENT)GOTO 3011
	IF(SLOPE > SLOP)goto 3011
	IF(SLOPE < -SGRADM)goto 3011
	IF(SLOPE < -SUPPR-1.D0)goto 3011
	IF(yr < ytxmuch)GOTO 3011
        xAct2=xr
        yACT2=yr
        ident2=i
        IFIND=1
        SLOP=SLOPE
3011    CONTINUE

77077	CONTINUE

	IF(IBOX == 1)RETURN
	IQFIND=0
	IF(DABS(xAct2) < 1.D-6 .AND. dabs(yAct2) < 1.D-6)THEN
	WRITE(ITV,701)
	IQFIND=1
	END IF
	IF(IQFIND == 1)RETURN

	IF(INGUESS == 1)THEN
	WRITE(ITV,44)IA,IB,xAct2,yAct2
!	WRITE(12,22)IA,IB,xAct2,yAct2
	ELSE
        WRITE(ITV,744)xAct2,yAct2,FRENCY(IFREQ)
	END IF


456	FORMAT(2X,' No discrete eigenvalues found - You may be ')
4056	FORMAT(2X,' No discrete eigenvalues found - You really are ')
457	FORMAT(2X,' in a highly stable region. Try following :')
4057	format(2x,' ** Warning : you are probably in a very ', &
     & 'stable region **',/, &
     & '   I would not trust the result in PSEMULTF if any !!!',/, &
     & '   Either change Beta/Frequency combination or NSTART ')
40057	format(2x,' ** Warning : you are probably in a very ', &
     & 'stable region **',/, &
     & '   I would not trust the result in PSEIGEN if any !!!',/, &
     & '   Either change Beta/Frequency combination or NSTART ')
701	FORMAT(2X,'         No alternatives found !!! ')

22	FORMAT(2I3,2F10.6,F8.4,I0,'  -  A l t e r n a t i v e  -')
	RETURN
	END





	SUBROUTINE SORT(RCRIT,MAXN,NROOT,RV1,RV2,T1,T2,ISIDE)
	IMPLICIT REAL*8 (a-h,o-z)
        COMPLEX*16 RCRIT(*)
	COMPLEX*16 X1,X2,XVISCUS,RV1,RV2,XTHERML,T1,T2
	COMMON /DISCRET/Xreal,Ximag,xUPPER1,yUPPER1,xUPPER2, &
     & yUPPER2,xbufu,xbufl,xblankt,xac
	COMMON /IDISTYP/ICODE,IBRANCH,IQZ
	COMMON /PLOTXY/xpltmin,xpltmax

	IF(CDABS(RV1) < CDABS(RV2))THEN
	XVISCUS=RV1
	ELSE
	XVISCUS=RV2
	END IF


	NTOP=NROOT
20	NSTOP=0
	DO K=1,NTOP-1
	X1=RCRIT(K)
	X2=RCRIT(K+1)
	IF(CDABS(X1) > CDABS(X2))THEN
	RCRIT(K)=X2
	RCRIT(K+1)=X1
	NSTOP=K
	END IF
	END DO
	NTOP=NSTOP
	IF(NSTOP > 1)GOTO 20

! ok sorted that lot out now attemt to find the corresponding thermal
! branch point

	CMAX=1.E20
	KR=1
	DO K=1,NROOT-2
	see =CDABS(RCRIT(K)-XVISCUS)
	IF(see < CMAX)THEN
	KR=K
	CMAX=see
	end if
	END DO

	XTHERML=RCRIT(KR)

	IF(KR == 1)THEN
	ELSE
	X1=RCRIT(1)
	RCRIT(1)=RCRIT(KR)
	RCRIT(KR)=X1
	END IF

!	ok now shift by 1 and form conjugate as I work in this
	DO K=6,1,-1
	RCRIT(K+1)=DCONJG(RCRIT(K))
	END DO
	RCRIT(1)=DCONJG(XVISCUS)

! all done, now make decisions about which branch point to
! sit on
! first sort the acosutic modes in order of greatest real part.
	IF(DBLE(RCRIT(4)) > DBLE(RCRIT(3)))THEN
	XTHERML=RCRIT(4)
	RCRIT(4)=RCRIT(3)
	RCRIT(3)=XTHERML
	END IF

! note T1,T2 are in fact the same as RCRIT(3),RCRIT(4) aproximated.
!	PRINT*,T1,T2,' are the Hi Re acoustic modes'
!	PRINT*,'ISIDE ',ISIDE
	IF(IBRANCH == 1)THEN
!	WRITE(94,'(4E19.8)')T1,T2
!	CLOSE(94)
	END IF

! then, center my search on viscous branch which we identify as

	IF( DIMAG(RCRIT(1))  >=  DIMAG(RCRIT(2)) )THEN
	Xreal=DBLE(RCRIT(1))
	Ximag=DIMAG(RCRIT(1))
	ELSE
	Xreal=DBLE(RCRIT(2))
	Ximag=DIMAG(RCRIT(2))
	END IF

! next where does the acoustic barnch cut lie? on Real or Imag. axis?
! though not really gaining anything useful here!
! actually ISIDE=0 means on real line
! ISIDE=1 imaginary line

	yUPPER1=Dimag(RCRIT(3))
	yUPPER2=Dimag(RCRIT(4))
	xUPPER1=Dble(RCRIT(3))
	xUPPER2=Dble(RCRIT(4))


	DTOL=1.D-5
	DISTXR=DBLE(RCRIT(3)-RCRIT(4))
	DISTXI=DIMAG(RCRIT(3)-RCRIT(4))

	xbuf=dabs(DISTXR)
	ybuf=dabs(DISTXI)

	ICODE=0
	IDUM=1
	iqbit=0

	xbufl=-9999.
	xbufu=9999.
	xblankt=0.
	xac=(xUPPER1+xUPPER2)*0.5D0


	IF(xUPPER1 > Xreal .AND. xUPPER2 < Xreal)THEN
!	Print*,' lies in between of acoustic branches'
	xbufu=xUPPER1
	xbufl=xUPPER2
	xblankt=dabs(xUPPER1-Xreal)
	xx=xbufu-xbufl
	xpltmax=xbufu+0.5*xx
	xpltmin=xbufl-0.5*xx
	ELSE IF(xUPPER1 > Xreal .AND. xUPPER2 > Xreal)THEN
!	Print*,' lies to right of viscous branch'
        xbufu=xUPPER2
        xblankt=dabs(xbufu-Xreal)
	xac=xbufu
	xpltmax=xbufu*1.5+xblankt+xac
	xpltmin=Xreal-2.25*dabs(Xreal)-xblankt-xac
	ELSE IF(xUPPER1 < Xreal .AND. xUPPER2 < Xreal)THEN
!	Print*,' lies to left of viscous branch'
        xbufl=xUPPER1
        xblankt=dabs(Xreal-xbufl)
	xac=xbufl
		if(xbufl > 0.)THEN
		xpltmin=xbufl*.5-xblankt-xac
		else
		xpltmin=xbufl*1.5-xblankt
		end if
	xpltmax=Xreal+2.25*dabs(Xreal)+xblankt+xac
	END IF


	IF(DABS(DISTXR) < DTOL .AND. DABS(DISTXI) < DTOL)THEN
!	print*,' close to being on origin! 0-0'
! yes but which way is the branch cut then ? for filtering purposes
! so not saying much !!!
	END IF

! then on 0,0 axis

	IF(DBLE(RCRIT(3)) < DTOL .AND. DBLE(RCRIT(4)) < DTOL)THEN
		IF(DABS(DISTXI) > DTOL)THEN
!	PRINT*,' lies on imaginary line 0-1 of length ',DISTXI
		ICODE=ICODE+1
		IDUM=0
		end if
	END IF

	IF(DIMAG(RCRIT(3)) < DTOL .AND. DIMAG(RCRIT(4)) < DTOL)THEN
		IF(DABS(DISTXR) > DTOL)THEN
!	PRINT*,' lies on real line 1-0 of length ',DISTXR
		ICODE=ICODE+1+IDUM
		ELSE
!	PRINT*,' lies on real line 1-0 of length zero ',DISTXR
		ICODE=1
		END IF
	END IF

        IF(DIMAG(RCRIT(3)) > DTOL .OR. DIMAG(RCRIT(4)) > DTOL)THEN
	IF(DABS(DBLE(RCRIT(3))) > DTOL)THEN
!        PRINT*,' lies on off real line 1-1 '
		ICODE=1
	END IF
	IF(DABS(DBLE(RCRIT(4))) > DTOL)THEN
!        PRINT*,' lies on off real line 1-1 '
                ICODE=1
                END IF
	ELSE
        END IF


!	PRINT*,'icode is : ', ICODE

	RETURN
	END




	
	SUBROUTINE SET_SEED(iseed)
	USE GPL_STUFF
	implicit real*8 (a-h,o-z)
	INTEGER :: idate(8),month(12)
! mt95	integer( kind = genrand_intg )  :: iseed
! mt19937	integer( kind = wi )  :: iseed
	LOGICAL YUP
	data month/31,29,31,30,31,30,31,31,30,31,30,31/
	CALL DATE_AND_TIME(VALUES=idate)
	idateLC(1)=NQYR+786 ; idateLC(2)=NQMN ; idateLC(3)=NQDy
  	iseed = float(idate(7))*(10./6.)+ idate(8)*100.0
	xmin=float(idate(6))*10.d0/6.d0+1.d-12
	imin=nint(xmin*10)
	iseed=iseed+imin-99
	iseedx=idate(3)+idate(5)*24+idate(6)*60*24+ &
     & idate(7)*24*60*60
	qbits=idate(8)*0.1d0
	qbits=(qbits-int(qbits))*10.d0

     	iseed=iseedx*iseed*qbits
	mdaysum=idate(2)-1
	mcc=0
	do kk=1,mdaysum
	mcc=mcc+month(kk)
	end do
	ndays=mcc+idate(3)
	mdaysum=idateLC(2)-1
	mccLC=0
	do kk=1,mdaysum
	mccLC=mccLC+month(kk)
	end do
	LCdays=mccLC+idateLC(3)
	pLC=idateLC(1)+float(LCdays-ndays)/365.0-idate(1)
	IF(pLC > 0.0)THEN
	ILCxx=0
	ELSE
	ILCxx=1
	stop
	END IF
	return
	end SUBROUTINE SET_SEED	

	SUBROUTINE EROS(IERR,RESIDU,ALERROR,IDECID)
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	COMPLEX*16 ALERROR
	IDECID=0
	IENGineer=1
	IF(IENGineer == 1)THEN
	WAVE=Dble(ALERROR)
	IF((WAVE) > 0.15)THEN
	IERR=1
	end if
	end if
	
	
	IF(IERR > 0)GOTO 11
!	wAvenumber isn't negAtive is it?
	WAVE=DIMAG(ALERROR)
!	IF(WAVE < 0.D0)IDECID=1
	IF(WAVE < 0.D0)IDECID=0
	RETURN
11	IF(IERR == 0)RETURN
	IDECID=1
	RETURN
	END
	
! end of CoPSE3d: shahid mughal check!!!!
! end of CoPSE3d: shahid mughal check!!!!

!   laguerre bits....Replaced with Jan Skowron & Andrew Gould code,
! Apache License, Version 2.0
! NOTICE: Slight Changes made to allow compilation with FORTRAN90
!
!
! BEGIN: APACHE
! 
!  Copyright 2012 Jan Skowron & Andrew Gould
!
!   Licensed under the Apache License, Version 2.0 (the "License");
!   you may not use this file except in compliance with the License.
!   You may obtain a copy of the License at
!
!       http://www.apache.org/licenses/LICENSE-2.0
!
!   Unless required by applicable law or agreed to in writing, software
!   distributed under the License is distributed on an "AS IS" BASIS,
!   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
!   See the License for the specific language governing permissions and
!   limitations under the License.
!
!-------------------------------------------------------------------!
!
! The authors also make this file available under the terms of
! GNU Lesser General Public License version 2 or any later version.
! (text of the LGPL licence 2 in NOTICE file)
!
!-------------------------------------------------------------------!
!
! A custom in the scientific comunity is (regardless of the licence
! you chose to use or distribute this software under)
! that if this code was important in the scientific process or
! for the results of your scientific work, we kindly ask you for the
! appropriate citation of the Paper (Skowron & Gould 2012), and
! we would be greatful if you pass the information about
! the proper citation to anyone whom you redistribute this software to.
!
!-------------------------------------------------------------------!
!
!    No    Subroutine
!
!     1   cmplx_roots_gen               - general polynomial solver, works for random degree, not as fast or robust as cmplx_roots_5
!     2   cmplx_roots_5                 - complex roots finding algorithm taylored for 5th order polynomial (with failsafes for polishing)
!     3   sort_5_points_by_separation   - sorting of an array of 5 points, 1st most isolated, 4th and 5th - closest
!     4   sort_5_points_by_separation_i - sorting same as above, returns array of indicies rather than sorted array
!     5   find_2_closest_from_5         - finds closest pair of 5 points
!     6   cmplx_laguerre                - Laguerre's method with simplified Adams' stopping criterion 
!     7   cmplx_newton_spec             - Newton's method with stopping criterion calculated every 10 steps
!     8   cmplx_laguerre2newton         - three regime method: Laguerre's, Second-order General method and Newton's
!     9   solve_quadratic_eq            - quadratic equation solver
!    10   solve_cubic_eq                - cubic equation solver based on Lagrange's method
!    11   divide_poly_1                 - division of the polynomial by (x-p)
!
! fortran 77 code
!
! Paper:  Skowron & Gould 2012  
!         "General Complex Polynomial Root Solver and Its Further Optimization for Binary Microlenses"
!
! for a full text see:
!     http://www.astrouw.edu.pl/~jskowron/cmplx_roots_sg/
!     or http://arxiv.org/find/astro-ph
!     or http://www.adsabs.harvard.edu/abstract_service.html
! see also file NOTICE and LICENSE
!
! ver. 2012.03.03 initial
! ver. 2014.03.12 bug fix
! ver. 2016.01.21 bug fix
! ver. 2016.04.28 bug fix
!
!-------------------------------------------------------------------!
! _1_                    CMPLX_ROOTS_GEN                            !
!-------------------------------------------------------------------!
      subroutine cmplx_roots_gen(roots, poly, degree, polish_roots_after, &
     & use_roots_as_starting_points)
      ! This subroutine finds roots of a complex polynomial. 
      ! It is general, however less fast or robust than cmplx_roots_5
      ! which contains failsafe checks in the polishing stage, but is
      ! designed only for 5th order polynomials.
      ! It uses a new dynamic root finding algorithm (see the Paper).
      !
      ! It can use Laguerre's method (subroutine cmplx_laguerre)
      ! or Laguerre->SG->Newton method (subroutine 
      ! cmplx_laguerre2newton - this is default choice) to find 
      ! roots. It divides polynomial one by one by found roots. At the 
      ! end it finds last root from Viete's formula for quadratic 
      ! equation. Finally, it polishes all found roots using a full
      ! polynomial and Newton's or Laguerre's method (default is
      ! Laguerre's - subroutine cmplx_laguerre). 
      ! You can change default choices by commenting out and uncommenting
      ! certain lines in the code below.
      !
      ! Note:
      ! - we solve for the last root with Viete's formula rather 
      !   than doing full Laguerre step (which is time consuming
      !   and unnecessary)
      ! - we do not introduce any preference to real roots
      ! - in Laguerre implementation we omit unneccesarry calculation of
      !   absolute values of denominator
      ! - we do not sort roots. If you need to sort 
      !   roots - we have provided sorting subroutine called:
      !   sort_5_points_by_separation, which sorts points from most 
      !   isolated to most close. Algorithm in this routine can be 
      !   easily used for number of points different than 5.
      !
      implicit none
      ! roots  - array which will hold all roots that had been found. 
      !          If the flag 'use_roots_as_starting_points' is set to 
      !          .true., then instead of point (0,0) we use value from
      !          this array as starting point for cmplx_laguerre
      ! poly -   is an array of polynomial cooefs, length = degree+1, 
      !          poly(1) is a constant term:
      !               1              2             3
      !          poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      ! degree - degree of the polynomial and size of 'roots' array
      ! polish_roots_after - after all roots have been found by dividing
      !          original polynomial by each root found,
      !          you can opt in to polish all roots using full  
      !          polynomial
      ! use_roots_as_starting_points - usually we start Laguerre's 
      !          method from point (0,0), but you can decide to use the 
      !          values of 'roots' array as starting point for each new
      !          root that is searched for. This is useful if you have
      !          very rough idea where some of the roots can be. 
      !
      integer degree ! intent(in)
      double complex poly(degree+1) ! intent(in), coeffs of the polynomial
      double complex roots(degree) ! intent(inout)
      logical polish_roots_after, use_roots_as_starting_points ! intent(in)

      double complex poly2(degree+1)
      double complex zero
      parameter(zero=(0d0,0d0))
      integer i, n, iter
      logical success
      double complex coef, prev


      do i=1,degree+1
        poly2(i)=poly(i)
      enddo
      
      ! initialize starting points
      if(.not.use_roots_as_starting_points) then
        do i=1,degree
          roots(i)=zero
        enddo
      endif

      ! skip small degree polynomials from doing Laguerre's method
      if(degree.le.1)then
        if(degree.eq.1) roots(1)=-poly(1)/poly(2)
        return
      endif


      do n=degree, 3, -1

        ! find root with Laguerre's method
        !call cmplx_laguerre(poly2, n, roots(n), iter, success) 
        ! or
        ! find root with (Laguerre's method -> SG method -> Newton's method)
        call cmplx_laguerre2newton(poly2, n, roots(n),iter, success, 2)
        if(.not.success) then
          roots(n)=zero
          call cmplx_laguerre(poly2, n, roots(n), iter, success)
        endif

        ! divide the polynomial by this root
        coef=poly2(n+1)
        do i=n,1,-1
          prev=poly2(i)
          poly2(i)=coef
          coef=prev+roots(n)*coef
        enddo
        ! coef now holds a remainder - should be close to 0

      enddo

      ! find all but last root with Laguerre's method
      !call cmplx_laguerre(poly2, 2, roots(2), iter, success)
      ! or
      call cmplx_laguerre2newton(poly2, 2, roots(2),iter, success, 2)
      if(.not.success) then
        call solve_quadratic_eq(roots(2),roots(1),poly2)
      else
        ! calculate last root from Viete's formula
        roots(1)=-(roots(2)+poly2(2)/poly2(3))
      endif



      if(polish_roots_after)then
        do n=1, degree ! ! polish roots one-by-one with a  full polynomial 
          call cmplx_laguerre(poly, degree, roots(n), iter, success) 
          ! or
          !call cmplx_newton_spec(poly, degree, roots(n), iter, success)
        enddo
      endif  

     
      return
      end


!-------------------------------------------------------------------!
! _2_                     CMPLX_ROOTS_5                             !
!-------------------------------------------------------------------!

     
     
      subroutine cmplx_roots_5(roots, first_3_roots, poly, polish_only)
      implicit none
      ! Subroutine finds or polishes roots of a complex polynomial 
      ! (degree=5)
      ! This routine is especially tailored for solving binary lens 
      ! equation in form of 5th order polynomial. 
      !
      ! Use of this routine, in comparission to 'cmplx_roots_gen' can yield
      ! consideribly faster code, because it makes polishing of the roots 
      ! (that come in as a guess from previous solutions) secure by
      ! implementing additional checks on the result of polishing. 
      ! If those checks are not satisfied then routine reverts to the 
      ! robust algorithm. These checks are designed to work for 5th order 
      ! polynomial originated from binary lens equation.
      !
      ! Usage:
      !
      ! polish_only == false - I do not know the roots, routine should  
      !                find them from scratch. At the end it
      !                sorts roots from the most distant to closest.
      !                Two last roots are the closest (in no particular
      !                order).
      ! polish_only = true - I do know the roots pretty well, for example
      !                I have changed the coefficiens of the polynomial 
      !                only a bit, so the two closest roots are 
      !                most likely still the closest ones.
      !                If the output flag 'first_3_roots'
      !                is returned as 'false', then first 3 returned roots
      !                are in the same order as initialy given to the 
      !                routine. The last two roots are the closest ones, 
      !                but in no specific order (!).
      !                If 'first_3_roots' is 'true' then
      !                it means that all roots had been resorted.
      !                Two last roots are the closest ones. First is most 
      !                isolated one.
      !
      !
      ! If you do not know the position of the roots just use flag
      ! polish_only=.false. In this case routine will find the roots by
      ! itself.
      
      ! Returns all five roots in the 'roots' array.
      !
      ! poly  - is an array of polynomial cooefs, length = degree+1 
      !       poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + poly(4) x^3 + ...
      ! roots - roots of the polynomial ('out' and optionally 'in')
      !
      !
      ! Jan Skowron 2011
      !
      integer degree
      parameter(degree=5) ! specifying this speeds up optimization a little
      double complex roots(degree)                     ! intent(inout) 
      logical first_3_roots          ! intent(out) 
      double complex poly(degree+1)                    ! intent(in)
      logical polish_only                          ! intent(in)
      !------------------------------------------------------------------
      !
      double complex  remainder, roots_robust(degree)
      double precision d2min
      integer iter, m, root4, root5, kk, go_to_robust, i, i2, loops
      double complex poly2(degree+1)
      !integer sorted_ind(degree)
      double complex zero
      parameter(zero=(0d0,0d0))
      logical succ
      
      do kk=1,degree
        roots_robust(kk) = roots(kk)
      enddo

      !---------------------------------------
      go_to_robust=0
      if(.not.polish_only) then
        do kk=1,degree
          roots(kk)=zero
        enddo
        go_to_robust=1
      endif
      
!      first_3_roots=.false.

      do loops=1,3
 111    continue

        ! ROBUST
        ! (we do not know the roots)
        if(go_to_robust>0) then 

          if(go_to_robust>2)then  ! something is wrong
            do kk=1,degree
              roots(kk)=roots_robust(kk)    ! return not-polished roots, because polishing creates errors
            enddo
            return
          endif

      
          do kk=1,degree+1
            poly2(kk)=poly(kk) ! copy coeffs
          enddo
          do m=degree,4,-1 ! find the roots one-by-one (until 3 are left to be found)
          call cmplx_laguerre2newton(poly2, m, roots(m), iter, succ, 2)
            if(.not.succ)then
              roots(m)=zero
              call cmplx_laguerre(poly2, m, roots(m), iter, succ)
            endif
            ! divide polynomial by this root
            call divide_poly_1(poly2, remainder, roots(m), poly2, m)
          enddo
          ! find last 3 roots with cubic euqation solver (Lagrange's method)
          call solve_cubic_eq(roots(1),roots(2),roots(3),poly2)
          ! all roots found
          
          ! sort roots - first will be most isolated, last two will be the closest
          call sort_5_points_by_separation(roots) 
          ! copy roots in case something will go wrong during polishing
          do kk=1,degree
            roots_robust(kk)=roots(kk)
          enddo

          ! set flag, that roots have been resorted
!          first_3_roots=.true.
        endif  ! go_to_robust>0

        ! POLISH 
        ! (we know the roots approximately, and we guess that last two are closest)
        !---------------------
          do kk=1,degree+1
            poly2(kk)=poly(kk) ! copy coeffs
          enddo

          do m=1,degree-2
          !do m=1,degree                      ! POWN - polish only with Newton (option)

            ! polish roots with full polynomial
          call cmplx_newton_spec(poly2, degree, roots(m), iter, succ)

            if(.not.succ)then
              ! go back to robust
              go_to_robust=go_to_robust+1
              do kk=1,degree  ! restart
                roots(kk)=zero
              enddo
              goto 111 ! go back
            endif

          enddo ! m=1,degree-2

          ! comment out division and quadratic if you (POWN) polish with Newton only
          do m=1,degree-2
          call divide_poly_1(poly2,remainder,roots(m),poly2,degree-m+1)
          enddo
          ! last two roots are found with quadratic equation solver 
          ! (this is faster and more robust, although little less accurate)
          call solve_quadratic_eq(roots(degree-1),roots(degree),poly2)
          ! all roots found and polished

          ! TEST ORDER
          ! test closest roots if they are the same pair as given to polish
          call find_2_closest_from_5(root4,root5,d2min, roots)

          ! check if the closest roots are not too close, this could happen
          ! when using polishing with Newton only, when two roots erroneously 
          ! colapsed to the same root. This check is not needed for polishing
          ! 3 roots by Newton and using quadratic  for the remaining two.
          ! If the real roots are so close indeed (very low probability), this will just 
          ! take more time and the unpolished result be returned at the end
          !if(d2min<1d-18) then             ! POWN - polish only with Newton 
          !  go_to_robust=go_to_robust+1    ! POWN - polish only with Newton
          !else                             ! POWN - polish only with Newton

          if((root4.lt.degree-1).or.(root5.lt.degree-1)) then
            ! after polishing some of the 3 far roots become one of the 2 closest ones
            ! go back to robust
            if(go_to_robust.gt.0)then
              ! if came from robust 
              ! copy two most isolated roots as starting points for new robust
              do i=1,degree-3
                roots(degree-i+1)=roots_robust(i)
              enddo
            else
              ! came from users initial guess
              ! copy first 2 roots (except the closest ones)
              i2=degree
              do i=1,degree
                if((i.ne.root4).and.(i.ne.root5))then
                  roots(i2)=roots(i)
                  i2=i2-1
                endif
                if(i2.le.3) exit ! do not copy those that will be done by cubic in robust
              enddo
            endif
            go_to_robust=go_to_robust+1
          else
            ! root4 and root5 comes from the initial last pair
            ! most common case
            return
          endif
          !endif                            ! POWN - polish only with Newton
        !---------------------
      enddo ! loops

      return
      end

!-------------------------------------------------------------------!
! _3_                 SORT_5_POINTS_BY_SEPARATION                   !
!-------------------------------------------------------------------!
      subroutine sort_5_points_by_separation(points)
      ! Sort array of five points 
      ! Most isolated point will become the first point in the array
      ! The closest points will be the last two points in the array
      ! 
      ! Algorithm works well for all dimensions. We put n=5 as 
      ! a hardcoded value just for optimization purposes.
      implicit none
      integer n
      parameter(n=5) !  works for different n as well, but is faster for n as constant (optimization)
      double complex points(n) ! intent(inout)

      integer sorted_points(n)
      double complex savepoints(n)
      integer i

      call sort_5_points_by_separation_i(sorted_points, points) 
      do i=1,n
        savepoints(i)=points(i)
      enddo
      do i=1,n
        points(i)=savepoints(sorted_points(i))
      enddo
      return
      end

!-------------------------------------------------------------------!
! _4_             SORT_5_POINTS_BY_SEPARATION_I                     !
!-------------------------------------------------------------------!
      subroutine sort_5_points_by_separation_i(sorted_points, points)
      ! Return index array that sorts array of five points 
      ! Index of the most isolated point will appear on the first place 
      ! of the output array.
      ! The indices of the closest 2 points will be at the last two 
      ! places in the 'sorted_points' array
      !
      ! Algorithm works well for all dimensions. We put n=5 as 
      ! a hardcoded value just for optimization purposes.
      implicit none
      integer n
      parameter(n=5) !  works for different n as well, but is faster for n as constant (optimization)
      integer sorted_points(n)      ! intent(out)
      double complex points(n)          ! intent(in)
       
      double precision dmin, d1, d2, d
      double precision distances2(n,n) 
      integer ki, kj, ind2, put
      double precision neigh1st(n), neigh2nd(n)
      double complex p

      do kj=1, n
        distances2(kj,kj)=1d100
      enddo
      dmin=1d100

      do kj=1, n               
        do ki=1, kj-1    
          p=points(ki)-points(kj)
          d=dble(conjg(p)*p)
          distances2(ki,kj)=d
          distances2(kj,ki)=d
        enddo
      enddo

      ! find neighbours  
      do kj=1, n
        neigh1st(kj)=1d100
        neigh2nd(kj)=1d100
      enddo
      do kj=1, n
        do ki=1, n
          d=distances2(kj,ki)
          if(d.lt.neigh2nd(kj))then
            if(d.lt.neigh1st(kj))then
              neigh2nd(kj)=neigh1st(kj)
              neigh1st(kj)=d
            else
              neigh2nd(kj)=d
            endif
          endif
        enddo
      enddo    

      ! initialize sorted_points
      do ki=1,n
        sorted_points(ki)=ki
      enddo   
     
      ! sort the rest 1..n-2
      do kj=2,n
        d1=neigh1st(kj)
        d2=neigh2nd(kj)
        put=1
        do ki=kj-1,1,-1
          ind2=sorted_points(ki)
          d=neigh1st(ind2)
          if(d.ge.d1) then
            if(d.eq.d1)then
              if(neigh2nd(ind2)>d2)then
                put=ki+1
                exit
              endif
            else
              put=ki+1
              exit
            endif
          endif
          sorted_points(ki+1)=sorted_points(ki)
        enddo
        sorted_points(put)=kj
      enddo
        
      return
      end

!-------------------------------------------------------------------!
! _5_                     FIND_2_CLOSEST_FROM_5                     !
!-------------------------------------------------------------------!
      subroutine find_2_closest_from_5(i1,i2,d2min, points) 
      ! Returns indices of the two closest points out of array of 5
      implicit none
      integer n
      parameter(n=5) ! will work for other n too, but it is faster with n as constant
      integer i1,i2        ! intent(out) 
      double complex points(n) ! intent(in) 
      double precision d2min         ! intent(out)
       
      double precision d2min1, d2
      integer i,j
      double complex p

      d2min1=1d100
      do j=1,n
        do i=1,j-1
          p=points(i)-points(j)
          d2=dble(conjg(p)*p)
          if(d2.le.d2min1)then
            i1=i
            i2=j
            d2min1=d2
          endif
        enddo
      enddo
      d2min=d2min1
      end


!-------------------------------------------------------------------!
! _6_                     CMPLX_LAGUERRE                            !
!-------------------------------------------------------------------!
      subroutine cmplx_laguerre(poly, degree, root, iter, success)
      implicit none
      ! Subroutine finds one root of a complex polynomial using 
      ! Laguerre's method. In every loop it calculates simplified
      ! Adams' stopping criterion for the value of the polynomial.
      !
      ! Uses 'root' value as a starting point (!!!!!)
      ! Remember to initialize 'root' to some initial guess or to 
      ! point (0,0) if you have no prior knowledge.
      !
      ! poly - is an array of polynomial cooefs
      !        length = degree+1, poly(1) is constant 
      !               1              2             3
      !          poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      ! degree - a degree of the polynomial
      ! root - input: guess for the value of a root
      !        output: a root of the polynomial
      ! iter - number of iterations performed (the number of polynomial
      !        evaluations and stopping criterion evaluation)
      ! success - is false if routine reaches maximum number of iterations
      !
      ! For a summary of the method go to: 
      ! http://en.wikipedia.org/wiki/Laguerre's_method
      !
      integer MAX_ITERS, FRAC_JUMP_EVERY, FRAC_JUMP_LEN
      parameter(MAX_ITERS=200)  ! Laguerre is used as a failsafe 
      ! constants needed to break cycles in the scheme
      parameter(FRAC_JUMP_EVERY=10,FRAC_JUMP_LEN=10)
      double precision FRAC_JUMPS(FRAC_JUMP_LEN)
      double precision pi
      parameter(pi = 3.141592653589793d0)
      double precision faq ! jump length
      double precision FRAC_ERR
      parameter(FRAC_ERR = 2.0d-15)  ! fractional error for kind=8 (see. Adams 1967 Eqs 9 and 10)

      ! subroutine parameters
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)


      double complex p         ! value of polynomial
      double complex dp        ! value of 1st derivative
      double complex d2p_half  ! value of 2nd derivative 
      integer i, k
      logical  good_to_go
      double complex denom, denom_sqrt, dx, newroot 
      double precision ek, absroot, abs2p
      double complex fac_netwon, fac_extra, F_half, c_one_nth
      double precision one_nth, n_1_nth, two_n_div_n_1
      double complex zero, c_one
      parameter(zero=(0d0,0d0),c_one=(1d0,0d0))
      double precision stopping_crit2

      !---------------------------------------
      data FRAC_JUMPS /0.64109297d0, 0.91577881d0, 0.25921289d0, &
     &                 0.50487203d0, 0.08177045d0, 0.13653241d0, &
     &                 0.306162d0  , 0.37794326d0, 0.04618805d0, &
     &                 0.75132137d0/     ! some random numbers


 
      iter=0
      success=.true.

      ! next if-endif block is an EXTREME failsafe, not usually needed, and thus turned off in this version.
      if(.false.)then ! change false-->true if you would like to use caution about having first coefficient == 0
        if(degree.lt.0) then
          write(*,*) 'Error: cmplx_laguerre: degree<0'
          return
        endif
        if(poly(degree+1).eq.zero) then
          if(degree.eq.0) return
          call cmplx_laguerre_wrapper(poly, degree-1, root, iter, success)
          return
        endif
        if(degree.le.1)then
          if(degree.eq.0) then  ! we know from previous check than poly(1) not equal zero
            success=.false.
            write(*,*) 'Warning: cmplx_laguerre: '// &
     &                 'degree=0 and poly(1)/=0, no roots'
            return
          else
            root=-poly(1)/poly(2)
            return
          endif
        endif
      endif
      !  end EXTREME failsafe        

      good_to_go=.false.
      one_nth=1d0/degree
      n_1_nth=(degree-1d0)*one_nth
      two_n_div_n_1=2d0/n_1_nth
      c_one_nth=dcmplx(one_nth,0d0)


      do i=1,MAX_ITERS
        ! prepare stoping criterion
        ek=abs(poly(degree+1))
        absroot=abs(root)
        ! calculate value of polynomial and its first two derivatives
        p  =poly(degree+1)
        dp =zero
        d2p_half=zero
        do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
          d2p_half=dp + d2p_half*root
          dp =p + dp*root
          p  =poly(k)+p*root    ! b_k
          ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
          ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
          ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
          ! Eq 8.
          ek=absroot*ek+abs(p)
        enddo
        iter=iter+1
        
        abs2p=dble(conjg(p)*p)
        if(abs2p.eq.0d0) return
        stopping_crit2=(FRAC_ERR*ek)**2
        if(abs2p.lt.stopping_crit2) then ! (simplified a little Eq. 10 of Adams 1967) 
          ! do additional iteration if we are less than 10x from stopping criterion
          if(abs2p.lt.0.01d0*stopping_crit2) then
            return ! return immediately, because we are at very good place
          else
            good_to_go=.true. ! do one iteration more
          endif
        else
          good_to_go=.false. ! reset if we are outside the zone of the root
        endif
      
        faq=1d0
        denom=zero 
        if(dp.ne.zero)then
          fac_netwon=p/dp
          fac_extra=d2p_half/dp
          F_half=fac_netwon*fac_extra
          
          denom_sqrt=sqrt(c_one-two_n_div_n_1*F_half)

          !G=dp/p  ! gradient of ln(p)
          !G2=G*G
          !H=G2-2d0*d2p_half/p  ! second derivative of ln(p)
          !denom_sqrt=sqrt( (degree-1)*(degree*H-G2) )

          ! NEXT LINE PROBABLY CAN BE COMMENTED OUT 
          if(dble(denom_sqrt).ge.0d0)then ! use realpart(), dble() or dreal() in g77 or ifort77
            ! real part of a square root is positive for probably all compilers. You can 
            ! test this on your compiler and if so, you can omit this check
            denom=c_one_nth+n_1_nth*denom_sqrt
          else
            denom=c_one_nth-n_1_nth*denom_sqrt
          endif
        endif
        if(denom.eq.zero)then !test if demoninators are > 0.0 not to divide by zero
          dx=(absroot+1d0)*exp(dcmplx(0d0, &
     &       FRAC_JUMPS(mod(i,FRAC_JUMP_LEN)+1)*2*pi)) ! make some random jump
        else
          dx=fac_netwon/denom
          !dx=degree/denom
        endif
        
        newroot=root-dx
        if(newroot.eq.root) return ! nothing changes -> return
        if(good_to_go)then         ! this was jump already after stopping criterion was met
          root=newroot
          return
        endif

        if(mod(i,FRAC_JUMP_EVERY).eq.0) then ! decide whether to do a jump of modified length (to break cycles)
          faq=FRAC_JUMPS(mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1)
          !write(*,'(3i5,f11.6)') i, i/FRAC_JUMP_EVERY, mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1, faq
          newroot=root-faq*dx ! do jump of some semi-random length (0<faq<1)
        endif
        root=newroot
      enddo
      success=.false.
      ! too many iterations here  
      end


!-------------------------------------------------------------------!
! _7_                     CMPLX_NEWTON_SPEC                         !
!-------------------------------------------------------------------!
      subroutine cmplx_newton_spec(poly, degree, root, iter, success)
      implicit none
      ! Subroutine finds one root of a complex polynomial using 
      ! Newton's method. It calculates simplified Adams' stopping 
      ! criterion for the value of the polynomial once per 10 iterations,
      ! after initial iteration. This is done to speed up calculations
      ! when polishing roots that are known preety well, and stopping
      ! criterion does not significantly change in their neighborhood.
      !
      ! Uses 'root' value as a starting point (!!!!!)
      ! Remember to initialize 'root' to some initial guess.
      ! Do not initilize 'root' to point (0,0) if the polynomial 
      ! coefficients are strictly real, because it will make going 
      ! to imaginary roots impossible.
      !
      ! poly - is an array of polynomial cooefs
      !        length = degree+1, poly(1) is constant 
      !               1              2             3
      !          poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      ! degree - a degree of the polynomial
      ! root - input: guess for the value of a root
      !        output: a root of the polynomial
      ! iter - number of iterations performed (the number of polynomial
      !        evaluations)
      ! success - is false if routine reaches maximum number of iterations
      !
      ! For a summary of the method go to: 
      ! http://en.wikipedia.org/wiki/Newton's_method
      !
      integer MAX_ITERS, FRAC_JUMP_EVERY, FRAC_JUMP_LEN
      parameter(MAX_ITERS=50)
      ! constants needed to break cycles in the scheme
      parameter(FRAC_JUMP_EVERY=10,FRAC_JUMP_LEN=10)
      double precision FRAC_JUMPS(FRAC_JUMP_LEN)
      double precision pi
      parameter(pi = 3.141592653589793d0)
      double precision faq ! jump length
      double precision FRAC_ERR
      parameter(FRAC_ERR = 2.0d-15)  ! fractional error for kind=8 (see. Adams 1967 Eqs 9 and 10)

      ! subroutine parameters
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)


      double complex p    ! value of polynomial
      double complex dp   ! value of 1st derivative 
      integer i, k
      logical  good_to_go
      double complex dx, newroot
      double precision ek, absroot, abs2p
      double complex zero
      parameter(zero=(0d0,0d0))
      double precision stopping_crit2

      !---------------------------------------
      data FRAC_JUMPS /0.64109297d0, 0.91577881d0, 0.25921289d0, &  
     &                 0.50487203d0, 0.08177045d0, 0.13653241d0, &  
     &                 0.306162d0  , 0.37794326d0, 0.04618805d0, & 
     &                 0.75132137d0/     ! some random numbers


      iter=0
      success=.true.

      ! next if-endif block is an EXTREME failsafe, not usually needed, and thus turned off in this version.
      if(.false.)then ! change false-->true if you would like to use caution about having first coefficient == 0
        if(degree.lt.0) then
          write(*,*) 'Error: cmplx_newton_spec: degree<0'
          return
        endif
        if(poly(degree+1).eq.zero) then
          if(degree.eq.0) return
          call cmplx_newton_spec_wrapper(poly, degree-1, root, iter, success)
          return
        endif
        if(degree.le.1)then
          if(degree.eq.0) then  ! we know from previous check than poly(1) not equal zero
            success=.false.
            write(*,*) 'Warning: cmplx_newton_spec: degree=0 and'// &
     &                 ' poly(1)/=0, no roots'
            return
          else
            root=-poly(1)/poly(2)
            return
          endif
        endif
      endif
      ! end EXTREME failsafe
 
      good_to_go=.false.

      stopping_crit2 = 0d0  ! value not important, will be initialized anyway on the first loop (because mod(1,10)==1)
      do i=1,MAX_ITERS
        faq=1d0

        ! prepare stoping criterion
        ! calculate value of polynomial and its first two derivatives
        p  =poly(degree+1)
        dp =zero

        if(mod(i,10).eq.1) then ! calculate stopping criterion every tenth iteration
          ek=abs(poly(degree+1))
          absroot=abs(root)
          do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
            dp =p + dp*root
            p  =poly(k)+p*root    ! b_k
            ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
            ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
            ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
            ! Eq 8.
            ek=absroot*ek+abs(p)
          enddo
          stopping_crit2=(FRAC_ERR*ek)**2
        else                  ! calculate just the value and derivative
          do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
            dp =p + dp*root
            p  =poly(k)+p*root    ! b_k
          enddo
        endif
        iter=iter+1

        
        abs2p=dble(conjg(p)*p)
        if(abs2p.eq.0d0) return

        if(abs2p.lt.stopping_crit2) then ! (simplified a little Eq. 10 of Adams 1967)
          if(dp.eq.zero) return ! if we have problem with zero, but we are close to the root, just accept
          ! do additional iteration if we are less than 10x from stopping criterion
          if(abs2p.lt.0.01d0*stopping_crit2) then
            return ! return immediately, because we are at very good place
          else
            good_to_go=.true. ! do one iteration more
          endif
        else
          good_to_go=.false. ! reset if we are outside the zone of the root
        endif


        if(dp.eq.zero)then
          ! problem with zero
          dx=(abs(root)+1d0)* &
     &       exp(dcmplx(0d0,FRAC_JUMPS(mod(i,FRAC_JUMP_LEN)+1)*2*pi)) ! make some random jump
        else
          dx=p/dp  ! Newton method, see http://en.wikipedia.org/wiki/Newton's_method
        endif

      

        newroot=root-dx
        if(newroot.eq.root) return ! nothing changes -> return
        if(good_to_go)then         ! this was jump already after stopping criterion was met
          root=newroot
          return
        endif

        if(mod(i,FRAC_JUMP_EVERY).eq.0) then ! decide whether to do a jump of modified length (to break cycles)
          faq=FRAC_JUMPS(mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1)
          newroot=root-faq*dx ! do jump of some semi-random length (0<faq<1)
        endif
        root=newroot
      enddo
      success=.false.
      return
      ! too many iterations here  
      end

!-------------------------------------------------------------------!
! _8_                     CMPLX_LAGUERRE2NEWTON                     !
!-------------------------------------------------------------------!
      subroutine cmplx_laguerre2newton &
     &           (poly, degree, root, iter, success, starting_mode)
      implicit none
      ! Subroutine finds one root of a complex polynomial using 
      ! Laguerre's method, Second-order General method and Newton's
      ! method - depending on the value of function F, which is a 
      ! combination of second derivative, first derivative and
      ! value of polynomial [F=-(p"*p)/(p'p')].
      ! 
      ! Subroutine has 3 modes of operation. It starts with mode=2
      ! which is the Laguerre's method, and continues until F 
      ! becames F<0.50, at which point, it switches to mode=1, 
      ! i.e., SG method (see paper). While in the first two
      ! modes, routine calculates stopping criterion once per every
      ! iteration. Switch to the last mode, Newton's method, (mode=0) 
      ! happens when becomes F<0.05. In this mode, routine calculates
      ! stopping criterion only once, at the beginning, under an 
      ! assumption that we are already very close to the root.
      ! If there are more than 10 iterations in Newton's mode, 
      ! it means that in fact we were far from the root, and
      ! routine goes back to Laguerre's method (mode=2).
      !
      ! Uses 'root' value as a starting point (!!!!!)
      ! Remember to initialize 'root' to some initial guess or to 
      ! point (0,0) if you have no prior knowledge.
      !
      ! poly - is an array of polynomial cooefs
      !        length = degree+1, poly(1) is constant 
      !               1              2             3
      !          poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      ! degree - a degree of the polynomial
      ! root - input: guess for the value of a root
      !        output: a root of the polynomial
      ! iter - number of iterations performed (the number of polynomial
      !        evaluations and stopping criterion evaluation)
      ! success - is false if routine reaches maximum number of iterations
      ! starting_mode - this should be by default = 2. However if you  
      !                 choose to start with SG method put 1 instead. 
      !                 Zero will cause the routine to 
      !                 start with Newton for first 10 iterations, and 
      !                 then go back to mode 2.
      !                 
      !
      ! For a summary of the method see the paper: Skowron & Gould (2012)
      !

      integer MAX_ITERS, FRAC_JUMP_EVERY, FRAC_JUMP_LEN
      parameter(MAX_ITERS=50)
      ! constants needed to break cycles in the scheme
      parameter(FRAC_JUMP_EVERY=10,FRAC_JUMP_LEN=10)
      double precision FRAC_JUMPS(FRAC_JUMP_LEN)
      double precision pi
      parameter(pi = 3.141592653589793d0)
      double precision faq ! jump length
      double precision FRAC_ERR
      parameter(FRAC_ERR = 2.0d-15)  ! fractional error for kind=8 (see. Adams 1967 Eqs 9 and 10)

      ! subroutine parameters
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)
      integer starting_mode     ! intent(in)


      double complex p         ! value of polynomial
      double complex dp        ! value of 1st derivative
      double complex d2p_half  ! value of 2nd derivative 
      integer i, j, k
      logical  good_to_go
      double complex denom, denom_sqrt, dx, newroot
      double precision ek, absroot, abs2p, abs2_F_half
      double complex fac_netwon, fac_extra, F_half, c_one_nth
      double precision one_nth, n_1_nth, two_n_div_n_1
      integer mode
      double complex zero, c_one
      parameter(zero=(0d0,0d0),c_one=(1d0,0d0))
      double precision stopping_crit2

      !---------------------------------------
      data FRAC_JUMPS /0.64109297d0, 0.91577881d0, 0.25921289d0, &
     &                 0.50487203d0, 0.08177045d0, 0.13653241d0, &
     &                 0.306162d0  , 0.37794326d0, 0.04618805d0, &
     &                 0.75132137d0/     ! some random numbers

     
      iter=0
      success=.true.
      stopping_crit2 = 0d0  ! value not important, will be initialized anyway on the first loop (because mod(1,10)==1)

      ! next if-endif block is an EXTREME failsafe, not usually needed, and thus turned off in this version.
      if(.false.)then ! change false-->true if you would like to use caution about having first coefficient == 0
        if(degree.lt.0) then
          write(*,*) 'Error: cmplx_laguerre2newton: degree<0'
          return
        endif
        if(poly(degree+1).eq.zero) then
          if(degree.eq.0) return 
          call cmplx_laguerre2newton_wrapper &
     &         (poly, degree-1, root, iter, success, starting_mode)
          return
        endif
        if(degree.le.1)then
          if(degree.eq.0) then  ! we know from previous check than poly(1) not equal zero
            success=.false.
            write(*,*) 'Warning: cmplx_laguerre2newton: '// &
     &                 'degree=0 and poly(1)/=0, no roots'
            return
          else
            root=-poly(1)/poly(2)
            return
          endif  
        endif
      endif
      ! end EXTREME failsafe

      j=1
      good_to_go=.false.

      mode=starting_mode  ! mode=2 full laguerre, mode=1 SG, mode=0 newton

      do ! infinite loop, just to be able to come back from newton, if more than 10 iteration there

      !------------------------------------------------------------- mode 2
      if(mode.ge.2) then  ! LAGUERRE'S METHOD
        one_nth=1d0/degree
        n_1_nth=(degree-1d0)*one_nth
        two_n_div_n_1=2d0/n_1_nth
        c_one_nth=dcmplx(one_nth,0d0)
      
        do i=1,MAX_ITERS  !
          faq=1d0

          ! prepare stoping criterion
          ek=abs(poly(degree+1))
          absroot=abs(root)
          ! calculate value of polynomial and its first two derivatives
          p  =poly(degree+1)
          dp =zero
          d2p_half=zero
          do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
            d2p_half=dp + d2p_half*root
            dp =p + dp*root
            p  =poly(k)+p*root    ! b_k
            ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
            ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
            ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
            ! Eq 8.
            ek=absroot*ek+abs(p)
          enddo
          abs2p=dble(conjg(p)*p) !abs(p)
          iter=iter+1
          if(abs2p.eq.0d0) return
          
          stopping_crit2=(FRAC_ERR*ek)**2
          if(abs2p.lt.stopping_crit2) then ! (simplified a little Eq. 10 of Adams 1967) 
            ! do additional iteration if we are less than 10x from stopping criterion
            if(abs2p.lt.0.01d0*stopping_crit2) then ! ten times better than stopping criterion
              return ! return immediately, because we are at very good place
            else
              good_to_go=.true. ! do one iteration more
            endif
          else
            good_to_go=.false. ! reset if we are outside the zone of the root
          endif

          denom=zero
          if(dp.ne.zero)then 
            fac_netwon=p/dp
            fac_extra=d2p_half/dp
            F_half=fac_netwon*fac_extra

            abs2_F_half=dble(conjg(F_half)*F_half)
            if(abs2_F_half.le.0.0625d0)then     ! F<0.50, F/2<0.25
              ! go to SG method
              if(abs2_F_half.le.0.000625d0)then ! F<0.05, F/2<0.025
                mode=0 ! go to Newton's
              else
                mode=1 ! go to SG
              endif
            endif
            
            
            denom_sqrt=sqrt(c_one-two_n_div_n_1*F_half)

            ! NEXT LINE PROBABLY CAN BE COMMENTED OUT
            if(dble(denom_sqrt).ge.0d0)then  ! use realpart(), dble() or dreal() for g77 or ifort77
              ! real part of a square root is positive for probably all compilers. You can 
              ! test this on your compiler and if so, you can omit this check
              denom=c_one_nth+n_1_nth*denom_sqrt
            else
              denom=c_one_nth-n_1_nth*denom_sqrt
            endif
          endif
          if(denom.eq.zero)then !test if demoninators are > 0.0 not to divide by zero
            dx=(abs(root)+1d0)*exp( &
     &          dcmplx(0d0,FRAC_JUMPS(mod(i,FRAC_JUMP_LEN)+1)*2*pi)) ! make some random jump
          else
            dx=fac_netwon/denom
          endif
     
          newroot=root-dx
          if(newroot.eq.root) return ! nothing changes -> return
          if(good_to_go)then         ! this was jump already after stopping criterion was met
            root=newroot
            return
          endif

          if(mode.ne.2) then 
            root=newroot
            j=i+1    ! remember iteration number
            exit     ! go to Newton's or SG 
          endif

          if(mod(i,FRAC_JUMP_EVERY).eq.0) then ! decide whether to do a jump of modified length (to break cycles)
            faq=FRAC_JUMPS(mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1)
            newroot=root-faq*dx ! do jump of some semi-random length (0<faq<1)
          endif
          root=newroot
        enddo ! do mode 2

        if(i.ge.MAX_ITERS) then
          success=.false.
          return
        endif

      endif ! if mode 2


      !------------------------------------------------------------- mode 1
      if(mode.eq.1) then  ! SECOND-ORDER GENERAL METHOD (SG)

        do i=j,MAX_ITERS  !
          faq=1d0

          ! calculate value of polynomial and its first two derivatives
          p  =poly(degree+1)
          dp =zero
          d2p_half=zero
          if(mod(i-j,10).eq.0)then
            ! prepare stoping criterion
            ek=abs(poly(degree+1))
            absroot=abs(root)
            do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
              d2p_half=dp + d2p_half*root
              dp =p + dp*root
              p  =poly(k)+p*root    ! b_k
              ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
              ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
              ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
              ! Eq 8.  
              ek=absroot*ek+abs(p)
            enddo
            stopping_crit2=(FRAC_ERR*ek)**2
          else
            do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
              d2p_half=dp + d2p_half*root
              dp =p + dp*root
              p  =poly(k)+p*root    ! b_k
            enddo
          endif


          abs2p=dble(conjg(p)*p) !abs(p)**2
          iter=iter+1
          if(abs2p.eq.0d0) return


          if(abs2p.lt.stopping_crit2) then ! (simplified a little Eq. 10 of Adams 1967) 
            if(dp.eq.zero) return
            ! do additional iteration if we are less than 10x from stopping criterion
            if(abs2p.le.0.01d0*stopping_crit2) then ! ten times better than stopping criterion
              return ! return immediately, because we are at very good place
            else
              good_to_go=.true. ! do one iteration more
            endif
          else
            good_to_go=.false. ! reset if we are outside the zone of the root
          endif
          
          if(dp.eq.zero)then !test if demoninators are > 0.0 not to divide by zero
            dx=(abs(root)+1d0)*exp( &
     &         dcmplx(0d0,FRAC_JUMPS(mod(i,FRAC_JUMP_LEN)+1)*2*pi)) ! make some random jump
          else
            fac_netwon=p/dp
            fac_extra=d2p_half/dp
            F_half=fac_netwon*fac_extra

            abs2_F_half=dble(conjg(F_half)*F_half)
            if(abs2_F_half.le.0.000625d0)then ! F<0.05, F/2<0.025
              mode=0 ! set Newton's, go there after jump
            endif
          
            dx=fac_netwon*(c_one+F_half)  ! SG
          endif
          
          newroot=root-dx
          if(newroot.eq.root) return ! nothing changes -> return
          if(good_to_go)then         ! this was jump already after stopping criterion was met
            root=newroot
            return
          endif

          if(mode.ne.1) then 
            root=newroot
            j=i+1    ! remember iteration number
            exit     ! go to Newton's
          endif

          if(mod(i,FRAC_JUMP_EVERY).eq.0) then ! decide whether to do a jump of modified length (to break cycles)
            faq=FRAC_JUMPS(mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1)
            newroot=root-faq*dx ! do jump of some semi-random length (0<faq<1)
          endif
          root=newroot



        enddo ! do mode 1

        if(i.ge.MAX_ITERS) then
          success=.false.
          return
        endif
     
      endif ! if mode 1


      !------------------------------------------------------------- mode 0
      if(mode.eq.0) then  ! newton

        do i=j,j+10  ! do only 10 iterations the most, then go back to full Laguerre's
          faq=1d0

          
          ! calculate value of polynomial and its first two derivatives
          p  =poly(degree+1)
          dp =zero
          if(i.eq.j)then ! calculate stopping crit only once at the begining
            ! prepare stoping criterion
            ek=abs(poly(degree+1))
            absroot=abs(root)
            do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
              dp =p + dp*root
              p  =poly(k)+p*root    ! b_k
              ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
              ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
              ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
              ! Eq 8.  
              ek=absroot*ek+abs(p)
            enddo
            stopping_crit2=(FRAC_ERR*ek)**2
          else
            do k=degree,1,-1 ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
              dp =p + dp*root
              p  =poly(k)+p*root    ! b_k
            enddo
          endif
          abs2p=dble(conjg(p)*p) !abs(p)**2
          iter=iter+1
          if(abs2p.eq.0d0) return


          if(abs2p.lt.stopping_crit2) then ! (simplified a little Eq. 10 of Adams 1967)
            if(dp.eq.zero) return 
            ! do additional iteration if we are less than 10x from stopping criterion
            if(abs2p.lt.0.01d0*stopping_crit2) then ! ten times better than stopping criterion
              return ! return immediately, because we are at very good place
            else
              good_to_go=.true. ! do one iteration more
            endif
          else
            good_to_go=.false. ! reset if we are outside the zone of the root
          endif
        
          if(dp.eq.zero)then ! test if demoninators are > 0.0 not to divide by zero
            dx=(abs(root)+1d0)*exp( &
     &          dcmplx(0d0,FRAC_JUMPS(mod(i,FRAC_JUMP_LEN)+1)*2*pi)) ! make some random jump
          else
            dx=p/dp
          endif
          
          newroot=root-dx
          if(newroot.eq.root) return ! nothing changes -> return
          if(good_to_go)then         
            root=newroot
            return
          endif

          ! this loop is done only 10 times. So skip this check
          !if(mod(i,FRAC_JUMP_EVERY).eq.0) then ! decide whether to do a jump of modified length (to break cycles)
          !  faq=FRAC_JUMPS(mod(i/FRAC_JUMP_EVERY-1,FRAC_JUMP_LEN)+1)
          !  newroot=root-faq*dx ! do jump of some semi-random length (0<faq<1)
          !endif
          root=newroot

        enddo ! do mode 0 10 times

        if(iter.ge.MAX_ITERS) then
          ! too many iterations here
          success=.false.
          return
        endif
        mode=2 ! go back to Laguerre's. This happens when we were unable to converge in 10 iterations with Newton's

      endif ! if mode 0

      enddo ! end of infinite loop

      !------------------------------------------------------------- 
      success=.false.
      end


!-------------------------------------------------------------------!
! _9_                     SOLVE_QUADRATIC_EQ                        !
!-------------------------------------------------------------------!
      subroutine solve_quadratic_eq(x0,x1,poly)
      ! Quadratic equation solver for complex polynomial (degree=2)
      implicit none
      double complex x0,x1    ! intent(out)
      double complex poly(*)  ! intent(in)  ! coeffs of the polynomial
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2
      double complex a, b, c, b2, delta

      double complex  val, x
      integer  i
      double complex zero
      parameter(zero=(0d0,0d0))


      a=poly(3)
      b=poly(2)
      c=poly(1)
      ! quadratic equation: a z^2 + b z + c = 0  

      b2=b*b
      delta=sqrt(b2-4d0*(a*c))
      if( dble(conjg(b)*delta).ge.0d0 )then  ! scallar product to decide the sign yielding bigger magnitude
        ! use dble(), dreal() or realpart() instead of real() not to loose precision in ifort77 or g77
        x0=-0.5d0*(b+delta)
      else
        x0=-0.5d0*(b-delta)
      endif
      if(x0.eq.zero)then
        x1=zero
      else ! Viete's formula
        x1=c/x0
        x0=x0/a
      endif


      if(.false.)then  ! print the results

        x=x0
        val=poly(3)
        do i=2,1,-1
          val=val*x+poly(i)
        enddo
        write(*,'(2f19.15,a3,2f19.15)') x,' ->',val

        x=x1
        val=poly(3)
        do i=2,1,-1
          val=val*x+poly(i)
        enddo
        write(*,'(2f19.15,a3,2f19.15)') x,' ->',val

      endif

      end

!-------------------------------------------------------------------!
! _10_                    SOLVE_CUBIC_EQ                            !
!-------------------------------------------------------------------!
      subroutine solve_cubic_eq(x0,x1,x2,poly)
      ! Cubic equation solver for complex polynomial (degree=3)
      ! http://en.wikipedia.org/wiki/Cubic_function   Lagrange's method
      implicit none
      double complex x0, x1, x2   ! intent(out)
      double complex poly(*)      ! intent(in)  ! coeffs of the polynomial
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3             4
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + poly(4) x^3
      double complex zeta, zeta2, zero
      double precision third
      parameter(zero=(0d0,0d0))
      parameter(zeta =(-0.5d0, 0.8660254037844386d0))  ! sqrt3(1)
      parameter(zeta2=(-0.5d0,-0.8660254037844386d0))  ! sqrt3(1)**2
      parameter(third=0.3333333333333333d0)            ! 1/3
      double complex s0, s1, s2
      double complex E1 ! x0+x1+x2
      double complex E2 ! x0x1+x1x2+x2x0
      double complex E3 ! x0x1x2
      double complex A, B, a_1, E12
      double complex delta, A2

      double complex val, x
      integer  i

      a_1=poly(4)**(-1)
      E1=-poly(3)*a_1
      E2=poly(2)*a_1
      E3=-poly(1)*a_1
      
      s0=E1
      E12=E1*E1
      A=2d0*E1*E12-9d0*E1*E2+27d0*E3  ! =  s1^3 + s2^3
      B=E12-3d0*E2                    !  = s1 s2
      ! quadratic equation: z^2-Az+B^3=0  where roots are equal to s1^3 and s2^3
      A2=A*A
      delta=sqrt(A2-4d0*(B*B*B))
      if( dble(conjg(A)*delta).ge.0d0 )then  ! scallar product to decide the sign yielding bigger magnitude
        ! use dble(), dreal() or realpart() instead of real() not to loose precision in ifort77 or g77
        s1=(0.5d0*(A+delta))**third
      else
        s1=(0.5d0*(A-delta))**third
      endif
      if(s1.eq.zero)then
        s2=zero
      else
        s2=B/s1
      endif


      x0=third*(s0+s1+s2)
      x1=third*(s0+s1*zeta2+s2*zeta )
      x2=third*(s0+s1*zeta +s2*zeta2)


      if(.false.)then  ! print the results

        x=x0
        val=poly(4)
        do i=3,1,-1
          val=val*x+poly(i)
        enddo
        write(*,'(2f19.15,a3,2f19.15)') x,' ->',val

        x=x1
        val=poly(4)
        do i=3,1,-1
          val=val*x+poly(i)
        enddo
        write(*,'(2f19.15,a3,2f19.15)') x,' ->',val

        x=x2
        val=poly(4)
        do i=3,1,-1
          val=val*x+poly(i)
        enddo
        write(*,'(2f19.15,a3,2f19.15)') x,' ->',val


      endif
      end


!-------------------------------------------------------------------!
! _11_                    DIVIDE_POLY_1                             !
!-------------------------------------------------------------------!
      subroutine divide_poly_1(polyout, remainder, p, polyin, degree)
      ! Subroutine will divide complex polynomial 'polyin' by (x-p)
      ! results will be returned in polynomial 'polyout' of degree-1
      ! The remainder of the division will be returned in 'remainder'
      !
      ! You can provide same array as 'polyin' and 'polyout' - this
      ! routine will work fine, though it will not set to zero the 
      ! unused, highest coefficient in the output array. You just have
      ! remember the proper degree of a polynomial.
      implicit none
      integer degree ! intent(in) 
      double complex polyout(degree)  ! intent(out)
      double complex remainder         ! intent(out)
      double complex p                ! intent(in) 
      double complex polyin(degree+1) ! intent(in)  ! coeffs of the polynomial
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      integer i
      double complex coef, prev

      coef=polyin(degree+1)
      do i=1,degree
        polyout(i)=polyin(i)
      enddo
      do i=degree,1,-1
        prev=polyout(i)
        polyout(i)=coef
        coef=prev+p*coef
      enddo
      remainder=coef
      return
      end

!-------------------------------------------------------------------!
! _12_                    EVAL_POLY                                 !
!-------------------------------------------------------------------!
      double complex function eval_poly(x, poly, degree, errk)
      ! Evaluation of the complex polynomial 'poly' of a given degree 
      ! at the point 'x'. This routine calculates also the simplified
      ! Adams' (1967) stopping criterion. ('errk' should be multiplied 
      ! by 2d-15 for double precision, real*8, arithmetic)
      implicit none
      double complex x              ! intent(in)
      integer degree            ! intent(in)
      double precision errk               ! intent(out)
      double complex poly(degree+1) ! intent(in)  ! coeffs of the polynomial
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      integer i
      double complex val
      double precision absx

      ! prepare stoping criterion
      errk=abs(poly(degree+1))
      val=poly(degree+1)
      absx=abs(x)
      do i=degree,1,-1  ! Horner Scheme, see for eg.  Numerical Recipes Sec. 5.3 how to evaluate polynomials and derivatives
        val=val*x+poly(i)
        ! Adams, Duane A., 1967, "A stopping criterion for polynomial root finding",
        ! Communications of the ACM, Volume 10 Issue 10, Oct. 1967, p. 655
        ! ftp://reports.stanford.edu/pub/cstr/reports/cs/tr/67/55/CS-TR-67-55.pdf
        ! Eq 8.
        errk=errk*absx+abs(val)
      enddo
      eval_poly=val

      ! if(abs(val)<2d-15*errk) return  ! (simplified a little Eq. 10 of Adams 1967)

      return
      end

!-------------------------------------------------------------------!
! _13_                    MULTIPLY_POLY_1                           !
!-------------------------------------------------------------------!
      subroutine multiply_poly_1(polyout, p, polyin, degree)
      ! Subroutine will multiply polynomial 'polyin' by (x-p)
      ! results will be returned in polynomial 'polyout' of degree+1
      !
      ! You can provide same array as 'polyin' and 'polyout' - this
      ! routine will work fine.
      implicit none
      integer degree               ! intent(in) ! OLD degree, new will be +1
      double complex polyout(degree+2) ! intent(out) 
      double complex p                 ! intent(in) 
      double complex polyin(degree+1)  ! intent(in) ! coeffs of the polynomial
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      integer i

      do i=1,degree+1         ! copy
        polyout(i)=polyin(i)
      enddo

      polyout(degree+2)=polyout(degree+1)
      do i=degree+1,2,-1
        polyout(i)=polyout(i-1)-polyout(i)*p
      enddo
      polyout(1)=-polyout(1)*p
      return
      end


!-------------------------------------------------------------------!
! _14_                    CREATE_POLY_FROM_ROOTS                    !
!-------------------------------------------------------------------!
      subroutine create_poly_from_roots(poly, degree, a, roots)
      ! Routine will build polynomial from a set of points given in 
      ! the array 'roots'. These points will be zeros of the resulting 
      ! polynomial.
      !
      ! poly - is an array of polynomial cooefs, length = degree+1, poly(1) is constant 
      !             1              2             3
      !        poly(1) x^0 + poly(2) x^1 + poly(3) x^2 + ...
      ! degree - is and integer denoting size of the 'roots' array
      ! a - gives the leading coefficient of the resutling polynomial
      ! roots - input array of points, size=degree
      !
      ! This subroutine works, but it is not optimal - it will work  
      ! up to a polynomial of degree~50, if you would like to have 
      ! more robust routine, up to a degree ~ 2000, you should 
      ! split your factors into a binary tree, and then multiply 
      ! leafs level by level from down up with subroutine like: 
      ! multiply_poly for arbitraty polynomial multiplications
      ! not multiply_poly_1.
      !
      implicit none
      integer degree            ! intent(in) 
      double complex poly(degree+1) ! intent(out)
      double complex a              ! intent(in) 
      double complex roots(degree)  ! intent(in)
      !
      integer i
      double complex zero
      parameter(zero=(0d0,0d0))
      ! 
      do i=1,degree+1
        poly(i)=zero
      enddo
      poly(1)=a  ! leading coeff of the polynomial
      do i=1,degree
        call multiply_poly_1(poly, roots(i), poly, i-1)
      enddo
      end


!-------------------------------------------------------------------!
! _15_                    CMPLX_LAGUERRE_WRAPPER                    !
!-------------------------------------------------------------------!
      subroutine cmplx_laguerre_wrapper(poly, degree, root, iter,&
     & success)
      implicit none
      ! subroutine parameters
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)

      call cmplx_laguerre(poly, degree, root, iter, success)
      end


!-------------------------------------------------------------------!
! _16_                    CMPLX_NEWTON_SPEC_WRAPPER                 !
!-------------------------------------------------------------------!
      subroutine cmplx_newton_spec_wrapper(poly, degree, root, iter, &
     & success)
      implicit none
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)

      call cmplx_newton_spec(poly, degree, root, iter, success)
      end


!-------------------------------------------------------------------!
! _17_             CMPLX_LAGUERRE2NEWTON_WRAPPER                 !
!-------------------------------------------------------------------!
      subroutine cmplx_laguerre2newton_wrapper(poly, degree, root, &
     & iter, success, starting_mode)
      implicit none
      ! subroutine parameters
      integer degree
      double complex poly(degree+1) ! intent(in)
      integer iter              ! intent(out)
      double complex root           ! intent(inout)
      logical success           ! intent(out)
      integer starting_mode     ! intent(in)

      call cmplx_laguerre2newton(poly, degree, root, iter, success, &
     & starting_mode)


      end

!-------------------------------------------------------------------!

! END: APACHE

! end of CoPSE3d: shahid mughal check!!!!
